@@ -1,7 +1,7 @@
 #   This module defines and implements a class that represents
 #   a connection to a lond daemon.
 #
-# $Id: LondConnection.pm,v 1.52 2011/01/24 11:02:32 foxr Exp $
+# $Id: LondConnection.pm,v 1.53 2012/10/01 11:00:43 foxr Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -342,9 +342,12 @@ sub Readable {
 	$self->Transition("Disconnected");
 	return -1;
     }
-    #  Append the data to the buffer.  And figure out if the read is done:
+    # If we actually got data, reset the timeout.
 
-    $self->{TimeoutRemaining}   = $self->{TimeoutValue}; # getting data resets the timeout period.
+    if (length $data) {
+	$self->{TimeoutRemaining}   = $self->{TimeoutValue}; # getting data resets the timeout period.
+    }
+    #  Append the data to the buffer.  And figure out if the read is done:
 
     &Debug(9,"Received from host: ".$data);
     $self->{TransactionReply} .= $data;
