@@ -1,7 +1,7 @@
 # The LearningOnline Network
 # Cookie Based Access Handler
 #
-# $Id: lonacc.pm,v 1.136 2011/06/13 02:44:36 raeburn Exp $
+# $Id: lonacc.pm,v 1.157 2014/06/17 23:22:10 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -89,8 +89,6 @@ store attempted access
 
 =head1 NOTABLE SUBROUTINES
 
-=over
-
 =cut
 
 
@@ -103,9 +101,9 @@ use Apache::lonnet;
 use Apache::loncommon();
 use Apache::lonlocal;
 use Apache::restrictedaccess();
-use Apache::blockedaccess(); 
+use Apache::blockedaccess();
 use Fcntl qw(:flock);
-use LONCAPA;
+use LONCAPA qw(:DEFAULT :match);
 
 sub cleanup {
     my ($r)=@_;
@@ -156,7 +154,15 @@ sub get_posted_cgi {
 	for ($i=0;$i<=$#lines;$i++) {
 	    if ($lines[$i]=~/^--\Q$contentsep\E/) {
 		if ($name) {
-		    $value=~s/[\r\n]+$//;
+                    chomp($value);
+                    if (($r->uri eq '/adm/portfolio') && 
+                        ($name eq 'uploaddoc')) {
+                        if (length($value) == 1) {
+                            $value=~s/[\r\n]$//;
+                        }
+                    } elsif ($fname =~ /\.(xls|doc|ppt)x$/i) {
+                        $value=~s/[\r\n]$//;
+                    }
                     if (ref($fields) eq 'ARRAY') {
                         next if (!grep(/^\Q$name\E$/,@{$fields}));
                     }
@@ -224,6 +230,8 @@ sub get_posted_cgi {
 
 =pod
 
+=over
+
 =item upload_size_allowed()
 
 	Perform size checks for file uploads to essayresponse items in course context.
@@ -258,20 +266,26 @@ sub upload_size_allowed {
 =item sso_login()
 
 	handle the case of the single sign on user, at this point $r->user 
-	will be set and valid now need to find the loncapa user info and possibly
-	balance them
-	returns OK if it was a SSO and user was handled
-        undef if not SSO or no means to hanle the user
+	will be set and valid; now need to find the loncapa user info, and possibly
+	balance them. If $r->user() is set this means either it was either set by
+        SSO or by checkauthen.pm, if a valid cookie was found. The latter case can
+        be identified by the third arg ($usename), except when lonacc is called in 
+        an internal redirect to /adm/switchserver (e.g., load-balancing following
+        successful authentication) -- no cookie set yet.  For that particular case
+        simply skip the call to sso_login(). 
+
+	returns OK if it was SSO and user was handled.
+        returns undef if not SSO or no means to handle the user.
         
 =cut
 
 sub sso_login {
-    my ($r,$handle) = @_;
+    my ($r,$handle,$username) = @_;
 
     my $lonidsdir=$r->dir_config('lonIDsDir');
-    if (!($r->user 
-	  && (!defined($env{'user.name'}) && !defined($env{'user.domain'}))
-	  && ($handle eq ''))) {
+    if (($r->user eq '') || ($username ne '') || ($r->user eq 'public:public') ||
+        (defined($env{'user.name'}) && (defined($env{'user.domain'}))
+	  && ($handle ne ''))) {
 	# not an SSO case or already logged in
 	return undef;
     }
@@ -281,7 +295,7 @@ sub sso_login {
     my $query = $r->args;
     my %form;
     if ($query) {
-        my @items = ('role','symb');
+        my @items = ('role','symb','iptoken');
         &Apache::loncommon::get_unprocessed_cgi($query,\@items);
         foreach my $item (@items) {
             if (defined($env{'form.'.$item})) {
@@ -290,6 +304,12 @@ sub sso_login {
         }
     }
 
+    my %sessiondata;
+    if ($form{'iptoken'}) {
+        %sessiondata = &Apache::lonnet::tmpget($form{'iptoken'});
+        my $delete = &Apache::lonnet::tmpdel($form{'token'});
+    }
+
     my $domain = $r->dir_config('lonSSOUserDomain');
     if ($domain eq '') {
         $domain = $r->dir_config('lonDefDomain');
@@ -297,7 +317,19 @@ sub sso_login {
     my $home=&Apache::lonnet::homeserver($user,$domain);
     if ($home !~ /(con_lost|no_host|no_such_host)/) {
 	&Apache::lonnet::logthis(" SSO authorized user $user ");
-	if ($r->dir_config("lonBalancer") eq 'yes') {
+        my ($is_balancer,$otherserver,$hosthere);
+        if ($form{'iptoken'}) {
+            if (($sessiondata{'domain'} eq $form{'udom'}) &&
+                ($sessiondata{'username'} eq $form{'uname'})) {
+                $hosthere = 1;
+            }
+        }
+        unless ($hosthere) {
+            ($is_balancer,$otherserver) =
+                &Apache::lonnet::check_loadbalancing($user,$domain);
+        }
+
+	if ($is_balancer) {
 	    # login but immediately go to switch server to find us a new 
 	    # machine
 	    &Apache::lonauth::success($r,$user,$domain,$home,'noredirect');
@@ -306,12 +338,24 @@ sub sso_login {
                 $env{'request.sso.reloginserver'} =
                     $r->dir_config('lonSSOReloginServer');
             }
-	    $r->internal_redirect('/adm/switchserver');
+            my $redirecturl = '/adm/switchserver';
+            if ($otherserver ne '') {
+                $redirecturl .= '?otherserver='.$otherserver;
+            }
+	    $r->internal_redirect($redirecturl);
 	    $r->set_handlers('PerlHandler'=> undef);
 	} else {
 	    # need to login them in, so generate the need data that
 	    # migrate expects to do login
-	    my %info=('ip'        => $r->connection->remote_ip(),
+	    my $ip;
+	    my $c = $r->connection;
+	    eval {
+	        $ip = $c->remote_ip();
+	    };
+	    if ($@) {
+	        $ip = $c->client_ip();
+	    }
+	    my %info=('ip'        => $ip,
 		      'domain'    => $domain,
 		      'username'  => $user,
 		      'server'    => $r->dir_config('lonHostID'),
@@ -322,6 +366,11 @@ sub sso_login {
                     $info{$item} = $form{$item};
                 }
             }
+            unless ($info{'symb'}) {
+                unless (($r->uri eq '/adm/roles') || ($r->uri eq '/adm/sso')) {
+                    $info{'origurl'} = $r->uri; 
+                }
+            }
             if ($r->dir_config("ssodirecturl") == 1) {
                 $info{'origurl'} = $r->uri;
             }
@@ -337,10 +386,8 @@ sub sso_login {
 	    $r->set_handlers('PerlHandler'=> undef);
 	}
 	return OK;
-    } elsif (defined($r->dir_config('lonSSOUserUnknownRedirect'))) {
+    } else {
 	&Apache::lonnet::logthis(" SSO authorized unknown user $user ");
-        $r->subprocess_env->set('SSOUserUnknown' => $user);
-        $r->subprocess_env->set('SSOUserDomain' => $domain);
         my @cancreate;
         my %domconfig =
             &Apache::lonnet::get_dom('configuration',['usercreation'],$domain);
@@ -354,13 +401,18 @@ sub sso_login {
                 }
             }
         }
-        if (grep(/^sso$/,@cancreate)) {
-            $r->internal_redirect('/adm/createaccount');
-        } else {
-	    $r->internal_redirect($r->dir_config('lonSSOUserUnknownRedirect'));
+        if ((grep(/^sso$/,@cancreate)) || (defined($r->dir_config('lonSSOUserUnknownRedirect')))) {
+            $r->subprocess_env->set('SSOUserUnknown' => $user);
+            $r->subprocess_env->set('SSOUserDomain' => $domain);
+            if (grep(/^sso$/,@cancreate)) {
+                $r->set_handlers('PerlHandler'=> [\&Apache::createaccount::handler]);
+                $r->handler('perl-script');
+            } else {
+	        $r->internal_redirect($r->dir_config('lonSSOUserUnknownRedirect'));
+                $r->set_handlers('PerlHandler'=> undef);
+            }
+	    return OK;
         }
-	$r->set_handlers('PerlHandler'=> undef);
-	return OK;
     }
     return undef;
 }
@@ -368,29 +420,31 @@ sub sso_login {
 sub handler {
     my $r = shift;
     my $requrl=$r->uri;
-    if (&Apache::lonnet::is_domainimage($requrl)) {
-        return OK;
-    }
 
     if ($requrl =~ m{^/res/adm/pages/[^/]+\.(gif|png)$}) {
         return OK;
     }
 
-    my $handle = &Apache::lonnet::check_for_valid_session($r);
-
-    my $result = &sso_login($r,$handle);
-    if (defined($result)) {
-	return $result;
+    if (&Apache::lonnet::is_domainimage($requrl)) {
+        return OK;
     }
 
+    my %user;
+    my $handle = &Apache::lonnet::check_for_valid_session($r,undef,\%user);
 
-    if ($r->dir_config("lonBalancer") eq 'yes') {
-	$r->set_handlers('PerlResponseHandler'=>
-			 [\&Apache::switchserver::handler]);
+    unless (($requrl eq '/adm/switchserver') && (!$r->is_initial_req())) {
+        my $result = &sso_login($r,$handle,$user{'name'});
+        if (defined($result)) {
+	    return $result;
+        }
     }
-    
+
+    my ($is_balancer,$otherserver);
+
     if ($handle eq '') {
-	$r->log_reason("Cookie $handle not valid", $r->filename); 
+        unless (($requrl eq '/adm/switchserver') && (!$r->is_initial_req())) {
+	    $r->log_reason("Cookie not valid", $r->filename);
+        }
     } elsif ($handle ne '') {
 
 # ------------------------------------------------------ Initialize Environment
@@ -407,6 +461,11 @@ sub handler {
     if ($env{'user.name'} ne '' && $env{'user.domain'} ne '') {
 # -------------------------------------------------------------- Resource State
 
+        my ($cdom,$cnum);
+        if ($env{'request.course.id'}) {
+            $cdom = $env{'course.'.$env{'request.course.id'}.'.domain'};
+            $cnum = $env{'course.'.$env{'request.course.id'}.'.num'};
+        }
 	if ($requrl=~/^\/+(res|uploaded)\//) {
 	    $env{'request.state'} = "published";
 	} else {
@@ -414,6 +473,7 @@ sub handler {
 	}
 	$env{'request.filename'} = $r->filename;
 	$env{'request.noversionuri'} = &Apache::lonnet::deversion($requrl);
+        my ($suppext,$checkabsolute);
         if ($requrl =~ m{^/adm/wrapper/ext/}) {
             my $query = $r->args;
             if ($query) {
@@ -423,17 +483,81 @@ sub handler {
                     unless ($name eq 'symb') {
                         $preserved .= $pair.'&';
                     }
+                    if (($env{'request.course.id'}) && ($name eq 'folderpath')) {
+                        if ($value =~ /^supplemental/) {
+                            $suppext = 1;
+                        }
+                    }
                 }
                 $preserved =~ s/\&$//;
                 if ($preserved) {
                     $env{'request.external.querystring'} = $preserved;
                 }
             }
+            if ($env{'request.course.id'}) {
+                $checkabsolute = 1;
+            }
+        } elsif ($env{'request.course.id'} &&
+                 (($requrl =~ m{^/adm/$match_domain/$match_username/aboutme$}) ||
+                  ($requrl =~ m{^/public/$cdom/$cnum/syllabus$}))) {
+            my $query = $r->args;
+            if ($query) {
+                foreach my $pair (split(/&/,$query)) {
+                    my ($name, $value) = split(/=/,$pair);
+                    if ($name eq 'folderpath') {
+                        if ($value =~ /^supplemental/) {
+                            $suppext = 1;
+                        }
+                    }
+                }
+            }
+            if ($requrl =~ m{^/public/$cdom/$cnum/syllabus$}) {
+                $checkabsolute = 1;
+            }
+        }
+        if ($checkabsolute) {
+            my $hostname = $r->hostname();
+            my $lonhost = &Apache::lonnet::host_from_dns($hostname);
+            if ($lonhost) {
+                my $actual = &Apache::lonnet::absolute_url($hostname);
+                my $expected = $Apache::lonnet::protocol{$lonhost}.'://'.$hostname;
+                unless ($actual eq $expected) {
+                    $env{'request.use_absolute'} = $expected;
+                }
+            }
         }
 # -------------------------------------------------------- Load POST parameters
 
 	&Apache::lonacc::get_posted_cgi($r);
 
+# ------------------------------------------------------ Check if load balancer 
+
+        my $checkexempt;
+        if ($env{'user.loadbalexempt'} eq $r->dir_config('lonHostID')) {
+            if ($env{'user.loadbalcheck.time'} + 600 > time) {
+                $checkexempt = 1;    
+            }
+        }
+        if ($env{'user.noloadbalance'} eq $r->dir_config('lonHostID')) {
+            $checkexempt = 1;
+        }
+        unless ($checkexempt) {
+            ($is_balancer,$otherserver) =
+                &Apache::lonnet::check_loadbalancing($env{'user.name'},
+                                                     $env{'user.domain'});
+        }
+        if ($is_balancer) {
+            $r->set_handlers('PerlResponseHandler'=>
+                             [\&Apache::switchserver::handler]);
+            if ($otherserver ne '') {
+                $env{'form.otherserver'} = $otherserver;
+            }
+            unless (($env{'form.origurl'}) || ($r->uri eq '/adm/roles') ||
+                    ($r->uri eq '/adm/switchserver') || ($r->uri eq '/adm/sso')) {
+                $env{'form.origurl'} = $r->uri;
+            }
+        }
+
 # ---------------------------------------------------------------- Check access
 	my $now = time;
 	if ($requrl !~ m{^/(?:adm|public|prtspool)/}
@@ -514,7 +638,7 @@ sub handler {
 # ------------------------------------- This is serious stuff, get symb and log
 		my $symb;
 		if ($query) {
-		    &Apache::loncommon::get_unprocessed_cgi($query,['symb']);
+		    &Apache::loncommon::get_unprocessed_cgi($query,['symb','folderpath']);
 		}
 		if ($env{'form.symb'}) {
 		    $symb=&Apache::lonnet::symbclean($env{'form.symb'});
@@ -542,19 +666,21 @@ sub handler {
                     if ($requrl=~m{^(/adm/.*/aboutme)/portfolio$}) {
                         $requrl = $1;
                     }
-		    $symb=&Apache::lonnet::symbread($requrl);
-		    if (&Apache::lonnet::is_on_map($requrl) && $symb &&
-			!&Apache::lonnet::symbverify($symb,$requrl)) {
-			$r->log_reason('Invalid symb for '.$requrl.': '.$symb);
-			$env{'user.error.msg'}=
-			    "$requrl:bre:1:1:Invalid Access";
-			return HTTP_NOT_ACCEPTABLE; 
-		    }
-		    if ($symb) {
-			my ($map,$mid,$murl)=
-			    &Apache::lonnet::decode_symb($symb);
-			&Apache::lonnet::symblist($map,$murl =>[$murl,$mid],
-						  'last_known' =>[$murl,$mid]);
+                    unless ($suppext) {
+		        $symb=&Apache::lonnet::symbread($requrl);
+		        if (&Apache::lonnet::is_on_map($requrl) && $symb &&
+			    !&Apache::lonnet::symbverify($symb,$requrl)) {
+			    $r->log_reason('Invalid symb for '.$requrl.': '.$symb);
+			    $env{'user.error.msg'}=
+			        "$requrl:bre:1:1:Invalid Access";
+			    return HTTP_NOT_ACCEPTABLE; 
+		        }
+		        if ($symb) {
+			    my ($map,$mid,$murl)=
+			        &Apache::lonnet::decode_symb($symb);
+			    &Apache::lonnet::symblist($map,$murl =>[$murl,$mid],
+						      'last_known' =>[$murl,$mid]);
+		        }
 		    }
 		}
 		$env{'request.symb'}=$symb;
@@ -563,9 +689,7 @@ sub handler {
 # ------------------------------------------------------- This is other content
 		&Apache::lonnet::courseacclog($requrl);    
 	    }
-            my $cdom = $env{'course.'.$env{'request.course.id'}.'.domain'};;
-            my $cnum = $env{'course.'.$env{'request.course.id'}.'.num'};;
-            if ($requrl =~ m{^/+uploaded/\Q$cdom\E/\Q$cnum\E/docs/.+\.html?$}) {
+            if ($requrl =~ m{^/+uploaded/\Q$cdom\E/\Q$cnum\E/(docs|supplemental)/.+\.html?$}) {
                 if (&Apache::lonnet::allowed('mdc',$env{'request.course.id'})) {
                     if ($query) {
                         &Apache::loncommon::get_unprocessed_cgi($query,['forceedit']);
@@ -574,9 +698,29 @@ sub handler {
                         }
                     }
                 }
+            } elsif ($requrl =~ m{^/+uploaded/\Q$cdom\E/\Q$cnum\E/portfolio/syllabus/.+\.html?$}) {
+                if (&Apache::lonnet::allowed('mdc',$env{'request.course.id'})) {
+                    if ($query) {
+                        &Apache::loncommon::get_unprocessed_cgi($query,['forceedit','editmode']);
+                        if (($env{'form.forceedit'}) || ($env{'form.editmode'})) {
+                            $env{'request.state'} = 'edit';
+                        }
+                    }
+                }
             }
 	}
 	return OK;
+    } else {
+        my $defdom=$r->dir_config('lonDefDomain');
+        ($is_balancer,$otherserver) =
+            &Apache::lonnet::check_loadbalancing(undef,$defdom);
+        if ($is_balancer) {
+            $r->set_handlers('PerlResponseHandler'=>
+                             [\&Apache::switchserver::handler]);
+            if ($otherserver ne '') {
+                $env{'form.otherserver'} = $otherserver;
+            }
+        }
     }
 # -------------------------------------------- See if this is a public resource
     if ($requrl=~m|^/+adm/+help/+|) {
