@@ -1,7 +1,7 @@
 # The LearningOnline Network
 # Login Screen
 #
-# $Id: lonlogin.pm,v 1.148 2011/06/11 18:21:07 raeburn Exp $
+# $Id: lonlogin.pm,v 1.158 2013/11/26 03:17:09 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -46,7 +46,7 @@ sub handler {
 	(join('&',$ENV{'QUERY_STRING'},$env{'request.querystring'},
 	      $ENV{'REDIRECT_QUERY_STRING'}),
 	 ['interface','username','domain','firsturl','localpath','localres',
-	  'token','role','symb']);
+	  'token','role','symb','iptoken']);
     if (!defined($env{'form.firsturl'})) {
         &Apache::lonacc::get_posted_cgi($r,['firsturl']);
     }
@@ -64,7 +64,8 @@ sub handler {
 
 
 # Are we re-routing?
-    if (-e '/home/httpd/html/lon-status/reroute.txt') {
+    my $londocroot = $r->dir_config('lonDocRoot'); 
+    if (-e "$londocroot/lon-status/reroute.txt") {
 	&Apache::lonauth::reroute($r);
 	return OK;
     }
@@ -91,7 +92,7 @@ sub handler {
             }
 	    $r->print(
                   $start_page
-                 .'<h1>'.&mt('You are already logged in!').'</h1>'
+                 .'<p class="LC_warning">'.&mt('You are already logged in!').'</p>'
                  .'<p>'.&mt('Please either [_1]continue the current session[_2] or [_3]log out[_4].',
                   '<a href="'.$dest.'">','</a>','<a href="/adm/logout">','</a>').'</p>'
                  .$end_page
@@ -102,15 +103,23 @@ sub handler {
 
 # ---------------------------------------------------- No valid token, continue
 
- # ---------------------------- Not possible to really login to domain "public"
+# ---------------------------- Not possible to really login to domain "public"
     if ($env{'form.domain'} eq 'public') {
 	$env{'form.domain'}='';
 	$env{'form.username'}='';
     }
+
+# ------ Is this page requested because /adm/migrateuser detected an IP change?
+    my %sessiondata;
+    if ($env{'form.iptoken'}) {
+        %sessiondata = &Apache::lonnet::tmpget($env{'form.iptoken'});
+        my $delete = &Apache::lonnet::tmpdel($env{'form.token'});
+    }
 # ----------------------------------------------------------- Process Interface
     $env{'form.interface'}=~s/\W//g;
 
-    my $httpbrowser=$ENV{"HTTP_USER_AGENT"};
+    (undef,undef,undef,undef,undef,undef,my $clientmobile) =
+        &Apache::loncommon::decode_user_agent();
 
     my $iconpath= 
 	&Apache::loncommon::lonhttpdurl($r->dir_config('lonIconsURL'));
@@ -118,17 +127,23 @@ sub handler {
     my $lonhost = $r->dir_config('lonHostID');
     my $domain = &Apache::lonnet::default_login_domain();
     if ($lonhost ne '') {
-        my $redirect = &check_loginvia($domain,$lonhost);
-        if ($redirect) {
-            $r->print($redirect);
-            return OK;
-        } 
+        unless ($sessiondata{'sessionserver'}) {
+            my $redirect = &check_loginvia($domain,$lonhost);
+            if ($redirect) {
+                $r->print($redirect);
+                return OK;
+            }
+        }
     }
 
-    if (($env{'form.domain'}) && 
+    if (($sessiondata{'domain'}) &&
+        (&Apache::lonnet::domain($env{'form.domain'},'description'))) {
+        $domain=$sessiondata{'domain'};
+    } elsif (($env{'form.domain'}) && 
 	(&Apache::lonnet::domain($env{'form.domain'},'description'))) {
 	$domain=$env{'form.domain'};
     }
+
     my $role    = $r->dir_config('lonRole');
     my $loadlim = $r->dir_config('lonLoadLim');
     my $uloadlim= $r->dir_config('lonUserLoadLim');
@@ -140,9 +155,20 @@ sub handler {
     my $host_name = &Apache::lonnet::hostname($lonhost);
 
 # --------------------------------------------- Default values for login fields
-
-    my $authusername=($env{'form.username'}?$env{'form.username'}:'');
-    my $authdomain=($env{'form.domain'}?$env{'form.domain'}:$domain);
+    
+    my ($authusername,$authdomain);
+    if ($sessiondata{'username'}) {
+        $authusername=$sessiondata{'username'};
+    } else {
+        $env{'form.username'} = &Apache::loncommon::cleanup_html($env{'form.username'});
+        $authusername=($env{'form.username'}?$env{'form.username'}:'');
+    }
+    if ($sessiondata{'domain'}) {
+        $authdomain=$sessiondata{'domain'};
+    } else {
+        $env{'form.domain'} = &Apache::loncommon::cleanup_html($env{'form.domain'});
+        $authdomain=($env{'form.domain'}?$env{'form.domain'}:$domain);
+    }
 
 # ---------------------------------------------------------- Determine own load
     my $loadavg;
@@ -228,23 +254,23 @@ sub handler {
         if ($spares) {
             $spares.= '<br />';
         }
-my %all_hostnames = &Apache::lonnet::all_hostnames();
-foreach my $hostid (sort
+        my %all_hostnames = &Apache::lonnet::all_hostnames();
+        foreach my $hostid (sort
 		    {
 			&Apache::lonnet::hostname($a) cmp
 			    &Apache::lonnet::hostname($b);
 		    }
 		    keys(%all_hostnames)) {
-    next if ($hostid eq $lonhost || $Apache::lonnet::spareid{$hostid});
-    my $hostname = &Apache::lonnet::hostname($hostid);
-    next if (($last eq $hostname) || ($hostname eq ''));
-    $spares.='<br /><a href="http://'.
-	$hostname.
-	'/adm/login?domain='.$authdomain.'">'.
-	$hostname.'</a>';
-    $last=$hostname;
-}
-$r->print(
+            next if ($hostid eq $lonhost || $Apache::lonnet::spareid{$hostid});
+            my $hostname = &Apache::lonnet::hostname($hostid);
+            next if (($last eq $hostname) || ($hostname eq ''));
+            $spares.='<br /><a href="http://'.
+	             $hostname.
+	             '/adm/login?domain='.$authdomain.'">'.
+	             $hostname.'</a>';
+            $last=$hostname;
+         }
+         $r->print(
    '<html>'
   .'<head><title>'
   .&mt('The LearningOnline Network with CAPA')
@@ -253,48 +279,54 @@ $r->print(
   .'<h1>'.&mt('The LearningOnline Network with CAPA').'</h1>'
   .'<img src="/adm/lonKaputt/lonlogo_broken.gif" align="right" />'
   .'<h3>'.&mt('This LON-CAPA server is temporarily not available for login.').'</h3>');
-    if ($spares) {
-        $r->print('<p>'.&mt('Please attempt to login to one of the following servers:')
-                 .'</p>'
-                 .$spares);
+        if ($spares) {
+            $r->print('<p>'.&mt('Please attempt to login to one of the following servers:')
+                     .'</p>'
+                     .$spares);
+        }
+        $r->print('</body>'
+                 .'</html>'
+        );
+        return OK;
     }
-    $r->print('</body>'
-  .'</html>'
-    );
-return OK;
-}
 
 # ----------------------------------------------- Apparently we are in business
-$servadm=~s/\,/\<br \/\>/g;
+    $servadm=~s/\,/\<br \/\>/g;
 
 # ----------------------------------------------------------- Front page design
-my $pgbg=&Apache::loncommon::designparm('login.pgbg',$domain);
-my $font=&Apache::loncommon::designparm('login.font',$domain);
-my $link=&Apache::loncommon::designparm('login.link',$domain);
-my $vlink=&Apache::loncommon::designparm('login.vlink',$domain);
-my $alink=&Apache::loncommon::designparm('login.alink',$domain);
-my $mainbg=&Apache::loncommon::designparm('login.mainbg',$domain);
-my $loginbox_bg=&Apache::loncommon::designparm('login.sidebg',$domain);
-my $loginbox_header_bgcol=&Apache::loncommon::designparm('login.bgcol',$domain);
-my $loginbox_header_textcol=&Apache::loncommon::designparm('login.textcol',$domain);
-my $logo=&Apache::loncommon::designparm('login.logo',$domain);
-my $img=&Apache::loncommon::designparm('login.img',$domain);
-my $domainlogo=&Apache::loncommon::domainlogo($domain);
-my $showbanner = 1;
-my $showmainlogo = 1;
-if (defined(&Apache::loncommon::designparm('login.showlogo_img',$domain))) {
-    $showbanner = &Apache::loncommon::designparm('login.showlogo_img',$domain);
-}
-if (defined(&Apache::loncommon::designparm('login.showlogo_logo',$domain))) {
-    $showmainlogo = &Apache::loncommon::designparm('login.showlogo_logo',$domain);
-}
-my $showadminmail=&Apache::loncommon::designparm('login.adminmail',$domain);
-my $showcoursecat =
-&Apache::loncommon::designparm('login.coursecatalog',$domain);
-my $shownewuserlink = 
-&Apache::loncommon::designparm('login.newuser',$domain);
-my $now=time;
-my $js = (<<ENDSCRIPT);
+    my $pgbg=&Apache::loncommon::designparm('login.pgbg',$domain);
+    my $font=&Apache::loncommon::designparm('login.font',$domain);
+    my $link=&Apache::loncommon::designparm('login.link',$domain);
+    my $vlink=&Apache::loncommon::designparm('login.vlink',$domain);
+    my $alink=&Apache::loncommon::designparm('login.alink',$domain);
+    my $mainbg=&Apache::loncommon::designparm('login.mainbg',$domain);
+    my $loginbox_bg=&Apache::loncommon::designparm('login.sidebg',$domain);
+    my $loginbox_header_bgcol=&Apache::loncommon::designparm('login.bgcol',$domain);
+    my $loginbox_header_textcol=&Apache::loncommon::designparm('login.textcol',$domain);
+    my $logo=&Apache::loncommon::designparm('login.logo',$domain);
+    my $img=&Apache::loncommon::designparm('login.img',$domain);
+    my $domainlogo=&Apache::loncommon::domainlogo($domain);
+    my $showbanner = 1;
+    my $showmainlogo = 1;
+    if (defined(&Apache::loncommon::designparm('login.showlogo_img',$domain))) {
+        $showbanner = &Apache::loncommon::designparm('login.showlogo_img',$domain);
+    }
+    if (defined(&Apache::loncommon::designparm('login.showlogo_logo',$domain))) {
+        $showmainlogo = &Apache::loncommon::designparm('login.showlogo_logo',$domain);
+    }
+    my $showadminmail;
+    my @possdoms = &Apache::lonnet::current_machine_domains();
+    if (grep(/^\Q$domain\E$/,@possdoms)) {
+        $showadminmail=&Apache::loncommon::designparm('login.adminmail',$domain);
+    }
+    my $showcoursecat =
+        &Apache::loncommon::designparm('login.coursecatalog',$domain);
+    my $shownewuserlink = 
+        &Apache::loncommon::designparm('login.newuser',$domain);
+    my $showhelpdesk =
+        &Apache::loncommon::designparm('login.helpdesk',$domain);
+    my $now=time;
+    my $js = (<<ENDSCRIPT);
 
 <script type="text/javascript" language="JavaScript">
 // <![CDATA[
@@ -338,7 +370,7 @@ ENDSCRIPT
 
 # --------------------------------------------------- Print login screen header
 
-my %add_entries = (
+    my %add_entries = (
 	       bgcolor      => "$mainbg",
 	       text         => "$font",
 	       link         => "$link",
@@ -346,14 +378,14 @@ my %add_entries = (
 	       alink        => "$alink",
                onload       => 'javascript:enableInput();',);
 
-$r->print(&Apache::loncommon::start_page('The LearningOnline Network with CAPA Login',$js,
+    $r->print(&Apache::loncommon::start_page('The LearningOnline Network with CAPA Login',$js,
 			       { 'redirect'       => [$expire,'/adm/roles'], 
 				 'add_entries' => \%add_entries,
 				 'only_body'   => 1,}));
 
 # ----------------------------------------------------------------------- Texts
 
-my %lt=&Apache::lonlocal::texthash(
+    my %lt=&Apache::lonlocal::texthash(
           'un'       => 'Username',
           'pw'       => 'Password',
           'dom'      => 'Domain',
@@ -371,29 +403,26 @@ my %lt=&Apache::lonlocal::texthash(
        );
 # -------------------------------------------------- Change password field name
 
-my $forgotpw = &forgotpwdisplay(%lt);
-$forgotpw .= '<br />' if $forgotpw;
-my $loginhelp = &loginhelpdisplay($authdomain,%lt);
-$loginhelp .= '<br />' if $loginhelp;
+    my $forgotpw = &forgotpwdisplay(%lt);
+    $forgotpw .= '<br />' if $forgotpw;
+    my $loginhelp = &Apache::lonauth::loginhelpdisplay($authdomain);
+    if ($loginhelp) {
+        $loginhelp = '<a href="'.$loginhelp.'">'.$lt{'help'}.'</a><br />';
+    }
 
 # ---------------------------------------------------- Serve out DES JavaScript
-{
-my $jsh=Apache::File->new($include."/londes.js");
-$r->print(<$jsh>);
-}
+    {
+    my $jsh=Apache::File->new($include."/londes.js");
+    $r->print(<$jsh>);
+    }
 # ---------------------------------------------------------- Serve rest of page
 
-$r->print(
+    $r->print(
     '<div class="LC_Box"'
    .' style="margin:0 auto; padding:10px; width:90%; height: auto; background-color:#FFFFFF;">'
 );
 
-#
-#  If the loadbalancing yielded just http:// because perhaps there's no loadbalancing?
-#  then just us a relative link to authenticate:
-#
-
-$r->print(<<ENDSERVERFORM);
+    $r->print(<<ENDSERVERFORM);
 <form name="server" action="/adm/authenticate" method="post" target="_top">
    <input type="hidden" name="logtoken" value="$logtoken" />
    <input type="hidden" name="serverid" value="$lonhost" />
@@ -406,38 +435,43 @@ $r->print(<<ENDSERVERFORM);
    <input type="hidden" name="localres" value="$env{'form.localres'}" />
   </form>
 ENDSERVERFORM
-my $coursecatalog;
-if (($showcoursecat eq '') || ($showcoursecat)) {
-    $coursecatalog = &coursecatalog_link($lt{'catalog'}).'<br />';
-}
-my $newuserlink;
-if ($shownewuserlink) {
-    $newuserlink = &newuser_link($lt{'newuser'}).'<br />';
-}
-my $logintitle =
-    '<h2 class="LC_hcell"'
-   .' style="background:'.$loginbox_header_bgcol.';'
-   .' color:'.$loginbox_header_textcol.'">'
-   .$lt{'log'}
-   .'</h2>';
+    my $coursecatalog;
+    if (($showcoursecat eq '') || ($showcoursecat)) {
+        $coursecatalog = &coursecatalog_link($lt{'catalog'}).'<br />';
+    }
+    my $newuserlink;
+    if ($shownewuserlink) {
+        $newuserlink = &newuser_link($lt{'newuser'}).'<br />';
+    }
+    my $logintitle =
+        '<h2 class="LC_hcell"'
+       .' style="background:'.$loginbox_header_bgcol.';'
+       .' color:'.$loginbox_header_textcol.'">'
+       .$lt{'log'}
+       .'</h2>';
 
-my $noscript_warning='<noscript><span class="LC_warning"><b>'
-                     .&mt('Use of LON-CAPA requires Javascript to be enabled in your web browser.')
-                    .'</b></span></noscript>';
-my $helpdeskscript;
-my $contactblock = &contactdisplay(\%lt,$servadm,$showadminmail,
-                                   $authdomain,\$helpdeskscript);
+    my $noscript_warning='<noscript><span class="LC_warning"><b>'
+                        .&mt('Use of LON-CAPA requires Javascript to be enabled in your web browser.')
+                        .'</b></span></noscript>';
+    my $helpdeskscript;
+    my $contactblock = &contactdisplay(\%lt,$servadm,$showadminmail,
+                                       $authdomain,\$helpdeskscript,
+                                       $showhelpdesk,\@possdoms);
 
-my $loginform=(<<LFORM);
+    my $mobileargs;
+    if ($clientmobile) {
+        $mobileargs = 'autocapitalize="off" autocorrect="off"'; 
+    }
+    my $loginform=(<<LFORM);
 <form name="client" action="" onsubmit="return(send())">
   <input type="hidden" name="lextkey" value="$lextkey" />
   <input type="hidden" name="uextkey" value="$uextkey" />
   <b><label for="uname">$lt{'un'}</label>:</b><br />
-  <input type="text" name="uname" id="uname" size="15" value="$authusername" readonly="readonly" /><br />
+  <input type="text" name="uname" id="uname" size="15" value="$authusername" readonly="readonly" $mobileargs /><br />
   <b><label for="upass$now">$lt{'pw'}</label>:</b><br />
   <input type="password" name="upass$now" id="upass$now" size="15" readonly="readonly" /><br />
   <b><label for="udom">$lt{'dom'}</label>:</b><br />
-  <input type="text" name="udom" id="udom" size="15" value="$authdomain" readonly="readonly" /><br />
+  <input type="text" name="udom" id="udom" size="15" value="$authdomain" readonly="readonly" $mobileargs /><br />
   <input type="submit" value="$lt{'log'}" />
 </form>
 LFORM
@@ -532,7 +566,7 @@ END
 END
     }
 
-$r->print(<<ENDDOCUMENT);
+    $r->print(<<ENDDOCUMENT);
     <div style="float: left;">
      <table border="0" cellspacing="0" cellpadding="0">
 $domainrow
@@ -619,12 +653,24 @@ sub redirect_page {
 }
 
 sub contactdisplay {
-    my ($lt,$servadm,$showadminmail,$authdomain,$helpdeskscript) = @_;
+    my ($lt,$servadm,$showadminmail,$authdomain,$helpdeskscript,$showhelpdesk,
+        $possdoms) = @_;
     my $contactblock;
-    my $showhelpdesk = 0;
-    my $requestmail = $Apache::lonnet::perlvar{'lonSupportEMail'};
-    if ($requestmail =~ m/^[^\@]+\@[^\@]+$/) {
-        $showhelpdesk = 1;
+    my $origmail;
+    if (ref($possdoms) eq 'ARRAY') {
+        if (grep(/^\Q$authdomain\E$/,@{$possdoms})) { 
+            $origmail = $Apache::lonnet::perlvar{'lonSupportEMail'};
+        }
+    }
+    my $requestmail = 
+        &Apache::loncommon::build_recipient_list(undef,'helpdeskmail',
+                                                 $authdomain,$origmail);
+    unless ($showhelpdesk eq '0') {
+        if ($requestmail =~ m/[^\@]+\@[^\@]+/) {
+            $showhelpdesk = 1;
+        } else {
+            $showhelpdesk = 0;
+        }
     }
     if ($servadm && $showadminmail) {
         $contactblock .= $$lt{'servadm'}.':<br />'.
@@ -637,7 +683,8 @@ sub contactdisplay {
 <script type="text/javascript">
 // <![CDATA[
 function helpdesk() {
-    var codedom = document.client.udom.value;
+    var possdom = document.client.udom.value;
+    var codedom = possdom.replace( new RegExp("[^A-Za-z0-9.\\-]","g"),'');
     if (codedom == '') {
         codedom = "$authdomain";
     }
@@ -661,25 +708,6 @@ sub forgotpwdisplay {
     return;
 }
 
-sub loginhelpdisplay {
-    my ($authdomain,%lt) = @_;
-    my $login_help = 1;
-    if ($login_help) {
-        my $dom = $authdomain;
-        if ($dom eq '') {
-            $dom = &Apache::lonnet::default_login_domain();
-        }
-        my %helpconfig = &Apache::lonnet::get_dom('configuration',['helpsettings'],$dom);
-        my $loginhelp_url = $helpconfig{'helpsettings'}{'loginhelpurl'};
-        if ($loginhelp_url ne '') {
-            return '<a href="'.$loginhelp_url.'">'.$lt{'help'}.'</a>';
-        } else {
-            return '<a href="/adm/loginproblems.html">'.$lt{'help'}.'</a>';
-        }
-    }
-    return;
-}
-
 sub coursecatalog_link {
     my ($linkname) = @_;
     return <<"END";
