@@ -1,7 +1,7 @@
 # The LearningOnline Network
 # Switch Servers Handler
 #
-# $Id: switchserver.pm,v 1.28.2.4 2011/08/02 14:20:42 raeburn Exp $
+# $Id: switchserver.pm,v 1.32 2013/09/29 00:49:24 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -59,7 +59,6 @@ sub do_redirect {
     my $start_page = 
 	&Apache::loncommon::start_page('Switching Server ...',undef,
 				       {'redirect'       => [0.5,$url],
-                                        'no_inline_link' => 1,
 					'only_body'      => $only_body,});
     my $end_page   = &Apache::loncommon::end_page();
     $r->print($start_page.$extra_text.$end_page);
@@ -67,7 +66,6 @@ sub do_redirect {
         $r->register_cleanup(\&flush_course_logs);
     }
     return OK;
-
 }
 
 sub flush_course_logs {
@@ -172,17 +170,16 @@ sub handler {
 	return OK;
     }
 # -------------------------------------------------------- Menu script and info
-
-    my $windowinfo=
-        &Apache::lonmenu::close().
-        &Apache::lonnavmaps::close();
+	   
 # ---------------------------------------------------------------- Get handover
 
+    my ($is_balancer) = &Apache::lonnet::check_loadbalancing($env{'user.name'},$env{'user.domain'});
     my %info=('ip'       => $ENV{'REMOTE_ADDR'},
 	      'domain'   => $env{'user.domain'},
 	      'username' => $env{'user.name'},
 	      'role'     => $env{'form.role'},
-	      'server'   => $r->dir_config('lonHostID'));
+	      'server'   => $r->dir_config('lonHostID'),
+	      'balancer' => $is_balancer);
     if ($env{'form.origurl'}) {
         $info{'origurl'} = $env{'form.origurl'};
     }
@@ -201,7 +198,7 @@ sub handler {
 	'&amp;username='.$env{'user.name'}.
 	'&amp;token='.$token;
 # --------------------------------------------------------------- Screen Output
-    return &do_redirect($r,$url,0,$windowinfo);
+    return &do_redirect($r, $url, 0);
 }
 
 1;
