@@ -1,5 +1,14 @@
+#
+# as this is a standalong file..that may ot be able to access the LONCAPA
+# module, I'm leaving the temp directory knoweldge uncentralized
+#    - R. Fox 7/1/2011
+#
+
 use strict;
-my $result=`ls -l /proc/*/fd/* | grep /home/httpd/perl/tmp/`;
+
+my $loncapa_tempdir = '/home/httpd/perl/tmp'; # configure the temp dir here.
+
+my $result=`ls -l /proc/*/fd/* | grep $loncapa_tempdir`;
 my @result=split("\n",$result);
 my (@pids,@files);
 foreach my $line (@result) {
