@@ -1,7 +1,7 @@
 #!/usr/bin/perl
 #
 #Automated Enrollment script
-# $Id: Autoenroll.pl,v 1.31 2009/01/08 16:58:04 raeburn Exp $
+# $Id: Autoenroll.pl,v 1.33 2013/03/01 05:20:01 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -54,7 +54,7 @@
         my $run_enroll = 0;
         my $domsettings;
         my %domconfig =
-            &Apache::lonnet::get_dom('configuration',['autoenroll'],$dom);
+            &Apache::lonnet::get_dom('configuration',['autoenroll','coursedefaults'],$dom);
         if (ref($domconfig{'autoenroll'}) eq 'HASH') {
             $domsettings = $domconfig{'autoenroll'};
             if ($domsettings->{'run'} eq '1') {
@@ -74,15 +74,22 @@
         my %enrollvar = ();
         my %reply = ();
         my %LC_code = ();
+        my ($showcredits,$domdefcredits);
+        if ((ref($domconfig{'coursedefaults'}) eq 'HASH') &&
+            (ref($domconfig{'coursedefaults'}{'coursecredits'}) eq 'HASH')) {
+            if ($domconfig{'coursedefaults'}{'coursecredits'}{'official'}) {
+                $showcredits = 1;
+                $domdefcredits = $domconfig{'coursedefaults'}{'coursecredits'}{'official'};
+            }
+        }
         foreach my $key (sort keys %courses) {
             my %args = (
                          one_time => 1,
                        );
-            my %coursehash = &Apache::lonnet::coursedescription($key,\%args);
-            my $crs = $coursehash{'num'};
-            next if ($coursehash{'domain'} ne $dom);
 # Get course settings
-            my %settings = &Apache::lonnet::dump('environment',$dom,$crs);
+            my %settings = &Apache::lonnet::coursedescription($key,\%args);
+            my $crs = $settings{'num'};
+            next if ($settings{'domain'} ne $dom);
             %{$enrollvar{$crs}} = ();
             @{$affiliates{$crs}} = ();
             %{$LC_code{$crs}} = ();
@@ -93,7 +100,7 @@
                     $enrollvar{$crs}{$item} = &HTML::Entities::decode($settings{$item});  
                 } elsif ($item eq 'default_enrollment_start_date') {
                     $enrollvar{$crs}{startdate} = $settings{$item};
-                }  elsif ($item eq 'default_enrollment_end_date') {
+                } elsif ($item eq 'default_enrollment_end_date') {
                     $enrollvar{$crs}{enddate} = $settings{$item};
                 }
             }
@@ -118,7 +125,7 @@
                         if ($sec =~ m/^(\w+):(\w*)$/ ) {
                             my $course_id = $enrollvar{$crs}{coursecode}.$1;
                             my $gp = $2;
-                            if (!grep/^$course_id$/,@{$affiliates{$crs}}) {
+                            if (!grep/^\Q$course_id\E$/,@{$affiliates{$crs}}) {
                                 push @{$affiliates{$crs}}, $course_id;
                                 $LC_code{$crs}{$course_id} = $gp; 
                             }
@@ -128,7 +135,7 @@
                         if ($xlist =~ m/^([^:]+):(\w*)$/) {
                             my $course_id = $1;
                             my $gp = $2;
-                            if (!grep/^$course_id$/,@{$affiliates{$crs}}) {
+                            if (!grep/^\Q$course_id\E$/,@{$affiliates{$crs}}) {
                                 push @{$affiliates{$crs}}, $course_id;
                                 $LC_code{$crs}{$course_id} = $gp;
                             }
@@ -149,7 +156,11 @@
             if ($reply{$crs} > 0) {
                 if ( ($enrollvar{$crs}{autostart} < $timenow) && ( ($enrollvar{$crs}{autoend} > $timenow) || ($enrollvar{$crs}{autoend} == 0) ) ) {
                     if (($enrollvar{$crs}{autoadds} == 1) || ($enrollvar{$crs}{autodrops} == 1)) {
-                        my ($changecount,$response) = &LONCAPA::Enrollment::update_LC($dom,$crs,$enrollvar{$crs}{autoadds},$enrollvar{$crs}{autodrops},$enrollvar{$crs}{startdate},$enrollvar{$crs}{enddate},$enrollvar{$crs}{authtype},$enrollvar{$crs}{autharg},\@{$affiliates{$crs}},\%{$LC_code{$crs}},\$logmsg,\$newusermsg,'automated');
+                        my $defaultcredits = $domdefcredits;
+                        if ($enrollvar{$crs}{defaultcredits}) {
+                            $defaultcredits = $enrollvar{$crs}{defaultcredits};
+                        }
+                        my ($changecount,$response) = &LONCAPA::Enrollment::update_LC($dom,$crs,$enrollvar{$crs}{autoadds},$enrollvar{$crs}{autodrops},$enrollvar{$crs}{startdate},$enrollvar{$crs}{enddate},$enrollvar{$crs}{authtype},$enrollvar{$crs}{autharg},$showcredits,$defaultcredits,$affiliates{$crs},$LC_code{$crs},\$logmsg,\$newusermsg,'automated');
                         print $fh &mt('Messages start for [_1]',$crs)."\n";
                         print $fh "$logmsg\n";
                         print $fh &mt('Messages end for [_1]',$crs)."\n";
