@@ -1,7 +1,7 @@
 #!/usr/bin/perl
 #
 # Automated Userinfo update script
-# $Id: Autoupdate.pl,v 1.19 2010/03/26 13:19:04 raeburn Exp $
+# $Id: Autoupdate.pl,v 1.22 2011/12/18 01:23:21 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -65,16 +65,22 @@
         # get courseIDs for domain
         my %courses=&Apache::lonnet::courseiddump($dom,'.',1,'.','.','.',1,[$hostid],'.');
         # get user information
-        my (%instusers,%instids);
-        next if (&localenroll::allusers_info($dom,\%instusers,\%instids) ne 'ok');
-        my (%users,%unamechg,%possnames);
+        my (%users,%instusers,%instids);
+        my $dir = $Apache::lonnet::perlvar{lonUsersDir}.'/'.$dom;
+        &descend_tree($dom,$dir,0,\%users,\%courses);
+        my $resp = &localenroll::allusers_info($dom,\%instusers,\%instids,\%users);
+        if ($resp ne 'ok') {
+            print $fh &mt('Problem retrieving institutional data for users in domain: [_1].',$dom)."\n".
+                      &mt('Error: [_1].',$resp)."\n".
+                      "-- \n".&Apache::lonlocal::locallocaltime(time).' '.
+                      &mt('Autoupdate messages end')."\n*******************\n\n";
+            next;
+        }
+        my (%unamechg,%possnames);
         my @types = ('active','future');
         my @roles = ('st');
         my @cdoms = ($dom);
-        my $dir = $Apache::lonnet::perlvar{lonUsersDir}.'/'.$dom;
-        &descend_tree($dir,0,\%users);
         foreach my $uname (sort(keys(%users))) {
-            next if ($courses{$dom.'_'.$uname});
             my %userhash = &Apache::lonnet::userenvironment($dom,$uname,@info);
             my (@inststatuses);
             if (!$userhash{'internalname'}) {
@@ -185,7 +191,7 @@
     }
 
 sub descend_tree {
-    my ($dir,$depth,$alldomusers) = @_;
+    my ($dom,$dir,$depth,$alldomusers,$coursesref) = @_;
     if (-d $dir) {
         opendir(DIR,$dir);
         my @contents = grep(!/^\./,readdir(DIR));
@@ -193,8 +199,11 @@ sub descend_tree {
         $depth ++;
         foreach my $item (@contents) {
             if ($depth < 4) {
-                &descend_tree($dir.'/'.$item,$depth,$alldomusers);
+                &descend_tree($dom,$dir.'/'.$item,$depth,$alldomusers,$coursesref);
             } elsif (-e $dir.'/'.$item.'/passwd') {
+                if (ref($coursesref) eq 'HASH') {
+                    next if (exists($coursesref->{$dom.'_'.$item}));
+                }
                 $$alldomusers{$item} = '';
             }
         }
