@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # input  definitons
 #
-# $Id: inputtags.pm,v 1.271.2.9 2011/10/10 17:47:58 raeburn Exp $
+# $Id: inputtags.pm,v 1.324 2014/06/19 19:37:35 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -125,6 +125,37 @@ sub initialize_inputtags {
     %Apache::inputtags::submission_display=();
 }
 
+#
+#  provides the onblur binding for spellchecking.  This could be an
+#  empty string if spellchecking was not enabled.
+#  Jquery selector binding is done rather than setting an onblur
+#  attribute because we'll need to set the element's spellcheck language
+#  option dynamically so we need $(this) to be defined.
+#
+# @param id   - The element id to bind.
+# @param lang - Language in which spellchecking is desired.
+#               if undef, nothing is generated.  
+# @return string - onblur specification to do the requested spellchecking.
+#
+sub spellcheck_onblur {
+    my ($id, $lang) = @_;
+    my $result = '';
+    if ($lang) {
+
+	$result = <<JAVASCRIPT;
+<script type="text/javascript">
+\$('\#$id').blur(function() {
+    doSpellcheck('\#$id', '$lang');
+ });
+</script>
+
+JAVASCRIPT
+
+
+    }
+    return $result;
+}
+
 sub check_for_duplicate_ids {
     my %check;
     foreach my $id (@Apache::inputtags::partlist,
@@ -185,7 +216,16 @@ sub start_textfield {
             }
         }
         unless ($newvariation) {
-	    $oldresponse = &HTML::Entities::encode($Apache::lonhomework::history{"resource.$partid.$resid.submission"},'<>&"');
+            if ((($env{'form.grade_username'} eq '') && ($env{'form.grade_domain'} eq '')) ||
+                (($env{'form.grade_username'} eq $env{'user.name'}) &&
+                 ($env{'form.grade_domain'} eq $env{'user.domain'}))) {
+                $oldresponse = $Apache::lonhomework::history{"resource.$partid.$resid.submission"};
+            } elsif (($Apache::lonhomework::history{"resource.$partid.type"} eq 'anonsurvey') ||
+                    ($Apache::lonhomework::history{"resource.$partid.type"} eq 'anonsurveycred')) {
+                $oldresponse = '* '.&mt('(only shown to submitter)').' *';
+            } else {
+                $oldresponse = $Apache::lonhomework::history{"resource.$partid.$resid.submission"};
+            }
         }
 	if ($Apache::inputtags::status[-1] eq 'CAN_ANSWER') {
 	    my $cols = &Apache::lonxml::get_param('cols',$parstack,$safeeval);
@@ -194,13 +234,19 @@ sub start_textfield {
 	    if ( $rows eq '') { $rows = 16; }
 	    my $addchars=&Apache::lonxml::get_param('addchars',$parstack,$safeeval);
 	    $result='';
+	    my $tagident = 'HWVAL_' . $resid;
 	    if ($addchars) {
-		$result.=&addchars('HWVAL_'.$resid,$addchars);
+		$result.=&addchars($tagident, $addchars);
 	    }
-            my $textareaclass = 'class="LC_richDetectHtml"';
-	    $result.= '<textarea wrap="hard" name="HWVAL_'.$resid.'" id="HWVAL_'.$resid.'" '.
-		      'rows="'.$rows.'" cols="'.$cols.'" '.$textareaclass.'>'.
-                      $oldresponse;
+            my $textareaclass;
+            unless (&Apache::londefdef::is_inside_of($tagstack,
+                                                    'externalresponse')) {
+                $textareaclass = 'class="LC_richDetectHtml spellchecked"';
+            }
+	    $result.= '<textarea wrap="hard" name="'.$tagident.'" id="'.$tagident.'" ' .
+		      'rows="'.$rows.'" cols="'.$cols.'" '.$textareaclass
+		      .'>'.
+                      &HTML::Entities::encode($oldresponse,'<>&"');
 	    if ($oldresponse ne '') {
 
 		#get rid of any startup text if the user has already responded
@@ -212,7 +258,8 @@ sub start_textfield {
 		&& &Apache::londefdef::is_inside_of($tagstack,
 						    'essayresponse') ) {
 		$result='<table class="LC_pastsubmission"><tr><td>'.
-		    $oldresponse.'</td></tr></table>';
+		    &HTML::Entities::encode($oldresponse,'"<>&').
+                    '</td></tr></table>';
 	    }
 	    #get rid of any startup text
 	    &Apache::lonxml::get_all_text("/textfield",$parser,$style);
@@ -233,10 +280,13 @@ sub start_textfield {
 	my $bodytext=&Apache::lonxml::get_all_text("/textfield",$parser,
 						   $style);
 	$result.=&Apache::edit::editfield($token->[1],$bodytext,'Text you want to appear by default:',80,2);
+        my $spell_langs = &spelling_languages();
+	$result .= &Apache::edit::select_arg('Spellcheck for:', 'spellcheck',
+					     $spell_langs, $token);
     } elsif ($target eq 'modified') {
 	my $constructtag=&Apache::edit::get_new_args($token,$parstack,
 						     $safeeval,'rows','cols',
-						     'addchars');
+						     'addchars', 'spellcheck');
 	if ($constructtag) {
 	    $result = &Apache::edit::rebuild_tag($token);
 	} else {
@@ -251,10 +301,18 @@ sub start_textfield {
 	    for (my $i=0;$i<int $number_of_lines*2;$i++) {$result.='\strut \\\\ ';}
 	    $result.='\strut \\\\\strut \\\\\strut \\\\\strut \\\\}}}';
 	} else {
-	    my $TeXwidth=$width_of_box/80;
-	    $result = '\vskip 1 mm \fbox{\fbox{\parbox{'.$TeXwidth.'\textwidth-5mm}{';
-	    for (my $i=0;$i<int $number_of_lines*2;$i++) {$result.='\strut \\\\ ';}
-	    $result.='}}}\vskip 2 mm ';
+            if ($env{'form.pdfFormFields'} eq 'yes') {
+                my $fieldname = $env{'request.symb'}.
+                                '&part_'. $Apache::inputtags::part.
+                                '&textresponse'.
+                                '&HWVAL_' . $Apache::inputtags::response['-1'];
+                $result.='\TextField[name='.$fieldname.',multiline=true,height=6\baselineskip,width=270,borderwidth=0,backgroundcolor={.85 .85 .85}]\\';
+            } else {
+                my $TeXwidth=$width_of_box/80;
+                $result = '\vskip 1 mm \fbox{\fbox{\parbox{'.$TeXwidth.'\textwidth-5mm}{';
+                for (my $i=0;$i<int $number_of_lines*2;$i++) {$result.='\strut \\\\ ';}
+                $result.='}}}\vskip 2 mm ';
+            }
 	}
     }
     return $result;
@@ -264,9 +322,14 @@ sub end_textfield {
     my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;
     my $result;
     if ($target eq 'web') {
+	my $spellcheck = &Apache::lonxml::get_param('spellcheck', $parstack, $safeeval);
 	$Apache::lonxml::evaluate++;
 	if ($Apache::inputtags::status[-1] eq 'CAN_ANSWER') {
-	    return "</textarea>";
+	    my $resid = $Apache::inputtags::response[-1];
+	    my $tagident = 'HWVAL_' . $resid;
+	    my $result =  "</textarea>";
+	    $result .= &spellcheck_onblur($tagident, $spellcheck);
+	    return $result;
 	}
     } elsif ($target eq 'edit') {
 	$result=&Apache::edit::end_table();
@@ -330,6 +393,12 @@ sub start_textline {
     my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;
     my $result = "";
     my $input_id = &start_input($parstack,$safeeval);
+
+    # The spellcheck attribute 
+    # 1. enables spellchecking.
+    # 2. Provides the language code in which the spellchecking will be performed.
+
+    my $spellcheck = &Apache::lonxml::get_param('spellcheck', $parstack, $safeeval);
     if ($target eq 'web') {
 	$Apache::lonxml::evaluate--;
 	my $partid=$Apache::inputtags::part;
@@ -352,9 +421,19 @@ sub start_textline {
                 }
             }
             unless ($newvariation) {
-	        $oldresponse = $Apache::lonhomework::history{"resource.$partid.$id.submission"};
+                if ((($env{'form.grade_username'} eq '') && ($env{'form.grade_domain'} eq '')) ||
+                    (($env{'form.grade_username'} eq $env{'user.name'}) &&
+                     ($env{'form.grade_domain'} eq $env{'user.domain'}))) {
+                    $oldresponse = $Apache::lonhomework::history{"resource.$partid.$id.submission"};
+                } elsif (($Apache::lonhomework::history{"resource.$partid.type"} eq 'anonsurvey') ||
+                        ($Apache::lonhomework::history{"resource.$partid.type"} eq 'anonsurveycred') ||
+                        ($Apache::lonhomework::type eq 'anonsurvey') ||
+                        ($Apache::lonhomework::type eq 'anonsurveycred')) {
+                        $oldresponse = '* '.&mt('(only shown to submitter)').' *';
+                } else {
+                    $oldresponse = $Apache::lonhomework::history{"resource.$partid.$id.submission"};
+                }
 	        &Apache::lonxml::debug("oldresponse $oldresponse is ".ref($oldresponse));
-
 	        if (ref($oldresponse) eq 'ARRAY') {
 		    $oldresponse = $oldresponse->[$#Apache::inputtags::inputlist];
 	        }
@@ -381,8 +460,15 @@ sub start_textline {
 		if ($Apache::inputtags::status[-1] eq 'CANNOT_ANSWER') {
 		    $name = "none";
 		}
-		$result.= '<input onkeydown="javascript:setSubmittedPart(\''.$partid.'\');" type="text" '.$readonly.' name="'.$name.'" value="'.
-		    $oldresponse.'" size="'.$size.'"'.$maxlength.' />';
+		$result.= '<input onkeydown="javascript:setSubmittedPart(\''.$partid.'\');"'
+		     . ' type="text" '
+		     . $readonly.' name="'. $name . '"'
+		     . ' id="' . $name . '"'
+		     . ' value="'.  $oldresponse.'"'
+		     . ' class="spellchecked"  size="'.$size.'"'.$maxlength
+		     . '/>';
+
+		$result .= &spellcheck_onblur($name, $spellcheck);
 	    }
 	    if ($Apache::lonhomework::type eq 'exam'
 		&& &needs_exam_box($tagstack)) {
@@ -401,12 +487,15 @@ sub start_textline {
 				    'addchars',$token,10);
         $result.=&Apache::edit::select_arg('Readonly:','readonly',
 					   ['no','yes'],$token);
+        my $spell_langs = &spelling_languages();
+	$result.=&Apache::edit::select_arg('Spellcheck for:', 'spellcheck',
+					   $spell_langs, $token);
 	$result.=&Apache::edit::end_row();
 	$result.=&Apache::edit::end_table();
     } elsif ($target eq 'modified') {
 	my $constructtag=&Apache::edit::get_new_args($token,$parstack,
 						     $safeeval,'size',
-						     'addchars','readonly');
+						     'addchars','readonly', 'spellcheck');
 	if ($constructtag) { $result = &Apache::edit::rebuild_tag($token); }
     } elsif ($target eq 'tex' 
 	     && $Apache::lonhomework::type ne 'exam') {
@@ -479,6 +568,7 @@ sub end_hiddenline {
     return "";
 }
 
+
 sub start_hiddensubmission {
     my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;
     my $result = "";
@@ -562,7 +652,7 @@ sub file_selector {
 	    &mt('Allowed filetypes: [_1]','<b>'.$uploadedfiletypes.'</b>').'<br />';
     }
     if ($maxfilesize) {
-        $constraints .= &mt('Combined size of all files not to exceed: [_1] MB[_2].',
+        $constraints .= &mt('Combined size of all files not to exceed: [_1] MB.',
                         '<b>'.$maxfilesize.'</b>').'<br />';
     }
     if ($constraints) {
@@ -571,13 +661,25 @@ sub file_selector {
     if ($which eq 'uploadonly' || $which eq 'both') { 
 	$result.=&mt('Submit a file: (only one file per submission)').
 	    ' <br /><input type="file" size="50" name="HWFILE'.
-	    $jspart.'_'.$id.'" /><br />';
+	    $jspart.'_'.$id.'" id="HWFILE'.$jspart.'_'.$id.'" /><br />';
     }
     if ( $which eq 'both') {
 	$result.='<br />'.'<strong>'.&mt('OR:').'</strong><br />';
     }
-    if ($which eq 'portfolioonly' || $which eq 'both') { 
-	$result.=$extratext.'<a href='."'".'javascript:void(window.open("/adm/portfolio?mode=selectfile&amp;fieldname='.$env{'form.request.prefix'}.'HWPORT'.$jspart.'_'.$id.'","cat","height=600,width=800,scrollbars=1,resizable=1,menubar=2,location=1"))'."'".'>'.
+    if ($which eq 'portfolioonly' || $which eq 'both') {
+        my $symb = $env{'request.symb'};
+        (undef,undef,my $res)=&Apache::lonnet::decode_symb($symb);
+        my $showsymb;
+        # If resource is a .task and URL is unencrypted, include symb in query string
+        # for url opened in portfolio file selection window. Can be used to override
+        # blocking of portfolio access resulting from an exam event in a different course. 
+        if ($res =~ /\.task$/i) {
+            my $encsymb = &Apache::lonenc::check_encrypt($symb);
+            if ($symb eq $encsymb) {
+                $showsymb = $symb;
+            }
+        }
+	$result.=$extratext.'<a href='."'".'javascript:void(window.open("/adm/portfolio?mode=selectfile&amp;fieldname='.$env{'form.request.prefix'}.'HWPORT'.$jspart.'_'.$id.'&amp;symb='.$showsymb.'","cat","height=600,width=800,scrollbars=1,resizable=1,menubar=2,location=1"))'."'".'>'.
 	    &mt('Select Portfolio Files: (one or more files per submission)').'</a><br />'.
 	    '<input type="text" size="50" name="HWPORT'.$jspart.'_'.$id.'" value="" />'.
 	    '<br />';
@@ -594,7 +696,8 @@ sub current_file_submissions {
     my $uploadedfile=$Apache::lonhomework::history{"resource.$part.$id.uploadedfile"};
     my $portfiles=$Apache::lonhomework::history{"resource.$part.$id.portfiles"};
     return if (($uploadedfile eq '') && ($portfiles !~/[^\s]/));
-    my $header = &Apache::loncommon::start_data_table().
+    my $header = &portpath_popup_js().
+                 &Apache::loncommon::start_data_table().
                  &Apache::loncommon::start_data_table_header_row();
     if ($Apache::inputtags::status[-1] eq 'CAN_ANSWER') {
         $header .= '<th>'.&mt('Delete?').'</th>';
@@ -660,13 +763,24 @@ sub current_file_submissions {
                                  ' value="'.$portfile.'" id="HWFILE'.$jspart.'_'.$id.'_'.$num.'_delete" /></td>'."\n";
                             $num ++;
                         }
-                        my $showname = $rows{$url}{path}.$name;
+                        my $pathid = 'HWFILE'.$jspart.'_'.$id.'_'.$num.'_path';
+                        my $pathidtext = $pathid.'text';
+                        my ($showname,$showpath);
                         if ($legacy{$url}) {
                             $showname = $name.' '.&mt('not in portfolio');
+                        } else {
+                            $showname = $name;
+                            $showpath = '<br />'. 
+                                        '<span id="'.$pathidtext.'" class="LC_cusr_subheading">'.
+                                        '<a href="javascript:showPortPath('."'$pathid','$pathidtext'".');" '.
+                                        'class="LC_menubuttons_link">'.
+                                        &mt('(Show path)').'</a></span>'.
+                                        '<div id="'.$pathid.'" class="LC_dccid">'.$rows{$url}{path}.$name.
+'</div>';
                         }
                         $result .= 
                             '<td><a href="'.$link.'"><img src="'.$icon.
-                            '" border="0" alt="" />'.$showname.'</a></td>'."\n".
+                            '" border="0" alt="" />'.$showname.'</a>'.$showpath.'</td>'."\n".
                             '<td align="right" valign="bottom">'.$rows{$url}{size}.'</td>'."\n".
                             '<td align="right" valign="bottom">'.$rows{$url}{lastmodified}.'</td>'."\n".
                             &Apache::loncommon::end_data_table_row();
@@ -676,9 +790,11 @@ sub current_file_submissions {
         }
     }
     if ($header_shown) {
-        $result .= &Apache::loncommon::end_data_table().
-                   '<br /><span class="LC_warning">'.
-                   &mt('Exclude existing file(s) from grading by checking the "Delete?" checkbox(es) and clicking "Submit Answer"').'</span>';
+        $result .= &Apache::loncommon::end_data_table();
+        if ($Apache::inputtags::status[-1] eq 'CAN_ANSWER') {
+            $result .= '<br /><span class="LC_warning">'.
+                       &mt('Exclude existing file(s) from grading by checking the "Delete?" checkbox(es) and clicking "Submit Answer"').'</span>';
+        }
     }
     if (@bad_file_list) {
         my $bad_files = '<span class="LC_filename">'.
@@ -719,6 +835,43 @@ sub current_file_info {
     return ($status,\%info,$error);
 }
 
+sub portpath_popup_js {
+    my %lt = &Apache::lonlocal::texthash(
+                                          show => '(Show path)',
+                                          hide => '(Hide)',
+                                        );
+    return <<"END";
+<script type="text/javascript"> 
+// <![CDATA[
+
+function showPortPath(id,idtext) {
+    document.getElementById(id).style.display='block';
+    document.getElementById(id).style.textAlign='left';
+    document.getElementById(id).style.textFace='normal';
+    if (document.getElementById(idtext)) {
+        document.getElementById(idtext).innerHTML ='<a href="javascript:hidePortPath(\\''+id+'\\',\\''+idtext+'\\'); '+
+                                                   '"class="LC_menubuttons_link">$lt{'hide'}</a>&nbsp;';
+    }
+    return;
+}
+
+function hidePortPath(id,idtext) {
+    if (document.getElementById(id)) {
+        document.getElementById(id).style.display='none';
+    }
+    if (document.getElementById(idtext)) {
+        document.getElementById(idtext).innerHTML ='<a href="javascript:showPortPath(\\''+id+'\\',\\''+idtext+'\\');" '+
+                                                   'class="LC_menubuttons_link">$lt{'show'}</a>';
+    }
+    return;
+}
+
+// ]]>
+</script>
+
+END
+}
+
 sub valid_award {
     my ($award) =@_;
     foreach my $possibleaward ('EXTRA_ANSWER','MISSING_ANSWER', 'ERROR',
@@ -728,7 +881,7 @@ sub valid_award {
 			       'UNIT_FAIL', 'NO_UNIT',
 			       'UNIT_NOTNEEDED', 'WANTED_NUMERIC',
 			       'BAD_FORMULA', 'NOT_FUNCTION', 'WRONG_FORMAT', 
-			       'INTERNAL_ERROR', 'SIG_FAIL', 'INCORRECT', 
+                               'INTERNAL_ERROR', 'SIG_FAIL', 'INCORRECT', 
 			       'MISORDERED_RANK', 'INVALID_FILETYPE',
                                'EXCESS_FILESIZE', 'FILENAME_INUSE', 
 			       'DRAFT', 'SUBMITTED', 'SUBMITTED_CREDIT', 
@@ -746,7 +899,7 @@ sub valid_award {
 		  'UNIT_INVALID_INSTRUCTOR', 'UNIT_INVALID_STUDENT',
 		  'UNIT_IRRECONCIBLE', 'UNIT_FAIL', 'NO_UNIT',
 		  'UNIT_NOTNEEDED', 'WANTED_NUMERIC', 'BAD_FORMULA',  'NOT_FUNCTION', 
-		  'WRONG_FORMAT', 'INTERNAL_ERROR',
+                  'WRONG_FORMAT', 'INTERNAL_ERROR',
 		  'COMMA_FAIL', 'SIG_FAIL', 'INCORRECT', 'MISORDERED_RANK',
 		  'INVALID_FILETYPE', 'EXCESS_FILESIZE', 'FILENAME_INUSE', 
                   'DRAFT', 'SUBMITTED',
@@ -862,13 +1015,13 @@ sub decideoutput {
     my $part = $Apache::inputtags::part;
     my $tohandgrade = &Apache::lonnet::EXT("resource.$part.handgrade");
     my $handgrade = ('yes' eq lc($tohandgrade)); 
- 
 #
 # Should "Computer's Answer" be displayed?
 # Should not be displayed if still answerable,
 # if the problem is handgraded,
 # or if the problem does not give a correct answer
 #
+    
     my $computer = ($handgrade || $nocorrect)? ''
 	                       : " ".&mt("Computer's answer now shown above.");
     &Apache::lonxml::debug("handgrade has :$handgrade:");
@@ -886,7 +1039,7 @@ sub decideoutput {
 	    $css_class=$possible_class{'charged_try'};
 	}
 	if ($handgrade || 
-            ($env{'request.filename'}=~ m{/res/lib/templates/(examupload|DropBox)\.problem$})) {
+            ($env{'request.filename'}=~/\/res\/lib\/templates\/(examupload|DropBox).problem$/)) {
 	    $message = &mt("A score has been assigned.");
 	    $added_computer_text=1;
 	} else {
@@ -911,7 +1064,13 @@ sub decideoutput {
 		}
 	    }
 	}
-	$button=0;
+        if ($awarded >= 1) {
+            $button=0;
+        } elsif (&Apache::lonnet::EXT("resource.$part.retrypartial") !~/^1|on|yes$/i) {
+            $button=0;
+        } else {
+            $button=1;
+        }
 	$previousmsg='';
     } elsif ($solved =~ /^excused/) {
 	if ($target eq 'tex') {
@@ -983,7 +1142,7 @@ sub decideoutput {
 	$css_class=$possible_class{'not_charged_try'};
 	$button=1;
     } elsif ($award eq 'EXCESS_FILESIZE') {
-        $message = &mt('Submission won\'t be graded. The combined size of submitted files exceeded the amount allowed.');
+        $message = &mt("Submission won't be graded. The combined size of submitted files exceeded the amount allowed.");
         $css_class=$possible_class{'not_charged_try'};
         $button=1;
     } elsif ($award eq 'FILENAME_INUSE') {
@@ -993,7 +1152,7 @@ sub decideoutput {
         } else {
             $message .= '<br />';
         }
-        $message .= &mt('Please use a different file name.');
+        $message .= &mt('Please use a different filename.');
         $css_class=$possible_class{'not_charged_try'};
         $button=1;
     } elsif ($award eq 'INVALID_FILETYPE') {
@@ -1047,7 +1206,7 @@ sub decideoutput {
         $message = &mt("Wrong format").'.';
         $css_class=$possible_class{'not_charged_try'};
         $button=1;
-    } elsif ($award eq 'INTERNAL_ERROR') {
+     } elsif ($award eq 'INTERNAL_ERROR') {
         $message = &mt("An internal error occurred while processing your answer. Please try again later.");
         $css_class=$possible_class{'not_charged_try'};
         $button=1;
@@ -1070,6 +1229,7 @@ sub decideoutput {
     } elsif ($award eq 'ANONYMOUS_CREDIT') {
         $message = &mt("Your anonymous submission has been recorded, and credit awarded.");
         $css_class=$possible_class{'correct'};
+        $button=1;
     } elsif ($award eq 'DRAFT') {
 	$message = &mt("Copy saved but not submitted.");
 	$css_class=$possible_class{'not_charged_try'};
@@ -1120,7 +1280,17 @@ sub decideoutput {
     if (&Apache::lonhomework::hide_problem_status()
 	&& $Apache::inputtags::status[-1] ne 'SHOW_ANSWER'
 	&& &hide_award($award)) {
-	$message = &mt("Answer Submitted: Your final submission will be graded after the due date.");
+        $message = &mt("Answer Submitted: Your final submission will be graded after the due date.");
+        my @interval= &Apache::lonnet::EXT("resource.$part.interval");
+        if ($interval[0] =~ /\d+/) {
+            my $first_access=&Apache::lonnet::get_first_access($interval[1]);
+            if (defined($first_access)) {
+                my $due_date= &Apache::lonnet::EXT("resource.$part.duedate");
+                unless (($due_date) && ($due_date < $first_access + $interval[0])) { 
+                    $message = &mt("Answer Submitted: Your final submission will be graded when the time limit is reached.");
+                }
+            }
+        }
 	$css_class=$possible_class{'no_grade'};
 	$button=1;
     }
@@ -1137,7 +1307,6 @@ sub decideoutput {
        }
        $message.=&mt('Submissions to practice problems are not permanently recorded.');
     }
-
     return ($button,$css_class,$message,$previousmsg);
 }
 
@@ -1190,8 +1359,7 @@ sub setgradedata {
 	$Apache::inputtags::status['-1'] ne 'CANNOT_ANSWER') {
 	$Apache::lonhomework::results{"resource.$id.afterduedate"}=$award;
 	return '';
-    } elsif ( $Apache::lonhomework::history{"resource.$id.solved"} !~
-	      /^correct/ 
+    } elsif ( $Apache::lonhomework::history{"resource.$id.awarded"} < 1
 	      || $Apache::lonhomework::scantronmode 
 	      || &Apache::lonhomework::hide_problem_status()  ) {
         # the student doesn't already have it correct,
@@ -1312,8 +1480,7 @@ sub setgradedata {
 		$Apache::lonhomework::results{"resource.$id.previous"} = '0';
 	    }
 	}
-    } elsif ( $Apache::lonhomework::history{"resource.$id.solved"} =~
-	      /^correct/ ) {
+    } elsif ( $Apache::lonhomework::history{"resource.$id.awarded"} == 1 ) {
 	#delete all data as they student already has it correct
 	&removealldata($id);
 	#and since they didn't do anything we were never here
@@ -1323,14 +1490,10 @@ sub setgradedata {
     if ($award eq 'SUBMITTED') {
 	&Apache::response::add_to_gradingqueue();
     }
-    if (($Apache::lonhomework::type eq 'anonsurvey') ||
-        ($Apache::lonhomework::type eq 'anonsurveycred') ||
-        ($Apache::lonhomework::type eq 'randomizetry')) {
-        $Apache::lonhomework::results{"resource.$id.type"} = $Apache::lonhomework::type;
-    }
-    if ($Apache::lonhomework::type eq 'randomizetry') {
-        $Apache::lonhomework::results{"resource.$id.rndseed"} = $env{'form.'.$id.'.rndseed'};
-    }
+    $Apache::lonhomework::results{"resource.$id.type"} = $Apache::lonhomework::type;
+    $Apache::lonhomework::results{"resource.$id.duedate"} = &Apache::lonnet::EXT("resource.$id.duedate");
+    $Apache::lonhomework::results{"resource.$id.hinttries"} = &Apache::lonnet::EXT("resource.$id.hinttries");
+    $Apache::lonhomework::results{"resourse.$id.version"} = &Apache::lonnet::usedversion(); 
 }
 
 sub find_which_previous {
@@ -1449,24 +1612,24 @@ sub get_grade_messages {
 	    if ($env{'request.state'} ne "construct"
 		&& $Apache::lonhomework::type ne 'exam'
 		&& $env{'form.suppress_tries'} ne 'yes') {
-                $trystr ='{\vskip 1 mm \small '
+		$trystr ='{\vskip 1 mm \small '
                         .&mt('[_1]'.$tries_text.'[_2] [_3]'
-                                ,'\textit{','}',$tries.'/'.$maxtries )
+				,'\textit{','}',$tries.'/'.$maxtries ) 
                         .'} \vskip 2 mm';
 	    } else {
 		$trystr = '\vskip 0 mm ';
 	    }
 	} else {
-            my $trial = $tries;
+	    my $trial =$tries;
 	    if ($Apache::lonhomework::parsing_a_task) {
 	    } elsif($env{'request.state'} ne 'construct') {
-		$trial.="/$maxtries";
+		$trial.="/".&Apache::lonhtmlcommon::direct_parm_link($maxtries,$env{'request.symb'},'maxtries',$id,$target);
 	    } else {
 		if (defined($Apache::inputtags::params{'maxtries'})) {
 		    $trial.="/".$Apache::inputtags::params{'maxtries'};
 		}
 	    }
-	    $trystr.='<td><span class="LC_nobreak">'.&mt("$tries_text [_1]",$trial).'</span></td>';
+	    $trystr = '<td><span class="LC_nobreak">'.&mt($tries_text.' [_1]',$trial).'</span></td>';
 	}
     }
 
@@ -1495,12 +1658,19 @@ sub gradestatus {
 	 && $status ne 'UNAVAILABLE' 
 	 && $status ne 'INVALID_ACCESS' 
 	 && $status ne 'NEEDS_CHECKIN' 
-	 && $status ne 'NOT_IN_A_SLOT') {  
+	 && $status ne 'NOT_IN_A_SLOT'
+         && $status ne 'RESERVABLE'
+         && $status ne 'RESERVABLE_LATER'
+         && $status ne 'NOTRESERVABLE') {
+
+	if ($status eq 'SHOW_ANSWER') {
+            $showbutton = 0;
+        }
 
 	($previousmsg,$latemessage,$message,$trystr) =
 	    &get_grade_messages($id,"resource.$id",$target,$status,
 				$showbutton);
-	if ( $status eq 'SHOW_ANSWER' || $status eq 'CANNOT_ANSWER') {
+	if ($status eq 'CANNOT_ANSWER') {
 	    $showbutton = 0;
 	}
 	if ( $status eq 'SHOW_ANSWER') {
@@ -1542,6 +1712,7 @@ sub previous_tries {
     my $count;
     my %count_lookup;
     my $lastrndseed;
+    my $numstamps = 0;
 
     foreach my $i (1..$Apache::lonhomework::history{'version'}) {
 	my $prefix = $i.":resource.$id";
@@ -1556,7 +1727,6 @@ sub previous_tries {
 	$count++;
 	$count_lookup{$i} = $count;
         my $curr_rndseed = $Apache::lonhomework::history{"$prefix.rndseed"};
-
 	my ($previousmsg,$latemessage,$message,$trystr);
 
 	($previousmsg,$latemessage,$message,$trystr) =
@@ -1592,17 +1762,24 @@ sub previous_tries {
                 if ($curr_rndseed ne $lastrndseed) {
                     $trystr .= '<br /><span style="color: green; white-space: nowrap; font-style: italic; font-weight: bold; font-size: 80%;">'.&mt('New problem variation this try.').'</span>';
                 }
-            }
+            } 
 	    $message =~ s{(</td>)}{ $trystr $1};
 	}
 	my ($class) = ($message =~ m{<td.*class="([^"]*)"}); #"
 	$message =~ s{(<td.*?>)}{<td>};
 	
 
-	$output.='<tr class="'.$class.'">';
-	$output.='<td align="center">'.$count.'</td>';
-	$output.=$message;
-
+	$output .= '<tr class="'.$class.'">'.
+	           '<td align="center">'.$count.'</td>'.$message;
+        if ((!$is_anon) && ($Apache::lonhomework::history{"$prefix.tries"}) &&
+            ($Apache::lonhomework::history{"$prefix.award"} ne 'ASSIGNED_SCORE') &&
+            ($Apache::lonhomework::history{$i.':timestamp'})) {
+            $output .= '<td>'.&Apache::lonlocal::locallocaltime(
+                             $Apache::lonhomework::history{$i.':timestamp'}).'</td>';
+            $numstamps ++;
+        } else {
+            $output .= '<td></td>';
+        }
 	foreach my $resid (@Apache::inputtags::response) {
 	    my $prefix = $prefix.".$resid";
 	    if (exists($Apache::lonhomework::history{"$prefix.submission"})) {
@@ -1625,36 +1802,28 @@ sub previous_tries {
         $lastrndseed = $curr_rndseed;
     }
     return if ($output eq '');
-    my $headers = 
-	'<tr>'.'<th>'.&mt('Submission #').'</th><th>'.&mt('Try').
-	'</th><th colspan="'.scalar(@Apache::inputtags::response).'">'.
-	&mt('Submitted Answer').'</th>';
+    my $headers = '<tr>'.
+                  '<th>'.&mt('Submission #').'</th>'.
+                  '<th>'.&mt('Try').'</th><th>';
+    if ($numstamps) {
+        $headers .= &mt('When');
+    }
+    $headers .= '</th>';
+    my $colspan = scalar(@Apache::inputtags::response);
+    if ($colspan > 1) {
+        $headers .= '<th colspan="'.$colspan.'">';
+    } else {
+        $headers .= '<th>';
+    }
+    $headers .= &mt('Submitted Answer').'</th></tr>';
     $output ='<table class="LC_prior_tries">'.$headers.$output.'</table>';
-    #return $output;
-    $output = &Apache::loncommon::js_ready($output); 
-    $output.='<br /><form action=""><center><input type="button" name="close" value="'.&mt('Close Window').'" onClick="window.close()" /></center></form>';
 
-    my $windowopen=&Apache::lonhtmlcommon::javascript_docopen();
     my $tries_text = &get_tries_text('link');
-    my $start_page =
-	&Apache::loncommon::start_page($tries_text, undef,
-				       {'only_body'      => 1,
-					'bgcolor'        => '#FFFFFF',
-					'js_ready'       => 1,
-				        'inherit_jsmath' => 1, });
-    my $end_page =
-	&Apache::loncommon::end_page({'js_ready' => 1,});
     my $prefix = $env{'form.request.prefix'};
     $prefix =~ tr{.}{_};
     my $function_name = "LONCAPA_previous_tries_".$prefix.
 	$Apache::lonxml::curdepth.'_'.$env{'form.counter'};
-    my $result ="<script type=\"text/javascript\">
-// <![CDATA[
-    function $function_name() {newWindow=open('','new_W','width=500,height=500,scrollbars=1,resizable=yes');newWindow.$windowopen;newWindow.document.writeln('$start_page $output $end_page');newWindow.document.close();newWindow.focus()}
-// ]]>
-</script><a href=\"javascript:$function_name();void(0);\">".&mt($tries_text)."</a><br />";
-    #use Data::Dumper;
-    #&Apache::lonnet::logthis(&Dumper(\%Apache::inputtags::submission_display));
+    my $result = &Apache::loncommon::modal_adhoc_window($function_name,420,410,$output,&mt($tries_text))."<br />";
     return $result;
 }
 
@@ -1680,6 +1849,21 @@ sub get_tries_text {
     return $tries_text;
 }
 
+sub spelling_languages {
+    my %langchoices;
+    foreach my $id (&Apache::loncommon::languageids()) {
+        my $code = &Apache::loncommon::supportedlanguagecode($id);
+        if ($code ne '') {
+            $langchoices{$code} =  &Apache::loncommon::plainlanguagedescription($id);
+        }
+    }
+    my @spelllangs = ('none');
+    foreach my $code ('en','de','he','es','fr','pt','tr') {
+        push(@spelllangs,[$code,$langchoices{$code}]);
+    }
+    return \@spelllangs;
+}
+
 1;
 __END__
 
