@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # The LON-CAPA Homework handler
 #
-# $Id: lonhomework.pm,v 1.317.4.4 2011/10/06 12:32:52 raeburn Exp $
+# $Id: lonhomework.pm,v 1.344 2014/02/25 19:31:48 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -112,7 +112,7 @@ sub get_target {
 	} elsif ( $env{'form.grade_target'} eq 'webgrade'
 		  && ($Apache::lonhomework::queuegrade eq 'F' )) {
 	    return ($env{'form.grade_target'});
-        } elsif ($env{'form.grade_target'} eq 'answer') {
+	} elsif ($env{'form.grade_target'} eq 'answer') {
             if ($env{'form.answer_output_mode'} eq 'tex') {
                 return ($env{'form.grade_target'});
             }
@@ -146,10 +146,10 @@ sub get_target {
 	    }
 	} else {
 	    if ($env{'form.problemstate'} eq 'WEB_GRADE') {
-                return ('grade','webgrade','answer');
+		return ('grade','webgrade','answer');
             } elsif ($env{'form.problemmode'} eq 'view') {
                 return ('grade','web','answer');
-            } elsif ($env{'form.problemmode'} eq 'saveview') {
+	    } elsif ($env{'form.problemmode'} eq 'saveview') {
                 return ('modified','web','answer');
             } elsif ($env{'form.problemmode'} eq 'discard') {
                 return ('web','answer');
@@ -157,13 +157,13 @@ sub get_target {
                      ($env{'form.problemmode'} eq 'undo')) {
                 return ('modified','no_output_web','edit');
             } elsif ($env{'form.problemmode'} eq 'edit') {
-                return ('no_output_web','edit');
-            } else {
-                return ('web');
-            }
-	}
+		return ('no_output_web','edit');
+	    } else {
+		return ('web');
+	    }
+        }
 #
-# End of Construction Space
+# End of Authoring Space
 #
     }
 #
@@ -178,16 +178,6 @@ sub setup_vars {
 #  return ';$external::target='.$target.';';
 }
 
-sub createmenu {
-    my ($which,$request)=@_;
-    if ($which eq 'grade') {
-	$request->print('<script type="text/javascript" language="JavaScript"> 
-          hwkmenu=window.open("/res/adm/pages/homeworkmenu.html","homeworkremote",
-                 "height=350,width=150,menubar=no");
-          </script>');
-    }
-}
-
 sub proctor_checked_in {
     my ($slot_name,$slot,$type)=@_;
     my @possible_proctors=split(",",$slot->{'proctor'});
@@ -256,20 +246,24 @@ sub check_slot_access {
 #    }
     my $slotstatus='NOT_IN_A_SLOT';
     my ($returned_slot,$slot_name);
+    my $now = time;
+    my $num_usable_slots = 0;
     foreach my $slot (@slots) {
 	$slot =~ s/(^\s*|\s*$)//g;
 	&Apache::lonxml::debug("getting $slot");
 	my %slot=&Apache::lonnet::get_slot($slot);
 	&Apache::lonhomework::showhash(%slot);
-	if ($slot{'starttime'} < time &&
-	    $slot{'endtime'} > time &&
+        next if ($slot{'endtime'} < $now);
+        $num_usable_slots ++;
+	if ($slot{'starttime'} < $now &&
+	    $slot{'endtime'} > $now &&
 	    &Apache::loncommon::check_ip_acc($slot{'ip'})) {
 	    &Apache::lonxml::debug("$slot is good");
 	    $slotstatus='NEEDS_CHECKIN';
 	    $returned_slot=\%slot;
 	    $slot_name=$slot;
 	    last;
-	}
+        } 
     }
     if ($slotstatus eq 'NEEDS_CHECKIN' &&
 	&proctor_checked_in($slot_name,$returned_slot,$type)) {
@@ -309,11 +303,36 @@ sub check_slot_access {
     # However, the problem is not closed, and potentially, another slot might be
     # used to gain access to it to work on it, until the due date is reached, and the
     # problem then becomes CLOSED.  Therefore return the slotstatus - 
-    # (which will be NOT_IN_SLOT).
-    if (!defined($slot_name)
-	&& $checkedin 
-	&& $type eq 'problem') {
-        return ($slotstatus);
+    # (which will be one of: NOT_IN_A_SLOT, RESERVABLE, RESERVABLE_LATER, or NOTRESERVABLE.
+    if (!defined($slot_name) && $type eq 'problem') {
+        if ($slotstatus eq 'NOT_IN_A_SLOT') {
+            if (!$num_usable_slots) {
+                if ($env{'request.course.id'}) {
+                    my $cdom = $env{'course.'.$env{'request.course.id'}.'.domain'};
+                    my $cnum = $env{'course.'.$env{'request.course.id'}.'.num'};
+                    my ($symb)=&Apache::lonnet::whichuser();
+                    $slotstatus = 'NOTRESERVABLE';
+                    my ($reservable_now_order,$reservable_now,$reservable_future_order,
+                        $reservable_future) = 
+                        &Apache::loncommon::get_future_slots($cnum,$cdom,$now,$symb);
+                    if ((ref($reservable_now_order) eq 'ARRAY') && (ref($reservable_now) eq 'HASH')) {
+                        if (@{$reservable_now_order} > 0) {
+                            $slotstatus = 'RESERVABLE';
+                            $datemsg = $reservable_now->{$reservable_now_order->[-1]}{'endreserve'};
+                        }
+                    }
+                    unless ($slotstatus eq 'RESERVABLE') {
+                        if ((ref($reservable_future_order) eq 'ARRAY') && (ref($reservable_future) eq 'HASH')) {
+                            if (@{$reservable_future_order} > 0) {
+                                $slotstatus = 'RESERVABLE_LATER';
+                                $datemsg = $reservable_future->{$reservable_future_order->[0]}{'startreserve'};
+                            }
+                        }
+                    }
+                }
+            }
+        }
+        return ($slotstatus,$datemsg);
     }
 
     if ($slotstatus eq 'NOT_IN_A_SLOT' 
@@ -386,7 +405,12 @@ sub check_access {
 	    $date=&mt("can not be accessed from your location.");
 	    return($status,$date);
 	}
-
+	if ($env{'form.grade_imsexport'}) {
+            if (($env{'request.course.id'}) && 
+                (&Apache::lonnet::allowed('mdc',$env{'request.course.id'}))) {
+                return ('SHOW_ANSWER');
+            }
+        }
 	foreach my $temp ("opendate","duedate","answerdate") {
 	    $lastdate = $date;
 	    if ($temp eq 'duedate') {
@@ -427,13 +451,14 @@ sub check_access {
 	    $datemsg=$date;
 	} elsif ($type eq 'opendate') {
 	    $status='CLOSED';
-	    $datemsg = &mt("will open on")." $date";
+	    $datemsg = &mt('will open on [_1]',$date);
 	} elsif ($type eq 'duedate') {
 	    $status='CAN_ANSWER';
-	    $datemsg = &mt("is due at")." $date";
+	    $datemsg = &mt('is due at [_1]',$date);
 	} elsif ($type eq 'answerdate') {
 	    $status='CLOSED';
-	    $datemsg = &mt("was due on")." $lastdate".&mt(", and answers will be available on")." $date";
+	    $datemsg = &mt('was due on [_1], and answers will be available on [_2]',
+                               $lastdate,$date);
 	}
     }
     if ($status eq 'CAN_ANSWER' ||
@@ -444,13 +469,16 @@ sub check_access {
 	if ( $tries eq '' ) { $tries = '0'; }
 	if ( $maxtries eq '' && 
 	     $env{'request.state'} ne 'construct') { $maxtries = '2'; } 
+	$Apache::lonhomework::results{'resource.'.$id.'.maxtries'}=$maxtries;
 	if ($maxtries && $tries >= $maxtries) { $status = 'CANNOT_ANSWER'; }
 	# if (correct and show prob status) or excused then CANNOT_ANSWER
-	if(($Apache::lonhomework::history{"resource.$id.solved"}=~/^correct/
-	    &&
-	    &show_problem_status())
-	   ||
-	   $Apache::lonhomework::history{"resource.$id.solved"}=~/^excused/) {
+	if ( ($Apache::lonhomework::history{"resource.$id.solved"}=~/^correct/)
+	      && (&show_problem_status()) ) {
+            if (($Apache::lonhomework::history{"resource.$id.awarded"} >= 1) ||
+                (&Apache::lonnet::EXT("resource.$id.retrypartial") !~/^1|on|yes$/i)) {
+	        $status = 'CANNOT_ANSWER';
+            }
+        } elsif ($Apache::lonhomework::history{"resource.$id.solved"}=~/^excused/) {
 	    $status = 'CANNOT_ANSWER';
 	}
 	if ($status eq 'CANNOT_ANSWER'
@@ -481,7 +509,6 @@ sub check_access {
   #    return ('UNCHECKEDOUT','needs to be checked out');
   #}
 
-
     &Apache::lonxml::debug("sending back :$status:$datemsg:");
     if (($Apache::lonhomework::browse eq 'F') && ($status eq 'CLOSED')) {
 	&Apache::lonxml::debug("should be allowed to browse a resource when closed");
@@ -581,6 +608,9 @@ sub showhashsubset {
 
 sub setuppermissions {
     $Apache::lonhomework::browse= &Apache::lonnet::allowed('bre',$env{'request.filename'});
+    unless ($Apache::lonhomework::browse eq 'F') {
+        $Apache::lonhomework::browse=&Apache::lonnet::allowed('bro',$env{'request.filename'}); 
+    }
     my $viewgrades = &Apache::lonnet::allowed('vgr',$env{'request.course.id'});
     if (! $viewgrades && 
 	exists($env{'request.course.sec'}) && 
@@ -640,7 +670,7 @@ sub setupheader {
 }
 
 sub handle_save_or_undo {
-    my ($request,$problem,$result) = @_;
+    my ($request,$problem,$result,$getobjref) = @_;
 
     my $file    = &Apache::lonnet::filelocation("",$request->uri);
     my $filebak =$file.".bak";
@@ -683,6 +713,30 @@ sub handle_save_or_undo {
 	my $fh=Apache::File->new(">$file");
 	if (defined($fh)) {
 	    print $fh $$result;
+            if (ref($getobjref) eq 'SCALAR') {
+                if ($file =~ m{([^/]+)\.(html?)$}) {
+                    my $fname = $1;
+                    my $ext = $2;
+                    my $path = $file;
+                    $path =~ s/\Q$fname\E\.\Q$ext\E$//; 
+                    my (%allfiles,%codebase);
+                    &Apache::lonnet::extract_embedded_items($file,\%allfiles,
+                                                           \%codebase,$result);
+                    if (keys(%allfiles) > 0) {
+                        my $url = $request->uri;
+                        my $state = <<STATE;
+    <input type="hidden" name="action" value="upload_embedded" />
+    <input type="hidden" name="url" value="$url" />
+STATE
+                        $$getobjref = "<h3>".&mt("Reference Warning")."</h3>".
+                                      "<p>".&mt("Completed upload of the file. This file contained references to other files.")."</p>".
+                                      "<p>".&mt("Please select the locations from which the referenced files are to be uploaded.")."</p>".
+                                      &Apache::loncommon::ask_for_embedded_content($url,$state,\%allfiles,\%codebase,
+                                      {'error_on_invalid_names'   => 1,
+                                       'ignore_remote_references' => 1,});
+                    }
+                }
+            }
 	} else {
 	    &Apache::lonxml::info('<span class="LC_error">'.
 				  &mt("Unable to write to [_1]",
@@ -700,8 +754,8 @@ sub analyze_header {
     my $js = &Apache::structuretags::setmode_javascript();
 
     # Breadcrumbs
-    my $brcrum = [{'href' => &Apache::loncommon::authorspace(),
-                   'text' => 'Construction Space'},
+    my $brcrum = [{'href' => &Apache::loncommon::authorspace($request->uri),
+                   'text' => 'Authoring Space'},
                   {'href' => '',
                    'text' => 'Problem Testing'},
                   {'href' => '',
@@ -752,13 +806,9 @@ sub analyze {
     my $rndseed=$env{'form.rndseed'};
     &analyze_header($request);
     my %prog_state=
-	&Apache::lonhtmlcommon::Create_PrgWin($request,&mt('Analyze Progress'),
-					      &mt('Getting Problem Variants'),
-					      $env{'form.numtoanalyze'},
-					      'inline',undef);
+	&Apache::lonhtmlcommon::Create_PrgWin($request,$env{'form.numtoanalyze'});
     for(my $i=1;$i<$env{'form.numtoanalyze'}+1;$i++) {
-	&Apache::lonhtmlcommon::Increment_PrgWin($request,\%prog_state,
-						 &mt('last problem'));
+	&Apache::lonhtmlcommon::Increment_PrgWin($request,\%prog_state,'last problem');
 	if (&Apache::loncommon::connection_aborted($request)) { return; }
         my $thisseed=$i+$rndseed;
 	my $subresult=&Apache::lonnet::ssi($request->uri,
@@ -767,7 +817,7 @@ sub analyze {
 	(my $garbage,$subresult)=split(/_HASH_REF__/,$subresult,2);
 	my %analyze=&Apache::lonnet::str2hash($subresult);
 	my @parts;
-	if (defined(@{ $analyze{'parts'} })) {
+        if (ref($analyze{'parts'}) eq 'ARRAY') {
 	    @parts=@{ $analyze{'parts'} };
 	}
 	foreach my $part (@parts) {
@@ -800,15 +850,15 @@ sub analyze {
 	    }
 	}
     }
-    &Apache::lonhtmlcommon::Update_PrgWin($request,\%prog_state,
-					  &mt('Analyzing Results'));
+    &Apache::lonhtmlcommon::Update_PrgWin($request,\%prog_state,&mt('Analyzing Results'));
     $request->print('<hr />'
                    .'<h3>'
                    .&mt('List of possible answers')
                    .'</h3>'
     );
     foreach my $part (sort(keys(%allparts))) {
-	if (defined(@{ $overall{$part.'.answer'} })) {
+        if ((ref($overall{$part.'.answer'}) eq 'ARRAY') &&
+            (@{$overall{$part.'.answer'}} > 0)) {
 	    for (my $i=0;$i<scalar(@{ $overall{$part.'.answer'} });$i++) {
 		my $num_cols=scalar(@{ $overall{$part.'.answer'}[$i][0] });
                 $request->print(&Apache::loncommon::start_data_table()
@@ -913,8 +963,9 @@ sub editxmlmode {
 
 	$problem='';
     }
+
     if (($env{'form.problemmode'} eq 'saveeditxml') ||
-        ($env{'form.problemmode'} eq 'saveviewxml') ||
+        ($env{'form.problemmode'} eq 'saveviewxml') || 
         ($env{'form.problemmode'} eq 'undoxml')) {
 	my $error=&handle_save_or_undo($request,\$problem,
 				       \$env{'form.editxmltext'});
@@ -935,24 +986,23 @@ sub editxmlmode {
 	    &Apache::loncommon::resize_textarea_js().
             &Apache::structuretags::setmode_javascript().
             &Apache::lonhtmlcommon::dragmath_js("EditMathPopup");
-	my $only_body =  ($env{'environment.remote'} eq 'off')? 0 : 1;
 
     # Breadcrumbs
-    my $brcrum = [{'href' => &Apache::loncommon::authorspace(),
-                   'text' => 'Construction Space'},
+    my $brcrum = [{'href' => &Apache::loncommon::authorspace($request->uri),
+                   'text' => 'Authoring Space'},
                   {'href' => '',
                    'text' => 'Problem Editing'}];
 
 	my $start_page = 
 	    &Apache::loncommon::start_page(&mt("EditXML [_1]",$file),$js,
 					   {'no_auto_mt_title' => 1,
-					    'only_body'        => $only_body,
+					    'only_body'        => 0,
 					    'add_entries'      => {
 						'onresize' => q[resize_textarea('LC_editxmltext','LC_aftertextarea')],
 						'onload'   => q[resize_textarea('LC_editxmltext','LC_aftertextarea')],
-                                    },
+                                                                  },
                                                 'bread_crumbs' => $brcrum,
-                                           });
+                                             });
 
     $result=$start_page
            .&Apache::loncommon::head_subbox(
@@ -971,8 +1021,8 @@ sub editxmlmode {
 
          $result.='<input type="hidden" name="problemmode" value="saveedit" />'.
                   &Apache::structuretags::problem_edit_buttons('editxml');
-
-         $result.='<hr style="clear:both;" />'.&Apache::lonxml::message_location().'</div>'.
+         
+         $result.='<hr style="clear:both;" />'.&Apache::lonxml::message_location().'</div>'.  
                   '<textarea '.&Apache::edit::element_change_detection().
 	              ' rows="'.$rows.'" cols="'.$cols.'" style="width:100%" '.
 		      ' name="editxmltext" id="LC_editxmltext">'.
@@ -1016,13 +1066,11 @@ sub renderpage {
 	    $problem='';
 	    my $filename=(split('/',$file))[-1];
 	    my $error =
-                '<p class="LC_error">'
-                .&mt('Unable to find [_1]',
-			   '<span class="LC_filename">'.$filename.'</span>')
-		."</p>";
+                &mt('Unable to find [_1]',
+			   '<span class="LC_filename">'.$filename.'</span>');
 	    $result.=
 		&Apache::loncommon::simple_error_page($request,'Not available',
-						      $error);
+						      $error,{'no_auto_mt_msg' => 1});
 	    return;
 	}
 
@@ -1083,9 +1131,10 @@ sub get_template_list {
 		     '/templates/*.'.$glob_extension);
     @files = map {[$_,&mt(&Apache::lonnet::metadata($_, 'title')),
                       (&Apache::lonnet::metadata($_, 'category')?&mt(&Apache::lonnet::metadata($_, 'category')):&mt('Miscellaneous')),
-                      &mt(&Apache::lonnet::metadata($_, 'help'))]} (@files);
+                      &Apache::lonnet::metadata($_, 'help')]} (@files);
     @files = sort {$a->[2].$a->[1] cmp $b->[2].$b->[1]} (@files);
     my ($midpoint,$seconddiv,$numfiles);
+    my @noexamplelink = ('blank.problem','blank.library','script.library');
     $numfiles = 0;
     foreach my $file (@files) {
         next if ($file->[1] !~ /\S/);
@@ -1101,6 +1150,7 @@ sub get_template_list {
     my $count = 0;
     my $currentcategory='';
     my $first = 1;
+    my $londocroot = $Apache::lonnet::perlvar{'lonDocRoot'};
     foreach my $file (@files) {
 	next if ($file->[1] !~ /\S/);
         if ($file->[2] ne $currentcategory) {
@@ -1123,11 +1173,16 @@ sub get_template_list {
         if ($file->[3]) {
            $result.=&Apache::loncommon::help_open_topic($file->[3]);
         }
+        # Provide example link
         my $filename=$file->[0];
-        $filename=~s/^\/home\/httpd\/html//;
-        $result.=' <span class="LC_fontsize_small">'
-                .'<a href="'.$filename.'?inhibitmenu=yes" target="sample">'.&mt('Example').'</a>'
-                .'</span><br />'."\n";
+        $filename=~s{^\Q$londocroot\E}{};
+        if (!(grep($filename =~ /\Q$_\E$/,@noexamplelink))) {
+            $result .= ' <span class="LC_fontsize_small">'
+                      .&Apache::loncommon::modal_link(
+                           $filename.'?inhibitmenu=yes',&mt('Example'),600,420,'sample')
+                      .'</span>';
+        }
+        $result .= '<br />'."\n";
         $count ++;
     }
     if ($numfiles > 0) {
@@ -1160,13 +1215,11 @@ sub newproblem {
 	&renderpage($request,$dest);
     } else {
 	my $url=&HTML::Entities::encode($request->uri,'<>&"');
-	my $shownurl=$url;	
-	$shownurl=~s-^/~-/priv/-;
 	my $dest = &Apache::lonnet::filelocation("",$request->uri);
 	my $errormsg;
 	my $instructions;
-        my $brcrum = [{'href' => &Apache::loncommon::authorspace(),
-                       'text' => 'Construction Space'},
+        my $brcrum = [{'href' => &Apache::loncommon::authorspace($request->uri),
+                       'text' => 'Authoring Space'},
                       {'href' => '',
                        'text' => "Create New $extension"}];
 	my $start_page = 
@@ -1180,7 +1233,7 @@ sub newproblem {
        .'<h1>'.&mt("Creating a new $extension resource.")."</h1>
 $errormsg
 ".&mt("The requested file [_1] currently does not exist.",
-      '<span class="LC_filename">'.$shownurl.'</span>').'
+      '<span class="LC_filename">'.$url.'</span>').'
 <p class="LC_info">
 '.&mt("To create a new $extension, select a template from the".
       " list below. Then click on the \"Create $extension\" button.").'
@@ -1251,11 +1304,11 @@ sub handler {
 		#first visit to problem in construction space
 		$env{'form.problemmode'}= 'view';
 		&renderpage($request,$file);
-            } elsif (($env{'form.problemmode'} eq 'editxml') ||
+	    } elsif (($env{'form.problemmode'} eq 'editxml') || 
                      ($env{'form.problemmode'} eq 'saveeditxml') ||
                      ($env{'form.problemmode'} eq 'saveviewxml') ||
                      ($env{'form.problemmode'} eq 'undoxml')) {
-                &editxmlmode($request,$file);
+		&editxmlmode($request,$file);
 	    } elsif ($env{'form.problemmode'} eq 'calcanswers') {
 		&analyze($request,$file);
 	    } else {
