@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # Interface routines to MAXIMA CAS
 #
-# $Id: lonmaxima.pm,v 1.29 2010/06/14 11:34:18 www Exp $
+# $Id: lonmaxima.pm,v 1.30 2012/02/17 15:31:04 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -51,7 +51,7 @@ sub maximareply {
 	print $socket &escape($cmd)."\n";
         my $reply=<$socket>;
         chomp($reply);
-        if ($reply=~/^Incorrect/) { $reply='Error: '.$reply; }
+        if ($reply=~/^Incorrect/i) { $reply='Error: '.$reply; }
         return &unescape($reply);
     } else {
         return 'Error: no connection.';
