@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # Full matching style response
 #
-# $Id: matchresponse.pm,v 1.81.2.1 2011/11/08 01:48:51 raeburn Exp $
+# $Id: matchresponse.pm,v 1.91 2014/05/05 17:40:54 bisitz Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -34,6 +34,7 @@ use Apache::optionresponse();
 use Apache::lonlocal;
 use Apache::lonnet;
 use Apache::lonxml;
+use POSIX qw(ceil);
 
 BEGIN {
     &Apache::lonxml::register('Apache::matchresponse',('matchresponse'));
@@ -103,20 +104,25 @@ sub start_itemgroup {
     if ($target eq 'edit') {
 	$result=&Apache::edit::tag_start($target,$token);
 	$result.=&Apache::edit::select_arg('Randomize Order:','randomize',
-					   ['yes','no'],$token).'&nbsp;'x 3;
+					   ['yes','no'],$token);
 	$result.=&Apache::edit::select_arg('Items Display Location:',
 					   'location',
 					   ['top','bottom','left','right'],
-					   $token).'&nbsp;'x 3;
+					   $token);
 	$result.=&Apache::edit::select_arg('Items Display Direction:',
 					   'direction',
 					   ['vertical','horizontal'],
 					   $token);
+        $result.=&Apache::edit::select_arg('Items Columns:',
+                                           'columns',
+                                           [['','default'],'1','2','3','4'],
+                                            $token);
 	$result.=&Apache::edit::end_row().&Apache::edit::start_spanning_row();
     } elsif ($target eq 'modified') {
 	my $constructtag=&Apache::edit::get_new_args($token,$parstack,
 						     $safeeval,'randomize',
-						     'location','direction');
+						     'location','direction',
+                                                     'columns');
 	if ($constructtag) { $result = &Apache::edit::rebuild_tag($token); }
     } elsif ($target eq 'web' or $target eq 'tex') {
 	$Apache::matchresponse::itemtable{'location'}=
@@ -158,28 +164,82 @@ sub end_itemgroup {
     $Apache::response::itemgroup{'letter_name_map'}=\%letter_name_map;
     $Apache::response::itemgroup{'name_letter_map'}=\%name_letter_map;
     my $direction=&Apache::lonxml::get_param('direction',$parstack,$safeeval);
+    my $columns=&Apache::lonxml::get_param('columns',$parstack,$safeeval);
+    
+    unless ($columns =~ /^\d+$/) {
+        undef($columns);
+    }
     if ($target eq 'web') {
 	
 	my $table='<br /><table>'; # extra space to match what latex does.
-	my $i=0;
-	if ($direction eq 'horizontal') { $table .='<tr>';}
-	foreach my $name (@names) {
-	    if ($direction ne 'horizontal') { $table.='<tr>'; }
-	    $table.='<td>'.$alphabet[$i].'</td><td>'.
-		$Apache::response::itemgroup{$name.'.text'}.'</td>';
-	    if ($direction ne 'horizontal') { $table.='</tr>'; }
-	    $i++;
-	}
-	if ($direction eq 'horizontal') { $table .='</tr>';}
+        if ((!$columns) || ($columns < 0)) {
+            if ($direction eq 'horizontal') {
+                if (@names > 0) { 
+                    $columns = scalar(@names);
+                } else {
+                    $columns = 1;
+                }
+            } else {
+                $columns = 1;
+            }
+        }
+        my $rows=ceil(scalar(@names)/$columns);
+        my $endloop = $columns*$rows;
+        for (my $i=0; $i<$endloop; $i++) {
+            my $label = '&nbsp;';
+            my $item = '&nbsp;';
+            my $index;
+            if ($direction eq 'horizontal') {
+                $index = $i;
+            } else {
+                $index = ($i % $columns)*$rows+int($i/$columns);
+            }
+            if ($index < scalar(@names)) {
+                $label = $alphabet[$index];
+                $item = $Apache::response::itemgroup{$names[$index].'.text'};
+            }
+            if ($i % $columns == 0) {
+                $table.='<tr>';
+            }
+            $table.= '<td>'.$label.'</td><td>'.$item.'</td>';
+            if ($columns > 1) {
+                $table .= '<td>&nbsp;</td>';
+            }
+            if ( ! (($i+1) % $columns) ) {
+                $table.='</tr>';
+            }
+        }
 	$table.='</table>';
 	$Apache::matchresponse::itemtable{'display'}=$table;
 	$Apache::lonxml::post_evaluate=0;
     } elsif ($target eq 'tex') {
 	my $table=' \begin{description}\setlength{\leftmargin}{2em}\setlength{\labelwidth}{1em}\setlength{\itemsep}{0.5pt plus1pt minus2pt}\setlength{\listparindent}{0em} ';
 	my $i=0;
+        my ($numrows,$bubbles_per_row);
+        if ($Apache::lonhomework::type eq 'exam') {
+            ($numrows,$bubbles_per_row) = 
+                &Apache::optionresponse::getnumrows(scalar(@names)); 
+        } else {
+            $numrows = 1;
+        }
 	foreach my $name (@names) {
 	    # $Apache::response::itemgroup{$name.'.text'}=~s/\$\$/\$/g;
-	    $table.='\item['.$alphabet[$i].'] '.
+            my $item;
+            if (($numrows > 1) && ($bubbles_per_row > 0)) {
+                my $num = 1+int($i/$bubbles_per_row);
+                my $idx = int($i % $bubbles_per_row);
+                if ($idx == 0) {
+                    if ($num == 1) {
+                        $table .= '\item[\footnotesize {'.&mt('(first line)').'}]';
+                    } else {
+                        $table .= '\item[\footnotesize {'.&mt('(next line)').'}]';
+                    }
+                }
+                $item = $alphabet[$idx];
+            } else {
+                $item = $alphabet[$i];
+            }
+	    $table.='\item['.$item.'] '.
 		$Apache::response::itemgroup{$name.'.text'};
 	    $i++;
 	}
@@ -202,7 +262,7 @@ sub start_item {
 	$result=&Apache::edit::tag_start($target,$token,"Item");
 	$result.=&Apache::edit::text_arg('Name:','name',$token);
 	if ($randomize ne 'no') {
-	    $result.='&nbsp;'x 3 .
+	    $result.=
                      &Apache::edit::select_arg('Location:','location',
 					       ['random','top','bottom'],
 					       $token);
@@ -292,8 +352,20 @@ sub end_foilgroup {
 						 ['text','value','location']);
 	    #FIXME need to store options in some way
 	}
-	&Apache::lonxml::increment_counter(&getfoilcounts($max), 
-					   "$part.$response_id");
+        my ($numrows,$bubbles_per_row);
+        if (($target eq 'tex') && ($Apache::lonhomework::type eq 'exam')) {
+            my $numitems;
+            if (ref($Apache::response::itemgroup{'names'}) eq 'ARRAY') {
+                $numitems = scalar(@{ $Apache::response::itemgroup{'names'} });
+                ($numrows,$bubbles_per_row) =
+                    &Apache::optionresponse::getnumrows($numitems);
+            }
+        }
+        if ($numrows < 1) {
+            $numrows = 1;
+        }
+        my $increment = &getfoilcounts($max) * $numrows;
+	&Apache::lonxml::increment_counter($increment,"$part.$response_id");
 	if ($target eq 'analyze') {
 	    &Apache::lonhomework::set_bubble_lines();
 	}
@@ -328,8 +400,8 @@ sub displayanswers {
     my %name_letter_map;
     if (ref($Apache::response::itemgroup{'name_letter_map'}) eq 'HASH') {
         if (%{$Apache::response::itemgroup{'name_letter_map'}}) {
-            %name_letter_map=
-                %{ $Apache::response::itemgroup{'name_letter_map'} };
+	    %name_letter_map=
+	        %{ $Apache::response::itemgroup{'name_letter_map'} };
         }
     }
     my $result;
@@ -369,13 +441,46 @@ sub grade_response {
     my %letter_name_map;
     if (ref($Apache::response::itemgroup{'letter_name_map'}) eq 'HASH') {
         if (%{$Apache::response::itemgroup{'letter_name_map'}}) {
-            %letter_name_map=
-                %{ $Apache::response::itemgroup{'letter_name_map'} };
+	    %letter_name_map=
+	        %{ $Apache::response::itemgroup{'letter_name_map'} };
         }
     }
     my @items;
+    my $numitems = scalar(@{ $Apache::response::itemgroup{'names'} });
+    my ($numrows,$bubbles_per_row);
+    if ($Apache::lonhomework::scantronmode) {
+        my $numitems = scalar(@{ $Apache::response::itemgroup{'names'} });
+        ($numrows,$bubbles_per_row) =
+            &Apache::optionresponse::getnumrows($numitems);
+    }
+    if ($numrows < 1) {
+        $numrows = 1;
+    }
+    my @alphabet=('A'..'Z');
+    my %nums_from_letters;
+    for (my $i=0; $i<@alphabet; $i++) {
+        $nums_from_letters{$alphabet[$i]} = $i;
+    }
     foreach my $name (@whichfoils) {
-	my $response = &Apache::response::getresponse($temp,'letter');
+        my $response;
+        if ($numrows > 1) {
+            my $num = $temp;
+            my $totalnum;
+            for (my $i=0; $i<$numrows; $i++) {
+                my $item = &Apache::response::getresponse($num,'letter');
+                if ($item =~ /^\w$/) {
+                    $totalnum = $i*$bubbles_per_row + $nums_from_letters{$item};
+                }
+                $num ++;
+            }
+            if ($totalnum =~ /^\d+$/) {
+                $response = $alphabet[$totalnum];
+            }
+            $temp += $numrows;
+        } else {
+	    $response = &Apache::response::getresponse($temp,'letter');
+            $temp ++;
+        }
 	push(@items,$response);
 	my $responsename = $letter_name_map{$response};
 	my $value=$Apache::response::foilgroup{$name.'.value'};
@@ -393,9 +498,9 @@ sub grade_response {
 	} else {
 	    $ignored++;
 	}
-	$temp++;
     }
     my $part=$Apache::inputtags::part;
+    my $nonlenient=&Apache::optionresponse::grading_is_nonlenient($part);
     my $id = $Apache::inputtags::response['-1'];
     my $responsestr=&Apache::lonnet::hash2str(%responsehash);
     my $itemstr    =&Apache::lonnet::array2str(@items);
@@ -432,34 +537,37 @@ sub grade_response {
 	    my $ad=$Apache::lonhomework::results{"resource.$part.$id.awarddetail"}='MISSING_ANSWER';
 	    &Apache::response::handle_previous(\%previous,$ad);
 	}
-    } elsif (!$Apache::lonhomework::scantronmode) {
-	my $ad;
-	if ($wrong==0 && $ignored==0) {
-	    $ad='EXACT_ANS';
-	} elsif ($wrong==0 && $right==0) {
-	    #nothing submitted
-	} else {
-	    if ($ignored==0) {
-		$ad='INCORRECT';
-	    } else {
-		$ad='MISSING_ANSWER';
-	    }
-	}
-	$Apache::lonhomework::results{"resource.$part.$id.awarddetail"}=$ad;
-	&Apache::response::handle_previous(\%previous,$ad);
+    } elsif ($nonlenient) {
+        my $ad;
+        if ($wrong==0 && $ignored==0) {
+            $ad='EXACT_ANS';
+        } elsif ($wrong==0 && $right==0) {
+            #nothing submitted
+        } else {
+            if ($ignored==0) {
+                $ad='INCORRECT';
+            } else {
+                $ad='MISSING_ANSWER';
+            }
+        }
+        $Apache::lonhomework::results{"resource.$part.$id.awarddetail"}=$ad;
+        &Apache::response::handle_previous(\%previous,$ad);
     } else {
-	my $ad;
-	if ($wrong==0 && $right==0) {
-	    #nothing submitted
-	} else {
-	    $ad='ASSIGNED_SCORE';
-	    $Apache::lonhomework::results{"resource.$part.$id.awarddetail"}=
-		$ad;
-	    $Apache::lonhomework::results{"resource.$part.$id.awarded"}=
-		$right/(scalar(@whichfoils));
-	    $Apache::lonhomework::results{"resource.$part.$id.numfoils"}=
-		scalar(@whichfoils);
-	}
+        my $ad;
+        if ($wrong==0 && $right==0) {
+            #nothing submitted only assign a score if we
+            #need to override a previous grade
+            if (defined($Apache::lonhomework::history{"resource.$part.$id.awarddetail"})) {
+                $ad='ASSIGNED_SCORE';
+            }
+        } else {
+            $ad='ASSIGNED_SCORE';
+        }
+        $Apache::lonhomework::results{"resource.$part.$id.awarddetail"}=$ad;
+        $Apache::lonhomework::results{"resource.$part.$id.awarded"}=
+            $right/(scalar(@whichfoils));
+        $Apache::lonhomework::results{"resource.$part.$id.numfoils"}=
+            scalar(@whichfoils);
     }
 }
 
@@ -472,7 +580,10 @@ sub format_prior_answer {
     my $output;
     
     foreach my $name (@{ $foil_order }) {
-	my $item=shift(@items);
+	my $item = &HTML::Entities::encode(shift(@items),'<>&"');
+        if ($item eq '') {
+            $item = '&nbsp;';
+        }
 	$output .= '<tr><td>'.$item.'</td></tr>';
     }
     return if (!defined($output));
@@ -508,15 +619,15 @@ sub displayfoils {
     my %letter_name_map;
     if (ref($Apache::response::itemgroup{'letter_name_map'}) eq 'HASH') {
         if (%{$Apache::response::itemgroup{'letter_name_map'}}) {
-            %letter_name_map=
-                %{ $Apache::response::itemgroup{'letter_name_map'} };
+	    %letter_name_map=
+	        %{ $Apache::response::itemgroup{'letter_name_map'} };
         }
     }
     my %name_letter_map;
     if (ref($Apache::response::itemgroup{'name_letter_map'}) eq 'HASH') {
         if (%{$Apache::response::itemgroup{'name_letter_map'}}) {
-            %name_letter_map=
-                %{ $Apache::response::itemgroup{'name_letter_map'} };
+	    %name_letter_map=
+	        %{ $Apache::response::itemgroup{'name_letter_map'} };
         }
     }
     if ( &Apache::response::show_answer() && ($target ne 'tex')) {
@@ -544,13 +655,30 @@ sub displayfoils {
                 $newvariation = 1;
             }
         }
-        unless ((($Apache::lonhomework::history{"resource.$part.type"} eq 'anonsurvey') || ($Apache::lonhomework::history{"resource.$part.type"} eq 'anonsurveycred') && (defined($env{'form.grade_symb'}))) || $newvariation) {
-	    $lastresponse=$Apache::lonhomework::history{"resource.$part.$id.submission"};
+        unless ($newvariation) {
+            if ((($env{'form.grade_username'} eq '') && ($env{'form.grade_domain'} eq '')) ||
+                (($env{'form.grade_username'} eq $env{'user.name'}) &&
+                 ($env{'form.grade_domain'} eq $env{'user.domain'}))) {
+                $lastresponse=$Apache::lonhomework::history{"resource.$part.$id.submission"};
+            } else {
+                unless (($Apache::lonhomework::history{"resource.$part.type"} eq 'anonsurvey') ||
+                        ($Apache::lonhomework::history{"resource.$part.type"} eq 'anonsurveycred')) {
+                    $lastresponse=$Apache::lonhomework::history{"resource.$part.$id.submission"};
+                }
+            }
         }
+
 	my %lastresponse=&Apache::lonnet::str2hash($lastresponse);    
 	my @alphabet=('A'..'Z');
 	my @used_letters=sort(keys(%letter_name_map));
 	my $internal_counter=$Apache::lonxml::counter;
+        my ($numrows,$bubbles_per_row);
+        if (($target eq 'tex') && ($Apache::lonhomework::type eq 'exam')) {
+            ($numrows,$bubbles_per_row) = 
+                &Apache::optionresponse::getnumrows(scalar(@used_letters));
+        } else {
+            $numrows = 1;
+        }
 	foreach my $name (@whichfoils) {
 	    my $lastopt=$lastresponse{$name};
 	    my $last_letter=$name_letter_map{$lastopt};
@@ -634,15 +762,27 @@ sub displayfoils {
 		    my @emptyItems = ();
 		    for (my $i=0;$i<=$#used_letters;$i++) {push @emptyItems, ' ';}
 		    $question.='\vskip -1 mm\noindent\begin{list}{}{\setlength{\listparindent}{0mm}\setlength{\leftmargin}{2mm}}'
-			.'\item \hskip -3mm \textbf{'.$internal_counter.'}';
+			.'\item \hskip -3mm ';
+                    if ($numrows == 1) {
+                        $question .= '\textbf{'.$internal_counter.'}';
+                    } else {
+                        my $linetext;
+                        for (my $i=0; $i<$numrows; $i++) {
+                            $linetext .= $internal_counter+$i.', ';
+                        }
+                        $linetext =~ s/,\s$//;
+                        $question .= '\small {\textbf{'.$linetext.'}} '.
+                                     '\hskip 2 mm {\footnotesize '.
+                                     &mt('(Bubble once in [_1] lines)',$numrows).
+                                     '} \vskip 3 mm';
+                    }
+                    my $max_width;
 		    if (&itemdisplay('left') || &itemdisplay('right')) {
-			$question .= '\vskip -4 mm' . &Apache::optionresponse::bubbles(\@used_letters,\@emptyItems, "", $righttabsize);
-		    }
-		    else {
-			$question .= '\vskip -4 mm' . &Apache::optionresponse::bubbles(\@used_letters,\@emptyItems);
-		    }
+                        $max_width = $righttabsize;
+                    }
+		    $question .= '\vskip -4 mm' . &Apache::optionresponse::bubbles(\@used_letters,\@emptyItems,'',$max_width,$numrows,$bubbles_per_row,$internal_counter);
 		    $question .= '\end{list} \vskip -8 mm \strut ';
-		    $internal_counter++;
+		    $internal_counter += $numrows;
 	        } else {
                     if($env{'form.pdfFormFields'} eq 'yes' 
                             && $Apache::inputtags::status['-1'] eq 'CAN_ANSWER') {
@@ -762,11 +902,11 @@ sub start_foil {
 	if ($$tagstack[-2] eq 'conceptgroup') { $level = '-3'; }
 	$result.=&Apache::edit::text_arg('Name:','name',$token);
 	my @names;
-        if (ref($Apache::response::itemgroup{'names'}) eq 'ARRAY') {
+	if (ref($Apache::response::itemgroup{'names'}) eq 'ARRAY') {
             if (@{ $Apache::response::itemgroup{'names'} }) {
-                @names=@{ $Apache::response::itemgroup{'names'} };
+	        @names=@{ $Apache::response::itemgroup{'names'} };
             }
-        }
+	}
 	$result.= &Apache::edit::select_or_text_arg('Correct Option:','value',['unused',@names],$token,'15');
 	my $randomize=&Apache::lonxml::get_param('randomize',$parstack,
 						 $safeeval,'-3');
@@ -841,5 +981,6 @@ sub insert_foil {
 <endouttext />
 </foil>';
 }
+
 1;
 __END__
