@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # rank style response
 #
-# $Id: rankresponse.pm,v 1.61 2008/12/12 09:19:34 bisitz Exp $
+# $Id: rankresponse.pm,v 1.71 2014/05/05 17:40:54 bisitz Exp $
 # Copyright Michigan State University Board of Trustees
 #
 # This file is part of the LearningOnline Network with CAPA (LON-CAPA).
@@ -96,7 +96,7 @@ sub start_foilgroup {
     my $result;
     %Apache::response::foilgroup=();
     $Apache::rankresponse::conceptgroup=0;
-    &Apache::response::pushrandomnumber();
+    &Apache::response::pushrandomnumber(undef,$target);
     return $result;
 }
 
@@ -127,8 +127,17 @@ sub end_foilgroup {
 	}
 	my $part = $Apache::inputtags::part;
 	my $id   = $Apache::inputtags::response[-1];
-	&Apache::lonxml::increment_counter(&getfoilcounts($max),
-					   "$part.$id");
+        my ($numrows,$bubbles_per_row);
+        if (($target eq 'tex') && ($Apache::lonhomework::type eq 'exam')) {
+            my (@whichfoils)=&whichfoils($max,$randomize);
+            ($numrows,$bubbles_per_row) =
+                &Apache::optionresponse::getnumrows(scalar(@whichfoils));
+        }
+        if ($numrows < 1) {
+            $numrows = 1;
+        }
+        my $increment = &getfoilcounts($max) * $numrows;
+	&Apache::lonxml::increment_counter($increment,"$part.$id");
 	if ($target eq 'analyze') {
 	    &Apache::lonhomework::set_bubble_lines();
 	}
@@ -177,8 +186,11 @@ sub get_correct_order {
 
 sub displayanswers {
     my ($max,$randomize,$tol,@opt)=@_;
-    if (!defined(@{ $Apache::response::foilgroup{'names'} })) { return; }
-    my @names = @{ $Apache::response::foilgroup{'names'} };
+    my @names;
+    if (ref($Apache::response::foilgroup{'names'}) eq 'ARRAY') {
+        @names = @{ $Apache::response::foilgroup{'names'} };
+    }
+    return if (!@names);
     my @whichfoils = &whichfoils($max,$randomize);
     my @correctorder=&get_correct_order($tol,@whichfoils);
     my $result;
@@ -227,8 +239,34 @@ sub grade_response {
     my %grade;
     my ($temp,$right,$wrong,$ignored)=(1,0,0,0);
     my @correctorder=&get_correct_order($tol,@whichfoils);
+    my ($numrows,$bubbles_per_row);
+    if ($Apache::lonhomework::scantronmode) {
+        my $numitems = scalar(@whichfoils);
+        ($numrows,$bubbles_per_row) =
+            &Apache::optionresponse::getnumrows($numitems);
+    }
+    if ($numrows < 1) {
+        $numrows = 1;
+    }
+
     foreach my $name (@whichfoils) {
-	my $response = &Apache::response::getresponse($temp,'A is 1');
+        my $response;
+        if ($numrows > 1) {
+            my $num = $temp;
+            my $totalnum;
+            for (my $i=0; $i<$numrows; $i++) {
+                my $item = &Apache::response::getresponse($num,'A is 1');
+                if ($item =~ /^\d+$/) {
+                    $totalnum = $i*$bubbles_per_row + $item;
+                }
+                $num ++;
+            }
+            $response = $totalnum;
+            $temp += $numrows;
+        } else {
+            $response = &Apache::response::getresponse($temp,'A is 1');
+            $temp ++;
+        }
 	my $value=shift(@correctorder);
 	if ( $response =~ /[^\s]/) {
 	    $responsehash{$name}=$response;
@@ -241,7 +279,6 @@ sub grade_response {
 	} else {
 	    $ignored++;
 	}
-	$temp++;
     }
     my $malformed=&check_response_order(%responsehash);
     my $part=$Apache::inputtags::part;
@@ -265,12 +302,26 @@ sub grade_response {
 	    $ad='MISSING_ANSWER';
 	}
     }
-    if ($Apache::lonhomework::type eq 'survey' &&
-	($ad eq 'INCORRECT' || $ad eq 'EXACT_ANS') ) {
-	$ad='SUBMITTED';
+    if (($ad eq 'INCORRECT' || $ad eq 'EXACT_ANS')) {
+        if ($Apache::lonhomework::type eq 'survey') {
+            $ad='SUBMITTED';
+        } elsif ($Apache::lonhomework::type eq 'surveycred') {
+	    $ad='SUBMITTED_CREDIT';
+        } elsif ($Apache::lonhomework::type eq 'anonsurvey') {
+            $ad='ANONYMOUS';
+        } elsif ($Apache::lonhomework::type eq 'anonsurveycred') {
+            $ad='ANONYMOUS_CREDIT';
+        } else {
+            $Apache::lonhomework::results{"resource.$part.$id.submissiongrading"}=$gradestr;
+        }
     } else {
 	$Apache::lonhomework::results{"resource.$part.$id.submissiongrading"}=$gradestr;
     }
+    if ($Apache::lonhomework::type eq 'randomizetry') {
+        if ($Apache::lonhomework::type eq 'randomizetry') {
+            $Apache::lonhomework::results{"resource.$part.$id.foilorder"} = &Apache::lonnet::array2str(@whichfoils);
+        }
+    }
     $Apache::lonhomework::results{"resource.$part.$id.submission"}=
 	$responsestr;
     $Apache::lonhomework::results{"resource.$part.$id.awarddetail"}=$ad;
@@ -284,8 +335,11 @@ sub format_prior_answer {
     my %grading     =&Apache::lonnet::str2hash($other_data->[1]);
     my $output;
     foreach my $name (@{ $foil_order }) {
-	next if (!defined($lastresponse{$name}));
-	$output .= '<tr><td>'.$lastresponse{$name}.'</td></tr>';
+        if (defined($lastresponse{$name})) {
+            $output .= '<tr><td>'.&HTML::Entities::encode($lastresponse{$name},'<>&"').'</td></tr>';
+        } else {
+            $output .= '<tr><td>&nbsp;</td></tr>';
+        }
     }
     return if (!defined($output));
     $output =
@@ -317,10 +371,37 @@ sub displayfoils {
 	my $temp=1;
 	my $id=$Apache::inputtags::response[-1];
 	my $part=$Apache::inputtags::part;
-	my $lastresponse=$Apache::lonhomework::history{"resource.$part.$id.submission"};
+        my ($lastresponse,$newvariation);
+        if ((($Apache::lonhomework::history{"resource.$part.type"} eq 'randomizetry') ||
+             ($Apache::lonhomework::type eq 'randomizetry')) &&
+        ($Apache::inputtags::status[-1] eq 'CAN_ANSWER')) {
+            if ($env{'form.'.$part.'.rndseed'} ne
+                $Apache::lonhomework::history{"resource.$part.rndseed"}) {
+                $newvariation = 1;
+            }
+        }
+        unless ($newvariation) {
+            if ((($env{'form.grade_username'} eq '') && ($env{'form.grade_domain'} eq '')) ||
+                (($env{'form.grade_username'} eq $env{'user.name'}) &&
+                 ($env{'form.grade_domain'} eq $env{'user.domain'}))) {
+                $lastresponse=$Apache::lonhomework::history{"resource.$part.$id.submission"};
+            } else {
+                unless (($Apache::lonhomework::history{"resource.$part.type"} eq 'anonsurvey') ||
+                        ($Apache::lonhomework::history{"resource.$part.type"} eq 'anonsurveycred')) {
+                    $lastresponse=$Apache::lonhomework::history{"resource.$part.$id.submission"};
+                }
+            }
+        }
 	my %lastresponse=&Apache::lonnet::str2hash($lastresponse); 
 	my @alp = splice @alphabet, 0, $#whichopt + 1;
 	my $internal_counter=$Apache::lonxml::counter;
+        my ($numrows,$bubbles_per_row);
+        if (($target eq 'tex') && ($Apache::lonhomework::type eq 'exam')) {
+            ($numrows,$bubbles_per_row) = 
+                &Apache::optionresponse::getnumrows(scalar(@alp));
+        } else {
+            $numrows = 1;
+        }
         if($target eq 'tex' && $env{'form.pdfFormFields'} eq 'yes') {
             $result .= '\strut \\\\ \strut \\\\' ;
         }
@@ -338,7 +419,7 @@ sub displayfoils {
 	    foreach $option (@whichopt) {
 		if ($option eq $lastopt) {
 		    if ($target ne 'tex' ) {
-                        $optionlist.="<option selected=\"on\">$option</option>\n";
+                        $optionlist.="<option selected=\"selected\">$option</option>\n";
                     } elsif ($target eq 'tex' && $env{'form.pdfFormFields'} eq 'yes') {
                         $optionlist .= &Apache::lonxml::print_pdf_add_combobox_option($option); 
                     }
@@ -373,9 +454,26 @@ sub displayfoils {
 		}
 	    } else {
 		if ($Apache::lonhomework::type eq 'exam') {
-		    $result.='\vskip 0 mm   '.$text.' \vskip 0 mm '."\n";
-		    $result.='\vskip -1 mm\noindent\begin{enumerate}\item[\textbf{'.$internal_counter.'}.]'.&Apache::optionresponse::bubbles(\@alp,\@whichopt,'rankresponse').'\end{enumerate} \vskip -8 mm \strut ';
-		    $internal_counter++;
+                    my $itemlabel;
+                    if ($numrows == 1) {
+                        $itemlabel = '\item[\textbf{'.$internal_counter.'}]';
+                    } else {
+                        my $linetext;
+                        for (my $i=0; $i<$numrows; $i++) {
+                            $linetext .= $internal_counter+$i.', ';
+                        }
+                        $linetext =~ s/,\s$//;
+                        $itemlabel = '\item[\small {\textbf{'.$linetext.'}}]'.
+                                     ' {\footnotesize '.
+                                     &mt('(Bubble once in [_1] lines)',$numrows).
+                                     '} \hspace*{\fill} \\\\';
+                    }
+		    $result .= '\vskip 0 mm '.$text.' \vskip 0 mm '."\n";
+		    $result .= '\vskip -1 mm';
+                    $result .= '\begin{enumerate}'.$itemlabel;
+		    $result .= &Apache::optionresponse::bubbles(\@alp,\@whichopt,'rankresponse',undef,$numrows,$bubbles_per_row,$internal_counter);
+                    $result .= '\end{enumerate} \vskip -8 mm \strut ';
+		    $internal_counter += $numrows;
 		} else {
                     if($env{'form.pdfFormFields'} ne 'yes') {
 		        $result.=' \vskip 0mm \framebox[5 mm][s]{\tiny\strut} '.$text."\n";
@@ -388,9 +486,14 @@ sub displayfoils {
 	}
     }
     if ($target eq 'web') {
+        my $questiontype;
+        if ($Apache::lonhomework::type eq 'randomizetry') {
+            $questiontype = $Apache::lonhomework::type;
+        }
 	&Apache::response::setup_prior_tries_hash(\&format_prior_answer,
 						  [\@whichfoils,
-						   'submissiongrading']);
+						   'submissiongrading'],
+                                                  $questiontype);
     }
     if ($target ne 'tex') {$result.="<br />";} else {$result.=' \vskip 0 mm ';}
     return $result;
