@@ -1,6 +1,6 @@
 # The LearningOnline Network with CAPA
 # 
-# $Id: imsimport.pm,v 1.32 2009/08/17 03:52:09 raeburn Exp $
+# $Id: imsimport.pm,v 1.45 2013/09/30 17:42:12 bisitz Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -29,7 +29,6 @@ package Apache::imsimport;
 
 use strict;
 use Apache::Constants qw(:common :http :methods);
-use Apache::loncacc;
 use Apache::loncommon();
 use Apache::lonnet;
 use Apache::imsprocessor;
@@ -43,7 +42,7 @@ use LONCAPA;
 
 # ----------------------------------------------------------------  Jscript One
 sub jscript_one {
-    my ($fullpath,$jsref,$formname) = @_;
+    my ($fullpath,$formname) = @_;
 
     my %body_layout = ('rightmargin'  => "0",
 		       'leftmargin'   => "0",
@@ -66,7 +65,7 @@ sub jscript_one {
                nndi => 'Enter the name of the new directory where you will store the contents of your IMS package.',
                go => 'Go', 
              );
-    $$jsref = <<"END_OF_ONE";
+    return <<"END_OF_ONE";
 function verify() {
  if ((document.forms.$formname.newdir.value == '')  || (!document.forms.$formname.newdir.value)) {
    alert('$lt{'ddir'}')
@@ -101,7 +100,7 @@ function createWin() {
   newWindow.document.write("<input type='hidden' name='action' value='newdir' />")
   newWindow.document.write("<input type='hidden' name='callingmode' value='imsimport' />")
   newWindow.document.write("$fullpath<input type='text' name='newfilename' value='' />")
-  newWindow.document.write("<input type='button' value='$lt{'go'}' onClick='document.fileaction.submit();' />")
+  newWindow.document.write("<input type='button' value='$lt{'go'}' onclick='document.fileaction.submit();' />")
   newWindow.document.write("</td></tr>\\n")
   newWindow.document.write("</table>")
   newWindow.document.write('$end_page')
@@ -115,7 +114,7 @@ END_OF_ONE
 
 # ----------------------------------------------------------------  Jscript Two
 sub jscript_two {
-    my ($javascript,$user,$dom,$numcrs) = @_;
+    my ($user,$dom,$numcrs) = @_;
     my %crsentry = ();
     my $course_list;
     my $title_list;
@@ -134,7 +133,21 @@ sub jscript_two {
     $course_list = '"'.join('","',@crslist).'"';
     $$numcrs = @crslist;
 
-    $$javascript = qq#
+    my %lt = &Apache::lonlocal::texthash(
+                  sel    => 'Please select',
+                  impto  => 'Import topics only',
+                  imptpa => 'Import topics + posts (with author)',
+                  imptpn => 'Import topics + posts (no author)',
+                  enrst  => 'Enroll students only',
+                  enrall => 'Enroll all users',
+                  notreq => 'Not required',
+                  errao  => 'You must select one of the additional options when importing Discussion Boards.',
+                  errtd  => 'You must select a target course when importing Discussion Boards.',
+                  errap  => 'You must select one of the additional options when importing Enrollment.',
+                  errte  => 'You must select a target course when importing enrollment information.',
+                  errcc  => 'You must check at least one Content Type.',
+       );
+    return <<"END_OF_TWO";
 
 function checkCourse() {
   courseID_array = new Array($course_list)
@@ -182,7 +195,7 @@ function setCourse(step2Form,call) {
     step2Form.targetcourse.length = 0
     if (call == 'add') {
         step2Form.targetcourse.length = 0
-        step2Form.targetcourse.options[0] = new Option("Please Select","0",true,true)
+        step2Form.targetcourse.options[0] = new Option("$lt{'sel'}","0",true,true)
         for (var i=0; i<courseID_array.length; i++) {
             step2Form.targetcourse.options[i+1] = new Option(courseTitle_array[i],courseID_array[i],false,false)
         }
@@ -202,22 +215,22 @@ function setOptions(caller,itemnum) {
   opForm.elements[menu].length = 0
   if (opForm.elements[itemnum*2].checked == true) {
     if (caller == "board") {
-      opForm.elements[menu].options[0] = new Option("Select","-1",true,true)
-      opForm.elements[menu].options[1] = new Option("Import topics only","topics",true,true)
-      opForm.elements[menu].options[2] = new Option("Import topics + posts (with author)","allpost",true,true)
-      opForm.elements[menu].options[3] = new Option("Import topics + posts (no author)","allanon",true,true)
+      opForm.elements[menu].options[0] = new Option("$lt{'sel'}","-1",true,true)
+      opForm.elements[menu].options[1] = new Option("$lt{'impto'}","topics",true,true)
+      opForm.elements[menu].options[2] = new Option("$lt{'imptpa'}","allpost",true,true)
+      opForm.elements[menu].options[3] = new Option("$lt{'imptpn'}","allanon",true,true)
     }
     else { 
       if (caller == "users") {
         opForm.elements[menu].length = 0
-        opForm.elements[menu].options[0] = new Option("Select","-1",true,true)
-        opForm.elements[menu].options[1] = new Option("Enroll students only","students",true,true)
-        opForm.elements[menu].options[2] = new Option("Enroll all users","all",true,true)
+        opForm.elements[menu].options[0] = new Option("$lt{'sel'}","-1",true,true)
+        opForm.elements[menu].options[1] = new Option("$lt{'enrst'}","students",true,true)
+        opForm.elements[menu].options[2] = new Option("$lt{'enrall'}","all",true,true)
       }
     }
   }
   else {
-    opForm.elements[menu].options[0] = new Option("Not required","0",true,true)
+    opForm.elements[menu].options[0] = new Option("$lt{'notreq'}","0",true,true)
   }
   opForm.elements[menu].selectedIndex = 0
   if (numCrs > 0) {
@@ -235,7 +248,7 @@ function verify(caller) {
       totcheck ++
       if (opForm.elements[2*i].name == "board") { 
         if (opForm.elements[2*i+1].selectedIndex == 0) {     
-          alert("You must select one of the additional options when importing Discussion Boards ")
+          alert("$lt{'errao'}")
           return false
         }
         if (numCrs == 0) {
@@ -244,14 +257,14 @@ function verify(caller) {
         }
         else {
           if (opForm.targetcourse.selectedIndex == 0) {
-            alert("You must select a target course when importing Discussion Boards")
+            alert("$lt{'errtd'}")
             return false
           }
         }
       }
       if (opForm.elements[2*i].name == "users") {
         if (opForm.elements[2*i+1].selectedIndex == 0) {     
-          alert("You must select one of the additional options when importing Enrollment")
+          alert("$lt{'errap'}")
           return false
         }
         if (numCrs == 0) {
@@ -260,7 +273,7 @@ function verify(caller) {
         }
         else {
           if (opForm.targetcourse.selectedIndex == 0) {
-            alert("You must select a target course when importing enrollment information")
+            alert("$lt{'errte'}")
             return false
           }
         }
@@ -268,7 +281,7 @@ function verify(caller) {
     }
   }
   if (totcheck == 0) {
-    alert("You must check the Checkbox for at least one Content Type");
+    alert("$lt{'errcc'}");
     return false
   }
   return true
@@ -280,23 +293,18 @@ function nextPage(caller) {
   }
 }
 
-#;
+END_OF_TWO
 
 }
 
-# ----------------------------------------------------------------  Jscript Three
-sub jscript_three {
-    my $javascript = shift;
-}
-
 # ---------------------------------------------------------------- Display One
 sub display_one {
-    my ($r,$uname,$fn,$fullpath,$formname) = @_;
-    $r->print('<form name="'.$formname.'" method="post">'.
+    my ($r,$fn,$fullpath,$formname) = @_;
+    $r->print('<form name="'.$formname.'" method="post" action="">'.
               &Apache::lonhtmlcommon::topic_bar(1,&mt('Specify the Course Management system used to create the package')).
         &mt('Choose the CMS used to create your IMS content package.').'&nbsp;&nbsp;
         <select name="source">
-         <option value="-1" selected="selected">Please select</option>
+         <option value="-1" selected="selected">'.&mt('Please select').'</option>
          <option value="bb5">Blackboard 5</option>
          <option value="bb6">Blackboard 6</option>
          <option value="angel5">ANGEL 5.5</option>
@@ -304,22 +312,20 @@ sub display_one {
          <option value="webctvista4">WebCT Vista 4</option>
         </select><br />'."\n".
         &Apache::lonhtmlcommon::topic_bar(2,&mt('Create a directory where you will unpack your IMS package'))."\n".
-         &mt('Create a destination LON-CAPA directory in which to store the contents of the IMS package file.').'&nbsp;&nbsp;<input type="button" name="createdir" value="Create Directory" onClick="javascript:createWin()" /><input type="hidden" name="newdir" value="" /><br /><br />
-          <input type="hidden" name="uploaduname" value="'.$uname.'" />
+         &mt('Create a destination LON-CAPA directory in which to store the contents of the IMS package file.').'&nbsp;&nbsp;<input type="button" name="createdir" value="'.&mt('Create Directory').'" onclick="javascript:createWin()" /><input type="hidden" name="newdir" value="" /><br /><br />
           <input type="hidden" name="filename" value="'.$fn.'" />
           <input type="hidden" name="phase" value="three" />
-          <input type="button" name="nextpage" value="'.&mt('Proceed').'" onClick="javascript:nextPage();" />&nbsp;&nbsp;&nbsp;&nbsp;
-          <input type="button" name="exitpage" value="'.&mt('Exit now').'" onClick="javascript:location.href='."'$fullpath'".'" />
+          <input type="button" name="nextpage" value="'.&mt('Proceed').'" onclick="javascript:nextPage();" />&nbsp;&nbsp;&nbsp;&nbsp;
+          <input type="button" name="exitpage" value="'.&mt('Exit now').'" onclick="javascript:location.href='."'$fullpath'".'" />
          </form>');
 }
 
 # ---------------------------------------------------------------- Display Two
 sub display_two {
-    my ($r,$zipupload,$areas,$areaname,$cmsmap,$uname,$newdir,$numcrs,$fullpath) = @_;
+    my ($r,$zipupload,$areas,$areaname,$cmsmap,$fn,$numcrs,$fullpath) = @_;
     &Apache::loncommon::get_unprocessed_cgi($ENV{'QUERY_STRING'},['folder','source']);
     my $cms = $env{'form.source'};
-    my $dirname = $env{'form.newdir'};
-    my $tempdir = &Apache::imsprocessor::create_tempdir('CSTR',$dirname,'');
+    my $tempdir = &Apache::imsprocessor::create_tempdir('CSTR',$env{'form.newdir'},'');
     my $fname = &Apache::imsprocessor::uploadzip('CSTR',$tempdir,$zipupload);
     my $unzip_result = '';
     my $manifest_result = '';
@@ -340,12 +346,12 @@ sub display_two {
                   type => 'Content type',
                   addo => 'Additional options',
                   chec => 'Check Import first',
-                  bbus => 'Choose a course to receive bulletin boards and user enrollment',
-                  list => 'A listing of possible course targets will be displayed if import of bulletin boards and/or enrollment is checked above (step 3). If you do not plan to import either of these content types, there is no need to specify a course.',
+                  bbus => 'Choose a course to receive discussion boards and user enrollment',
+                  list => 'A listing of possible course targets will be displayed if import of discussion boards and/or enrollment is checked above (step 3). If you do not plan to import either of these content types, there is no need to specify a course.',
                   chco => 'Choose course:',
                   nreq => 'Not required',
-                  yodo => 'You do not have active course coordinator status in any LON-CAPA courses currently, so bulletin boards and enrollment information included in your IMS package will be discarded, regardless of your import choice for these two items above (step 3).',
-                  ifyo => "If you wish to import bulletin boards and/or user information into LON-CAPA please click 'Exit now' to quit the current IMS import process, and contact your domain coordinator and request a course coordinator role in a LON-CAPA course into which you can upload bulletin boards and/or enroll users.",
+                  yodo => 'You do not have active course coordinator status in any LON-CAPA courses currently, so discussion boards and enrollment information included in your IMS package will be discarded, regardless of your import choice for these two items above (step 3).',
+                  ifyo => "If you wish to import discussion boards and/or user information into LON-CAPA please click 'Exit now' to quit the current IMS import process, and contact your domain coordinator and request a course coordinator role in a LON-CAPA course into which you can upload discussion boards and/or enroll users.",
                   impa => 'Import package',
                   unpa => 'Unpacking of your IMS package failed because an IMS manifest file was not located in the package',
                   proc => 'Processing of your IMS package failed because the file you uploaded could not be unzipped',
@@ -404,7 +410,7 @@ sub display_two {
                 $conditions = 'none';
             }
 
-            $r->print('<form name="pickoptions" method="post">'.
+            $r->print('<form name="pickoptions" method="post" action="">'.
                       &Apache::lonhtmlcommon::topic_bar(3,$lt{'cont'}).
                       &Apache::loncommon::start_data_table().
                       &Apache::loncommon::start_data_table_header_row().
@@ -417,7 +423,7 @@ sub display_two {
                     $r->print(&Apache::loncommon::start_data_table_row()."\n".
                               '<td><input name="'.$area.'" type="checkbox" ');
                     if ($area eq 'board' || $area eq 'users') {
-                        $r->print('onClick="javascript:setOptions('."'$area','$counter'".');" ');
+                        $r->print('onclick="javascript:setOptions('."'$area','$counter'".');" ');
                     }
                     $r->print('/></td>'.
                               '<td>&nbsp;&nbsp;'.$$areaname{$area}.'&nbsp;&nbsp; - '.
@@ -459,8 +465,7 @@ sub display_two {
           <input type="hidden" name="conditions" value="'.$conditions.'" />
           <input type="hidden" name="source" value="'.$cms.'" />
           <input type="hidden" name="tempdir" value="'.$tempdir.'" />
-          <input type="hidden" name="uploaduname" value="'.$uname.'" />
-          <input type="hidden" name="filename" value="'.$fname.'" />
+          <input type="hidden" name="filename" value="'.$fn.'" />
           <input type="hidden" name="phase" value="four" />'."\n");
             if ($count{board} == 0) {
                 $r->print('<input type="hidden" name="board" value="" />'."\n");
@@ -468,19 +473,23 @@ sub display_two {
             if ($count{users} == 0) {
                 $r->print('<input type="hidden" name="users" value="" />'."\n");
             }
-            $r->print('<input type="button" name="nextpage" value="'.$lt{'impa'}.'" onClick="javascript:nextPage('."'$counter.'".')" />&nbsp;&nbsp;&nbsp;
-           <input type="button" name="exitpage" value="'.$lt{'exit'}.'" onClick="javascript:location.href='."'$fullpath'".'" /></form>');
+            $r->print('<input type="button" name="nextpage" value="'.$lt{'impa'}.'" onclick="javascript:nextPage('."'$counter.'".')" />&nbsp;&nbsp;&nbsp;
+           <input type="button" name="exitpage" value="'.$lt{'exit'}.'" onclick="javascript:location.href='."'$fullpath'".'" /></form>');
         } else {
             $r->print($lt{'unpa'});
         }
     } else {
-        $r->print($lt{'proc'});
+        $r->print(
+            '<br />'.&Apache::loncommon::confirmwrapper(
+                &Apache::lonhtmlcommon::confirm_success($lt{'proc'},1)
+               .'<br />'.&mt('Error: [_1]',$unzip_result))
+        );
     }
 }
 
 # ---------------------------------------------------------------- Display Three
 sub display_three {
-    my ($r,$uname,$udom,$areas,$areaname,$cmsmap,$destdir,$newdir) = @_;
+    my ($r,$uname,$udom,$areas,$areaname,$cmsmap,$destdir,$dirpath) = @_;
     my $crs = '';
     my $cdom = '';
     my $db_handling = '';
@@ -496,7 +505,7 @@ sub display_three {
     if ( defined($env{'form.user_crs'}) ) {
         ($user_cdom,$user_crs) = split/\//,$env{'form.user_crs'};
     }
-    my $seqstem = "/res/$udom/$uname/$newdir";
+    my $seqstem = "/res/$udom/$uname/$dirpath";
     my %importareas = ();
     my %includedres = ();
     my %includeditems = ();
@@ -545,14 +554,16 @@ sub display_three {
                  yims => 'Your IMS package has been processed successfully.',
                  plsv => 'Please view the imported items and use the LON-CAPA editing tools to make changes.',
                  tseq => "The sequences directory contains a file named 'Top.sequence' which includes links to the items found at the top level of your IMS package. From there you can follow links to display all the imported items. Alternatively, you can browse the pages, sequences, problems and resfiles directories directly. Note if you rename a file, you will need to modify any .sequence files or .page files which include a reference to the renamed file.",
-                 tfin => 'The final step in the IMS import process is to publish the materials you have imported into your Construction Space so that you can use them in a course. Once your files are published, subsequent re-publication will result in the storage of information about changes between the different versions.',
+                 tfin => 'The final step in the IMS import process is to publish the materials you have imported into your Authoring Space so that you can use them in a course. Once your files are published, subsequent re-publication will result in the storage of information about changes between the different versions.',
                  disp => 'Display new directory',
                  proc => 'Processing of your IMS package failed, because the IMS content package did not contain an IMS manifest file.'
              );
     my $manifest_result = &Apache::imsprocessor::process_manifest($cms,$tempdir,
                           \%resources,\%items,\%hrefs,\%resinfo,'prepare',
                           \%includedres);
+
     if ($manifest_result eq 'ok') {
+        my %possibledep;
         foreach my $res (sort keys %resources) {
             if ($importareas{$resources{$res}{type}}) {
                 $includedres{$res} = 1;
@@ -561,6 +572,13 @@ sub display_three {
                      $resources{$res}{type} eq 'webct.question') {
                     push(@assessmentfiles,$res);
                 }   
+            } elsif ($resources{$res}{usedby}) {
+                $possibledep{$res} = 1;
+            }
+        }
+        foreach my $res (sort(keys(%possibledep))) {
+            if ($includedres{$resources{$res}{usedby}}) {
+                $includedres{$res} = 1;
             }
         }
         foreach my $itm (sort keys %items) {
@@ -575,6 +593,7 @@ sub display_three {
     $manifest_result = &Apache::imsprocessor::process_manifest($cms,$tempdir,
                        \%resources,\%items,\%hrefs,\%resinfo,'build',
                        \%includedres,\%includeditems);
+
     if ($manifest_result eq 'ok') {
         &Apache::imsprocessor::target_resources(\%resources,\%importareas,\@targets);
 
@@ -593,86 +612,78 @@ sub display_three {
 
         &Apache::imsprocessor::process_resinfo($cms,'CSTR',$tempdir,$destdir,\%items,\%resources,\@targets,\@boards,\@announcements,\@quizzes,\@surveys,\@pools,\@groups,\%messages,\@timestamp,\%boardnum,\%resinfo,$udom,$uname,$cdom,$crs,$db_handling,$user_handling,\%total,$seqstem,$seqstem,\@resrcfiles,\@packages,\%hrefs,\@pages,\@sequences,\%randompicks);
 
-        my $copy_result = &Apache::imsprocessor::copy_resources('CSTR',$cms,\%hrefs,$tempdir,\@targets,\%urls,$crs,$cdom,$destdir,$timenow,\@assessmentfiles,\%total);
+        my $copy_result = &Apache::imsprocessor::copy_resources('CSTR',$cms,\%hrefs,\%resources,$tempdir,\@targets,\%urls,$crs,$cdom,$destdir,$timenow,\@assessmentfiles,\%total);
    
-        &Apache::imsprocessor::build_structure($cms,'CSTR',$destdir,\%items,\%resinfo,\%resources,\@targets,\%hrefs,$udom,$uname,$newdir,$timenow,$cdom,$crs,\@timestamp,\%total,\@boards,\@announcements,\@quizzes,\@surveys,\@pools,\%boardnum,\@pages,\@sequences,\@topurls,\@topnames,\@packages,\%includeditems,\%randompicks);
+        &Apache::imsprocessor::build_structure($cms,'CSTR',$destdir,\%items,\%resinfo,\%resources,\@targets,\%hrefs,$udom,$uname,$dirpath,$timenow,$cdom,$crs,\@timestamp,\%total,\@boards,\@announcements,\@quizzes,\@surveys,\@pools,\%boardnum,\@pages,\@sequences,\@topurls,\@topnames,\@packages,\%includeditems,\%randompicks);
 
-        $r->print("<h3>IMS import completed</h3>");
+        my $message =
+            &Apache::lonhtmlcommon::confirm_success(
+                &mt('IMS import completed'))
+           .'<br />'.$lt{'yims'}.' ';
 
         if ($cms eq 'angel5') {
-            $r->print($lt{'yims'}.' '.&mt('A total of [quant,_1,sequence], [quant,_2,composite page], and [quant,_3,bulletin board] have been created, and [quant,_4,file] copied.',$total{seq},$total{page},$total{board},$total{file})."\n");
+            $message .= &mt('A total of [quant,_1,sequence], [quant,_2,composite page], and [quant,_3,discussion board] have been created, and [quant,_4,file] copied.',$total{seq},$total{page},$total{board},$total{file})."\n";
         } else {
-            $r->print($lt{'yims'}.' '.&mt('A total of [quant,_1,sequence], [quant,_2,composite page], [quant,_3,bulletin board], [quant,_4,quiz,quizzes], [quant,_5,survey], and [quant,_6,problem] have been created, and [quant,_7,file] copied.',$total{seq},$total{page},$total{board},$total{quiz},$total{surv},$total{prob},$total{file})."\n");
+            $message .= &mt('A total of [quant,_1,sequence], [quant,_2,composite page], [quant,_3,discussion board], [quant,_4,quiz,quizzes], [quant,_5,survey], and [quant,_6,problem] have been created, and [quant,_7,file] copied.',$total{seq},$total{page},$total{board},$total{quiz},$total{surv},$total{prob},$total{file})."\n";
         }
-        $r->print('<br /><br />'.$lt{'plsv'}.' '.$lt{'tseq'}.'<br /><br />'.$lt{'tfin'}.'<br /><br /><a href="/priv/'.$uname.'/'.$newdir.'">'.$lt{'disp'}.'</a>');
-        if ($destdir =~ m-^/home/$uname/public_html/-) {
+        $r->print(
+            '<br />'.&Apache::loncommon::confirmwrapper($message)
+           .'<p>'.$lt{'plsv'}.' '.$lt{'tseq'}.'</p>'
+           .'<p>'.$lt{'tfin'}.'</p>'
+           .&Apache::lonhtmlcommon::actionbox(
+                ['<a href="/priv/'.$udom.'/'.$uname.'/'.$dirpath.'/">'.$lt{'disp'}.'</a>'])
+        );
+        my $londocroot = $r->dir_config('lonDocRoot');
+        if ($destdir =~ m{^\Q$londocroot/priv/$udom/$uname/$dirpath\E}) {
             system (" rm -r -f $destdir/temp");
         }
     } elsif ($manifest_result eq 'nomanifest') {
-        $r->print($lt{'proc'});
+        $r->print(
+            '<br />'.&Apache::loncommon::confirmwrapper(
+                &Apache::lonhtmlcommon::confirm_success($lt{'proc'},1))
+        );
     }
 }
 
 # ---------------------------------------------------------------- Get LON-CAPA Course Coordinator roles for this user
 sub get_ccroles {
     my ($user,$dom,$crsentry,$crslist) = @_;
-    my %roles = ();
+    my %roles;
     unless ($user eq '') {
-        %roles = &Apache::lonnet::dump('roles',$dom,$user);
+        my $ccrole = 'cc';
+        %roles = &Apache::lonnet::get_my_roles($user,$dom,'userroles',undef,[$ccrole]);
     }
     my $iter = 0;
     my @codes = ();
     my %courses = ();
     my @crslist = ();
     my %descrip =();
-    foreach my $key (keys %roles ) {
+    foreach my $key (keys(%roles)) {
         if ($key =~ m{^/($LONCAPA::domain_re)/($LONCAPA::username_re)_cc$}) {
             my $cdom = $1;
             my $crs = $2;
-            my $role_end = 0;
-            my $role_start = 0;
-            my $active_chk = 1;
-            if ( $roles{$key} =~ m/^cc_(\d+)/ ) {
-                $role_end = $1;
-                if ( $roles{$key} =~ m/^cc_($role_end)_(\d+)$/ )
-                {
-                    $role_start = $2;
-                }
+            my $currcode = '';
+            my %settings = &Apache::lonnet::get('environment',['internal.coursecode','description'],$cdom,$crs);
+            if (defined($settings{'description'}) ) {
+                $descrip{$crs} = $settings{'description'};
+            } else {
+                $descrip{$crs} = 'Unknown';
             }
-            if ($role_start > 0) {
-                if (time < $role_start) {
-                    $active_chk = 0;
-                }
-            }
-            if ($role_end > 0) {
-                if (time > $role_end) {
-                    $active_chk = 0;
-                }
-            }
-            if ($active_chk) {
-                my $currcode = '';
-                my %settings = &Apache::lonnet::get('environment',['internal.coursecode','description'],$cdom,$crs);
-                if (defined($settings{'description'}) ) {
-                    $descrip{$crs} = $settings{'description'};
-                } else {
-                    $descrip{$crs} = 'Unknown';
-                }
-                if (defined($settings{'internal.coursecode'}) ) {
-                    $currcode = $settings{'internal.coursecode'};
-                    if ($currcode eq '') {
-                        $currcode = "____".$iter;
-                        $iter ++;
-                    }
-                } else {
+            if (defined($settings{'internal.coursecode'}) ) {
+                $currcode = $settings{'internal.coursecode'};
+                if ($currcode eq '') {
                     $currcode = "____".$iter;
                     $iter ++;
                 }
-                unless (grep/^$currcode$/,@codes) {
-                    push @codes,$currcode;
-                    @{$courses{$currcode}} = ();
-                }
-                push @{$courses{$currcode}}, $cdom.'/'.$crs;
+            } else {
+                $currcode = "____".$iter;
+                $iter ++;
             }
+            unless (grep/^$currcode$/,@codes) {
+                push @codes,$currcode;
+                @{$courses{$currcode}} = ();
+            }
+            push @{$courses{$currcode}}, $cdom.'/'.$crs;
         }
     }
     foreach my $code (sort @codes) {
@@ -690,54 +701,43 @@ sub get_ccroles {
 # ---------------------------------------------------------------- Main Handler
 sub handler {
     my $r=shift;
-    my $uname;
-    my $udom;
-    my $javascript = '';
-    my $page_name = '';
-    my $current_page = '';
-    my $qcount = '';
+
+    my $fn=$env{'form.filename'};
+
+    if ($env{'form.filename1'}) {
+        $fn=$env{'form.filename1'}.$env{'form.filename2'};
+    }
+    $fn=~s{\+}{}g;
+
+    unless ($fn) {
+        $r->log_reason($env{'user.name'}.' at '.$env{'user.domain'}.
+                       ' unspecified filename for upload', $r->filename);
+        return HTTP_NOT_FOUND;
+    }
+
+    my ($uname,$udom) = &Apache::lonnet::constructaccess($fn);
+    if (($uname eq '') || ($udom eq '')) {
+        $r->log_reason($uname.' at '.$udom.
+                       ' trying to publish file '.$fn.' - not authorized',
+                       $r->filename);
+        return HTTP_NOT_ACCEPTABLE;
+    }
+
+    my $londocroot = $r->dir_config('lonDocRoot');
+    my $zipupload = $londocroot.$fn;
+    my $fullpath = &File::Basename::dirname($fn);
+    unless ($fullpath =~ m{/$}) {
+        $fullpath .= '/';
+    }
 
 # get personal information for this user
     my $user=$env{'user.name'};
     my $dom=$env{'user.domain'};
 
-#
-# re-attach user
-#
-    if ($env{'form.uploaduname'}) {
-        $env{'form.filename'}='/priv/'.$env{'form.uploaduname'}.'/'.
-            $env{'form.filename'};
-    }
-    ($uname,$udom)=
-        &Apache::loncacc::constructaccess($env{'form.filename'},
-                                          $r->dir_config('lonDefDomain'));
-    unless (($uname) && ($udom)) {
-        $r->log_reason($uname.' at '.$udom.
-                       ' trying to publish file '.$env{'form.filename'}.
-                       ' - not authorized',
-                       $r->filename);
-        return HTTP_NOT_ACCEPTABLE;
-    }
-                                                                                             
-    my $fn;
-    if ($env{'form.filename'}) {
-        $fn=$env{'form.filename'};
-        $fn=~s/^https?\:\/\/[^\/]+\///;
-        $fn=~s/^\///;
-        $fn=~s/(\~|priv\/)($LONCAPA::username_re)//;
-        $fn=~s/\/+/\//g;
-    } else {
-        $r->log_reason($env{'user.name'}.' at '.$env{'user.domain'}.
-                       ' unspecified filename for upload', $r->filename);
-        return HTTP_NOT_FOUND;
-    }
-    my $zipupload = '/home/'.$uname.'/public_html'.$fn;
-    my $pathname = &File::Basename::dirname($fn);
-    my $fullpath = '/priv/'.$uname.$pathname;
-    unless ($pathname eq '/') {
-        $fullpath .= '/';
-    }
-
+    my $javascript = '';
+    my $page_name = '';
+    my $current_page = '';
+    my $qcount = '';
     my @areas = ();
     my %cmsmap = ();
     my %areaname = ();
@@ -750,41 +750,54 @@ sub handler {
 
     my $formname_one = 'info';
     if ($env{'form.phase'} eq 'two') {
-        &jscript_one($fullpath,\$javascript,$formname_one);
+        $javascript = &jscript_one($fullpath,$formname_one);
     } elsif ($env{'form.phase'} eq 'three') {
-        &jscript_two(\$javascript,$user,$dom,\$numcrs);
-    } elsif ($env{'form.phase'} eq 'four') {
-        &jscript_three(\$javascript);
+        $javascript = &jscript_two($user,$dom,\$numcrs);
     }
-    $javascript = "<script type=\"text/javascript\">\n//<!--\n$javascript\n// --></script>\n";
+    if ($javascript ne '') {
+        $javascript = <<"END_JS";
+<script type="text/javascript">
+// <![CDATA[
 
-    my $title = 'Upload IMS package to Construction Space';
+$javascript
+
+// ]]>
+</script>
+END_JS
+    }
+
+    my $title = 'Upload IMS package to Authoring Space';
     $r->print(&Apache::loncommon::start_page($title, $javascript));
 
     if (($uname ne $env{'user.name'}) || ($udom ne $env{'user.domain'})) {
-        $r->print('<p><span class="LC_warning">'
+        $r->print('<p><span class="LC_info">'
                   .&mt('Co-Author [_1]',$uname.':'.$udom)
                   .'</span></p>'
         );
-    }   
+    }
     if ($env{'form.phase'} eq 'two') {
-        my $flag = &Apache::lonupload::phasetwo($r,$fn,$uname,$udom,'imsimport');
+        my $flag = &Apache::lonupload::phasetwo($r,$fn,'imsimport');
         if ($flag eq 'ok') {
-            &display_one($r,$uname,$fn,$fullpath,$formname_one);
+            &display_one($r,$fn,$fullpath,$formname_one);
+        } else {
+            $r->print(
+                '<br />'.&Apache::loncommon::confirmwrapper(
+                    &Apache::lonhtmlcommon::confirm_success(
+                        &mt('Error uploading IMS package'),1))
+            );
         }
     } elsif ( ($env{'form.phase'} eq 'three') || ($env{'form.phase'} eq 'four') ) {
-        my $docroot = $env{'form.newdir'};
-        my $newdir = '';
-        if ($docroot =~ m|public_html/(.+)$|) {
-            $newdir = $1;
-        }
+        my $destdir = $env{'form.newdir'};
+        my $dirpath = $destdir;
+        $dirpath =~ s{^\Q$londocroot/priv/$udom/$uname/\E}{};
+
         if ($env{'form.phase'} eq 'three') {
-            &display_two ($r,$zipupload,\@areas,\%areaname,\%cmsmap,$uname,$newdir,\$numcrs,$fullpath);
+            &display_two($r,$zipupload,\@areas,\%areaname,\%cmsmap,$fn,\$numcrs,$fullpath);
         } elsif ($env{'form.phase'} eq 'four') {
-            &display_three ($r,$uname,$udom,\@areas,\%areaname,\%cmsmap,$docroot,$newdir);
+            &display_three($r,$uname,$udom,\@areas,\%areaname,\%cmsmap,$destdir,$dirpath);
         }
     } else {
-        &Apache::lonupload::phaseone($r,$fn,$uname,$udom,'imsimport');
+        &Apache::lonupload::phaseone($r,$fn,'imsimport',$uname,$udom);
     }
     $r->print(&Apache::loncommon::end_page());
     return OK;
