@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # Utilities for classifying courses based on institutional code
 #
-# $Id: courseclassifier.pm,v 1.14 2011/03/03 00:06:38 raeburn Exp $
+# $Id: courseclassifier.pm,v 1.18 2014/03/17 02:33:32 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -37,7 +37,8 @@ use LONCAPA;
 sub retrieve_instcodes {
     my ($coursecodes,$codedom) = @_;
     my $totcodes;
-    my %courses = &Apache::lonnet::courseiddump($codedom,'.',1,'.','.','.',                                                undef,undef,'Course');
+    my %courses = &Apache::lonnet::courseiddump($codedom,'.',1,'.','.','.',
+                                                undef,undef,'Course');
     foreach my $course (keys(%courses)) {
         if (ref($courses{$course}) eq 'HASH') {
             if ($courses{$course}{'inst_code'} ne '') {
@@ -488,12 +489,27 @@ sub instcode_selectors_data {
                 if ($officialjs) {
                     $jscript .= '
 
-function toggleOfficial() {';
+function toggleOfficial() {
+   var choice; 
+   for (var i=0; i<document.'.$formname.'.official.length; i++) {
+       if (document.'.$formname.'.official[i].checked) {
+           choice = document.'.$formname.'.official[i].value;
+       }
+   }
+   if (choice == "off") {
+       if (document.getElementById("instcodes")) {
+           document.getElementById("instcodes").style.display = "none";
+       }';
                     for (my $i=0; $i<@{$codetitles}-1; $i++) {
                         $jscript .= '
-            document.'.$formname.'.'.$codetitles->[$i].'.selectedIndex = 0;';
+       document.'.$formname.'.'.$codetitles->[$i].'.selectedIndex = 0;';
                     }
-    $jscript .=  '
+    $jscript .= '
+   } else {
+       if (document.getElementById("instcodes")) { 
+           document.getElementById("instcodes").style.display = "block";
+       }
+   }
 }
 
 function setOfficial(caller) {
@@ -563,7 +579,11 @@ sub build_instcode_selectors {
     my $output;
     my @standardnames = &Apache::loncommon::get_standard_codeitems();
     if ($numtitles > 0) {
-        $output .= '<table><tr>';
+        my $style;
+        if ($env{'form.official'} eq 'off') {
+            $style = ' style="display: none"';
+        }
+        $output .= '<div id="instcodes"'.$style.'><table><tr>';
         for (my $k=0; $k<$lasttitle-1; $k++) {
             my (@items,@unsorted);
             if (ref($cat_items->{$codetitles->[$k]}) eq 'ARRAY') {
@@ -579,7 +599,7 @@ sub build_instcode_selectors {
                 @longitems = @items;
             }
             $output .= '<td align="center">'.$codetitles->[$k].'<br />'."\n".
-                       '<select name="'.$standardnames[$k].'" onChange="courseSet(this)"'.
+                       '<select name="'.$standardnames[$k].'" onchange="courseSet(this)"'.
                        '>'."\n".'<option value="0">'.&mt('All').'</option>'."\n";
             for (my $i=0; $i<@items; $i++) {
                 if ($longitems[$i] eq '') {
@@ -599,11 +619,38 @@ sub build_instcode_selectors {
                        '<input type="text" name="'.$standardnames[$numtitles-1].
                        '" /><br />'."\n";
         }
-        $output .= '<br />';
+        $output .= '</div>';
     }
     return $output;
 }
 
+sub instcode_selectors {
+    my ($codedom,$formname,$officialjs,$codetitles) = @_;
+    my ($output,%cat_titles,%cat_order,%cat_items);
+    my ($jscript,$totcodes,$numtitles,$lasttitle) =
+        &instcode_selectors_data($codedom,$formname,\%cat_items,$codetitles,
+                                 \%cat_titles,\%cat_order,$officialjs);
+    if ($numtitles > 0) {
+        my $official = ' checked="checked" ';
+        my $unofficial = '';
+        if (($formname eq 'filterpicker') && ($env{'form.official'} eq 'off')) {
+            $unofficial = $official;
+            $official = '';
+        }
+        $output .= '<span class="LC_nobreak">'.&mt('Official course:').'&nbsp;<label>'.
+                   '<input type="radio" name="official" value="on"'.$official.
+                   ' onclick="toggleOfficial();" />'.&mt('Yes').'</label>'.
+                   ('&nbsp;'x3).'<label>'.
+                   '<input type="radio" name="official" value="off"'.$unofficial.
+                   ' onclick="toggleOfficial();" />'.&mt('No').'</label></span><br />'.
+                   &build_instcode_selectors($numtitles,$lasttitle,\%cat_items,$codetitles,
+                                             \%cat_titles,\%cat_order)."\n".
+                   '<input type="hidden" name="numtitles" value="'.$numtitles.'" />'."\n".
+                   '<input type="hidden" name="state" value="listing" />'."\n";
+    }
+    return ($output,$jscript,$numtitles);
+}
+
 sub recurse_options {
     my ($currkey,$currlist,$level,$cat,$cat_options,$data,$by_year,$by_sem,$by_dept) = @_;
     if (ref($currlist) eq 'HASH') {
@@ -802,7 +849,7 @@ function courseSet(caller) {
                     }
                 }
                 if (iddept != -1) {
-                    document.$formname.Number.options[0] =  new Option('$all','0',false,false);
+                    document.$formname.Number.options[0] = new Option('$all','0',false,false);
                     for (var i=0; i<idcourses[idyr][idsem][iddept].length; i++) {
                         var display = idcourses[idyr][idsem][iddept][i];
                         if (longtitles[3] == 1) {
