@@ -2,7 +2,7 @@
 # The LON-CAPA group sort handler
 # Allows for sorting prior to import into RAT.
 #
-# $Id: groupsort.pm,v 1.66 2010/05/03 13:53:47 bisitz Exp $
+# $Id: groupsort.pm,v 1.68.6.6 2013/08/08 12:08:01 raeburn Exp $
 # 
 # Copyright Michigan State University Board of Trustees
 #
@@ -37,7 +37,7 @@ use GDBM_File;
 use Apache::loncommon;
 use Apache::lonlocal;
 use Apache::lonnet;
-use LONCAPA;
+use LONCAPA qw(:DEFAULT :match);
 
 my $iconpath; # variable to be accessible to multiple subroutines
 my %hash; # variable to tie to user specific database
@@ -86,8 +86,8 @@ sub update_actions_hash {
 sub readfromdb {
     my ($r,$resources)=@_;
 
-    my $diropendb = 
-       "/home/httpd/perl/tmp/$env{'user.domain'}_$env{'user.name'}_sel_res.db";
+    my $diropendb = LONCAPA::tempdir() .
+       "$env{'user.domain'}_$env{'user.name'}_sel_res.db";
 
 # ----------------------------- diropendb is now the filename of the db to open
     if (tie(%hash,'GDBM_File',$diropendb,&GDBM_WRCREAT(),0640)) {
@@ -152,12 +152,23 @@ sub readfromfile {
 		  &Apache::lonnet::filelocation('',$env{'form.readfile'}));
     } else {
         my $parser = HTML::TokeParser->new(\$cont);
-        my $token;
+        my ($token,$donechk,$allmaps);
+        $allmaps = {};
         while ($token = $parser->get_token) {
 	    if ($token->[0] eq 'S') {
                 if ($token->[1] eq 'resource') {
 		    if ($env{'form.recover'}) {
 			if ($token->[2]->{'type'} ne 'zombie') { next; }
+                        if ($token->[2]->{'src'} =~ /\.(page|sequence)$/) {
+                            if (($env{'request.course.id'}) &&
+                                ($env{'form.readfile'} =~ m{/default(|_\d+)\.(page|sequence)$})) {
+                                unless ($donechk) {
+                                    $allmaps = &Apache::loncommon::allmaps_incourse();
+                                    $donechk = 1;
+                                }
+                            }
+                            if ($allmaps->{$token->[2]->{'src'}}) { next; }
+                        }
 		    } else {
 			if ($token->[2]->{'type'} eq 'zombie') { next; }
 		    }
@@ -181,31 +192,12 @@ sub readfromfile {
     }
 }
 
-# --------------------------------------------------------- Read from bookmarks
-
-sub readfrombookmarks {
-    my ($r,$resources)=@_;
-    my %bookmarks=&Apache::lonnet::dump('bookmarks');
-# the bookmark "hash" is just one entry
-# it's a javascript program code with arguments like ('title','url');
-    my @bookmarks=($bookmarks{'bookmarks'}=~/\((?:\'([^\']+)\'\,\'([^\']+)\'|\"([^\"]+)\"\,\"([^\"]+)\")\)\;/g);
-    for (my $index=0;$index<($#bookmarks+1)/2;$index++) {
-        if ($bookmarks[$index*2+1]) {
-	    my $url  = $bookmarks[$index*2+1];
-	    my $name = $bookmarks[$index*2];
-	    $name =~ s/^LON\-CAPA\s+//;
-
-	    push(@{$resources},{'url' => $url, 'title' => $name});
-	}
-    }
-}
-
 # ---------------------------------------------------------------- Main Handler
 sub handler {
     my $r = shift;
  
    &Apache::loncommon::get_unprocessed_cgi($ENV{'QUERY_STRING'},
-			     ['acts','mode','readfile','recover','bookmarks']);
+			     ['acts','mode','readfile','recover']);
 
     &Apache::loncommon::content_type($r,'text/html');
     $r->send_http_header;
@@ -215,7 +207,7 @@ sub handler {
     my $finishimport='';
     my $begincondition='';
     my $endcondition='';
-    if (($env{'form.readfile'}) || ($env{'form.bookmarks'}))  {
+    if (($env{'form.readfile'}))  {
         $begincondition='if (eval("document.forms.groupsort.include"+num+".checked")) {';
 	$endcondition='}';
     }
@@ -299,8 +291,6 @@ END
 
     if ($env{'form.readfile'}) {
 	&readfromfile($r,\@resources);
-    } elsif ($env{'form.bookmarks'}) {
-	&readfrombookmarks($r,\@resources);
     } else {
 	&readfromdb($r,\@resources);
     }
@@ -313,10 +303,7 @@ END
     } else {
         $title = 'Sort Imported Resources';
     }
-    if ($env{'form.bookmarks'}) {
-        $title = 'Import Resources from Bookmarks';
-    }
-    if (($clen > 1) || ($env{'form.readfile'}) || ($env{'form.bookmarks'})) {
+    if (($clen > 1) || ($env{'form.readfile'})) {
 	my %lt=&Apache::lonlocal::texthash(
 		'fin'=> 'Finalize order of resources',
 		'ci' => 'Continue Import',
@@ -342,7 +329,6 @@ END
 <input type="hidden" name="newval" value="" />
 <input type="hidden" name="mode" value="$env{'form.mode'}" />
 <input type="hidden" name="readfile" value="$env{'form.readfile'}" />
-<input type="hidden" name="bookmarks" value="$env{'form.bookmarks'}" />
 <input type="hidden" name="recover" value="$env{'form.recover'}" />
 END
 
@@ -350,14 +336,11 @@ END
         # ---
 
         my $buttontext = $lt{'re'};
-        if ($env{'form.bookmarks'}) {
-            $buttontext = $lt{'ip'}
-        }
-        if ($env{'form.recover'} || $env{'form.bookmarks'}) {
+        if ($env{'form.recover'}) {
 	    $r->print(<<END);
 <input type="button" name="alter" value="$buttontext"
- onClick="finish_import()" />&nbsp;
-<input type="button" name="alter" value="$lt{'ca'}" onClick="self.close()" />
+ onclick="finish_import()" />&nbsp;
+<input type="button" name="alter" value="$lt{'ca'}" onclick="self.close()" />
 END
 	} else {
         # --- Continue Buttons
@@ -367,12 +350,12 @@ END
 <h2>$lt{'fin'}</h2>
 <div>
 <input type="button" name="alter" value="$lt{'ci'}"
- onClick="window.location='$resurl?inhibitmenu=yes&amp;catalogmode=import'" />&nbsp;
+ onclick="window.location='$resurl?inhibitmenu=yes&amp;catalogmode=import'" />&nbsp;
 <input type="button" name="altersearch" value="$lt{'cs'}"
- onClick="window.location='/adm/searchcat?inhibitmenu=yes&amp;catalogmode=import'" />&nbsp;
+ onclick="window.location='/adm/searchcat?inhibitmenu=yes&amp;catalogmode=import'" />&nbsp;
 <input type="button" name="alter" value="$lt{'fi'}"
- onClick="finish_import()" />&nbsp;
-<input type="button" name="alter" value="$lt{'ca'}" onClick="self.close()" />
+ onclick="finish_import()" />&nbsp;
+<input type="button" name="alter" value="$lt{'ca'}" onclick="self.close()" />
 </div>
 <br />
 END
@@ -382,7 +365,7 @@ END
         if ($clen > 0) {
             $r->print(&Apache::loncommon::start_data_table()
                      .&Apache::loncommon::start_data_table_header_row());
-            if (($env{'form.readfile'}) || ($env{'form.bookmarks'})) { 
+            if (($env{'form.readfile'})) { 
                 $r->print("<th>$lt{'in'}</th>\n");
             } else { 
                 $r->print('<th colspan="2">'.$lt{'co'}.'</th>'."\n"); 
@@ -397,9 +380,6 @@ END
             } else {
                 $errtxt = 'There are no resources to import.';
             }
-            if ($env{'form.bookmarks'}) {
-                $errtxt = 'There are no resources in your bookmarks to import.';
-            }
             $r->print('<p class="LC_info">'.&mt($errtxt).'</p>');
         }
     } else {
@@ -420,10 +400,10 @@ END
     foreach my $resource (@resources) {
 	$ctr++;
 	my $iconname=&Apache::loncommon::icon($resource->{'url'});
-	if (($clen > 1) || ($env{'form.readfile'}) || ($env{'form.bookmarks'})) {
+	if (($clen > 1) || ($env{'form.readfile'})) {
 	    $r->print(&Apache::loncommon::start_data_table_row()
                      ."<td>");
-            if (($env{'form.readfile'}) || ($env{'form.bookmarks'})) {
+            if (($env{'form.readfile'})) {
 		$r->print(&checkbox($ctr-1));
 	    } else {
 		$r->print(&movers($clen,$ctr));
@@ -431,9 +411,9 @@ END
 	}
 	$r->print(&hidden($ctr-1,$resource->{'title'},$resource->{'url'},
 			  $resource->{'id'}));
-	if (($clen > 1)  || ($env{'form.readfile'}) || ($env{'form.bookmarks'})) {
+	if (($clen > 1)  || ($env{'form.readfile'})) {
 	    $r->print("</td>");
-            unless (($env{'form.readfile'}) || ($env{'form.bookmarks'})) {
+            unless (($env{'form.readfile'})) {
 		$r->print("<td>".
 			  &select_box($clen,$ctr).
 			  "</td>");
@@ -441,13 +421,20 @@ END
 	    $r->print("<td>");
 	    $r->print("<img src='$iconname' />");
 	    $r->print("</td><td>");
-	    $r->print($resource->{'title'}.$resource->{'notes'}."</td><td>\n");
+            if (($env{'form.recover'}) &&
+                ($resource->{'url'} =~ m{/uploaded/$match_domain/$match_courseid/supplemental/})) {
+                my $title = &Apache::loncommon::parse_supplemental_title($resource->{'title'});
+                $r->print($title);
+            } else {
+                $r->print($resource->{'title'});
+            }
+            $r->print($resource->{'notes'}."</td><td>\n");
 	    $r->print($resource->{'url'}."</td>"
                      .&Apache::loncommon::end_data_table_row()
                      ."\n");
 	} 
     }
-    if (($clen > 1) || ($env{'form.readfile'}) || ($env{'form.bookmarks'})) {
+    if (($clen > 1) || ($env{'form.readfile'})) {
         if ($clen > 0) {
             $r->print(&Apache::loncommon::end_data_table());
         }
@@ -501,7 +488,7 @@ sub select_box {
     my ($total,$sel) = @_;
     my $string;
     $string = '<select name="alt'.$sel.'"';
-    $string .= " onChange='selectchange($sel)'>";
+    $string .= " onchange='selectchange($sel)'>";
     $string .= "<option name='o0' value='0'>".&mt('discard')."</option>";
     for my $cur (1..$total) {
 	$string .= "<option name='o$cur' value='$cur'";
