@@ -1,7 +1,7 @@
 # The LearningOnline Network
 # Announce
 #
-# $Id: lonannounce.pm,v 1.82 2010/03/16 20:10:13 raeburn Exp $
+# $Id: lonannounce.pm,v 1.87 2013/05/27 00:19:32 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -68,14 +68,14 @@ sub editfield {
 
     $r->print(<<ENDFORM);
 <h2>$lt{'annon'} $help</h2>
-<form name="anno" method="post">
+<form name="anno" method="post" action="">
 <input type="hidden" value='' name="action" />
 <table><tr><td>$lt{'start'}:</td><td>$startdateform</td></tr>
 <tr><td>$lt{'end'}:</td><td>$enddateform</td></tr></table>
 <textarea name="msg" rows="4" cols="60">$text</textarea>
 <br />
 <label><input type="checkbox" name="rsspost" /> $lt{'incrss'}</label>
-<br /><input type="button" onClick="trysubmit()" value="$lt{'post'}" /><hr />
+<br /><input type="button" onclick="trysubmit()" value="$lt{'post'}" /><hr />
 ENDFORM
 }
 
@@ -228,7 +228,7 @@ sub normalcell {
 	       && ($env{'form.pickdate'} ne 'yes')
 	       && (!$internalflag)) {
                $output.='<input type="checkbox" name="remove_'.$start.'_'.
-		   $end.'">';
+		   $end.'" />';
                $showedcheck{$start.'_'.$end}=1;
 	      }
 	    }
@@ -569,16 +569,30 @@ ENDDOCUMENT
     }
 # ------------------------------------------------------------ Process commands
     if ($serverpost) {
-	if ($env{'form.serveraction'}) {
-	    foreach my $key (keys(%env)) {
-		if ($key=~/^form\.postto\_(\w+)/) {
-		    $r->print( 
-			'<br />Posting '.$1.': '.&Apache::lonnet::postannounce
-			($1,$env{'form.serverannnounce'}));
-		}
-	    }
-	}
-	$r->print('<form name="serveranno" method="post">'
+        if ($env{'form.serveraction'}) {
+            my $rc;
+            my $message;
+            foreach my $key (keys(%env)) {
+                if ($key=~/^form\.postto\_(\w+[\w\-]*)/) {
+                     $rc = &Apache::lonnet::postannounce
+                           ($1,$env{'form.serverannnounce'});
+                    if ($rc eq 'ok') {
+                        $message .= 
+                            &Apache::lonhtmlcommon::confirm_success(
+                                &mt('Announcement posted to [_1]',$1))
+                               .'<br />';
+                    } else {
+                        $message .=
+                            &Apache::lonhtmlcommon::confirm_success(
+                                &mt('Posting announcement to [_1] failed: [_2]'
+                                    ,$1,$rc), 1)
+                               .'<br />';
+                    }
+                }
+            }
+            $r->print(&Apache::loncommon::confirmwrapper($message));
+        }
+	$r->print('<form name="serveranno" method="post" action="">'
                  .'<h2>'.&mt('Post Server Announcements').'</h2>'
                  .&mt('Post announcements to the system login and roles screen').'<br />'
                  .'<i>'.&mt('(leave blank to delete announcement)').'</i><br />'
@@ -761,7 +775,7 @@ ENDDOCUMENT
 # ----------------------------------------------------------------- Check marks
     undef(%showedcheck);
 # --------------------------------------------------------------- Remove button
-    if ($allowed) { $r->print('<br /><input type="button" onClick="removesub()" value="'.&mt('Remove Checked Entries').'">'.
+    if ($allowed) { $r->print('<br /><input type="button" onclick="removesub()" value="'.&mt('Remove Checked Entries').'" />'.
 			      &Apache::loncommon::help_open_topic('Calendar_Remove_Announcement').'</form>'); }
     $r->print('<p>'.
  '<a href="/adm/announcements?month='.$pm.'&amp;year='.$py.
