@@ -1,7 +1,7 @@
 # The LearningOnline Network
 # routines for clone a course
 #
-# $Id: lonclonecourse.pm,v 1.7 2008/09/11 13:19:28 raeburn Exp $
+# $Id: lonclonecourse.pm,v 1.9 2012/03/10 02:45:35 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -48,20 +48,23 @@ sub innercrsdirlist {
     unless ($path)  { $path=''; } else { $path.='/'; }
     my %crsdata=&Apache::lonnet::coursedescription($courseid);
     my $getpropath = 1;
-    my @listing=&Apache::lonnet::dirlist
-	($which,$crsdata{'domain'},$crsdata{'num'},$getpropath);
-    foreach (@listing) {
-	unless ($_=~/^\./) {
-	    my @unpackline = split (/\&/,$_);
-	    if ($unpackline[3]&$dirptr) {
+    my ($dirlistref,$listerror) = 
+        &Apache::lonnet::dirlist($which,$crsdata{'domain'},
+                                 $crsdata{'num'},$getpropath);
+    if (ref($dirlistref) eq 'ARRAY') {
+        foreach (@{$dirlistref}) {
+	    unless ($_=~/^\./) {
+	        my @unpackline = split (/\&/,$_);
+	        if ($unpackline[3]&$dirptr) {
 # is a directory, recurse
-		&innercrsdirlist($courseid,$which.$unpackline[0],
-				            $path.$unpackline[0]);
-	    } else { 
+		    &innercrsdirlist($courseid,$which.$unpackline[0],
+				     $path.$unpackline[0]);
+	        } else { 
 # is a file, put into output
-		push (@output,$path.$unpackline[0]);
+		    push (@output,$path.$unpackline[0]);
+	        }
 	    }
-	}
+        }
     }
     return @output;
 }
@@ -225,7 +228,7 @@ sub copydbfiles {
     foreach (&crsdirlist($origcrsid)) {
 	if ($_=~/\.db$/) {
 	    unless 
-             ($_=~/^(nohist\_|discussiontimes|classlist|versionupdate|resourcedata|\Q$origcrs_discussion\E|slots|slot_reservations|gradingqueue|reviewqueue|CODEs|groupmembership)/) {
+             ($_=~/^(nohist\_|disclikes|discussiontimes|classlist|versionupdate|resourcedata|\Q$origcrs_discussion\E|slots|slot_reservations|gradingqueue|reviewqueue|CODEs|groupmembership)/) {
 		 &copydb($origcrsid,$newcrsid,$_);
 	     }
 	}
