@@ -1,7 +1,7 @@
 # The LearningOnline Network
 # Communicate
 #
-# $Id: loncommunicate.pm,v 1.46 2009/07/25 23:16:04 raeburn Exp $
+# $Id: loncommunicate.pm,v 1.47 2012/04/18 19:56:59 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -102,7 +102,7 @@ sub menu {
 			 help => 'Course_Face_To_Face_Records,Course_Critical_Message',
              linktitle => "User notes, records of face-to-face discussions, critical messages, broadcast messages and archived messages in $crstype."},
                     
-            {url => '/adm/email?block=display',
+            {url => '/adm/setblock?caller=email',
 			 permission => "$can_dcm",
              icon => 'comblock.png',
 			 linktext => 'Communication Blocking',
