@@ -1,7 +1,7 @@
 # The LearningOnline Network
 # Create a course
 #
-# $Id: loncreatecourse.pm,v 1.137.4.1 2010/05/24 06:43:32 raeburn Exp $
+# $Id: loncreatecourse.pm,v 1.157 2014/02/28 19:20:06 bisitz Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -93,6 +93,7 @@ sub print_course_creation_page {
         $crstype = 'Course';
     }
     my $defdom=$env{'request.role.domain'};
+    my %domdefaults = &Apache::lonnet::get_domain_defaults($defdom);
     my %host_servers = &Apache::lonnet::get_servers($defdom,'library');
     my $course_home = '<select name="course_home" size="1">'."\n";
     foreach my $server (sort(keys(%host_servers))) {
@@ -154,11 +155,13 @@ function validate(formname) {
                     'iinf' => "Institutional Information",
                     'stat' => "The following entries will be used to identify the course according to the naming scheme adopted by your institution. Your choices will be used to map an internal LON-CAPA course ID to the corresponding course section ID(s) used by the office responsible for providing official class lists for courses at your institution. This mapping is required if you choose to employ automatic population of class lists.",
                     'ccod' => "Course Code",
-                    'toin' => "to interface with institutional data, e.g., fs03glg231 for Fall 2003 Geology 231",
                     'snid' => "Section Numbers and corresponding LON-CAPA section IDs",
                     'csli' => "a comma separated list of institutional section numbers, each separated by a colon from the (optional) corresponding section ID to be used in LON-CAPA e.g., 001:1,002:2",
                     'crcs' => "Crosslisted courses",
-                    'cscs' => "a comma separated list of course sections crosslisted with the current course, with each entry including the institutional course section name followed by a colon and then the (optional) sectionID to be used in LON-CAPA, e.g., fs03ent231001:ent1,fs03bot231001:bot1,fs03zol231002:zol2",
+                    'cscs' => "a comma separated list of course sections crosslisted with the current course, with each entry including the institutional course section name followed by a colon and then the (optional) section ID to be used in LON-CAPA, e.g., fs03ent231001:ent1,fs03bot231001:bot1,fs03zol231002:zol2",
+                    'ccre' => "Course credits",
+                    'crcd' => "Create six character course identifier to share with students",
+                    'cred' => "the number of institutional credits students will earn by completing this course",
                     'crco' => "Course Content",
                     'cncr' => "Completely new course",
                     'cecr' => "Clone an existing course", 
@@ -203,8 +206,6 @@ function validate(formname) {
                     'psam' => "Authentication mechanism",
                     'pcda' => "Default authentication method for new users added to this domain by the automated enrollment process",
                     'nech' => "Notification of enrollment changes",
-                    'nccl' => "Notification to course coordinator via LON-CAPA message when enrollment changes occur during the automated update?",
-                    'ndcl' => "Notification to domain coordinator via LON-CAPA message when enrollment changes occur during the automated update?",
                     'irsp' => "Include retrieval of student photographs?",
 		    'rshm' => 'Resource Space Home',
                     'cgrs' => "Course Group Settings",
@@ -213,6 +214,9 @@ function validate(formname) {
                     'opcm' => 'Open Community',
 	       );
     $lt{'kaut'} = &mt('Key authority ([_1]id:domain[_2]) if other than course','<tt>','</tt>');
+    $lt{'toin'} = &mt('to interface with institutional data, e.g., [_1]fs13glg231[_2] for [_3]Fall 2013 Geology 231[_4]','<tt>','</tt>','<i>','</i>');
+    $lt{'nccl'} = &mt('Notification to [_1]course coordinator[_2] via LON-CAPA message when enrollment changes occur during the automated update?','<b>','</b>');
+    $lt{'ndcl'} = &mt('Notification to [_1]domain coordinator[_2] via LON-CAPA message when enrollment changes occur during the automated update?','<b>','</b>');
 
     if ($crstype eq 'Community') {
         $lt{'cinf'} = &mt('Community Information');
@@ -277,7 +281,7 @@ END
 					    'Create_Courses');
     $r->print($start_page.$crumbs);
 
-    $r->print('<form action="/adm/createcourse" method="post" name="ccrs"><div class="LC_createcourse">'
+    $r->print('<form action="/adm/createcourse" method="post" name="ccrs" action=""><div class="LC_createcourse">'
              .&Apache::lonhtmlcommon::start_pick_box()
 
              .&Apache::lonhtmlcommon::row_headline()
@@ -299,7 +303,7 @@ END
              .&Apache::lonhtmlcommon::row_closure()
 
              .&Apache::lonhtmlcommon::row_title($lt{'ctit'})
-             .'<input type="text" size="50" name="title" />'
+             .'<input type="text" size="80" name="title" />'
              .&Apache::lonhtmlcommon::row_closure()
 
              .&Apache::lonhtmlcommon::row_title($lt{'chsr'})
@@ -313,7 +317,9 @@ END
     if ($crstype eq 'Course') {
         $r->print(&Apache::lonhtmlcommon::row_headline()
                  .'<h3>'.$lt{'iinf'}.'</h3>'
-                 .$lt{'stat'}
+                 .$lt{'stat'}.' '
+                 .&mt('Default credits can also be specified, if different from the domain defaults (official courses: [_1]; unofficial courses: [_2]; textbook courses: [_3]).',
+                      $domdefaults{'officialcredits'},$domdefaults{'unofficialcredits'},$domdefaults{'textbookcredits'})
                  .&Apache::lonhtmlcommon::row_closure()
 
                  .&Apache::lonhtmlcommon::row_title($lt{'ccod'})
@@ -331,8 +337,20 @@ END
                  .'<br />('.$lt{'cscs'}.')'
                  .&Apache::lonhtmlcommon::row_closure()
         );
+        if ($domdefaults{'officialcredits'} || $domdefaults{'unofficialcredits'} || $domdefaults{'textbookcredits'}) {
+            $r->print(&Apache::lonhtmlcommon::row_title($lt{'ccre'})
+                     .'<input type="text" size="3" name="defaultcredits" />'
+                     .'<br />('.$lt{'cred'}.')'
+                     .&Apache::lonhtmlcommon::row_closure()
+            );
+        }
+        $r->print(&Apache::lonhtmlcommon::row_title($lt{'crcd'})
+                     .'<span class="LC_nobreak">'
+                     .'<input type="radio" name="uniquecode" value="1" />'.&mt('Yes').('&nbsp;'x2)
+                     .'<input type="radio" name="uniquecode" value="0" checked="checked" />'.&mt('No')
+                     .&Apache::lonhtmlcommon::row_closure()
+            );
     }
-
     # Table: New Course / Clone Course
     $r->print(&Apache::lonhtmlcommon::row_headline()
                  .'<h3>'.$lt{'crco'}.'</h3>'
@@ -405,7 +423,7 @@ END
                  .' <input type="text" size="5" name="dateshift" value="365" />'
                  .&Apache::lonhtmlcommon::row_closure()
                  .&Apache::lonhtmlcommon::row_headline()
-                 .$lt{'asov'}
+                 .'<span class="LC_info">'.$lt{'asov'}.'</span>'
                  .&Apache::lonhtmlcommon::row_closure(1)
                  .&Apache::lonhtmlcommon::end_pick_box()
                  .'</td>'
@@ -537,7 +555,7 @@ END
 <p>
 <input type="hidden" name="prevphase" value="courseone" />
 <input type="hidden" name="phase" value="coursetwo" />
-<input type="button" onClick="verify_message(this.form)" value="'.$lt{'opco'}.'" />'
+<input type="button" onclick="verify_message(this.form)" value="'.$lt{'opco'}.'" />'
         );
     } elsif ($crstype eq 'Community') {
         $r->print('
@@ -560,6 +578,8 @@ sub create_course {
     my $ccdomain=&LONCAPA::clean_domain($env{'form.ccdomain'});
     my $crstype = 'Community';
     my ($enrollstart,$enrollend,$startaccess,$endaccess);
+    my %domdefaults = 
+        &Apache::lonnet::get_domain_defaults($env{'request.role.domain'});
 
     if ($env{'form.phase'} eq 'coursetwo') {
         $crstype='Course';
@@ -635,6 +655,15 @@ sub create_course {
                firstres => $env{'form.firstres'}
                };
 
+    if (($crstype eq 'Course') && 
+        ($domdefaults{'officialcredits'} || $domdefaults{'unofficialcredits'} || $domdefaults{'textbookcredits'})) {
+        $args->{'defaultcredits'} = $env{'form.defaultcredits'};
+    }
+
+    if ($env{'form.uniquecode'}) {
+         $args->{'uniquecode'} = 1;
+    }
+
     #
     # Verify data
     #
@@ -663,12 +692,12 @@ sub create_course {
                      ,$env{'form.course_home'}.&Apache::loncommon::end_page()));
         return;
     }
-    my ($courseid,$crsudom,$crsunum);
+    my ($courseid,$crsudom,$crsunum,$code);
     my ($success,$output) = 
 	&Apache::loncommon::construct_course($args,\$logmsg,\$courseid,
 					     \$crsudom,\$crsunum,
 					     $env{'user.domain'},
-					     $env{'user.name'},'dc_create');
+					     $env{'user.name'},'dc_create',undef,undef,\$code);
     $r->print($output);
     if ($success) {
         #
@@ -688,8 +717,13 @@ sub create_course {
 	    $r->print(
 		      '<p><a href="/adm/managekeys?cid='.$crsudom.'_'.$crsunum.'">'.&mt('Manage Access Keys').'</a></p>');
 	}
+        if ($code) {
+            $r->print(
+                      '<p>'.&mt('Course identifier to share with students: [_1]',$code).'</p>'
+                     );
+        }
         # Flush the course logs so reverse user roles immediately updated
-	&Apache::lonnet::flushcourselogs();
+	$r->register_cleanup(\&Apache::lonnet::flushcourselogs);
 	$r->print('<p>'.&mt('Roles will be active at next login').'.</p>');
     }
     $r->print('<p><a href="/adm/createcourse?phase='.lc($crstype).'one">'.
@@ -705,62 +739,79 @@ sub print_intro_page {
     my $end_page =
         &Apache::loncommon::end_page();
     my $helplink=&Apache::loncommon::help_open_topic('Create_Course_Community',&mt('Help on Creating Courses and Communities'));
-    my @choices = ({ internal_name     => 'courseone',
-                     name              => &mt('Create a single course'),
-                     short_description => &mt('Create a new course by completing an online form.'),
-                   });
-    if ($show_all_choices) {
-        push(@choices,{ internal_name     => 'groupone',
-                        name              => &mt('Create a single community'),
-                        short_description =>
-    &mt('Create a new collaborative community by completing an online form.'),
-                      },
-                      { internal_name     => 'batchone',
-                        name              => &mt('Create courses/communities by uploading an attributes file'),
-                        short_description =>
-    &mt('Upload an attributes file containing specifications for one or more courses or communities in XML format.'),
-                        help => 'Batch_Creation',
-                      },
-                      { internal_name     => 'requestdisplay',
-                        name              => &mt('Approve or reject requests'),
-                        short_description =>
-    &mt('Display course and community creation requests submitted by authorized users, held pending approval by a Domain Coordinator.'),
-                      },
-                      { internal_name     => 'pendingdisplay',
-                        name              => &mt('Display official course requests'),
-                        short_description =>
-    &mt('Display official course requests submitted by authorized users, held pending validation of instructor status.'),
-                      },
-                      { internal_name     => 'creationlog',
-                        name              => &mt('View creation log'),
-                        short_description =>
-    &mt('Display information about when, how and by whom courses and communities were created in this domain.'),
-                      });
-    } else {
-        push(@choices,{ internal_name     => 'batchone',
-                        name              => &mt('Create courses by uploading an attributes file'),
-                        short_description =>
-    &mt('Upload an attributes file containing specifications for one or more courses in XML format.'),
-                        help => 'Batch_Creation',
-                      });
-    }
-    my $options;
-    foreach my $choice (@choices) {
-        $options .='    <h3><a href="/adm/createcourse?phase='.
-            $choice->{'internal_name'}.'" >'.
-            $choice->{'name'}.'</a>';
-        if (exists($choice->{'help'})) {
-            $options .= &Apache::loncommon::help_open_topic($choice->{'help'});
-        }
-        $options .= "</h3>\n".'    '.('&nbsp;'x8).$choice->{'short_description'}.
-            "\n";
-    }
-    $r->print(<<ENDDOCUMENT);
-$start_page
-$crumbs
-$options
-$end_page
-ENDDOCUMENT
+
+    # Create menu
+    my @menu;
+    @menu =
+       ({ categorytitle => 'Course/Community Creation',
+       items => [
+          {
+               linktext => 'Create a single course',
+               url => '/adm/createcourse?phase=courseone',
+               permission => 1,
+               #help => '',
+               icon => 'crsnew.png',
+               linktitle => 'Create a new course by completing an online form.'
+          },
+          {
+               linktext => 'Create a single community',
+               url => '/adm/createcourse?phase=groupone',
+               permission => $show_all_choices,
+               #help => '',
+               icon => 'crsnew.png',
+               linktitle => 'Create a new collaborative community by completing an online form.'
+          },
+          {
+               linktext => 'Create courses/communities by uploading an attributes file',
+               url => '/adm/createcourse?phase=batchone',
+               permission => 1,
+               help => 'Batch_Creation',
+               icon => 'uplcrs.png',
+               linktitle => 'Upload an attributes file containing specifications for one or more courses or communities in XML format.'
+          },
+       ]
+       },
+       { categorytitle => 'Course/Community Requests',
+       items => [
+           {
+               linktext => 'Approve or reject requests',
+               url => '/adm/createcourse?phase=requestdisplay',
+               permission => $show_all_choices,
+               #help => '',
+               icon => 'rcrs.png',
+               linktitle => 'Display course and community creation requests submitted by authorized users, held pending approval by a Domain Coordinator.'
+           },
+           {
+               linktext => 'View pending official course requests',
+               url => '/adm/createcourse?phase=pendingdisplay',
+               permission => $show_all_choices,
+               #help => '',
+               icon => 'edit-find.png',
+               alttext => '',
+               linktitle => 'Display official course requests submitted by authorized users, held pending validation of instructor status.'
+           },
+       ]
+       },
+       { categorytitle => 'Administration',
+       items => [
+           {
+               linktext => 'View creation log',
+               url => '/adm/createcourse?phase=creationlog',
+               permission => $show_all_choices,
+               #help => '',
+               icon => 'document-properties.png',
+               linktitle => 'Display information about when, how and by whom courses and communities were created in this domain.'
+           },
+       ]
+       },
+       );
+
+    $r->print(
+        $start_page
+       .$crumbs
+       .&Apache::lonhtmlcommon::generate_menu(@menu)
+       .$end_page
+    );
 }
 
 sub upload_batchfile {
@@ -777,7 +828,7 @@ sub upload_batchfile {
               '<input type="file" name="coursecreatorxml" />'.
               '<input type="hidden" name="phase" value="batchtwo" /><br /><br />'.
               '<input type="submit" name="batchsubmit" '.
-              'value="Create Courses/Communities" /></form>');
+              'value="'.&mt('Create Courses/Communities').'" /></form>');
     $r->print($end_page);
     return;
 }
@@ -915,7 +966,7 @@ sub print_creation_logs {
     if ($numtitles) {
         if (($curr{'type'} eq 'official') || ($curr{'type'} eq 'unofficial')) {
             $instcodefilter = &instcode_srchstr($dom,$numtitles);
-                &Apache::courseclassifier::instcode_search_str($dom,$numtitles);
+                &Apache::courseclassifier::instcode_search_str($dom,$numtitles,\@codetitles);
             if ($curr{'type'} eq 'official') {
                 $regexpok = 1;
             } else {
@@ -1107,7 +1158,7 @@ sub creation_display_filter {
                         $typename = $typenames->{$type};
                     }
                 }
-                $output .= '<option value="'.$type.'"'.$selstr.'>'.$typename.'</option>'."\n";
+                $output .= '<option value="'.$type.'"'.$selstr.'>'.&mt($typename).'</option>'."\n";
             }
             $output .= '</select></td>';
         }
@@ -1265,7 +1316,7 @@ sub handler {
                    faq=>9,bug=>'Dom Coord Interface',});
            my $js = &courserequestbrowser_javascript();
            my $start_page=&Apache::loncommon::start_page('Display Requests',$js);
-           my $crumbs = &Apache::lonhtmlcommon::breadcrumbs('Display Requests','Course_Request',undef,'Course_Request');
+           my $crumbs = &Apache::lonhtmlcommon::breadcrumbs('Display Requests','Course_Requests',undef,'Course_Requests');
            $r->print($start_page.$crumbs."\n".'<div>'.
                      &Apache::loncoursequeueadmin::display_queued_requests(
                          'domain',$env{'request.role.domain'}).'</div>'.
@@ -1287,7 +1338,7 @@ sub handler {
                  text=>&mt('[_1] Requests Updated',),
                  faq=>9,bug=>'Dom Coord Interface',},);
            my $start_page=&Apache::loncommon::start_page('Update Requests Result');
-           my $crumbs = &Apache::lonhtmlcommon::breadcrumbs('Requests Updated','Course_Request',undef,'Course_Request');
+           my $crumbs = &Apache::lonhtmlcommon::breadcrumbs('Requests Updated','Course_Requests',undef,'Course_Requests');
            $r->print($start_page.$crumbs."\n".'<div>'.
                      &Apache::loncoursequeueadmin::update_request_queue(
                      'domain',$env{'request.role.domain'}).'</div>'.
@@ -1299,7 +1350,7 @@ sub handler {
                    faq=>9,bug=>'Dom Coord Interface',});
            my $js = &courserequestbrowser_javascript();
            my $start_page=&Apache::loncommon::start_page('Display Pending Queue',$js);
-           my $crumbs = &Apache::lonhtmlcommon::breadcrumbs('Display Pending Queue','Course_Request',undef,'Course_Request');
+           my $crumbs = &Apache::lonhtmlcommon::breadcrumbs('Display Pending Queue','Course_Requests',undef,'Course_Requests');
            $r->print($start_page.$crumbs."\n".'<div>'.
                      &Apache::loncoursequeueadmin::display_queued_requests(
                          'pending',$env{'request.role.domain'}).'</div>'.
@@ -1312,7 +1363,7 @@ sub handler {
                  {href=>"/adm/createcourse?phase=requestvalidation",
                    text=>&mt('Validation Attempted'),},);
            my $start_page=&Apache::loncommon::start_page('Requests Validation Result',$js);
-           my $crumbs = &Apache::lonhtmlcommon::breadcrumbs('Validation Attempted','Course_Request',undef,'Course_Request');
+           my $crumbs = &Apache::lonhtmlcommon::breadcrumbs('Validation Attempted','Course_Requests',undef,'Course_Requests');
            $r->print($start_page.$crumbs."\n".'<div>'.
                      &Apache::loncoursequeueadmin::process_official_reqs('domain',$env{'request.role.domain'}).'</div>'.
                      &Apache::loncommon::end_page());
@@ -1326,7 +1377,7 @@ sub handler {
                     '// ]]>'."\n".
                     '</script>';
            my $start_page=&Apache::loncommon::start_page('Course/Community Creation Logs',$js);
-           my $crumbs = &Apache::lonhtmlcommon::breadcrumbs('Created Courses/Communities','Creation_Log',undef,'Creation_Log');
+           my $crumbs = &Apache::lonhtmlcommon::breadcrumbs('Created Courses/Communities','Course_Creation_Log',undef,'Course_Creation_Log');
            $r->print($start_page.$crumbs."\n".'<div>');
            &print_creation_logs($r);
            $r->print('</div>'.&Apache::loncommon::end_page());
