@@ -1,6 +1,6 @@
 # The LearningOnline Network with CAPA
 #
-# $Id: lonhelp.pm,v 1.40 2011/01/04 21:53:23 raeburn Exp $
+# $Id: lonhelp.pm,v 1.44 2014/04/23 17:57:57 bisitz Exp $
 #
 # .tex help system web server handler
 #
@@ -57,31 +57,56 @@ sub processLabelName {
 
 # Serve out the text
 sub servetext {
-    my ($r,$text) = @_;
-    my $bugs=&Apache::loncommon::help_open_bug('Documentation',&mt('Report a documentation bug'));
-    my $morehelp;#=&Apache::loncommon::help_open_menu(undef,undef,1,undef,undef,undef,undef,'Help and Assistance Menu');
-    my $header=&mt('LON-CAPA Help');
+    my ($r,$uri,$text,$is_mobile,$firstfile) = @_;
+    my $bugs;
+    my %helpconfig = &Apache::lonnet::get_dom('configuration',['helpsettings'],
+                                              $env{'request.role.domain'});
+    if (ref($helpconfig{'helpsettings'}) eq 'HASH') {
+        if ($helpconfig{'helpsettings'}{'submitbugs'} eq '1') {
+            $bugs = &Apache::loncommon::help_open_bug('Documentation',&mt('Report a documentation bug'));
+        }
+    }
+    my %lt = &Apache::lonlocal::texthash(
+                                           header  => 'LON-CAPA Help',
+                                           search  => 'Search LON-CAPA help',
+                                           query   => 'Search',
+                                        );
     $r->print(<<HEADER);
     <h3 style="font: sans-serif"><img align="right" alt="help logo"
-    src="/adm/help/gif/lonhelpheader.gif"/>$header</h3><hr />
+    src="/adm/help/gif/lonhelpheader.gif" />$lt{'header'}</h3><hr />
     <!-- BEGIN -->
 HEADER
-
-    $r->print($text);
+    if ($is_mobile) {
+        my $width = 500;
+        my $height = 400;
+        my $machine = &Apache::lonnet::absolute_url();
+        $r->print(&Apache::loncommon::nicescroll_javascript('helpwrapper',
+                                                            {cursorcolor => '#00F',
+                                                             railalign => 'right',
+                                                             railoffset => '{top:5,left:40}'},
+                                                             undef,1,$machine.$firstfile));
+        $r->print('<div id="helpwrapper" style="height:'.$height.'px; width:'.$width.'px; overflow: auto;">'.
+                  $text.
+                  '</div>');
+    } else {
+        $r->print($text);
+    }
 
     if (&Apache::lonnavmaps::advancedUser()) {
-	my $search=&mt('Search LON-CAPA help');
-        my $query=&mt('Search');
-	my $uri = $r->uri;
-	$r->print(<<FOOTER);
-    <hr /><form action="$uri" method="post">
-<label>$search: <input type="text" name="searchterm" size="40" /></label><input type="submit" value="$query" />
-</form>
-<br />
+        $r->print(<<FOOTER);
+    <hr /><div class="LC_left_float">
+    <form action="$uri" method="post">
+    <fieldset><legend>$lt{'search'}</legend>
+    <input type="text" name="searchterm" size="40" />
+    <input type="submit" value="$lt{'query'}" />
+    </fieldset>
+    </form>
+    </div>
+    <div class="LC_left_float">
+    </div>
+    <div style="padding:0;clear:both;margin:0;border:0"></div>
 $bugs
-$morehelp
 FOOTER
- 
     }
 
     $r->print(<<ENDBODY);
@@ -125,7 +150,7 @@ sub render {
              '\\begin{html}<a href="/adm/help/'.
               $label .
               $ext . 
-             '"><img src="'.$icon.'" border="0" /></a>' .
+             '"><img src="'.$icon.'" border="0" alt="'.&mt('Help').'" /></a>' .
              '\\end{html}'
              |gxe;
 
@@ -199,10 +224,30 @@ sub handler {
 
      &Apache::lonlocal::get_language_handle($r);
      &Apache::loncommon::content_type($r,"text/html");
+     my $caller;
+     if ($env{'form.searchterm'}=~/\w/) {
+        $caller = 'search';
+     }
+
+     my $starthash;
+
+     if ($env{'browser.mobile'}) {
+         $starthash = {
+                        only_body   => 1,
+                        add_entries => {
+                                         'onload' => "javascript:expand_div('$caller');",
+                                       },
+                      };
+     } else {
+         $starthash = {
+                        only_body   => 1,
+                      };
+     }
+     my $firstfile;
      my $start_page=
-	 &Apache::loncommon::start_page('LON-CAPA Help',undef,
-					{'only_body' => 1,});
+	 &Apache::loncommon::start_page('LON-CAPA Help',undef,$starthash);
      my $text='';
+     my $uri = $r->uri;
      if ($env{'form.searchterm'}=~/\w/) {
 	 ($text,my $matches)=&listmatches($docroot,$env{'form.searchterm'},&Apache::lonlocal::current_language().'/');
          if ($matches) {
@@ -214,8 +259,7 @@ sub handler {
              $text=&listmatches($docroot,$env{'form.searchterm'}); 
          }
      } else {
-	 my $filenames = &unescape(substr($ENV{'REQUEST_URI'} , 
-					  rindex($ENV{'REQUEST_URI'}, '/') + 1, -4));
+         my $filenames = &unescape(substr($uri,rindex($uri,'/')+1,-4));
 	 
 	 # Security check on the file; the whole filename must consist
 	 # of nothing but alphanums, ' ,, or ., or the file
@@ -231,7 +275,8 @@ sub handler {
 	 # so replace : with ,
 	 $filenames =~ s/:/,/g;
 	 my @files = split(/,/, $filenames);
-	 
+         $firstfile = '/adm/help/'.$files[0].'.hlp';
+
 	 for my $filename (@files) {
 	     if (-e $docroot.'/adm/help/tex/'.
 		 &Apache::lonlocal::current_language().'/'.
@@ -249,7 +294,7 @@ sub handler {
 
      $r->send_http_header;
      $r->print($start_page);
-     &servetext($r,$text);
+     &servetext($r,$uri,$text,$env{'browser.mobile'},$firstfile);
      $r->print(&Apache::loncommon::end_page());
 
      return OK;
