@@ -1,7 +1,7 @@
 # The LearningOnline Network
 # Pick a course
 #
-# $Id: lonpickcourse.pm,v 1.103 2011/09/01 02:32:50 raeburn Exp $
+# $Id: lonpickcourse.pm,v 1.115 2014/05/30 01:50:05 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -31,11 +31,11 @@ package Apache::lonpickcourse;
 use strict;
 use Apache::Constants qw(:common);
 use Apache::loncommon;
+use Apache::lonhtmlcommon;
 use Apache::loncoursedata;
 use Apache::lonnet;
 use Apache::lonlocal;
 use Apache::longroup;
-use Apache::courseclassifier;
 use LONCAPA qw(:DEFAULT :match);
 
 sub handler {
@@ -105,9 +105,22 @@ sub handler {
 
     if ((($env{'form.form'} eq 'cu') || ($env{'form.form'} eq 'studentform')) && 
         ($env{'form.pickedcourse'})) {
+        if ($type ne 'Community') {
+            my %coursedescription =
+                &Apache::lonnet::coursedescription($env{'form.pickedcourse'},
+                                                   {'one_time' => '1'});
+            my $cdom = $coursedescription{'domain'};
+            my %domdefs = &Apache::lonnet::get_domain_defaults($cdom);
+            if (($domdefs{'officialcredits'} || $domdefs{'unofficialcredits'} || $domdefs{'textbookcredits'})) {
+                $loaditem{'onload'} .= 'setDefaultCredits();';
+            }
+        }
         $loaditem{'onload'} .= 'setRoles();setSections();';
     }
-    my $js = &js_changer();
+    if ((($env{'form.gosearch'}) && ($env{'form.updater'} eq '')) && (!$onlyown)) {
+        $loaditem{'onload'} .=  'hideSearching(); ';
+    }
+    my $js = &Apache::loncommon::js_changer();
     $r->print(&Apache::loncommon::start_page($title,$js,
 					     {'add_entries' => \%loaditem,
 					      'no_nav_bar'  => 1, }));
@@ -133,16 +146,15 @@ sub handler {
         $onlyown) {
         $r->print(&gochoose_javascript($type,$multiple,$autosubmit,$lastaction));
     }
-    $r->print('<script type="text/javascript" language="Javascript">'.$jscript.
-              '</script>');
+    $r->print(&Apache::lonhtmlcommon::scripttag($jscript));
     $r->print($submitopener);
 
 # ------------------------------------------ Display of filters to limit search
     my $filter = {};
     my $action = '/adm/pickcourse';
     my ($numtitles,$showroles,$nohost,@codetitles);
-    if (!$onlyown) {
-        my $filterlist = ['domainfilter'];
+    unless ($onlyown) {
+        my $filterlist = ['domainfilter','sincefilter'];
         # created filter for DCs only
         if ($env{'user.adv'} && $env{'form.domainfilter'} &&
             exists($env{'user.role.dc./'.$env{'form.domainfilter'}.'/'})
@@ -154,7 +166,15 @@ sub handler {
         }
         push(@{$filterlist},('descriptfilter','instcodefilter'));
         if ($env{'form.form'} eq 'rules') {
-            push(@{$filterlist},'personfilter');
+            push(@{$filterlist},('personfilter','persondomfilter'));
+            if ($env{'form.persondomfilter'} eq '') {
+                unless ($env{'form.gosearch'}) {
+                    $filter->{'persondomfilter'} = $env{'request.role.domain'};
+                }
+            } else {
+                $filter->{'persondomfilter'} =
+                    &LONCAPA::clean_domain($env{'form.persondomfilter'});
+            }
             if (($env{'form.personfilter'} ne '') && ($env{'form.persondomfilter'} ne '')) {
                 if (&Apache::lonnet::homeserver($env{'form.personfilter'},
                                                  $env{'form.persondomfilter'}) eq 'no_host') {
@@ -164,7 +184,7 @@ sub handler {
                 } 
             }
         } else {
-            push(@{$filterlist},'ownerfilter');
+            push(@{$filterlist},('ownerfilter','ownerdomfilter'));
         }
         # course ID filter for DCs only
         if ($env{'user.adv'} && $env{'form.domainfilter'} &&
@@ -174,10 +194,40 @@ sub handler {
         if ($cloneruname ne '' && $clonerudom ne '') {
             push(@{$filterlist},'cloneableonly');
         }
-
-        $r->print(&build_filters($filterlist,$type,$roleelement,$multelement,
-                                 $filter,$action,\$numtitles,undef,$cloneruname,
-                                 $clonerudom,$typeelement,\@codetitles));
+        if ((ref($filterlist) eq 'ARRAY') && (ref($filter) eq 'HASH')) {
+            foreach my $item (@{$filterlist}) {
+                $filter->{$item} = $env{'form.'.$item};
+            }
+        }
+        if ($env{'form.form'} eq 'portform') {
+            $filter->{'domainfilter'} ||= $env{'user.domain'};
+        } elsif ($env{'form.form'} eq 'studentform') {
+            $filter->{'domainfilter'} ||= $env{'request.role.domain'};
+        }
+        my $codedom;
+        if ($env{'form.fixeddom'}) {
+            $codedom = $env{'request.role.domain'};
+        } else {
+            $codedom = $filter->{'domainfilter'};
+        }
+        my ($clonetext,$clonewarning);
+        if (($env{'form.form'} eq 'ccrs') || ($env{'form.form'} eq 'requestcrs')) {
+            (my $coord_cloneable,$clonewarning) =
+                &get_coordinator_cloneable($cloneruname,$clonerudom,$type);
+            if ($env{'form.form'} eq 'ccrs') {
+                $clonetext = '<input type="hidden" name="cloner" value="'.$env{'form.cloner'}.'" />'."\n";
+            }
+            if ($coord_cloneable) {
+                $clonetext .= '<input type="hidden" name="cc_clone" value="'.$coord_cloneable.'" />';
+            }
+        }
+        $r->print(&Apache::loncommon::build_filters($filterlist,$type,$roleelement,$multelement,
+                                                    $filter,$action,\$numtitles,undef,$cloneruname,
+                                                    $clonerudom,$typeelement,\@codetitles,$codedom,
+                                                    $env{'form.form'},$env{'form.fixeddom'},
+                                                    $env{'form.prevphase'},$env{'form.cnameelement'},
+                                                    $env{'form.cnumelement'},$env{'form.cdomelement'},
+                                                    $env{'form.setroles'},$clonetext,$clonewarning));
     }
 
 # ---------------------------------------------------------------- Get the data
@@ -190,8 +240,21 @@ sub handler {
                 $domcloner = 1;
             }
         }
-        my %courses = &search_courses($r,$type,$onlyown,$filter,$numtitles,
-                                      $cloneruname,$clonerudom,$domcloner,\@codetitles);
+        my %courses;
+        if (!$onlyown) {
+            $r->print('<div id="searching">'.&mt('Searching ...').'</div>');
+            $r->rflush();
+            my $srchdom = $filter->{'domainfilter'};
+            %courses = &Apache::loncommon::search_courses($srchdom,$type,$filter,$numtitles,
+                                                          $cloneruname,$clonerudom,$domcloner,
+                                                          \@codetitles,$env{'form.cc_clone'});
+        } else {
+            $r->print('<br />');
+            my %coursehash = &Apache::loncommon::findallcourses();
+            foreach my $cid (sort(keys(%coursehash))) {
+                $courses{$cid}{'description'} = $env{'course.'.$cid.'.description'};
+            }
+        }
         if ($nohost) {
             $r->print ('<span class="LC_warning">'.
                        &mt('User does not exist - username: [_1], domain: [_2].',
@@ -206,23 +269,6 @@ sub handler {
     return OK;
 }
 
-sub js_changer {
-    return <<"ENDJS";
-<script type="text/javascript">
-
-function updateFilters(caller) {
-    if (typeof(caller) != "undefined") {
-        document.filterpicker.updater.value = caller.name;
-    }
-    document.filterpicker.submit();
-}
-
-</script>
-
-ENDJS
-
-}
-
 sub processpick {
     my $openerform = 'rolechoice';
     if ($env{'form.form'} eq 'studentform') {
@@ -282,6 +328,13 @@ sub create_user_javascript {
         my $cdom = $coursedescription{'domain'};
         my $cnum = $coursedescription{'num'};
         my $crstype = $coursedescription{'type'};
+        my %domdefs = &Apache::lonnet::get_domain_defaults($cdom);
+        my ($showcredits,$credits);
+        if (($crstype ne 'Community') &&
+            (($domdefs{'officialcredits'} || $domdefs{'unofficialcredits'} || $domdefs{'textbookcredits'}))) {
+            $showcredits = 1; 
+            $credits = $coursedescription{'internal.defaultcredits'};
+        }
         my $sec_element = 'currsec';
         my $grplist_element = 'groups';
         my ($sections,$groups) =
@@ -292,6 +345,7 @@ sub create_user_javascript {
         my $groupslist = join(',',@{$groups});
         $output = qq|
 <script type="text/javascript">
+// <![CDATA[
 function setSections() {
     opener.document.$env{"form.form"}.$grplist_element.value='$groupslist';
     window.opener.setSect('$seclist');
@@ -300,9 +354,18 @@ function setSections() {
 function setRoles() {
     window.opener.setRole('$crstype');
 }
+|;
+        if ($showcredits) {
+            $output .= qq|
+function setDefaultCredits() {
+    window.opener.setCredits('$credits');
+}
+|;
+        }
+        $output .= qq|
+// ]]>
 </script>
 |;
-;
     }
     return $output;
 }
@@ -325,9 +388,10 @@ sub display_matched_courses {
             my $ccrolechk = ' ';
             my $menuchk = ' checked="checked" ';
             $r->print(
-                '<fieldset>'
-               .'<legend>'.&mt('Action').'</legend>'
-               .'<div class="LC_nobreak"><label>'
+                '<div class="LC_left_float">'
+               .'<fieldset>'
+               .'<legend>'.&mt('Pick action').'</legend>'
+               .'<span class="LC_nobreak"><label>'
                .'<input type="radio" name="phase" value="ccrole"'.$ccrolechk.'/>'
                .'&nbsp;');
             if ($type eq 'Community') {
@@ -335,17 +399,17 @@ sub display_matched_courses {
             } else {
                 $r->print(&mt('Enter the course with the role of [_1].',$cctitle));
             }
-            $r->print('</label></div>'
-               .'<div class="LC_nobreak"><label>'
+            $r->print('</label></span><br />'
+               .'<span class="LC_nobreak"><label>'
                .'<input type="radio" name="phase" value="menu"'.$menuchk.'/>&nbsp;');
             if ($type eq 'Community') {
                 $r->print(&mt('View or modify community settings which only a [_1] may modify.',$dctitle));
             } else {
                 $r->print(&mt('View or modify course settings which only a [_1] may modify.',$dctitle));
             }
-            $r->print('</label></div>'
-               .'</fieldset>'
-               .'<br />'
+            $r->print('</label></span>'
+               .'</fieldset></div>'
+               .'<br clear="all" />'
             );
         }
     }
@@ -364,9 +428,9 @@ sub display_matched_courses {
     }
  
     if ($numcourses > 1 && $multiple) {
-        $r->print('<input type="button" value="check all"
+        $r->print('<input type="button" value="'.&mt('check all').'"
                   onclick="javascript:checkAll(document.courselist.course_id)" />
-                  &nbsp;&nbsp;<input type="button" value="uncheck all"
+                  &nbsp;&nbsp;<input type="button" value="'.&mt('uncheck all').'"
                   onclick="javascript:uncheckAll(document.courselist.course_id)" />
                   <br /><br />');
     }
@@ -494,9 +558,9 @@ sub display_matched_courses {
     }
 
     if (!%courses) {
-        $r->print(&mt('None found'));
+        $r->print('<p class="LC_info">'.&mt('None found').'</p>');
     } elsif ($multiple) {
-        $r->print('<input type="button" value="Submit" onClick="gochoose('."'','','')".'" />');
+        $r->print('<input type="button" value="'.&mt('Submit').'" onclick="gochoose('."'','','')".'" />');
     }
     $r->print('<input type="hidden" name="form" value="'.$env{'form.form'}.'" />'.
               "\n".'<input type="hidden" name="pickedcourse" value="" />'."\n".
@@ -541,293 +605,6 @@ sub multiples_tag {
     return ($jscript,$multelement);
 }
 
-sub build_filters {
-    my ($filterlist,$type,$roleelement,$multelement,$filter,$action,
-        $numtitlesref,$caller,$cloneruname,$clonerudom,$typeelement,$codetitlesref) = @_;
-    my ($list,$formname,$fixeddom,$codedom,$jscript);
-    $codedom = $env{'request.role.domain'};
-    if (defined($env{'form.form'})) {
-        $formname = $env{'form.form'};
-    } else {
-        $formname = $caller;
-    }
-    my $onchange = 'javascript:updateFilters(this)';
-    my ($domainselectform,$sincefilterform,$createdfilterform,
-        $ownerdomselectform,$persondomselectform,$instcodeform,
-        $typeselectform,$instcodetitle);
-    foreach my $item (@{$filterlist}) {
-        $filter->{$item} = $env{'form.'.$item};
-        if ($item ne 'descriptfilter' && $item ne 'instcodefilter') {
-	    if ($item eq 'domainfilter') {
-		$filter->{$item} = &LONCAPA::clean_domain($filter->{$item});
-	    } elsif ($item eq 'coursefilter') {
-		$filter->{$item} = &LONCAPA::clean_courseid($filter->{$item});
-	    } elsif ($item eq 'ownerfilter') {
-		$filter->{$item} = &LONCAPA::clean_username($filter->{$item});
-                $filter->{'ownerdomfilter'} = 
-                    &LONCAPA::clean_domain($env{'form.ownerdomfilter'});
-                $ownerdomselectform =
-                    &Apache::loncommon::select_dom_form($filter->{'ownerdomfilter'},
-                                                        'ownerdomfilter',1);
-            } elsif ($item eq 'personfilter') {
-                $filter->{$item} = &LONCAPA::clean_username($filter->{$item});
-                if ($env{'form.persondomfilter'} eq '') {
-                    unless ($env{'form.gosearch'}) {
-                        $filter->{'persondomfilter'} = $env{'request.role.domain'};
-                    }
-                } else {
-                    $filter->{'persondomfilter'} =
-                        &LONCAPA::clean_domain($env{'form.persondomfilter'});
-                }
-                $persondomselectform =
-                    &Apache::loncommon::select_dom_form($filter->{'persondomfilter'},
-                                                        'persondomfilter',1);
-	    } else {
-		$filter->{$item} =~ s/\W//g;
-	    }
-            if (!$filter->{$item}) {
-                $filter->{$item} = '';
-            }
-        }
-        if ($item eq 'domainfilter') {
-	    my $allow_blank = 1;
-	    if ($formname eq 'portform') {
-		$filter->{$item} ||= $env{'user.domain'};
-		$allow_blank=0;
-	    } elsif ($formname eq 'studentform') {
-                $filter->{$item} ||= $env{'request.role.domain'};
-                $allow_blank=0;
-            }
-            if ($env{'form.fixeddom'}) {
-                $domainselectform = '<input type="hidden" name="domainfilter"'.
-                                    'value="'.$env{'request.role.domain'}.'" />'.
-                                    &Apache::lonnet::domain($env{'request.role.domain'},
-                                                            'description');
-                $codedom = $env{'request.role.domain'};
-            } else {
-                $domainselectform =
-		    &Apache::loncommon::select_dom_form($filter->{$item},
-		    				        'domainfilter',
-						        $allow_blank,'',$onchange);
-                $codedom = $filter->{'domainfilter'};
-            }
-        } else {
-            $list->{$item} = &HTML::Entities::encode($filter->{$item},'<>&"');
-        }
-    }
-
-    # last course activity filter and selection
-    $sincefilterform = &timebased_select_form('sincefilter',$filter);
-
-    # course created filter and selection
-    if (exists($filter->{'createdfilter'})) {
-        $createdfilterform = &timebased_select_form('createdfilter',$filter);
-    }
-
-    my %lt = &Apache::lonlocal::texthash(
-                'cac' => "$type Activity",
-                'ccr' => "$type Created", 
-                'cde' => "$type Title",
-                'cdo' => "$type Domain",
-                'ins' => 'Institutional Code',
-                'inc' => 'Institutional Categorization',
-                'cow' => "$type Owner/Co-owner",
-                'cop' => "$type Personnel Includes",
-                'cog' => 'Type',
-             );
-
-    if (($env{'form.form'} eq 'ccrs') || ($env{'form.form'} eq 'requestcrs')) {
-        my $typeval = 'Course';
-        if ($type eq 'Community') {
-            $typeval = 'Community';
-        }
-        $typeselectform = '<input type="hidden" name="type" value="'.$typeval.'" />';
-    } else {
-        $typeselectform =  '<select name="type" size="1"';
-        if ($onchange) {
-            $typeselectform .= 'onchange="'.$onchange.'"';
-        }
-        $typeselectform .= '>'."\n";
-        foreach my $posstype ('Course','Community') {
-            $typeselectform.='<option value="'.$posstype.'"'.
-                ($posstype eq $type ? ' selected="selected" ' : ''). ">".&mt($posstype)."</option>\n";
-        }
-        $typeselectform.="</select>";
-    }
-    
-    my ($cloneableonlyform,$cloneabletitle);
-    if (exists($filter->{'cloneableonly'})) {
-        my $cloneableon = '';
-        my $cloneableoff = ' checked="checked"';
-        if ($filter->{'cloneableonly'}) {
-            $cloneableon = $cloneableoff;
-            $cloneableoff = '';
-        }
-        $cloneableonlyform = '<span class="LC_nobreak"><label><input type="radio" name="cloneableonly" value="1" '.$cloneableon.'/>&nbsp;'.&mt('Required').'</label>'.('&nbsp;'x3).'<label><input type="radio" name="cloneableonly" value="" '.$cloneableoff.' />&nbsp;'.&mt('No restriction').'</label></span>';
-        if ($env{'form.form'} eq 'ccrs') {
-            $cloneabletitle = &mt('Cloneable for').' '.$cloneruname.':'.$clonerudom;
-        } else {
-            $cloneabletitle = &mt('Cloneable by you');
-        }
-    }
-    my $officialjs;
-    if ($type eq 'Course') {
-        if (exists($filter->{'instcodefilter'})) {
-            if (($env{'form.fixeddom'}) || ($formname eq 'requestcrs')
-                || ($formname eq 'modifycourse')) {
-                $officialjs = 1;
-                ($instcodeform,$jscript,$$numtitlesref) = 
-                    &instcode_selectors($codedom,'filterpicker',$officialjs,$codetitlesref);
-                if ($jscript) {
-                    $jscript = '<script type="text/javascript" language="Javascript">'.
-                               $jscript.'</script>'."\n";
-                }
-            }
-            if ($instcodeform eq '') {
-                $instcodeform = 
-                    '<input type="text" name="instcodefilter" size="10" value="'.
-                    $list->{'instcodefilter'}.'" />';
-                $instcodetitle = $lt{'ins'};
-            } else {
-                $instcodetitle = $lt{'inc'};
-            }
-            if ($env{'form.fixeddom'}) {
-                $instcodetitle .= '<br />('.$codedom.')';
-            }
-        }
-    }
-
-    my $output = qq|
-<form method="post" name="filterpicker" action="$action">
-<input type="hidden" name="form" value="$formname" />
-|;
-    if ($formname eq 'modifycourse') {
-        $output .= '<input type="hidden" name="phase" value="courselist" />'."\n".
-                   '<input type="hidden" name="prevphase" value="'.
-                   $env{'form.prevphase'}.'" />'."\n";
-    } else {
-        my $name_input;
-        if ($env{'form.cnameelement'} ne '') {
-            $name_input = '<input type="hidden" name="cnameelement" value="'.
-                          $env{'form.cnameelement'}.'" />';
-        }
-        $output .= qq|
-<input type="hidden" name="cnumelement" value="$env{'form.cnumelement'}" />
-<input type="hidden" name="cdomelement" value="$env{'form.cdomelement'}" />
-$name_input
-$roleelement
-$multelement
-$typeelement
-|;
-        if ($formname eq 'portform') {
-            $output .= '<input type="hidden" name="setroles" value="'.$env{'form.setroles'}.'" />'."\n";
-        }
-    }
-    if ($env{'form.fixeddom'}) { 
-        $output .= '<input type="hidden" name="fixeddom" value="'.$env{'form.fixeddom'}.'" />'."\n";
-    }
-    $output .= "<br />\n".&Apache::lonhtmlcommon::start_pick_box();
-    if ($sincefilterform) { 
-        $output .= &Apache::lonhtmlcommon::row_title($lt{'cac'})
-                  .$sincefilterform
-                  .&Apache::lonhtmlcommon::row_closure();
-    }
-    if ($createdfilterform) {
-        $output .= &Apache::lonhtmlcommon::row_title($lt{'ccr'})
-                  .$createdfilterform
-                  .&Apache::lonhtmlcommon::row_closure();
-    }
-    if ($domainselectform) {
-        $output .= &Apache::lonhtmlcommon::row_title($lt{'cdo'})
-                  .$domainselectform
-                  .&Apache::lonhtmlcommon::row_closure();
-    } 
-    if ($typeselectform) {
-        if (($env{'form.form'} eq 'ccrs') || ($env{'form.form'} eq 'requestcrs')) {
-            $output .= $typeselectform;
-        } else {
-            $output .= &Apache::lonhtmlcommon::row_title($lt{'cog'})
-                      .$typeselectform
-                      .&Apache::lonhtmlcommon::row_closure();
-        }
-    }
-    if ($instcodeform) {
-        $output .= &Apache::lonhtmlcommon::row_title($instcodetitle)
-                  .$instcodeform
-                  .&Apache::lonhtmlcommon::row_closure();
-    }
-    if (exists($filter->{'ownerfilter'})) {
-        $output .= &Apache::lonhtmlcommon::row_title($lt{'cow'}).
-                   '<table><tr><td>'.&mt('Username').'<br />'.
-                   '<input type="text" name="ownerfilter" size="20" value="'.
-                   $list->{'ownerfilter'}.'" /></td><td>'.&mt('Domain').'<br />'.
-                   $ownerdomselectform.'</td></tr></table>'.
-                   &Apache::lonhtmlcommon::row_closure();
-    }
-    if (exists($filter->{'personfilter'})) {
-        $output .= &Apache::lonhtmlcommon::row_title($lt{'cop'}).
-                   '<table><tr><td>'.&mt('Username').'<br />'.
-                   '<input type="text" name="personfilter" size="20" value="'.
-                   $list->{'personfilter'}.'" /></td><td>'.&mt('Domain').'<br />'.
-                   $persondomselectform.'</td></tr></table>'.
-                   &Apache::lonhtmlcommon::row_closure();
-    }
-    if (exists($filter->{'coursefilter'})) {
-        $output .= &Apache::lonhtmlcommon::row_title(&mt('LON-CAPA course ID'))
-                  .'<input type="text" name="coursefilter" size="25" value="'
-                  .$list->{'coursefilter'}.'" />'
-                  .&Apache::lonhtmlcommon::row_closure();
-    }
-    if ($cloneableonlyform) {
-        $output .= &Apache::lonhtmlcommon::row_title($cloneabletitle).
-                   $cloneableonlyform.&Apache::lonhtmlcommon::row_closure();
-    }
-    if (exists($filter->{'descriptfilter'})) {
-        $output .= &Apache::lonhtmlcommon::row_title($lt{'cde'})
-                  .'<input type="text" name="descriptfilter" size="40" value="'
-                  .$list->{'descriptfilter'}.'" />'
-                  .&Apache::lonhtmlcommon::row_closure(1);
-    }
-    $output .= &Apache::lonhtmlcommon::end_pick_box().'<p>';
-    my ($coord_cloneable,$warning);
-    if (($env{'form.form'} eq 'ccrs') || ($env{'form.form'} eq 'requestcrs')) {
-        ($coord_cloneable,$warning) = 
-            &get_coordinator_cloneable($cloneruname,$clonerudom,$type);
-        if ($env{'form.form'} eq 'ccrs') {
-            $output .= '<input type="hidden" name="cloner" value="'.$env{'form.cloner'}.'" />'."\n";
-        }
-        if ($coord_cloneable) {
-            $output .= '<input type="hidden" name="cc_clone" value="'.$coord_cloneable.'" />';
-        }
-    }
-    $output .= '<input type="hidden" name="updater" value="">'."\n".
-               '<input type="submit" name="gosearch" value="'. 
-               &mt('Search').'" /></p>'."\n".'</form>'."\n".'<hr />'."\n";
-    return $jscript.$warning.$output;
-}
-
-sub timebased_select_form {
-    my ($item,$filter) = @_;
-    if (ref($filter) eq 'HASH') {
-        $filter->{$item} = $env{'form.'.$item};
-        $filter->{$item} =~ s/[^\d-]//g;
-        if (!$filter->{$item}) { $filter->{$item}=-1; }
-        return &Apache::loncommon::select_form( 
-                            $filter->{$item},
-                            $item,
-                            {      '-1' => '',
-                                '86400' => &mt('today'),
-                               '604800' => &mt('last week'),
-                              '2592000' => &mt('last month'),
-                              '7776000' => &mt('last three months'),
-                             '15552000' => &mt('last six months'),
-                             '31104000' => &mt('last year'),
-                    'select_form_order' =>
-                           ['-1','86400','604800','2592000','7776000',
-                            '15552000','31104000']});
-    }
-}
-
 sub get_coordinator_cloneable {
     my ($cloneruname,$clonerudom,$type) = @_;
     if (($cloneruname!~/\w/) || ($clonerudom!~/\w/)) {
@@ -856,129 +633,6 @@ sub get_coordinator_cloneable {
     }
 }
 
-sub instcode_selectors {
-    my ($codedom,$formname,$officialjs,$codetitles) = @_;
-    my ($output,%cat_titles,%cat_order,%cat_items);
-    my ($jscript,$totcodes,$numtitles,$lasttitle) =
-        &Apache::courseclassifier::instcode_selectors_data($codedom,$formname,
-                           \%cat_items,$codetitles,\%cat_titles,\%cat_order,$officialjs);
-    if ($numtitles > 0) {
-        my $official = ' checked="checked" ';
-        my $unofficial = '';
-        if ($env{'form.official'} eq 'off') {
-            $unofficial = $official;
-            $official = '';
-        }
-        $output .= '<span class="LC_nobreak">'.&mt('Official course:').'&nbsp;<label>'.
-                   '<input type="radio" name="official" value="on"'.$official.' />'.
-                   &mt('Yes').'</label>'.('&nbsp;'x3).'<label>'.
-                   '<input type="radio" name="official" value="off"'.$unofficial.
-                   ' onclick="toggleOfficial();" />'.&mt('No').'</label></span><br />'.
-                   &Apache::courseclassifier::build_instcode_selectors($numtitles,
-                   $lasttitle,\%cat_items,$codetitles,\%cat_titles,\%cat_order)."\n".
-                   '<input type="hidden" name="numtitles" value="'.$numtitles.'" />'."\n".
-                   '<input type="hidden" name="state" value="listing" />'."\n";
- 
-    }
-    return ($output,$jscript,$numtitles);
-}
-
-sub search_courses {
-    my ($r,$type,$onlyown,$filter,$numtitles,$cloneruname,$clonerudom,$domcloner,$codetitles) = @_;
-    my (%courses,%showcourses,$cloner,$ccrole);
-    if ($type eq 'Community') {
-        $ccrole = 'co';
-    } else {
-        $ccrole = 'cc';
-    }
-    if (!$onlyown) {
-        $r->print(&mt('Searching ...').'<br />&nbsp;<br />');
-        $r->rflush();
-        if (($filter->{'ownerfilter'} ne '') ||
-            ($filter->{'ownerdomfilter'} ne '')) {
-            $filter->{'combownerfilter'} = $filter->{'ownerfilter'}.':'.
-                                           $filter->{'ownerdomfilter'};
-        }
-        foreach my $item ('descriptfilter','coursefilter','combownerfilter') {
-            if (!$filter->{$item}) {
-                $filter->{$item}='.';
-            }
-        }
-        my $now = time;
-        my $timefilter = 
-           ($filter->{'sincefilter'}==-1?1:$now-$filter->{'sincefilter'});
-        my ($createdbefore,$createdafter);
-        if (($filter->{'createdfilter'} ne '') && ($filter->{'createdfilter'} !=-1)) {
-            $createdbefore = $now;
-            $createdafter = $now-$filter->{'createdfilter'};
-        }
-        my ($instcodefilter,$regexpok);
-        if ($numtitles) {
-            if ($env{'form.official'} eq 'on') {
-                $instcodefilter = 
-                    &Apache::courseclassifier::instcode_search_str($filter->{'domainfilter'},$numtitles,$codetitles);
-                $regexpok = 1;
-            } elsif ($env{'form.official'} eq 'off') {
-                $instcodefilter = &Apache::courseclassifier::instcode_search_str($filter->{'domainfilter'},$numtitles,$codetitles);
-                unless ($instcodefilter eq '') {
-                    $regexpok = -1;
-                }
-            }
-        } else {
-            $instcodefilter = $filter->{'instcodefilter'};
-        }
-        if ($instcodefilter eq '') { $instcodefilter = '.'; }
-        if ($type eq '') { $type = '.'; }
- 
-        if (($clonerudom ne '') && ($cloneruname ne '')) {
-            $cloner = $cloneruname.':'.$clonerudom;  
-        }
-        %courses = 
-            &Apache::lonnet::courseiddump($filter->{'domainfilter'},
-                                          $filter->{'descriptfilter'},
-                                          $timefilter,
-                                          $instcodefilter,
-                                          $filter->{'combownerfilter'},
-                                          $filter->{'coursefilter'},
-                                          undef,undef,$type,$regexpok,undef,undef,
-                                          undef,undef,$cloner,$env{'form.cc_clone'},
-                                          $filter->{'cloneableonly'},
-                                          $createdbefore,$createdafter,undef,
-                                          $domcloner);
-        if (($filter->{'personfilter'} ne '') && ($filter->{'persondomfilter'} ne '')) {
-            my %rolehash = &Apache::lonnet::get_my_roles($filter->{'personfilter'},
-                                                         $filter->{'persondomfilter'},
-                                                         'userroles',undef,
-                                                         [$ccrole,'in','ad','ep','ta','cr'],
-                                                         $filter->{'domainfilter'});
-            foreach my $role (keys(%rolehash)) {
-                my ($cnum,$cdom,$courserole) = split(':',$role);
-                my $cid = $cdom.'_'.$cnum;
-                if (exists($courses{$cid})) {
-                    if (ref($courses{$cid}) eq 'HASH') {
-                        if (ref($courses{$cid}{roles}) eq 'ARRAY') {
-                            if (!grep(/^\Q$courserole\E$/,@{$courses{$cid}{roles}})) { 
-                                push (@{$courses{$cid}{roles}},$courserole);
-                            }
-                        } else {
-                            $courses{$cid}{roles} = [$courserole];
-                        }
-                        $showcourses{$cid} = $courses{$cid};
-                    }
-                }
-            }
-            %courses = %showcourses;
-        }
-    } else {
-        $r->print('<br />');
-        my %coursehash = &Apache::loncommon::findallcourses(); 
-        foreach my $cid (sort(keys(%coursehash))) {
-            $courses{$cid}{'description'} = $env{'course.'.$cid.'.description'};
-        }
-    }
-    return %courses;
-}
-
 sub course_chooser {
     my ($multiple,$cdom,$cnum,$cleandesc,$canclone) = @_;
     my $output; 
@@ -986,12 +640,12 @@ sub course_chooser {
         $output = '<label><input type="checkbox" name="course_id" value="'.$cdom.'_'.$cnum.'" />'."\n";
     } elsif ((($env{'form.form'} eq 'ccrs') || ($env{'form.form'} eq 'requestcrs')) && (!$canclone))  {
         if ($env{'form.form'} eq 'ccrs') {
-            $output = &mt('No cloning for ').$env{'form.cloner'}."\n";
+            $output = &mt('No cloning for [_1]',$env{'form.cloner'})."\n";
         } else {
             $output = &mt('No rights to clone')."\n";
         }
     } else {
-        $output = '<input type="button" value="'.&mt('Select').'" onClick="gochoose('.
+        $output = '<input type="button" value="'.&mt('Select').'" onclick="gochoose('.
                   "'".$cnum."','".$cdom."','".$cleandesc."')".'" />'."\n";
     }
     return $output;
@@ -1011,8 +665,7 @@ sub gochoose_javascript {
                                  list  => 'courselist',
                               },
                     );
-    my $output .= qq|
-<script type="text/javascript">
+    my $output = qq|
 function gochoose(cname,cdom,cdesc) {
     var openerForm = "$env{'form.form'}";
     courseCount = 0;
@@ -1098,9 +751,8 @@ ENDNAMECODE
     $autosubmit
     $lastaction
 }
-</script>
 |;
-    return $output;
+    return &Apache::lonhtmlcommon::scripttag($output);
 }
 
 1;
@@ -1216,30 +868,6 @@ Output: 2 - $jscript - javascript for check all/uncheck all checkboxes; $multele
 
 Side Effects: None
 
-=item *
-X<build_filters()>
-B<build_filters($filterlist,$type,$roleelement,$multelement,$filter,$action,$numfiltersref,$caller,$cloneruname,$clonerudom,$typeelem,$codetitlesref>:
-
-
-Input: 12 - anonymous array of search criteria; course type; $roleelement ; $multelement ; anonymous hash of criteria and their values; form action; ref to scalar (count of number of elements in institutional codes -- e.g., 4 for year, semester, department, and number); caller context (e.g., set to 'modifycourse' when routine is called from lonmodifycourse.pm); username and domain of new course owner who wants to clone, $typeelem; reference to array of titles of components in institutional codes (official courses). 
-
-Output: 1 - $output - HTML for display of search criteria, and hidden form elements. 
-
-Side Effects: None
-
-=item *
-X<search_courses()>
-B<search_courses($r,$type,$onlyown,$filter,$numtitles,$cloneruname,$clonerudom,
-                 $domcloner,$codetitles)>:
-
-
-Input: 9 -  request object, course type, search scope: only courses in which user has active role (1), or any course (0); anonymous hash of criteria and their values; for institutional codes - number of categories; optional username of new course owner, optional domain of new course owner). Seventh and eighth needed when search is for courses to clone from course request form, or course creation form (DC). Optional "domcloner" flag (has value=1 if user has ccc priv in  
-domain being filtered by (used when DC is using course creation form). Ninth is reference to array of titles of components in institutional codes (official courses).
-
-Output: 1 - %courses - hash of courses satisfying search criteria, keys = course IDs, values are corresponding colon-separated escaped description, institutional code, owner and type.
-
-Side Effects: None
-
 
 =item *
 X<course_chooser()>
