@@ -1,7 +1,7 @@
 # The LearningOnline Network
 # Preferences
 #
-# $Id: lonpreferences.pm,v 1.186.2.8 2011/10/10 22:38:20 raeburn Exp $
+# $Id: lonpreferences.pm,v 1.196.4.15 2014/06/09 22:39:01 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -36,8 +36,6 @@ use strict;
 use LONCAPA;
 use Apache::Constants qw(:common);
 use Apache::File;
-use Crypt::DES;
-use DynaLoader; # for Crypt::DES version
 use Apache::loncommon();
 use Apache::lonhtmlcommon();
 use Apache::lonlocal;
@@ -45,50 +43,6 @@ use Apache::lonnet;
 use LONCAPA::lonauthcgi();
 use LONCAPA();
 
-#
-# Write lonnet::passwd to do the call below.
-# Use:
-#   my $answer=reply("encrypt:passwd:$udom:$uname:$upass",$tryserver);
-#
-##################################################
-#          password associated functions         #
-##################################################
-sub des_keys {
-    # Make a new key for DES encryption.
-    # Each key has two parts which are returned separately.
-    # Please note:  Each key must be passed through the &hex function
-    # before it is output to the web browser.  The hex versions cannot
-    # be used to decrypt.
-    my @hexstr=('0','1','2','3','4','5','6','7',
-                '8','9','a','b','c','d','e','f');
-    my $lkey='';
-    for (0..7) {
-        $lkey.=$hexstr[rand(15)];
-    }
-    my $ukey='';
-    for (0..7) {
-        $ukey.=$hexstr[rand(15)];
-    }
-    return ($lkey,$ukey);
-}
-
-sub des_decrypt {
-    my ($key,$cyphertext) = @_;
-    my $keybin=pack("H16",$key);
-    my $cypher;
-    if ($Crypt::DES::VERSION>=2.03) {
-        $cypher=new Crypt::DES $keybin;
-    } else {
-        $cypher=new DES $keybin;
-    }
-    my $plaintext=
-	$cypher->decrypt(unpack("a8",pack("H16",substr($cyphertext,0,16))));
-    $plaintext.=
-	$cypher->decrypt(unpack("a8",pack("H16",substr($cyphertext,16,16))));
-    $plaintext=substr($plaintext,1,ord(substr($plaintext,0,1)) );
-    return $plaintext;
-}
-
 ################################################################
 #                       Handler subroutines                    #
 ################################################################
@@ -152,32 +106,16 @@ sub languagechanger {
                 text => 'Change Language'});
     $r->print(Apache::loncommon::start_page('Content Display Settings'));
     $r->print(Apache::lonhtmlcommon::breadcrumbs('Change Language')); 
-    my $user       = $env{'user.name'};
-    my $domain     = $env{'user.domain'};
-    my %userenv = &Apache::lonnet::get
-        ('environment',['languages']);
+    my %userenv = &Apache::lonnet::get('environment',['languages']);
     my $language=$userenv{'languages'};
 
-    my $pref=&mt('Preferred language');
-    my %langchoices=('' => 'No language preference');
-    foreach (&Apache::loncommon::languageids()) {
-	if (&Apache::loncommon::supportedlanguagecode($_)) {
-	    $langchoices{&Apache::loncommon::supportedlanguagecode($_)}
-	               = &Apache::loncommon::plainlanguagedescription($_);
-	}
-    }
-    %langchoices = &Apache::lonlocal::texthash(%langchoices);
-    my $selectionbox=
-           &Apache::loncommon::select_form(
-               $language,
-               'language',
-               \%langchoices);
-    $r->print(<<ENDLSCREEN);
-<form name="prefs" action="/adm/preferences" method="post">
-<input type="hidden" name="action" value="verify_and_change_languages" />
-<br />$pref: $selectionbox
-ENDLSCREEN
-    $r->print('<br /><input type="submit" value="'.&mt('Save').'" />');
+    $r->print(
+        '<form name="prefs" action="/adm/preferences" method="post">'."\n".
+        '<input type="hidden" name="action" value="verify_and_change_languages" />'.
+        '<br /><span class="LC_nobreak">'.&mt('Preferred language').':&nbsp;'.
+        &Apache::loncommon::select_language('language',$language,1).'</span>'."\n".
+        '<br /><input type="submit" value="'.&mt('Save').'" /></form>'
+    );
 }
 
 
@@ -227,6 +165,7 @@ sub texenginechanger {
 		     'tth' => 'tth (TeX to HTML)',
 		     #'ttm' => 'TeX to MathML',
 		     'jsMath' => 'jsMath',
+                     'MathJax' => 'MathJax',
 		     'mimetex' => 'mimetex (Convert to Images)',
                      'raw' => 'Raw (Screen Reader)'
                      );
@@ -236,12 +175,15 @@ sub texenginechanger {
                $texengine,
                'texengine',
                \%mathchoices);
+    my $MathJax_start=&Apache::lontexconvert::MathJax_header();
     my $jsMath_start=&Apache::lontexconvert::jsMath_header();
     my %lt=&Apache::lonlocal::texthash(
       'headline' => 'Change how math is displayed',
       'preftxt'  => 'Preferred method to display math',
       'change'   => 'Save',
       'exmpl'    => 'Examples',
+      'mathjax'  => 'MathJax:',
+      'mathjaxinfo' =>  'MathJax provides rendered equations whose source code can be extracted in TeX and MathML formats by right clicking the equation.',
       'jsmath'   => 'jsMath:',
       'tth'      => 'tth (TeX to HTML):',
       'mimetex'  => 'mimetex (Convert to Images):',
@@ -276,6 +218,14 @@ $lt{'preftxt'}: $selectionbox
 <hr />
 $lt{'exmpl'}
 
+<h3>$lt{'mathjax'}</h3>
+</script>
+<iframe src="/res/adm/pages/math_example.tex?inhibitmenu=yes&texengine=MathJax" width="400" height="150"></iframe>
+</p>
+<p>
+$lt{'mathjaxinfo'}
+</p>
+
 <h3>$lt{'jsmath'}</h3> 
 <p>
 $jsMath_start
@@ -283,18 +233,19 @@ $jsMath_start
 if (jsMath.nofonts == 1) {
     document.writeln($jsMathWarning);
 }
+
 </script>
-<iframe src="/res/adm/pages/math_example.tex?inhibitmenu=yes&texengine=jsMath" width="400" height="120"></iframe>
+<iframe src="/res/adm/pages/math_example.tex?inhibitmenu=yes&texengine=jsMath" width="400" height="150"></iframe>
 </p>
 
 <h3>$lt{'mimetex'}</h3>
 <p>
-<iframe src="/res/adm/pages/math_example.tex?inhibitmenu=yes&texengine=mimetex" width="400" height="100"></iframe>
+<iframe src="/res/adm/pages/math_example.tex?inhibitmenu=yes&texengine=mimetex" width="400" height="150"></iframe>
 </p>
 
 <h3>$lt{'tth'}</h3>
 <p>
-<iframe src="/res/adm/pages/math_example.tex?inhibitmenu=yes&texengine=tth" width="400" height="220"></iframe>
+<iframe src="/res/adm/pages/math_example.tex?inhibitmenu=yes&texengine=tth" width="400" height="150"></iframe>
 </p>
 ENDLSCREEN
     if ($env{'environment.texengine'} ne 'jsMath') {
@@ -342,16 +293,25 @@ sub rolesprefchanger {
     my $domain     = $env{'user.domain'};
     my %userenv = &Apache::lonnet::get
         ('environment',['recentroles','recentrolesn']);
+    my $brtext = 'Change '.$role.' Page Pref';
+    my $brtitle;
+    if ($env{'form.returnurl'} eq '/adm/roles') {
+        $brtext = 'Configure Hotlist';
+    } else {
+        $brtitle = $brtext;
+    }
     Apache::lonhtmlcommon::add_breadcrumb(
 	    {	href => '/adm/preferences?action=changerolespref',
-                text => 'Change '.$role.' Page Pref'});
+                text => $brtext});
     $r->print(Apache::loncommon::start_page('Content Display Settings'));
-    $r->print(Apache::lonhtmlcommon::breadcrumbs('Change '.$role.' Page Pref'));
+    $r->print(Apache::lonhtmlcommon::breadcrumbs($brtitle));
     my $hotlist_flag=$userenv{'recentroles'};
     my $hotlist_n=$userenv{'recentrolesn'};
-    my $checked;
+    my ($checkedon,$checkedoff);
     if ($hotlist_flag) {
-	$checked = 'checked="checked"';
+        $checkedon = 'checked="checked"'; 
+    } else {
+        $checkedoff = 'checked="checked"';
     }
     
     if (!$hotlist_n) { $hotlist_n=3; }
@@ -378,7 +338,7 @@ sub rolesprefchanger {
 	    &Apache::loncommon::start_data_table().
 	    &Apache::loncommon::start_data_table_header_row().
 	    "<th>".&mt('Freeze '.$role)."</th>".
-	    "<th>".&mt($role)."</td>".
+	    "<th>".&mt($role)."</th>".
 	    &Apache::loncommon::end_data_table_header_row().
 	    "\n";
 	my $count;
@@ -401,28 +361,51 @@ sub rolesprefchanger {
         $roles_check_list .= "</table>\n";
     }
 
+    my $actionurl = '/adm/preferences';
+    if ($env{'form.returnurl'} eq '/adm/roles') {
+        $actionurl = '/adm/roles';
+    }
+    $r->print('<h3>'.&mt('Recent Roles Hotlist').'</h3>');
+    unless ($checkedon) {
+        $r->print(&mt('LON-CAPA users with several '.$lc_role.'s may wish to enable the Hotlist.').'<br />');
+    }
     $r->print('
-<p>'.&mt('Some LON-CAPA users have a long list of '.$lc_role.'s. The Recent '.$role.'s Hotlist feature keeps track of the last N '.$lc_role.'s which have been visited and places a table of these at the top of the '.$lc_role.'s page. People with very few '.$lc_role.'s should leave this feature disabled.').'
-</p>
-<form name="prefs" action="/adm/preferences" method="post">
+<form name="prefs" action="'.$actionurl.'" method="post">
 <input type="hidden" name="action" value="verify_and_change_rolespref" />
-<br />'.
-'<h2>'.&mt('Recent '.$role.'s Hotlist').'</h2>'.
+<input type="hidden" name="returnurl" value="'.$env{'form.returnurl'}.'" />
+<div class="LC_left_float"><h4>'.&mt('Hotlist options').'</h4>
+<p>'.
+&mt('When enabled, the Hotlist keeps track of the last N '.$lc_role.'s visited.').'<br />'.
+&mt('Those N '.$lc_role.'s are then shown in a table at the top of the '.$lc_role.'s page.').'</p>'.
 &Apache::lonhtmlcommon::start_pick_box().
-&Apache::lonhtmlcommon::row_title('<label for="Hotlist">'.&mt('Enable Recent '.$role.'s Hotlist').'</label>').
-'<input id="Hotlist" type="checkbox"'.$checked.' name="recentroles" value="true" />'.
+&Apache::lonhtmlcommon::row_title(&mt('Use Recent '.$role.'s Hotlist')).
+'<span class="LC_nobreak">
+<label><input id="Hotliston" type="radio" '.$checkedon.' name="recentroles" value="1" />'.&mt('Yes').'</label>'.
+('&nbsp;'x2).
+'<label><input id="Hotlistoff" type="radio" '.$checkedoff.' name="recentroles" value="0" />'.&mt('No').'</label>
+</span>'.
 &Apache::lonhtmlcommon::row_closure().
 &Apache::lonhtmlcommon::row_title('<label for="NumberOfRoles">'.&mt('Number of '.$role.'s in Hotlist').'</label>').
 '<select name="recentrolesn" size="1" id ="NumberOfRoles">'.
-$options.
-'</select>'.
+$options.'
+</select>'.
 &Apache::lonhtmlcommon::row_closure(1).
-&Apache::lonhtmlcommon::end_pick_box().
-'<br/><h2>'.&mt('Freeze Roles').'</h2>'.
-'<p>'.&mt('This list below can be used to <q>freeze</q> '.$lc_role.'s on your screen. Those marked as frozen will not be removed from the list, even if they have not been used recently.').'
+&Apache::lonhtmlcommon::end_pick_box().'
+</div>');
+    if ($roles_check_list) {
+        $r->print('<div class="LC_left_float">
+<h4>'.&mt('Freeze Roles').'</h4>
+<p>'.&mt('The table below can be used to [_1]freeze[_2] '.$lc_role.'s in the Hotlist.','<q>','</q>').'<br />'.
+&mt('Those '.$lc_role.'s marked frozen will not be removed from the list, even if not recently used.').'
 </p>
 '.$roles_check_list.'
-<br />
+</div>');
+     } else {
+         $r->print('<br clear="all" />'.
+                   &mt('Once the Hotlist contains recently visited '.$lc_role.'s you can return to this page to also set frozen roles.'));
+     }
+     $r->print('
+<br clear="all" />
 <input type="submit" value="'.&mt('Save').'" />
 </form>');
 }
@@ -513,7 +496,11 @@ sub verify_and_change_rolespref {
         }
     }
     $message=&Apache::loncommon::confirmwrapper($message);
-    &print_main_menu($r, $message);
+    if ($env{'form.returnurl'} eq '/adm/roles') {
+        return $message;
+    } else {
+        &print_main_menu($r, $message);
+    }
 }
 
 
@@ -620,7 +607,7 @@ sub iconchanger {
     my ($inlinetools,$toolsorder) = &icon_previews();
     if ((ref($inlinetools) eq 'HASH') && (ref($toolsorder) eq 'ARRAY')) {
         foreach my $tool (@{$toolsorder}) {
-            my ($command,$row,$col,$img,$top,$bot,$act,$desc) =
+            my ($command,$row,$col,$img,$top,$bot,$act,$desc) = 
                 split(/\&/,$inlinetools->{$tool});
             $iconic_preview .= '<li><a title="'.$desc.'" class="LC_menubuttons_link" href=""><img alt="'.$desc.'" src="/res/adm/pages/'.$img.'"  class="LC_icon" /><span class="LC_menubuttons_inline_text">'.$top.('&nbsp;' x 2).'</span></a></li>';
             $iconsonly_preview .= '<li><a title="'.$desc.'" class="LC_menubuttons_link" href=""><img alt="'.$desc.'" src="/res/adm/pages/'.$img.'"  class="LC_icon" />&nbsp;</a></li>';
@@ -633,7 +620,7 @@ sub iconchanger {
     $iconic_preview = '<ul class="LC_breadcrumb_tools_outerlist"><li>'.
                       '<ul>'.
                       $iconic_preview.
-                      '</ul></li></ul>';
+                      '</ul></li></ul>'; 
     $r->print(<<ENDSCREEN);
 $classic
 <form name="prefs" action="/adm/preferences" method="post">
@@ -663,17 +650,16 @@ sub verify_and_change_icons {
 }
 
 sub icon_options {
-    my %lt = &Apache::lonlocal::texthash(
-                 iconic    => 'Use icons and text',
-                 iconsonly => 'Use icons only',
-             );
-    return %lt;
+    return &Apache::lonlocal::texthash(
+                                        iconic    => 'Use icons and text',
+                                        iconsonly => 'Use icons only',
+                                      );
 }
 
 sub icon_previews {
      my %icon_text = (
                       annotate => 'Notes',
-                      bookmark => 'Bookmark',
+                      wishlist => 'Stored Links',
                       catalog  => 'Info',
                       evaluate => 'Evaluate',
                       feedback => 'Communicate',
@@ -681,13 +667,13 @@ sub icon_previews {
                      );
     my %inlinetools = (
         printout => "s&8&3&prt.png&$icon_text{'printout'}&printout[_1]&gopost('/adm/printout',currentURL)&Prepare a printable document",
-        bookmark => "s&9&1&sbkm.png&$icon_text{'bookmark'}&bookmark[_2]&set_bookmark()&Set a bookmark for this resource",
+        wishlist => "s&9&1&wishlist-link.png&$icon_text{'wishlist'}&wishlistlink[_2]&set_wishlistlink()&Save a link for this resource in your personal Stored Links repository",
         evaluate => "s&8&1&eval.png&$icon_text{'evaluate'}&this[_1]&gopost('/adm/evaluate',currentURL,1)&Provide my evaluation of this resource",
         feedback => "s&8&2&fdbk.png&$icon_text{'feedback'}&discuss[_1]&gopost('/adm/feedback',currentURL,1)&Provide feedback messages or contribute to the course discussion about this resource",
         annotate => "s&9&3&anot.png&$icon_text{'annotate'}&tations[_1]&annotate()&Make notes and annotations about this resource",
         catalog  => "s&6&3&catalog.png&$icon_text{'catalog'}&info[_1]&catalog_info()&Show Metadata",
     );
-    my @toolsorder = qw(annotate bookmark evaluate feedback printout catalog);
+    my @toolsorder = qw(annotate wishlist evaluate feedback printout catalog);
     return (\%inlinetools,\@toolsorder);
 }
 
@@ -760,8 +746,8 @@ sub domcoordchanger {
     if ($userenv{'domcoord.author'} eq 'blocked') {
        $constchecked=' checked="checked"';
     }
-    my $text=&mt('By default, the Domain Coordinator can enter your construction space.');
-    my $construction=&mt('Block access to construction space');
+    my $text=&mt('By default, the Domain Coordinator can enter your Authoring Space.');
+    my $construction=&mt('Block access to Authoring Space');
     my $change=&mt('Save');
     $r->print(<<ENDSCREEN);
 <form name="prefs" action="/adm/preferences" method="post">
@@ -787,7 +773,7 @@ sub verify_and_change_domcoord {
     } else {
         $status=&mt('off');
     }
-    my $message=&Apache::lonhtmlcommon::confirm_success(&mt('Set [_1] to [_2]','<i>'.&mt('Block access to construction space').'</i>','<tt>'.$status.'</tt>'));
+    my $message=&Apache::lonhtmlcommon::confirm_success(&mt('Set [_1] to [_2]','<i>'.&mt('Block access to Authoring Space').'</i>','<tt>'.$status.'</tt>'));
     $message=&Apache::loncommon::confirmwrapper($message);
     &print_main_menu($r,$message);
 }
@@ -845,15 +831,15 @@ sub msgforwardchanger {
                                           foad  => 'Forward to account(s)',
                                           fwdm  => 'Forward messages to other account(s) in LON-CAPA',
                                           noti  => 'E-mail notification of LON-CAPA messages',
-                                          foad_exmpl => 'e.g. <tt>userA:domain1,userB:domain2,...</tt>',
                                           mnot  => 'E-mail address(es) which should be notified about new LON-CAPA messages',
-                                          mnot_exmpl => 'e.g. <tt>joe@doe.com</tt>',
                                           chg   => 'Save',
                                           email => 'The e-mail address entered in row ',
                                           notv => 'is not a valid e-mail address',
                                           toen => "To enter multiple addresses, enter one address at a time, click 'Change' and then add the next one", 
                                           prme => 'Back',
                                         );
+    $lt{'foad_exmpl'} = &mt('e.g. [_1]userA:domain1,userB:domain2,...[_2]','<tt>','</tt>');
+    $lt{'mnot_exmpl'} = &mt('e.g. [_1]joe@doe.com[_2]','<tt>','</tt>');
     Apache::lonhtmlcommon::add_breadcrumb(
 	    {	href => '/adm/preferences?action=changemsgforward',
 		text => 'Messages & Notifications'});
@@ -957,7 +943,7 @@ ENDMSG
                    '" onclick="javscript:delete_address('."'$num'".')" />'.
                    &mt('Delete').'</label></span></td>'.
                    '<td><input type="text" value="'.$item.'" name="address_'.
-                   $num.'" onFocus="javascript:address_changes('."'$num'".
+                   $num.'" onfocus="javascript:address_changes('."'$num'".
                    ')" /></td><td>';
         my %chk;
         if (defined($allnot{$item}{'crit'})) {
@@ -1006,7 +992,7 @@ ENDMSG
                '<input type="checkbox" name="add_notify_'.$num.
                '" value="1" />'.&mt('Add new address').'</label></span></td>'.
                '<td><input type="text" value="" name="address_'.$num.
-               '" onFocus="javascript:new_address('."'$num'".')" /></td><td>';
+               '" onfocus="javascript:new_address('."'$num'".')" /></td><td>';
     foreach my $type ('all','crit','reg') {
         $output .= '<span class="LC_nobreak"><label>'.
                    '<input type="radio" name="notify_type_'.$num.
@@ -1180,47 +1166,25 @@ sub colorschanger {
     foreach my $item (sort(keys(%colortypes))) {
        my $curcol=&Apache::loncommon::designparm($function.'.'.$item,$domain);
        $chtable.=&Apache::loncommon::start_data_table_row().
-	   '<td>'.$colortypes{$item}.'</td><td style="background: '.$curcol.
-        '">&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td><td><input name="'.$item.
-        '" size="10" value="'.$curcol.
-'" /></td><td><a href="javascript:pjump('."'color_custom','".$colortypes{$item}.
-"','".$curcol."','"
-	    .$item."','parmform.pres','psub'".');">'.&mt('Select').'</a></td>'.
-	    &Apache::loncommon::end_data_table_row()."\n";
+                 '<td>'.$colortypes{$item}.'<td><input name="'.$item.
+                 '" class="colorchooser"  size="10" value="'.$curcol.
+                 '" /></td>'.
+	         &Apache::loncommon::end_data_table_row()."\n";
     }
     my $end_data_table = &Apache::loncommon::end_data_table();
     my $pjump_def = &Apache::lonhtmlcommon::pjump_javascript_definition();
     my $savebutton = &mt('Save');
     my $resetbutton = &mt('Reset All');
     my $resetbuttondesc = &mt('Reset All Colors to Default');
-    $r->print(<<ENDCOL);
-<script type="text/javascript" language="JavaScript">
-
-    function pclose() {
-        parmwin=window.open("/adm/rat/empty.html","LONCAPAparms",
-                 "height=350,width=350,scrollbars=no,menubar=no");
-        parmwin.close();
-    }
-
-    $pjump_def
-
-    function psub() {
-        pclose();
-        if (document.parmform.pres_marker.value!='') {
-            if (document.parmform.pres_type.value!='') {
-                eval('document.prefs.'+
-                     document.parmform.pres_marker.value+
-		     '.value=document.parmform.pres_value.value;');
-	    }
-        } else {
-            document.parmform.pres_value.value='';
-            document.parmform.pres_marker.value='';
-        }
-    }
-
-
+    my $colorchooser=&Apache::lonhtmlcommon::color_picker();
+    $r->print('<script type="text/javascript" language="JavaScript">
+' . $colorchooser . '
 </script>
-<form name="parmform">
+');
+
+    $r->print(<<ENDCOL);
+
+<form name="parmform" action="">
 <input type="hidden" name="pres_marker" />
 <input type="hidden" name="pres_type" />
 <input type="hidden" name="pres_value" />
@@ -1258,6 +1222,9 @@ sub verify_and_change_colors {
     my $message='';
     foreach my $item (keys %colortypes) {
         my $color=$env{'form.'.$item};
+        if (!($color =~ /^#/)) {
+            $color = '#' . $color;
+        }
         my $entry='color.'.$function.'.'.$item;
 	if (($color=~/^\#[0-9A-Fa-f]{6}$/) && (!$env{'form.resetall'})) {
 	    &Apache::lonnet::put('environment',{$entry => $color});
@@ -1305,8 +1272,11 @@ sub passwordchanger {
     } elsif ($caller eq 'reset_by_email') {
             my %data = &Apache::lonnet::tmpget($mailtoken);
             if (keys(%data) == 0) {
-                $r->print(&mt('Sorry, the URL you provided to complete the reset of your password was invalid. Either the token included in the URL has been deleted or the URL you provided was invalid. Please submit a [_1]new request[_2] for a password reset, and follow the link to the new URL included in the e-mail that will be sent to you, to allow you to enter a new password.'
+                $r->print(
+                    '<p class="LC_warning">'
+                   .&mt('Sorry, the URL you provided to complete the reset of your password was invalid. Either the token included in the URL has been deleted or the URL you provided was invalid. Please submit a [_1]new request[_2] for a password reset, and follow the link to the new URL included in the e-mail that will be sent to you, to allow you to enter a new password.'
                              ,'<a href="/adm/resetpw">','</a>')
+                   .'</p>'
                 );
                 return;
             }
@@ -1316,18 +1286,32 @@ sub passwordchanger {
                     $domain = $data{'domain'};
                     $currentpass = $data{'temppasswd'};
                 } else {
-                    $r->print(&mt('Sorry, the token generated when you requested a password reset has expired.').'<br />');
+                    $r->print(
+                        '<p class="LC_warning">'
+                       .&mt('Sorry, the token generated when you requested'
+                           .' a password reset has expired.')
+                       .'</p>'
+                    );
                     return;
                 }
             } else {
-                $r->print(&mt('Sorry, the URL generated when you requested reset of your password contained incomplete information.').'<br />');
+                $r->print(
+                   '<p class="LC_warning">'
+                  .&mt('Sorry, the URL generated when you requested reset of'
+                      .' your password contained incomplete information.')
+                   .'</p>'
+                );
                 return;
             }
             if (&Apache::lonnet::domain($domain) eq '') {
                 $domain = $r->dir_config('lonDefDomain');
             }
-   } else {
-        $r->print(&mt('Page requested in unexpected context').'<br />');
+    } else {
+        $r->print(
+            '<p class="LC_error">'
+           .&mt('Page requested in unexpected context')
+           .'</p>'
+        );
         return;
     }
     my $currentauth=&Apache::lonnet::queryauthenticate($user,$domain);
@@ -1335,9 +1319,9 @@ sub passwordchanger {
     return if ($currentauth !~ /^(unix|internal):/);
     #
     # Generate keys
-    my ($lkey_cpass ,$ukey_cpass ) = &des_keys();
-    my ($lkey_npass1,$ukey_npass1) = &des_keys();
-    my ($lkey_npass2,$ukey_npass2) = &des_keys();
+    my ($lkey_cpass ,$ukey_cpass ) = &Apache::loncommon::des_keys();
+    my ($lkey_npass1,$ukey_npass1) = &Apache::loncommon::des_keys();
+    my ($lkey_npass2,$ukey_npass2) = &Apache::loncommon::des_keys();
     # Store the keys in the log files
     my $lonhost = $r->dir_config('lonHostID');
     my $logtoken=Apache::lonnet::reply('tmpput:'
@@ -1430,7 +1414,7 @@ sub client_form {
                 'changepass' => 'Save',
     );
 
-    my $output = '<form name="client">'
+    my $output = '<form name="client" action="">'
                 .&Apache::lonhtmlcommon::start_pick_box();
     if ($caller eq 'reset_by_email') {
         $output .= &Apache::lonhtmlcommon::row_title(
@@ -1461,7 +1445,7 @@ sub client_form {
               .'<input type="password" name="newpass_2" size="10" />'
               .&Apache::lonhtmlcommon::row_closure(1)
               .&Apache::lonhtmlcommon::end_pick_box();
-    $output .= '<p><input type="button" value="'.$lt{'changepass'}.'" onClick="send();" /></p>'
+    $output .= '<p><input type="button" value="'.$lt{'changepass'}.'" onclick="send();" /></p>'
               .qq|
 <input type="hidden" name="ukey_cpass"  value="$hexkey->{'ukey_cpass'}" />
 <input type="hidden" name="lkey_cpass"  value="$hexkey->{'lkey_cpass'}" />
@@ -1576,9 +1560,9 @@ ENDERROR
     }
     my ($ckey,$n1key,$n2key)=split(/&/,$tmpinfo);
     # 
-    $currentpass = &des_decrypt($ckey ,$currentpass);
-    $newpass1    = &des_decrypt($n1key,$newpass1);
-    $newpass2    = &des_decrypt($n2key,$newpass2);
+    $currentpass = &Apache::loncommon::des_decrypt($ckey ,$currentpass);
+    $newpass1    = &Apache::loncommon::des_decrypt($n1key,$newpass1);
+    $newpass2    = &Apache::loncommon::des_decrypt($n2key,$newpass2);
     #
     if ($caller eq 'reset_by_email') {
         my %data = &Apache::lonnet::tmpget($mailtoken);
@@ -1599,14 +1583,14 @@ ENDERROR
     } 
     if ($newpass1 ne $newpass2) {
 	&passwordchanger($r,
-			 '<span class="LC_error">'.
+			 '<span class="LC_warning">'.
 			 &mt('The new passwords you entered do not match.').'  '.
 			 &mt('Please try again.').'</span>',$caller,$mailtoken);
 	return 1;
     }
     if (length($newpass1) < 7) {
 	&passwordchanger($r,
-			 '<span class="LC_error">'.
+			 '<span class="LC_warning">'.
 			 &mt('Passwords must be a minimum of 7 characters long.').'  '.
 			 &mt('Please try again.').'</span>',$caller,$mailtoken);
 	return 1;
@@ -1619,7 +1603,7 @@ ENDERROR
     }
     if ($badpassword) {
 	# I can't figure out how to enter bad characters on my browser.
-	my $errormessage ='<span class="LC_error">'.
+	my $errormessage ='<span class="LC_warning">'.
            &mt('The password you entered contained illegal characters.').'<br />'.
            &mt('Valid characters are').(<<"ENDERROR");
 : space and <br />
@@ -1791,7 +1775,7 @@ sub verify_and_change_discussion {
     if (defined($env{'form.discmark'}) ) {
         my $newmark = $env{'form.newmark'};
         if ($newmark eq 'ondisp') {
-            $message.=&Apache::lonhtmlcommon::confirm_success(&mt('In discussions: new posts will be cease to be identified as "NEW" after display.')).'<br />';
+            $message.=&Apache::lonhtmlcommon::confirm_success(&mt('In discussions: new posts will cease to be identified as "NEW" after display.')).'<br />';
             &Apache::lonnet::put('environment',{'discmarkread' => $newmark});
             &Apache::lonnet::appenv({'environment.discmarkread' => $newmark});
         } else {
@@ -2117,7 +2101,6 @@ push(@{ $menu[1]->{items} }, {
         });
     }
 
-
     if (&can_toggle_namelocking()) {
         push(@{ $menu[0]->{items} }, {
         linktext => 'Automatic name changes',
@@ -2191,11 +2174,19 @@ sub handler {
                                    ['action','wysiwyg','returnurl','refpage']);
     #
     Apache::lonhtmlcommon::clear_breadcrumbs();
+    my ($brlink,$brtxt,$brhelp);
+    if (($env{'form.action'} eq 'changerolespref') && ($env{'form.returnurl'} eq '/adm/roles')) {
+        $brlink ='/adm/roles';
+        $brtxt = 'User Roles';
+    } else {
+        $brlink ='/adm/preferences';
+        $brtxt = 'Set User Preferences';
+        $brhelp = 'Prefs_About_Me,Prefs_Language,Prefs_Screen_Name_Nickname,Change_Colors,Change_Password,Prefs_Messages,Change_Discussion_Display';
+    }
     Apache::lonhtmlcommon::add_breadcrumb
-        ({href => '/adm/preferences',
-          text => 'Set User Preferences',
-          help =>
-          'Prefs_About_Me,Prefs_Language,Prefs_Screen_Name_Nickname,Change_Colors,Change_Password,Prefs_Messages,Change_Discussion_Display'});
+        ({href => $brlink,
+          text => $brtxt,
+          help => $brhelp,});
     if(!exists $env{'form.action'}) {
 	    &print_main_menu($r);
     }elsif($env{'form.action'} eq 'changepass'){
