@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # Source Code handler
 #
-# $Id: lonsource.pm,v 1.24 2009/08/13 11:00:38 raeburn Exp $
+# $Id: lonsource.pm,v 1.32 2014/02/11 15:49:15 bisitz Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -40,7 +40,7 @@ use Apache::lonmeta;
 use Apache::File;
 use Apache::lonlocal;
 use HTML::Entities;
-use LONCAPA;
+use LONCAPA qw(:DEFAULT :match);
 
 sub make_link {
     my ($filename, $listname) = @_;
@@ -50,12 +50,12 @@ sub make_link {
 }
 
 sub stage_2 {
-    my ($r, $filename, $author, $listname) = @_;
-    my ($uname, $udom) = &Apache::loncacc::constructaccess('/~'.$author.'/',$r->dir_config('lonDefDomain'));
-    $r->print(&Apache::loncommon::start_page('Copy Problem Source Code to Construction Space')
+    my ($r, $filename, $listname) = @_;
+    my ($author)=($filename=~/\/res\/[^\/]+\/([^\/]+)\//);
+    $r->print(&Apache::loncommon::start_page('Copy Problem Source Code to Authoring Space')
              .&mt('Please enter the directory that you would like the source code to go into.')
              .'<p>'
-             .&mt('Note: the path is in reference to the root of your construction space,'
+             .&mt('Note: the path is in reference to the root of your Authoring Space,'
                  .' and new directories will be automatically created.')
              .'</p>');
     $r->print('<form name="copy" action="/adm/source" target="_parent" method="post">
@@ -68,11 +68,7 @@ sub stage_2 {
     return OK;
 }
 
-
-sub copy_stage {
-    my ($r, $filename, $listname, $newpath) = @_;
-
-#Figure out if we are author or co-author
+sub copy_author {
     my $role;
     my $domain;
     my $author_name;
@@ -83,11 +79,26 @@ sub copy_stage {
         $domain = $env{'user.domain'};
         $author_name = $env{'user.name'};
     }
+    return ($role,$author_name,$domain);
+}
+
+
+sub copy_stage {
+    my ($r, $filename, $listname, $newpath) = @_;
+
+#Figure out if we are author or co-author
+    my ($role,$author_name,$domain)=&copy_author();
+
+# Construct path to copy and filter out any possibly nasty stuff
+    my $path_to_new_file = $r->dir_config('lonDocRoot').
+                           "/priv/$domain/$author_name/$newpath/$listname";
+    $path_to_new_file=~s/\.\.//g;
+    $path_to_new_file=~s/\~//g;
+    $path_to_new_file=~s/\/+/\//g;
 
-    my $path_to_new_file = '/home/'.$author_name.'/public_html/'.$newpath.'/'.$listname;
 #Just checking again for access as we want to make sure that it is really ok now that we have the real path
 
-    my ($uname,$udom)= &Apache::loncacc::constructaccess($path_to_new_file,$domain);
+    my ($uname,$udom)= &Apache::lonnet::constructaccess($path_to_new_file);
 
     if (!$uname || !$udom) {
 	$r->print(&Apache::loncommon::start_page('Not Allowed'));
@@ -96,29 +107,27 @@ sub copy_stage {
 	return;
     }
 
-
     #allowed
     $r->print(&Apache::loncommon::start_page('Copying Source'));
     my $result = &Apache::loncfile::exists($uname, $udom, $path_to_new_file);
     $r->print($result);
     if(($result) && ($result =~ m|published|) ) {
-	&delete_copy_file($r, $author_name, $newpath, $filename, $path_to_new_file, '1');
+	&delete_copy_file($r, $newpath, $filename, $path_to_new_file, '1');
     } elsif(($result) && ($result =~ m|exists!|)) {
-	&confirm($r, $author_name, $newpath, $filename, $path_to_new_file);
+	&confirm($r, $newpath, $filename, $path_to_new_file);
     } else {
-	&copy_file($r, $author_name, $newpath, $filename, $path_to_new_file);
+	&copy_file($r, $newpath, $filename, $path_to_new_file);
     }
     
     $r->print(&Apache::loncommon::end_page());
 }
 
 sub confirm {
-    my ($r, $author_name, $newpath, $filename, $path_to_new_file) = @_;
-    $r->print("<b>Press delete to remove file and replace it with a copy of the source you are viewing</b><br /><br /	>");
+    my ($r, $newpath, $filename, $path_to_new_file) = @_;
+    $r->print('<b>'.&mt('Press delete to remove file and replace it with a copy of the source you are viewing.').'</b><br /><br />');
     $r->print('<form name="delete_confirm" action="/adm/source" target="_parent" method="post">
               <input type="hidden" name="filename" value="'.$filename.'" />
               <input type="hidden" name="path" value="'.$path_to_new_file.'" />
-              <input type="hidden" name="author" value="'.$author_name.'" />
               <input type="hidden" name="newpath" value="'.$newpath.'" />
               <input type="hidden" name="action" value="delete_confirm" />
 
@@ -127,14 +136,14 @@ sub confirm {
 }
 
 sub delete_copy_file {
-    my ($r, $author_name, $newpath, $filename, $path_to_new_file, $type) = @_;
+    my ($r, $newpath, $filename, $path_to_new_file, $type) = @_;
     if($type eq '1') {
         $r->print('<p><span class="LC_warning">'
                  .&mt('Cannot delete non-obsolete published file.')
                  .'</span><br />'
                  .&mt('Please use the code view in previous window to use shared code.')
                  .'<br /><br />');
-        $r->print('<input type="button" value="Close Window" name="close"'
+        $r->print('<input type="button" value="'.&mt('Close Window').'" name="close"'
                  .' onclick="window.close()" />'
                  .'</p>');
     } else {
@@ -147,15 +156,20 @@ sub delete_copy_file {
             $r->print('<p class="LC_error">'.&mt('No such file').'</p></form>');
             return 0;
         }
-        &copy_file($r, $author_name, $newpath, $filename, $path_to_new_file);
+        &copy_file($r, $newpath, $filename, $path_to_new_file);
     }
 }
 
 sub copy_file {
-    my ($r, $author_name, $newpath, $filename, $path_to_new_file) = @_;
-    $r->print("<b>Creating directories</b>");
-    my $path = '/home/'.$author_name.'/public_html/';
+    my ($r, $newpath, $filename, $path_to_new_file) = @_;
+    $r->print('<b>'.&mt('Creating directories').'</b>');
+
+#Figure out if we are author or co-author
+    my ($role,$author_name,$domain)=&copy_author();
+
+    my $path = $r->dir_config('lonDocRoot')."/priv/$domain/$author_name/";
     my @directories = split(/\//,$newpath);
+
     foreach my $now_checking (@directories) {
         if($now_checking ne '') {
             $path = $path.'/'.$now_checking;
@@ -174,7 +188,7 @@ sub copy_file {
         } else { } #Just move along
 
     }
-    $r->print("<br /><b>Copying File</b>");
+    $r->print('<br /><b>'.&mt('Copying File').'</b>');
     my $problem_filename = $Apache::lonnet::perlvar{'lonDocRoot'}.$filename;
     my $file_output = &includemeta(&Apache::lonnet::getfile($problem_filename),$filename);
     my $fs=Apache::File->new(">$path_to_new_file");
@@ -182,28 +196,86 @@ sub copy_file {
         print $fs $file_output;
     }
     $r->print("<br /><br />");
-    $r->print('<input type="button" value="Close Window" name="close" onclick="window.close()" />');
+    $r->print('<input type="button" value="'.&mt('Close Window').'" name="close" onclick="window.close()" />');
     #Some 1.3'ish feature is to include the derivative feature, will go here..'
 }
 
 sub print_item {
-    my ($r, $filename) = @_;
-    my $file_output = &includemeta(&Apache::lonnet::getfile($Apache::lonnet::perlvar{'lonDocRoot'}.$filename),$filename);
-    my $count=0;
-    my $maxlength=-1;
-    foreach (split ("\n", $file_output)) {
-        $count+=int(length($_)/79);
-        $count++;
-        if (length($_) > $maxlength) {
-            $maxlength = length($_);
-        }
-    }
-    my $rows = $count;
-    my $cols = $maxlength;
-    $r->print('<textarea rows="'.$rows.'" cols="'.$cols.'" name="editxmltext">'.
-              &HTML::Entities::encode($file_output,'<>&"').'</textarea>');
-    return OK;
+    my ($r,$filename,$listname) = @_;
+    my $file_output = 
+        &includemeta(&Apache::lonnet::getfile($Apache::lonnet::perlvar{'lonDocRoot'}.$filename),
+                                              $filename);
+    $r->print(&Apache::loncommon::start_page('View Source Code',undef,
+                                             {'only_body' => 1}));
+    if ($file_output ne '') {
+        my $access_to_cstr;
+        my $lonhost = $r->dir_config('lonHostID');
+        if (&Apache::lonnet::is_library($lonhost)) {
+            my @possdoms = &Apache::lonnet::current_machine_domains();
+            foreach my $dom (@possdoms) {
+                if ($env{"user.role.au./$dom/"}) {
+                    $access_to_cstr = 1;
+                    last;  
+                }
+            }
+            unless ($access_to_cstr) {
+                foreach my $key (keys(%env)) {
+                    if ($key =~ m{^\Quser.role.ca./\E($match_domain)/}) {
+                        my $adom = $1;
+                        if (grep(/^\Q$adom\E$/,@possdoms)) {
+                            $access_to_cstr = 1;
+                            last;
+                        }
+                    }
+                }
+            }
+            if ($access_to_cstr) {
+                $r->print('
+             <form name="copy" action="/adm/source" target="_parent" method="post">
+              <input type="button" value="'.&mt('Close Window').'" name="close" onclick="window.close();" />
+              <input type="hidden" name="filename" value="'.$filename.'" />
+              <input type="hidden" name="listname" value="'.$listname.'" />
+              <input type="hidden" name="action" value="stage2" />
+              <input type="submit" value="'.&mt('Copy to Authoring Space').'" />
+             </form><hr />
+                ');
+            } else {
+                $r->print('<p><span class="LC_info">'.
+                          &mt('Source code is displayed, but you can not copy to Authoring Space, as you do not have an author or co-author role on this server.').
+                          '</span></p><a href="javascript:window.close();">'.&mt('Close Window').
+                          '</a><br /><hr />'
+                         );
+            }
+        } else {
+            $r->print('<p><span class="LC_info">'.
+                      &mt('Source code is displayed, but you can not copy to Authoring Space on this server.').
+                          '</span></p><a href="javascript:window.close();">'.&mt('Close Window').
+                          '</a><br /><hr />'
+                     );
 
+        }
+        my $count=0;
+        my $maxlength=-1;
+        foreach (split ("\n", $file_output)) {
+            $count+=int(length($_)/79);
+            $count++;
+            if (length($_) > $maxlength) {
+                $maxlength = length($_);
+            }
+        }
+        my $rows = $count;
+        my $cols = $maxlength;
+        $r->print('<form name="showsrc" action="" method="post" onsubmit="return false">'."\n".
+                  '<textarea rows="'.$rows.'" cols="'.$cols.'" name="editxmltext">'.
+                  &HTML::Entities::encode($file_output,'<>&"').'</textarea></form>');
+    } else {
+        $r->print('<p class="LC_warning">'.
+                  &mt('Unable to retrieve file contents.').
+                  '</p><a href="javascript:window.close();">'.&mt('Close Window').'</a>'
+                 );
+    }
+    $r->print(&Apache::loncommon::end_page());
+    return;
 }
 
 sub includemeta {
@@ -220,7 +292,11 @@ sub includemeta {
     } else {
 	$file_output=~s/(\<(?:html|problem)[^\>]*\>)/$1\n\<meta name=\"isbasedontime\" content=\"\Q$copytime\E\" \/\>/i;
     }
-    return $file_output;
+    if ($file_output eq '-1') {
+        return;
+    } else {
+        return $file_output;
+    }
 }
 
 sub handler {
@@ -233,39 +309,27 @@ sub handler {
     if ($source ne 'open') {
         $env{'user.error.msg'}="$filename:cre:1:1:Source code not available";
         return HTTP_NOT_ACCEPTABLE;
-    } 
-    unless ((&Apache::lonnet::allowed('bre',$filename)) &&
-	    (&Apache::lonnet::allowed('cre','/'))) {
+    }
+    unless (&Apache::lonnet::allowed('bre',$filename)) {
         $env{'user.error.msg'}="$filename:bre:1:1:Access to resource denied";
         return HTTP_NOT_ACCEPTABLE;
-    } 
+    }
+    unless (&Apache::lonnet::allowed('cre','/')) {
+        $env{'user.error.msg'}="$filename:cre:1:1:Access to source code denied";
+        return HTTP_NOT_ACCEPTABLE;
+    }
 
     &Apache::loncommon::content_type($r,'text/html');
     $r->send_http_header;
 
     if ($env{'form.action'} eq 'stage2') {
-        my $author = &Apache::lonnet::metadata($filename,'authorspace');
-	#strip the domain of the author name
-	if ($author =~ /:/) {
-	    ($author) = split(/:/,$author);
-	} else {
-	    ($author) = split(/@/,$author);
-	}
-        &stage_2($r, $env{'form.filename'}, $author, $listname);
+        &stage_2($r,$filename,$listname);
     } elsif($env{'form.action'} eq 'copy_stage') {
-        &copy_stage($r, $filename,$env{'form.listname'},$env{'form.newpath'});
+        &copy_stage($r,$filename,$listname,$env{'form.newpath'});
     } elsif($env{'form.action'} eq 'delete_confirm') {
-        &delete_copy_file($r, $env{'form.author'}, $env{'form.newpath'}, $env{'form.filename'}, $env{'form.path'}, '0');
+        &delete_copy_file($r,$env{'form.newpath'},$filename, $env{'form.path'}, '0');
     } else {
-	$r->print('<form name="copy" action="/adm/source" target="_parent" method="post">
-                  <input type="button" value="'.&mt('Close Window').'" name="close" onclick="window.close()" />
-                  <input type="hidden" name="filename" value="'.$filename.'" />
-                  <input type="hidden" name="listname" value="'.$listname.'" />
-                  <input type="hidden" name="action" value="stage2" />
-                  <input type="submit" value="'.&mt('Copy to Construction Space').'" />
-                  </form>');
-        $r->print('<hr />');
-        &print_item($r, $env{'form.filename'});
+        &print_item($r,$filename,$listname);
     }
     return OK;
 }
