@@ -1,5 +1,5 @@
 #
-# $Id: lonsupportreq.pm,v 1.66 2011/03/03 17:29:29 bisitz Exp $
+# $Id: lonsupportreq.pm,v 1.78 2014/01/20 17:25:41 bisitz Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -36,9 +36,10 @@ use Apache::lonhtmlcommon;
 use Apache::lonnet;
 use Apache::lonlocal;
 use Apache::lonacc();
+use Apache::lonauth();
 use Apache::courseclassifier;
 use LONCAPA qw(:DEFAULT :match);
- 
+use HTML::Entities;
 
 sub handler {
     my ($r) = @_;
@@ -56,8 +57,19 @@ sub handler {
     if ($r->uri eq '/adm/helpdesk') {
         &Apache::lonacc::get_posted_cgi($r);
     }
-    my $function = $env{'form.function'};
+    my $function;
+    if ($env{'form.function'}) {
+        if (($env{'form.function'} eq 'norole')  ||
+            ($env{'form.function'} eq 'student') ||
+            ($env{'form.function'} eq 'admin')   ||
+            ($env{'form.function'} eq 'author')) {
+            $function = $env{'form.function'};
+        }
+    }
     my $origurl = $env{'form.origurl'};
+    $origurl =~ s{^https?://}{};
+    $origurl =~ s/(`)//g;
+    $origurl =~ s/\$/\(\$\)/g;
     my $command = $env{'form.command'};
 
     if ($command eq 'process') {
@@ -72,7 +84,7 @@ sub print_request_form {
     my ($r,$origurl,$function) = @_;
     my ($os,$browser,$bversion,$uhost,$uname,$udom,$uhome,$urole,$usec,$email,$cid,
         $cdom,$cnum,$ctitle,$ccode,$sectionlist,$lastname,$firstname,$server,
-        $formname,$public);
+        $formname,$public,$homeserver);
     $function = &Apache::loncommon::get_users_function() if (!$function);
     $ccode = '';
     $os = $env{'browser.os'};
@@ -82,22 +94,28 @@ sub print_request_form {
     if (($env{'user.name'} eq 'public') && ($env{'user.domain'} eq 'public')) {
         $public = 1;
     } else {
-        $uname = $env{'user.name'};
-        $udom = $env{'user.domain'};
+        if (($env{'user.name'} ne '') && ($env{'user.domain'} ne '')) {
+            $homeserver = &Apache::lonnet::homeserver($env{'user.name'},
+                                                      $env{'user.domain'});
+            if ($homeserver eq 'no_host') {
+                undef($homeserver); 
+            } else {
+                $uname = $env{'user.name'};
+                $udom = $env{'user.domain'};
+            }
+        }
+    }
+    if ($homeserver) {
+        $uhome = $env{'user.home'};
+        $urole = $env{'request.role'};
+        $usec = $env{'request.course.sec'};
+        $cid = $env{'request.course.id'};
     }
-    $uhome = $env{'user.home'};
-    $urole = $env{'request.role'};
-    $usec = $env{'request.course.sec'};
-    $cid = $env{'request.course.id'};
     $formname = 'logproblem';
     my $machine = &Apache::lonnet::absolute_url();
-    if ($origurl =~ m-^https?://-) {
-        $server = $origurl;
-    } else {
-        $server = $machine.$origurl;
-    }
-    my $showserver = $server;
-    $showserver =~ s/\?.*$//;
+    my $sourceurl = $machine.$origurl;
+    $server = $machine.&Apache::loncommon::cleanup_html($origurl);
+    $server =~ s/\?.*$//;
     my %lt = &Apache::lonlocal::texthash (
                   email => 'The e-mail address you entered',
                   notv => 'is not a valid e-mail address',
@@ -159,18 +177,20 @@ END
         $sectionlist = $csettings{'internal.sectionnums'};
     }
 
-    if ($env{'environment.permanentemail'}) {
-        $email = $env{'environment.permanentemail'};
-    } elsif ($env{'environment.critnotification'}) {
-        $email = $env{'environment.critnotification'};
-    } elsif ($env{'environment.notification'}) {
-        $email = $env{'environment.notification'};
-    }
-    if ($env{'environment.lastname'}) {
-        $lastname = $env{'environment.lastname'};
-    }
-    if ($env{'environment.firstname'}) {
-        $firstname = $env{'environment.firstname'};
+    if ($homeserver) {
+        if ($env{'environment.permanentemail'}) {
+            $email = $env{'environment.permanentemail'};
+        } elsif ($env{'environment.critnotification'}) {
+            $email = $env{'environment.critnotification'};
+        } elsif ($env{'environment.notification'}) {
+            $email = $env{'environment.notification'};
+        }
+        if ($env{'environment.lastname'}) {
+            $lastname = $env{'environment.lastname'};
+        }
+        if ($env{'environment.firstname'}) {
+            $firstname = $env{'environment.firstname'};
+        }
     }
     my @sections = split(/,/,$sectionlist);
     my %groupid;
@@ -243,11 +263,19 @@ function initialize_codes() {
         }
     }
 	
-    my $js = '<script type="text/javascript">'."\n$scripttag\n$jscript\n".
-	     $loaditems.'</script>';
-    my %add_entries = (topmargin    => "0",
-		       marginheight => "0",
-		       onLoad       =>"initialize_codes()",);
+    my $js = <<"ENDJS";
+<script type="text/javascript">
+// <![CDATA[
+$scripttag
+$jscript
+$loaditems
+// ]]>
+</script>
+ENDJS
+    my %add_entries = (
+                       style    => "margin-top:0px;margin-bottom:0px;",
+                       onload   => "initialize_codes();",
+                      );
 
     
     $r->print(&Apache::loncommon::start_page('Support Request',$js,
@@ -260,43 +288,57 @@ function initialize_codes() {
     my @css = ('LC_evenrow_value','LC_oddrow_value');
     my $num = 1;
     my $i = $num%2;
-    $r->print('<form method="post" name="logproblem" enctype="multipart/form-data">'."\n");
+    my $formtype;
+    if ($homeserver) {
+        $formtype = ' enctype="multipart/form-data"';
+    }
+    $r->print('<form method="post" action="" name="logproblem"'.$formtype.'>'."\n");
     my $output = &Apache::lonhtmlcommon::start_pick_box().
+                 &Apache::lonhtmlcommon::row_headline().
+                 '<span class="LC_info">'.
+                 &mt('(All fields marked with * are required.)').
+                 '</span>'.
+                 &Apache::lonhtmlcommon::row_closure().
                  &Apache::lonhtmlcommon::row_title($lt{'name'},undef,$css[$num])."\n";
     my $fullname = '';
     if ((defined($lastname) && $lastname ne '') && (defined($firstname) && $firstname ne '')) {
         $fullname = "$firstname $lastname"; 
-        $output .= $fullname.'<input type="hidden" name="username" value="'.&HTML::Entities::encode($fullname,'"<>&').'" />';
+        $output .= $fullname.'<input type="hidden" name="username" value="'.&HTML::Entities::encode($fullname,'"<>&').'" />'."\n";
     } else {
         if (defined($firstname) && $firstname ne '') {
             $fullname = $firstname;
         } elsif (defined($lastname) && $lastname ne '') {
             $fullname = " $lastname";
         }
-        $output .= '<input type="text" size="20" name="username" value="'.&HTML::Entities::encode($fullname,'"<>&').'" />';
+        $output .= '<input type="text" size="20" name="username" value="'.&HTML::Entities::encode($fullname,'"<>&').'" />'."\n";
     }
     $output .= '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<input type="button" value="'.$lt{'subm'}.'" onclick="validate()" />&nbsp;'.
                 &Apache::lonhtmlcommon::row_closure()."\n";
     $num ++;
     $i = $num%2;
-    $output .= &Apache::lonhtmlcommon::row_title($lt{'emad'},undef,$css[$i]).
+    $output .= &Apache::lonhtmlcommon::row_title(
+                   '<span title="'.&mt('required').'">'.
+                   $lt{'emad'}.' <span class="LC_info">*</span></span>'
+                  ,undef,$css[$i]).
                '<input type="text" size="20" name="email" value="'.
                &HTML::Entities::encode($email,'"<>&').'" /><br />'."\n".
                &Apache::lonhtmlcommon::row_closure();
     $num ++;
     $i = $num%2;
     if (($env{'user.name'} =~ /^$match_username$/) && (!$public)) {
-        $output .= &Apache::lonhtmlcommon::row_title($lt{'emac'},undef,$css[$i]).
-                   '<input type="text" size="50" name="cc" value="" /><br />'."\n".
-                   &Apache::lonhtmlcommon::row_closure();
-        $num ++;
-        $i = $num%2;
+        if ($homeserver) { 
+            $output .= &Apache::lonhtmlcommon::row_title($lt{'emac'},undef,$css[$i]).
+                       '<input type="text" size="50" name="cc" value="" /><br />'."\n".
+                       &Apache::lonhtmlcommon::row_closure();
+            $num ++;
+            $i = $num%2;
+        }
     }
     $output .= &Apache::lonhtmlcommon::row_title("$lt{'unme'}/$lt{'doma'}",undef,$css[$i]);
     my $udom_input = '<input type="hidden" name="udom" value="'.
-                     &HTML::Entities::encode($udom,'"<>&').'" />';
+                     &HTML::Entities::encode($udom,'"<>&').'" />'."\n";
     my $uname_input = '<input type="hidden" name="uname" value="'.
-                      &HTML::Entities::encode($uname,'"<>&').'" />'; 
+                      &HTML::Entities::encode($uname,'"<>&').'" />'."\n"; 
     if (($env{'user.name'} =~ /^$match_username$/) && 
         ($env{'user.domain'} =~ /^$match_domain$/) && (!$public)) {
         $output .= '<i>'.$lt{'unme'}.'</i>:&nbsp;'.$uname.'&nbsp;&nbsp;<i>'.$lt{'doma'}.'</i>:&nbsp;'.$udom.$udom_input.$uname_input;
@@ -310,7 +352,7 @@ function initialize_codes() {
         } else {
             $output .= $lt{'entr'};
         }
-        $output .= '<br />';
+        $output .= '<br />'."\n";
         if (!$public) {
             if ($env{'user.domain'} =~ /^$match_domain$/) {
                 $udomform = '<i>'.$lt{'doma'}.'</i>:&nbsp;'.$udom.$udom_input;
@@ -320,7 +362,7 @@ function initialize_codes() {
         }
         if ($udomform eq '') {
             $udomform = '<i>'.$lt{'doma'}.'</i>:&nbsp;';
-            $udomform .= &Apache::loncommon::select_dom_form($codedom,'udom');
+            $udomform .= &Apache::loncommon::select_dom_form($codedom,'udom')."\n";
         }
         if ($unameform eq '') {
             $unameform= '<i>'.$lt{'unme'}.'</i>:&nbsp;<input type="text" size="20" name="uname" value="'.$uname.'" />&nbsp;&nbsp;';
@@ -331,11 +373,11 @@ function initialize_codes() {
     $num ++;
     $i = $num%2;
     $output .= &Apache::lonhtmlcommon::row_title("$lt{'urlp'}",undef,$css[$i]).
-               $showserver.'<input type="hidden" name="sourceurl" value="'.
-               &HTML::Entities::encode($server,'"<>&').'" />'.
+               $server."\n".'<input type="hidden" name="sourceurl" value="'.
+               &HTML::Entities::encode($sourceurl,'"<>&').'" />'."\n".
                &Apache::lonhtmlcommon::row_closure().
                &Apache::lonhtmlcommon::row_title("$lt{'phon'}",undef,'LC_evenrow_value').
-               '<input type="text" size="15" name="phone" /><br />'.
+               '<input type="text" size="15" name="phone" /><br />'."\n".
                &Apache::lonhtmlcommon::row_closure();
     $num ++;
     $i = $num%2; 
@@ -345,17 +387,17 @@ function initialize_codes() {
             foreach my $item (@codetitles) {
                 $output .= '<i>'.$item.'</i>:&nbsp;'.$codes{$cnum}{$item}.';&nbsp;';
             }
-            $output .= '&nbsp;<input type="hidden" name="coursecode" value="'.&HTML::Entities::encode($coursecodes{$cnum},'"<>&').'" />';
+            $output .= '&nbsp;<input type="hidden" name="coursecode" value="'.&HTML::Entities::encode($coursecodes{$cnum},'"<>&').'" />'."\n";
         } else {
             $output .= $lt{'enin'}.':&nbsp;
-                  <input type="text" name="coursecode" size="15" value="" />';
+                  <input type="text" name="coursecode" size="15" value="" />'."\n";
         }
     } else {
         if ($totcodes > 0) {
             my $numtitles = @codetitles;
             if ($numtitles == 0) {
                 $output .= $lt{'enin'}.':&nbsp;
-                  <input type="text" name="coursecode" size="15" value="" />';
+                  <input type="text" name="coursecode" size="15" value="" />'."\n";
             } else {
                 my @standardnames = &Apache::loncommon::get_standard_codeitems();
                 my $lasttitle = $numtitles;
@@ -364,7 +406,7 @@ function initialize_codes() {
                 } 
                 $output .= '<table><tr><td>'.$codetitles[0].'<br />'."\n".
                       '<select name="'.$standardnames[0].'" onchange="courseSet('."'$codetitles[0]'".')">'."\n".
-                      ' <option value="-1" />'.$lt{'sele'}."\n";
+                      ' <option value="-1">'.$lt{'sele'}."</option>\n";
                 my @items = ();
                 my @longitems = ();
                 if ($idlist{$codetitles[0]} =~ /","/) {
@@ -407,7 +449,7 @@ function initialize_codes() {
             }
         } else {
             $output .= $lt{'enin'}.':&nbsp;
-                  <input type="text" name="coursecode" size="15" value="" />';
+                  <input type="text" name="coursecode" size="15" value="" />'."\n";
         }
     }
     if ($ctitle) {
@@ -438,25 +480,50 @@ function initialize_codes() {
         }
         $output .= "</select>";
     } else {
-        $output .= '<input type="text" name="section" size="10" />';
+        $output .= '<input type="text" name="section" size="10" />'."\n";
     }
     $output .= &Apache::lonhtmlcommon::row_closure();
     $num ++;
     $i = $num%2; 
-    $output .= &Apache::lonhtmlcommon::row_title($lt{'subj'},undef,'LC_oddrow_value').
-               '  <input type="text" size="40" name="subject" />'."\n".
+    $output .= &Apache::lonhtmlcommon::row_title(
+                   '<span title="'.&mt('required').'">'.
+                   $lt{'subj'}.' <span class="LC_info">*</span></span>'
+                  ,undef,'LC_oddrow_value').
+               '<input type="text" size="40" name="subject" />'."\n".
                &Apache::lonhtmlcommon::row_closure().
-               &Apache::lonhtmlcommon::row_title($lt{'detd'},undef,'LC_evenrow_value').
-               '  <textarea rows="10" cols="45" name="description" wrap="virtual"></textarea>'.
+               &Apache::lonhtmlcommon::row_title(
+                   '<span title="'.&mt('required').'">'.
+                   $lt{'detd'}.' <span class="LC_info">*</span></span>'
+                  ,undef,'LC_evenrow_value').
+               '<textarea rows="10" cols="45" name="description" style="word-wrap:normal;">'.
+               '</textarea>'."\n".
                &Apache::lonhtmlcommon::row_closure();
     $num ++;
     $i = $num%2; 
     if (($env{'user.name'} =~ /^$match_username$/) && (!$public)) {
-        $output .= &Apache::lonhtmlcommon::row_title($lt{'opfi'},undef,$css[$i]).
-                   ' <input type="file" name="screenshot" size="20" /><br />'.$lt{'uplf'}."\n".
-        &Apache::lonhtmlcommon::row_closure();
-        $num ++;
-        $i = $num%2; 
+        if ($homeserver) {
+            $output .= &Apache::lonhtmlcommon::row_title($lt{'opfi'},undef,$css[$i]).
+                       ' <input type="file" name="screenshot" size="20" /><br />'.
+                       "\n".$lt{'uplf'}."\n".
+                       &Apache::lonhtmlcommon::row_closure();
+            $num ++;
+            $i = $num%2;
+        }
+    } else {
+        my $lonhost = $r->dir_config('lonHostID');
+        my ($captchaform,$error) = 
+            &Apache::loncommon::captcha_display('login',$lonhost);
+        if ($captchaform) {
+            $output .= &Apache::lonhtmlcommon::row_title(
+                           '<span title="'.&mt('required').'">'.
+                           &mt('Validation').
+                           ' <span class="LC_info">*</span></span>'
+                      ,undef,$css[$i]).
+                       $captchaform."\n".
+                       &Apache::lonhtmlcommon::row_closure();
+            $num ++;
+            $i = $num%2;
+        }
     }
     $output .= &Apache::lonhtmlcommon::row_title($lt{'fini'},undef,$css[$i]);
     $output .= <<END;
@@ -486,6 +553,37 @@ END
 
 sub print_request_receipt {
     my ($r,$url,$function) = @_;
+    my $public;
+    if (($env{'user.name'} eq 'public') && ($env{'user.domain'} eq 'public')) {
+        $public = 1;
+    }
+    unless (($env{'user.name'} =~ /^$match_username$/) && (!$public)) {
+        my $lonhost = $r->dir_config('lonHostID');
+        my ($captcha_chk,$captcha_error) = 
+            &Apache::loncommon::captcha_response('login',$lonhost);
+        if ($captcha_chk != 1) {
+            $r->print(&Apache::loncommon::start_page('Support request failed',undef,
+                                       {'function'    => $function,
+                                        'add_entries' => {
+                                            topmargin    => "0",
+                                            marginheight => "0",
+                                        },
+                                        'only_body'   => 1,}));
+            if ($r->uri eq '/adm/helpdesk') {
+                &print_header($r,$url,'process');
+            }
+            $r->print(
+                '<h2>'.&mt('Support request failed').'</h2>'.
+                      &Apache::lonhtmlcommon::confirm_success(
+                        &mt('Validation of the code you entered failed.'),1).
+                '<br /><br />'.
+                &Apache::lonhtmlcommon::actionbox([
+                    &mt('[_1]Go back[_2] and try again',
+                        '<a href="javascript:history.go(-1)">','</a>')]).
+                      &Apache::loncommon::end_page());
+            return;
+        }
+    }
     my @ENVvars = ('HTTP_HOST','HTTP_USER_AGENT','REMOTE_ADDR','SERVER_ADDR','SERVER_NAME');
     my @envvars = ('browser.os','browser.type','browser.version','user.home','request.role');
     my @loncvars = ('user.name','user.domain','request.course.sec','request.course.id');
@@ -565,8 +663,15 @@ sub print_request_receipt {
                  adin        => 'Additional information recorded',
     );
 
-    my (@ok_ccs,@bad_ccs,$badccmsg,$okcclist);
-    if ((defined($env{'user.name'})) && (defined($env{'form.cc'}))) {
+    my (@ok_ccs,@bad_ccs,$badccmsg,$okcclist,$homeserver);
+    unless ($public) {
+        if (($env{'user.name'} ne '') && ($env{'user.domain'} ne '')) {
+            $homeserver = &Apache::lonnet::homeserver($env{'user.name'},
+                                                      $env{'user.domain'});
+        }
+    }
+
+    if (($homeserver) && (defined($env{'form.cc'}))) {
         my @ccs;
         if ($env{'form.cc'} =~ /,/) {
             @ccs = split(/,/,$env{'form.cc'});
@@ -592,10 +697,11 @@ sub print_request_receipt {
            $okcclist = join(', ',@ok_ccs); 
         } 
         if (@bad_ccs == 1) {
-            $badccmsg .= '<br />'.&mt('The following Cc e-mail address is invalid: ').$bad_ccs[0];
+            if ($bad_ccs[0] ne '') {
+                $badccmsg .= '<br />'.&mt('The following Cc e-mail address is invalid: ').&Apache::loncommon::cleanup_html($bad_ccs[0]);
+            }
         } elsif (@bad_ccs > 1) {
-            my $bad_cc_string = join(', ',@bad_ccs);
-            $badccmsg .= '<br />'.&mt('The following Cc e-mail addresses are invalid: ').$bad_cc_string;
+            $badccmsg .= '<br />'.&mt('The following Cc e-mail addresses are invalid: '). &Apache::loncommon::cleanup_html(join(', ',@bad_ccs));
         }
     }
     $env{'form.user'} = "'".$env{'form.uname'}.':'.$env{'form.udom'}."'";
@@ -618,7 +724,7 @@ END
         if ($env{'form.'.$item} ne '') {
             if ($item eq 'description') {
                 my $descrip = $env{'form.description'};
-                $descrip =  &cleanup_html($descrip);
+                $descrip =  &Apache::loncommon::cleanup_html($descrip);
                 $descrip =~ s|[\n\r\f]|<br />|g;
                 $displaymsg .= 
                     '<span class="LC_helpform_receipt_cat">'.
@@ -626,17 +732,20 @@ END
             } elsif ($item eq 'sourceurl') {
                 my $showurl = $env{'form.sourceurl'};
                 $showurl =~ s/\?.*$//;
-                $showurl =  &cleanup_html($showurl);
+                $showurl =  &Apache::loncommon::cleanup_html($showurl);
                 $displaymsg .= 
                     '<span class="LC_helpform_receipt_cat">'.
                     "$lt{$item}</span>: $showurl<br />\n";
             } elsif ($item eq 'cc') {
-                $displaymsg .=
-                    '<span class="LC_helpform_receipt_cat">'.
-                    "$lt{$item}</span>: $okcclist<br />\n";
+                if ($okcclist) {
+                    my $showcclist = &Apache::loncommon::cleanup_html($okcclist);
+                    $displaymsg .=
+                        '<span class="LC_helpform_receipt_cat">'.
+                        "$lt{$item}</span>: $showcclist<br />\n";
+                }
             } else {
                 my $showitem = $env{'form.'.$item};
-                $showitem = &cleanup_html($showitem);
+                $showitem = &Apache::loncommon::cleanup_html($showitem);
                 $displaymsg .= 
                     '<span class="LC_helpform_receipt_cat">'.
                     "$lt{$item}</span>: $showitem<br />\n";
@@ -657,7 +766,7 @@ END
 
     $r->print(<<"END");
 $start_page
-<form name="logproblem">
+<form name="logproblem" action="">
 <input type="hidden" name="command" value="result" />
 </form>
 END
@@ -727,12 +836,12 @@ END
 
     my $attachmentpath = '';
     my $attachmentsize = '';
-    if ((defined($env{'user.name'})) && ($env{'user.name'} ne 'public')
-        && ($env{'user.domain'} ne 'public')) {
-        if ($env{'form.screenshot.filename'}) {
+    if ((defined($env{'user.name'})) && (!$public)) {
+        if ($homeserver && $env{'form.screenshot.filename'}) {
             $attachmentsize = length($env{'form.screenshot'});
             if ($attachmentsize > 131072) {
-                $displaymsg .= '<br />'.&mt('The uploaded screenshot file ([_1] bytes) included with your request exceeded the maximum allowed size - 128 KB, and has therefore been discarded.',$attachmentsize);
+                $displaymsg .= '<br /><span class="LC_warning">'.
+                               &mt('The uploaded screenshot file ([_1] bytes) included with your request exceeded the maximum allowed size - 128 KB, and has therefore been discarded.',$attachmentsize).'</span>';
             } else {
                 $attachmentpath=&Apache::lonnet::userfileupload('screenshot',undef,'helprequests');
             }
@@ -772,9 +881,11 @@ END
                  Type    =>'TEXT',
                  Data    => $supportmsg,
                  );
-    if (@ok_ccs > 0) {
-        my $cc_string = join(', ',@ok_ccs);
-        $msg->add("Cc" => $cc_string);
+    if ($homeserver) {
+        if (@ok_ccs > 0) {
+            my $cc_string = join(', ',@ok_ccs);
+            $msg->add("Cc" => $cc_string);
+        }
     }
     if ($bcc ne '') {
         $msg->add("Bcc" => $bcc);
@@ -782,7 +893,7 @@ END
     $msg->attr("content-type"         => "text/plain");
     $msg->attr("content-type.charset" => "UTF-8");
 
-    if ($attachmentpath) {
+    if ($homeserver && $attachmentpath) {
         my ($type, $encoding) = MIME::Types::by_suffix($attachmentpath);
         $msg->attach(Type     => $type,
                      Path     => $attachmentpath,
@@ -855,11 +966,14 @@ sub print_header {
         $helpdesk_link = '<a href="/adm/helpdesk">';
     }
     my %lt = &Apache::lonlocal::texthash (
-                                           login => 'Log-in help',
-                                           ask   => 'Ask helpdesk',
-                                           getst => 'Getting started guide',
-                                           back =>  'Back to last location',
+                                           login    => 'Log-in help',
+                                           ask      => 'Ask helpdesk',
+                                           getst    => 'Getting started guide',
+                                           back     =>  'Back to last location',
                                            headline => 'help/support',
+                                           stud     => 'Students',
+                                           ifyo     => 'If your problem is still unresolved, the form below can be used to send a question to the LON-CAPA helpdesk.',
+                                           cont     => 'Contact your instructor instead.',
                                          );
     my ($getstartlink,$reviewtext);
     if (-e $Apache::lonnet::perlvar{'lonDocRoot'}.'/adm/gettingstarted.html') {
@@ -868,42 +982,55 @@ sub print_header {
     } else {
         $reviewtext = &mt('Please review the information in "Log-in help" if you are unable to log-in.');
     }
+    my $linkback;
     if ($origurl eq '') {
-        $origurl = 'javascript:history.go(-1)';
+        $linkback = 'javascript:history.go(-1)';
+    } else {
+        $linkback = &HTML::Entities::encode($origurl,'"<>&');
     }
-    $r->print(<<END);
-<table width="620" border="0" cellspacing="0" cellpadding="0" height="55">   <tr height="50">    <td width='5'>&nbsp;</td>
-   <td>
+    my $loginhelp = &Apache::lonauth::loginhelpdisplay();
+    if ($loginhelp eq '') {
+        $loginhelp = '/adm/loginproblems.html';
+    }
+    $r->print(<<"END");
+<table width="620" border="0" cellspacing="0" cellpadding="0" style="height: 55px;">
+ <tr>
+   <td width="5" height="50">&nbsp;</td>
+   <td height="50">
     <fieldset>
       <legend>
-        <img src="$location/lonIcons/minilogo.gif" height="20" width="29" valign="bottom" />
+        <img src="$location/lonIcons/minilogo.gif" height="20" width="29" alt="logo" style="vertical-align:bottom" />
         LON-CAPA $lt{'headline'}
       </legend>
  <table id="LC_helpmenu_links">
    <tr>
-    <td align="center"><span class="LC_nobreak"><img src="$location/help/help.png" border="0" alt="($lt{'login'})" valign="middle" />&nbsp;<b><a href="/adm/loginproblems.html">$lt{'login'}</a></b>&nbsp;</span></td>
-    <td align="center"><span class="LC_nobreak">&nbsp;<b>$helpdesk_link<img src="$location/lonIcons/helpdesk.gif" border="0" alt="($lt{'ask'})" valign="middle" />&nbsp;$lt{'ask'}</a></b>&nbsp;</span></td>$getstartlink
-    <td align="center"><span class="LC_nobreak">&nbsp;<b><a href="$origurl" target="_top"><img src="$location/lonIcons/move_up.gif" border="0" alt="($lt{'back'})" valign="middle" />&nbsp;$lt{'back'}</a></b>&nbsp;</span></td>
+    <td align="center"><span class="LC_nobreak"><img src="$location/help/help.png" border="0" alt="($lt{'login'})" style="vertical-align:middle" />&nbsp;<b><a href="$loginhelp">$lt{'login'}</a></b>&nbsp;</span></td>
+    <td align="center"><span class="LC_nobreak">&nbsp;<b>$helpdesk_link<img src="$location/lonIcons/helpdesk.gif" border="0" alt="($lt{'ask'})" style="vertical-align:middle" />&nbsp;$lt{'ask'}</a></b>&nbsp;</span></td>$getstartlink
+    <td align="center"><span class="LC_nobreak">&nbsp;<b><a href="$linkback" target="_top"><img src="$location/lonIcons/move_up.gif" border="0" alt="($lt{'back'})" style="vertical-align:middle" />&nbsp;$lt{'back'}</a></b>&nbsp;</span></td>
    </tr>
  </table>
 </fieldset>
   </td>
-  <td width='5'>&nbsp;</td>
+  <td width="5">&nbsp;</td>
  </tr>
- <tr height='5'>
-  <td colspan='3' height='5'>&nbsp;</td>
+ <tr>
+  <td colspan="3" height="5">&nbsp;</td>
  </tr>
 END
     if  ($command ne 'process') {
-        $r->print('
+        my $stuwarn = &mt('Do [_1]not[_2] use this form to ask about course content.',
+                          '<b>','</b>');
+        $r->print(<<"END");
  <tr>
-  <td colspan="3">'.$reviewtext.' '
-.&mt('If your problem is still unresolved, the form below can be used to send a question to the LON-CAPA helpdesk.').'<br />'
-.'<font size="-1"><b>'.&mt('Students').'</b>: '.&mt('Do <b>not</b> use this form to ask questions about course content.').' '
-.&mt('Contact your instructor instead.')
-.'</font><br /><br />
+  <td colspan="3">$reviewtext 
+  $lt{'ifyo'}
+<p class="LC_info">
+<b>$lt{'stud'}:</b> 
+$stuwarn $lt{'cont'}
+</p><br />
   </td>
- </tr>');
+ </tr>
+END
     }
     $r->print('
 </table>');
@@ -913,8 +1040,11 @@ END
 sub get_domain {
     my $codedom;
     if (exists($env{'form.codedom'})) {
-        $codedom = $env{'form.codedom'};
-    } elsif ($env{'request.course.id'}) {
+        if (&Apache::lonnet::domain($env{'form.codedom'}) ne '') {
+            return $env{'form.codedom'};
+        }
+    }
+    if ($env{'request.course.id'}) {
         $codedom = $env{'course.'.$env{'request.course.id'}.'.domain'};
     } elsif ($env{'request.role.domain'}) {
         $codedom = $env{'request.role.domain'};
@@ -924,22 +1054,4 @@ sub get_domain {
     return $codedom;
 }
 
-sub cleanup_html {
-    my ($incoming) = @_;
-    my $outgoing;
-    if ($incoming ne '') {
-        $outgoing = $incoming;
-        $outgoing =~ s/\#/&#035;/g;
-        $outgoing =~ s/\&/&#038;/g;
-        $outgoing =~ s/</&#060;/g;
-        $outgoing =~ s/>/&#062;/g;
-        $outgoing =~ s/\(/&#040/g;
-        $outgoing =~ s/\)/&#041;/g;
-        $outgoing =~ s/"/&#034;/g;
-        $outgoing =~ s/'/&#039;/g;
-        $outgoing =~ s/\$/&#036;/g;
-    }
-    return $outgoing;
-}
-
 1;
