@@ -1,7 +1,7 @@
 #!/usr/bin/perl
 # CGI-script to run LaTeX, dvips, ps2ps, ps2pdf etc.
 #
-# $Id: printout.pl,v 1.146.4.2 2011/03/04 16:53:06 raeburn Exp $
+# $Id: printout.pl,v 1.159 2014/02/28 19:20:06 bisitz Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -44,7 +44,7 @@ use LONCAPA::Configuration;
 use strict;
 
 my $busy_wait_timeout = 30; 
-
+my $pdfs_converted    = 0;	# non zero if PDF includes (need to fixps).
 
 my $debugging = 0;
 
@@ -127,8 +127,8 @@ sub send_error_mail {
 
 	    # Todo: Convert badurl into a url from file path:
 
-	    my $subject  = "Error [$badurl] Print failed for $user".'@'.$domain;
-	    my $message .= "Print failed to render LaTeX for $user".'@'."$domain\n";
+	    my $subject  = "Error [$badurl] Print failed for $user".':'.$domain;
+	    my $message .= "Print failed to render LaTeX for $user".':'."$domain\n";
 	    $message    .= "  User was attempting to print: \n";
 	    foreach my $resource (split(/:/,$resources)) {
 		$message    .= "       $resource\n";
@@ -237,7 +237,7 @@ if (-e $figfile) {
     close $temporary_file;  
     my $noteps;
     my %prog_state;
-    if ($advanced_role) { %prog_state=&Apache::lonhtmlcommon::Create_PrgWin('','Converting Images to EPS','Picture Conversion Status',$#content_of_file,'inline','80');  }
+    if ($advanced_role) { %prog_state=&Apache::lonhtmlcommon::Create_PrgWin('',$#content_of_file);  }
     print('<br />');
     foreach my $not_eps (@content_of_file) {
 	chomp($not_eps);
@@ -283,7 +283,7 @@ my $ind=-1;
 
 my %prog_state;
 if ($advanced_role) { 
-    %prog_state=&Apache::lonhtmlcommon::Create_PrgWin('','Print Status',&mt('Class Print Status'),$number_of_files,'inline','80'); 
+    %prog_state=&Apache::lonhtmlcommon::Create_PrgWin('',$number_of_files); 
 }
 print "<br />";
 my $num_files = @texfile;	# How does this differ from $number_of_files , can that be 0?
@@ -308,14 +308,23 @@ foreach $texfile (@texfile) {
   my $name;
   my $name_range='';
 
-  # $name       -> Either user's full name or username@domain
-  # $name_range -> Either user's last name or usrname.
+  # $name       -> Either user's full name or username:domain
+  # $name_range -> Either user's last name or username.
 
   if ($tempo_array[3]) {
       $name=$tempo_array[3];
-      ($name_range) = split(/,/,$name, 2);
+      $name =~ s{^\s+|\s+$}{}g;
+      if ($name =~ /,/) { 
+          ($name_range) = split(/,/,$name, 2);
+      } elsif ($name =~ /\s/) {
+          $name_range = $name;
+          $name_range =~ s/\s+/_/;
+      } else {
+          $name_range = $name;  
+      }
+      $name_range =~ s/[^\w\:\-]+//g;
   } else {
-      $name=$tempo_array[0].'@'.$tempo_array[1];
+      $name=$tempo_array[0].':'.$tempo_array[1];
       $name_range = $tempo_array[0];
   }
 
@@ -323,7 +332,7 @@ foreach $texfile (@texfile) {
   # user is getting printed.
   #
 
-  if (($name ne "") && ($name ne '@') ) { # Could be printing codes...
+  if (($name ne "") && ($name ne ':') ) { # Could be printing codes...
       $link_text='<b>'.$name.'</b>';
       $status_statement.=$name;
   }
@@ -336,13 +345,23 @@ foreach $texfile (@texfile) {
       @tempo_array=split(/:/,$stud_info[-1]);
       if ($tempo_array[3]) {
 	  $name=$tempo_array[3];
-	  my ($lastname) = split(/,/, $name,2);
+          $name =~ s{^\s+|\s+$}{}g;
+          my $lastname;
+          if ($name =~ /,/) {
+	      ($lastname) = split(/,/, $name,2);
+          } elsif ($name =~ /\s/) {
+              $lastname = $name;
+              $lastname =~ s/\s+/_/;
+          } else {
+              $lastname = $name;
+          }
 	  $name_range .= "-".$lastname;
+          $name_range =~ s/[^\w\:\-]+//g;
       } else {
-	  $name=$tempo_array[0].'@'.$tempo_array[1];
+	  $name=$tempo_array[0].':'.$tempo_array[1];
 	  $name_range .= '-'.$tempo_array[0];
       }
-      if (($name ne "") && ($name ne '@')) {
+      if (($name ne "") && ($name ne ':')) {
 	  $link_text.=' - <b>'.$name.'</b>';
 	  $status_statement.=' -  '.$name;
   
@@ -372,7 +391,7 @@ foreach $texfile (@texfile) {
 
   print "<br/>";
   if ($advanced_role) { 
-      &Apache::lonhtmlcommon::Update_PrgWin('',\%prog_state,'Creating PDF for '.$status_statement); 
+      &Apache::lonhtmlcommon::Update_PrgWin('',\%prog_state,&mt('Creating PDF for: [_1]',$status_statement)); 
   }
 
   if (-e $texfile) {		# Ensure the tex file exists:
@@ -477,6 +496,31 @@ foreach $texfile (@texfile) {
 	      my $ps_file = my $tempo_file = $1.'temporar.ps';
 	      my $pdf_file = $1.'.pdf';
 	      $papera=~s/t/p/;
+#----
+# The code below uses fixps to make pdf include in sequences work.
+#
+#              $comma = "fixps --force $new_name_file";
+#              &debug("FIXPS command: $comma");
+#              &busy_wait_command("$comma 1>$tempo_file  2>/dev/null",
+#                                 "for $status_statement now validating PS",
+#                                 \%prog_state,$tempo_file);
+
+#--- 
+#  The code below uses gs to make pdf includes in sequences work
+
+	      # Use gs to fix the postscript -> level 1.5 
+	      # .. if pdfs were included
+
+	      if ($pdfs_converted > 0) {
+		  $comma = "gs -sDEVICE=pswrite -dLanguageLevel=1.5 ";
+		  &busy_wait_command("$comma -o $tempo_file $new_name_file 2>/dev/null 1>/dev/null",
+				     "for $status_statement now validating PS",
+				     \%prog_state, $tempo_file);
+		  
+#---
+		  &busy_wait_command("mv $tempo_file $new_name_file",
+				     'File move', \%prog_state, $new_name_file);
+	      }
 	      if ($laystyle eq 'album' and $numberofcolumns eq '2') {
 		  $comma = "psnup $papera -2 -s1.0 $new_name_file";
 		  &debug("PSNUP command: $comma");
@@ -528,14 +572,21 @@ foreach $texfile (@texfile) {
 	      my @garb = ($texlog,$texaux,$texdvi,$texps);
 #	  unlink @garb;
 	      unlink($duefile);
-	      print "<a href=\"/prtspool/$pdf_file\">$link_text - click here to download pdf</a>";
-	      print "\n";
+	      print
+                  '<p>'
+                 .&mt('[_1] - [_2]Your PDF file[_3] is ready for download.',
+                      $link_text,'<a href="/prtspool/'.$pdf_file.'">','</a>')
+                 .'</p>'."\n";
 	  }
 	  unlink($missfonts_file);
 
       }  
   } else {
-      print "LaTeX file $texfile was not created successfully";
+      print
+          '<p class="LC_error">'
+         .&mt('The LaTeX file [_1] was not created successfully.',
+              '<span class="LC_filename">'.$texfile.'</span>')
+         .'</p>';
   }
 }
 print "<br />";
@@ -547,11 +598,15 @@ if ($number_of_files>1) {
 	$file=~s/\.tex/.\pdf/;
 	$statement.=' '.$file; 
     }
-    print("<pre>Zip Output:\n");
+    print('<p>'.&mt('Zip Output:')."\n<pre>\n");
     system($statement);
-    print("</pre>");
+    print("</pre></p>\n");
     $zipfile=~s{^\Q$perlvar{'lonPrtDir'}\E}{/prtspool};
-    print "<br /> A <a href=\"$zipfile\">ZIP file</a> of all the PDFs.";
+    print
+        '<p>'
+       .&mt('A [_1]ZIP file[_2] of all the PDF files is ready for download.',
+            '<a href="'.$zipfile.'">','</a>')
+       .'</p>';
 }
 if ($advanced_role) { &Apache::lonhtmlcommon::Close_PrgWin('',\%prog_state); }
 print(&Apache::loncommon::end_page());
@@ -589,7 +644,7 @@ sub busy_wait_command {
 	    if ($size == $last_size) {
 		$unchanged_time++;
 		if ($timeout && ($unchanged_time > $timeout)) {
-		    print "<h1>Operation timed out!</h1>\n";
+		    print '<p class="LC_error">'.&mt('Operation timed out!')."</p>\n";
 		    print "<p>Executing $command, the output file $output_file did not grow\n";
 		    print "after $timeout seconds.  This <em>may</em> indicate $command\n";
 		    print "is in an infinite loop.\n";
@@ -604,8 +659,7 @@ sub busy_wait_command {
 	    }
 	}
 	if ($advanced_role) {
-	    &Apache::lonhtmlcommon::Update_PrgWin('',$progress_win,
-						  $message.$extra_msg);
+	    &Apache::lonhtmlcommon::Update_PrgWin('',$progress_win,$message.$extra_msg);
 	}
     }
     $SIG{CHLD}='IGNORE';
@@ -702,8 +756,7 @@ sub repaginate {
     my @pages_in_student;	# For each student his/her initial page count.
     my $max_pages = 0;		# Pages in 'longest' student.
     my $page_number = 0;
-    &Apache::lonhtmlcommon::Update_PrgWin('',\%prog_state, 
-					  &mt("Counting pages for student: [_1]",1));
+    &Apache::lonhtmlcommon::Update_PrgWin('',\%prog_state,&mt("Counting pages for student: [_1]",1));
 
     while ($line = <PSFILE>) {
 	
@@ -731,8 +784,7 @@ sub repaginate {
 		# - save the page_number, reset and, if necessary
 		#    update max_pages.
 		push(@pages_in_student, $page_number);
-		&Apache::lonhtmlcommon::Update_PrgWin('',\%prog_state, 
-						      &mt("Counting pages for student: [_1]", scalar(@pages_in_student)));
+		&Apache::lonhtmlcommon::Update_PrgWin('',\%prog_state,&mt("Counting pages for student: [_1]", scalar(@pages_in_student)));
 		if ($page_number > $max_pages) {
 		    $max_pages = $page_number;
 		}
@@ -744,8 +796,7 @@ sub repaginate {
     }
     # file ended so one more student
     push(@pages_in_student, $page_number);
-    &Apache::lonhtmlcommon::Update_PrgWin('',\%prog_state, 
-					  &mt("Counting pages for student: [_1]",scalar(@pages_in_student)));
+    &Apache::lonhtmlcommon::Update_PrgWin('',\%prog_state,&mt("Counting pages for student: [_1]",scalar(@pages_in_student)));
     if ($page_number > $max_pages) {
 	$max_pages = $page_number;
     }
@@ -772,8 +823,7 @@ sub repaginate {
 
 
     my $student_number    = 0;	# Index of student we're working on.
-    &Apache::lonhtmlcommon::Update_PrgWin('',\%prog_state, 
-					  "Repaginating student ".$student_number+1);
+    &Apache::lonhtmlcommon::Update_PrgWin('',\%prog_state,&mt("Repaginating student: [_1]",$student_number+1));
 
     while (my $line = <LATEXIN>) {
 	if ($line eq "\\special{ps:ENDOFSTUDENTSTAMP}\n") {
@@ -790,8 +840,7 @@ sub repaginate {
 	    }
 	    
 	    $student_number++;
-	    &Apache::lonhtmlcommon::Update_PrgWin('',\%prog_state, 
-						  "Repaginating student ".$student_number+1);
+	    &Apache::lonhtmlcommon::Update_PrgWin('',\%prog_state,&mt("Repaginating student: [_1]",$student_number+1));
 	    
 	} else {
 	    print LATEXOUT $line;
@@ -872,8 +921,8 @@ sub convert_figure {
     my $status_statement='EPS picture for '.$not_eps;
     my $eps_f = $not_eps;
 
-    if ($eps_f=~/\/home\/([^\/]+)\/public_html\//) {
-	$eps_f=~s/\/home\/([^\/]+)\/public_html/$1/;
+    if ($eps_f=~/\/home\/httpd\/html\/priv\/[^\/]+\/([^\/]+)\//) {
+	$eps_f=~s/\/home\/httpd\/html\/priv\/[^\/]+\/([^\/]+)/$1/;
     } elsif ($eps_f=~/$perlvar{'lonDocRoot'}\/res\//) {
 	$eps_f=~ s/$perlvar{'lonDocRoot'}\/res\/(.+)/$1/;
     } elsif ($eps_f=~/$perlvar{'lonUsersDir'}\//) {
@@ -904,12 +953,10 @@ sub convert_figure {
 	$not_eps =~ s/^\s+//;
 	$not_eps =~ s/\s+$//;
 	$not_eps =~ s/ /\\ /g;
-        my $prettyname=$not_eps;
+    my $prettyname=$not_eps;
 	if ($advanced_role) {
-	    $prettyname=~s|/home/([^/]+)/public_html|/priv/$1|;
 	    $prettyname=~s|$perlvar{'lonDocRoot'}/|/|;
-	    &Apache::lonhtmlcommon::Update_PrgWin('',\%prog_state,
-						  'Converting to EPS '.$prettyname);
+	    &Apache::lonhtmlcommon::Update_PrgWin('',\%prog_state,&mt('Converting to EPS: [_1]',$prettyname));
 	}
 	#
 	#  If the file is a PDF, need to use pdftops to convert it to a ps file.
@@ -926,14 +973,17 @@ sub convert_figure {
 
 	    &debug("Converting pdf $not_eps to postscript: $eps_f");
 	    system("pdftops $not_eps $eps_f");
+	    $pdfs_converted++;	# Need to fix ps in last pass.
 	} else {
 	    system("convert $not_eps $eps_f");
-            if ($? and $advanced_role) {
-                print '<p class="LC_warning">'
-                      .&mt('An error occured during the conversion of [_1].[_2]If possible try to save this image using different settings and republish it.',
-                      '<span class="LC_filename">'.$prettyname.'</span>','<br/>')
-                      .'</p>';
-            }
+        if($? and $advanced_role){
+            print '<p class="LC_warning">'
+                 .&mt('An error occurred during the conversion of [_1].',
+                          '<span class="LC_filename">'.$prettyname.'</span>')
+                 .'<br />'
+                 .&mt('If possible try to save this image using different settings and republish it.')
+                 .'</p>';
+        }
 	}
 
 	if (not -e $eps_f) {
@@ -1021,7 +1071,7 @@ sub analyze_logfile {
 	    print "<h2>".&mt('LaTeX could not successfully parse your TeX file.')."</h2>";
 	    print &mt('It probably has errors in it.')."<br />";
 	    if ($badtext) {
-                print &mt('With very high probability this error occured in [_1].',$badtext)
+                print &mt('With very high probability this error occurred in [_1].',$badtext)
                      ."<br /><br />";
             }
 	    print &mt('Here are the error messages in the LaTeX log file:')
@@ -1110,7 +1160,7 @@ sub analyze_logfile {
 	    my $whereitends = index $body_log_file,'STAMPOFPASSEDRESOURCEEND',$whereitbegins;
 	    print "<br />"
                  .&mt('It has found an error in [_1][_2]and corrected it.',substr($body_log_file,$whereitbegins+26,$whereitends-$whereitbegins-26),"<br />")."\n";
-	    print &mt('Usually this correction is valid but you probably need to check the indicated resource one more time and implement neccessary corrections by yourself.')."\n";
+	    print &mt('Usually this correction is valid but you probably need to check the indicated resource one more time and implement necessary corrections by yourself.')."\n";
 	    $whereitbegins = index $body_log_file,'<inserted text>',$tempobegin+10;
 	}
 
