@@ -1,5 +1,5 @@
 #
-# $Id: classcalc.pm,v 1.29 2008/10/23 09:07:57 bisitz Exp $
+# $Id: classcalc.pm,v 1.32 2014/02/12 16:53:09 bisitz Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -91,16 +91,10 @@ sub html_header {
         $toprow .= '<th align="center"><b>'.&mt($name).'</b></th>';
         $bottomrow .= '<td>'.$selector.'</td>';
     }
-    my $status .= '<p><span class="LC_nobreak">'
-                 .&mt('Status: [_1]',
-                          '<input type="text" name="spreadsheet_status"'
-                         .' size="60" value="" readonly="readonly" />')
-                 .'</span></p>';
-
     return "<p>\n<table>\n".
         "<tr>".$toprow."</tr>\n".
         "<tr>".$bottomrow."</tr>\n".
-        "</table>\n".$status."\n".
+        "</table>\n".
         "</p>";
 }
 
@@ -322,13 +316,13 @@ sub outsheet_htmlclasslist {
     #
     if ($show_expire_link) {
         $r->print('<a href="/adm/classcalc?output_format=htmlclasslist&'.
-                  'recalc=ilovewastingtime&not_first_run=1">'.
+                  'recalc=expireallsheets&not_first_run=1">'.
                   &mt('Expire all student spreadsheets').'</a>'.$/);
     }
     #
-    $r->print('<h3>'.
+    $r->print('<p class="LC_info">'.
               &mt('Click on a student to be taken to their spreadsheet').
-              '</h3>');
+              '</p>');
     #
     my %header=&Apache::lonlocal::texthash(
                                            'student'      => 'Student',
@@ -363,7 +357,7 @@ END
 	    $r->print("</table>\n".$tableheader);
 	}
         my $link = '<a href="/adm/studentcalc?sname='.$student->{'username'}.
-            '&sdomain='.$student->{'domain'}.'">';
+            '&amp;sdomain='.$student->{'domain'}.'">';
         $student->{'section'} = 'none' if ($student->{'section'} eq '-1');
 	$r->print('<tr>'.
                   '<td>'.$num_output.'</td>'.
@@ -376,9 +370,9 @@ END
         if ($show_expire_link) {
             $r->print('<td>'.
                       '<a href="/adm/classcalc?recalc=student:'.
-                      $student->{'username'}.':'.$student->{'domain'}.'&'.
-                      'output_format=htmlclasslist&'.
-                      'not_first_run=1'.'">Expire Record</a>'.$/);
+                      $student->{'username'}.':'.$student->{'domain'}.'&amp;'.
+                      'output_format=htmlclasslist&amp;'.
+                      'not_first_run=1'.'">'.&mt('Expire Record').'</a>'.$/);
         }
         $r->print("</tr>\n");
     }
@@ -386,16 +380,6 @@ END
     return;
 }
 
-sub update_status {
-    my ($r,$message) = @_;
-    $r->print('<script>'.
-              'window.document.sheet.spreadsheet_status.value="'.
-              $message.
-              '";</script>');
-    $r->rflush();
-    return;
-}
-
 sub compute {
     my $self = shift;
     my ($r) = @_;
@@ -404,14 +388,9 @@ sub compute {
     $self->initialize_safe_space();
     my %c = $self->constants();
     my %f = $self->formulas();
-    &update_status($r,'Initializing Course Structure');
     &Apache::studentcalc::initialize_package();
-    my %prog_state=&Apache::lonhtmlcommon::Create_PrgWin
-        ($r,&mt('Spreadsheet Computation Status'),
-         &mt('Spreadsheet Computation'), scalar(@Students),'inline',undef,
-         'sheet','spreadsheet_status');
-    &Apache::lonhtmlcommon::Update_PrgWin($r,\%prog_state,
-                                          &mt('Processing first student'));
+    my %prog_state=&Apache::lonhtmlcommon::Create_PrgWin($r,scalar(@Students));
+    &Apache::lonhtmlcommon::Update_PrgWin($r,\%prog_state,&mt('Processing first student'));
     foreach my $student (@Students) {
         if ($connection->aborted()) { $self->cleanup(); return; }
         my $sname = $student->{'username'}.':'.$student->{'domain'};
@@ -437,17 +416,14 @@ sub compute {
                 $c{$cell} = $data;
             }
 	}
-        &Apache::lonhtmlcommon::Increment_PrgWin($r,\%prog_state,
-                                                 'last student');
+        &Apache::lonhtmlcommon::Increment_PrgWin($r,\%prog_state,'last student');
     }
     &Apache::lonhtmlcommon::Close_PrgWin($r,\%prog_state);
-    &update_status($r,'Done computing student sheets');
     $r->rflush();
     $self->constants(\%c);
     $self->formulas(\%f);
     $self->calcsheet();
     $self->save() if ($self->need_to_save());
-    &update_status($r,'Done!');
 }
 
 1;
