@@ -1,5 +1,5 @@
 #
-# $Id: lonspreadsheet.pm,v 1.55.6.1 2011/10/09 00:28:03 raeburn Exp $
+# $Id: lonspreadsheet.pm,v 1.61.6.2 2013/08/09 23:05:18 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -64,6 +64,7 @@ use Apache::lonnet;
 use Apache::lonhtmlcommon;
 use Apache::lonlocal;
 use Apache::loncoursedata();
+use Apache::lonquickgrades();
 use HTML::Entities();
 
 ##
@@ -202,16 +203,6 @@ END
 
 sub handler {
     my $r=shift;
-    #
-    # Overload checking
-    #
-    # Check this server
-    my $loaderror=&Apache::lonnet::overloaderror($r);
-    if ($loaderror) { return $loaderror; }
-    # Check the course homeserver
-    $loaderror= &Apache::lonnet::overloaderror($r,
-                      $env{'course.'.$env{'request.course.id'}.'.home'});
-#    if ($loaderror) { return $loaderror; } 
     #
     # HTML Header
     #
@@ -251,7 +242,7 @@ sub handler {
 
     #
     # Do not allow users without vgr or mgr priv to continue unless 
-    # grading type is set to spreadsheet.
+    # grading type is set to spreadsheet. 
     #
 
     if ((!$allowed_to_view) && (!$allowed_to_edit)) {
@@ -354,10 +345,10 @@ sub handler {
     ##
     my $js;
     if ($allowed_to_edit) {
-	my %lt=(
+	my %lt=&Apache::lonlocal::texthash(
 		'ce' => 'Cell',
-		'ac' => 'Accept',
-		'dc' => 'Discard Changes'
+		'ac' => 'Save',
+		'dc' => 'Cancel'
 	);
         my $extra_javascript = 
             &Apache::loncommon::browser_and_searcher_javascript();
@@ -386,14 +377,14 @@ sub handler {
         // cellformula may contain less-than and greater-than symbols, so
         // we need to escape them?  
         edit_text +='$cell_edit_start';
-        edit_text += '<form name="editwinform">';
+        edit_text += '<form name="editwinform" action="">';
         edit_text += '<center><h3>$lt{'ce'} '+cellname+'</h3>';
         edit_text += '<textarea id="LC_newformula" name="newformula" ';
         edit_text += ' cols="60" rows="12"; wrap="off" style="width:100%">';
 	edit_text += cellformula+'</textarea>';
         edit_text += '<div id="LC_aftertextarea"><br />';
         edit_text += '<input type="button" name="accept" value="$lt{'ac'}"';
-        edit_text += ' onClick=\\\'javascript:';
+        edit_text += ' onclick=\\\'javascript:';
         edit_text += 'opener.document.sheet.cell.value=';
         edit_text +=     '"'+cellname+'";';
         edit_text += 'opener.document.sheet.newformula.value=';
@@ -403,7 +394,7 @@ sub handler {
         edit_text += '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;';
         edit_text += '<input type="button" name="abort" ';
         edit_text +=     'value="$lt{'dc'}"';
-        edit_text += ' onClick="javascript:self.close()" />';
+        edit_text += ' onclick="javascript:self.close()" />';
         edit_text += '</center></div></form>$cell_edit_end';
 
         if (editwin != null && !(editwin.closed) ) {
@@ -425,15 +416,15 @@ ENDSCRIPT
           text  => 'Spreadsheet',
           faq   => 134,
           bug   => 'Spreadsheet'});
-    my $settingslink = &Apache::lonhtmlcommon::coursepreflink(&mt('Grade display settings'),
-                                                              'grading');
-    &Apache::lonhtmlcommon::add_breadcrumb_tool('advtools',$settingslink);
     $r->print(&Apache::loncommon::start_page('Grades Spreadsheet',$js).
               &Apache::lonhtmlcommon::breadcrumbs('Spreadsheet',
-						  'Spreadsheet_About'));
+						  'Spreadsheet_About').
+              &Apache::lonhtmlcommon::coursepreflink(&mt('Grade display settings'),
+                                                     'grading'));
+
     #
     # Open the form
-    #
+    # 
     if ($warning) {
         $r->print('<p class="LC_info">'.$warning.'</p>');
     }
@@ -459,7 +450,7 @@ ENDSCRIPT
         if (exists($env{'form.recalc'})) {
             &Apache::loncoursedata::delete_caches($env{'requres.course.id'});
         }
-        if ($env{'form.recalc'} eq 'ilovewastingtime') {
+        if ($env{'form.recalc'} eq 'expireallsheets') {
             &Apache::lonnet::logthis('spreadsheet expired: entire course');
             # expire ALL spreadsheets
             &Apache::lonnet::expirespread('','','studentcalc');
@@ -541,14 +532,11 @@ ENDSCRIPT
                   '</td>'.
                   '<td valign="center">'.$html."</td></tr></table>\n");
         if ($action_message ne '') {
-            $r->print(<<END);
-<table>
-<tr><td valign="top"><b>Last Action:</b></td>
-    <td>&nbsp;</td>
-    <td>$action_message</td>
-</tr>
-</table>
-END
+            $r->print(
+                &Apache::loncommon::confirmwrapper(
+                    &mt('Last Action:')
+                   .$action_message)
+            );
         }
         $r->rflush();
     } else {
@@ -564,13 +552,13 @@ END
     if ($allowed_to_view) {
         $r->print('<td>'.
                   &Apache::loncommon::help_open_topic("Spreadsheet_About",
-                                                      'Spreadsheet Help').
+                                                      &mt('Spreadsheet Help')).
                   '</td>');
     }
     if ($allowed_to_edit) {
         $r->print('<td>'.
                   &Apache::loncommon::help_open_topic("Spreadsheet_Editing",
-                                                      'Editing Help').
+                                                      &mt('Editing Help')).
                   '</td>');
     }
     $r->print('</tr></table>');
@@ -579,7 +567,7 @@ END
     $r->print(&hiddenfield('filename',$filename));
     #
     # Keep track of the number of times we have been called, sort of.
-    $r->print(&hiddenfield('not_first_run','whatever'));
+    $r->print(&hiddenfield('not_first_run','1'));
     #
     if (exists($env{'form.not_first_run'}) || $sheettype ne 'classcalc') {
         $r->print($spreadsheet->get_html_title());
@@ -589,7 +577,8 @@ END
         $r->rflush();
         $spreadsheet->display($r);
     }
-    $r->print('</form>'.&Apache::loncommon::end_page());
+    $r->print('</form>');
+    $r->print(&Apache::loncommon::end_page());
     $spreadsheet->clear_package();
     return OK;
 }
