@@ -1,6 +1,6 @@
 # The LearningOnline Network with CAPA
 #
-# $Id: lonsurveyreports.pm,v 1.24 2010/02/28 23:58:55 raeburn Exp $
+# $Id: lonsurveyreports.pm,v 1.26.4.1 2012/05/12 03:59:53 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -67,6 +67,8 @@ sub BuildSurveyReportsPage {
                                                 \%Saveable_Parameters);
     #
     &Apache::lonstatistics::PrepareClasslist();
+    $r->print(&Apache::lonhtmlcommon::breadcrumbs('Student Submission Reports'));
+
     #
     $r->print(&CreateInterface());
     #
@@ -230,10 +232,7 @@ sub SurveyProblemSelector {
 sub compile_student_answers {
     my ($r,$problem,$ProblemData,$Students) = @_;
     my $resource = $problem->{'resource'};
-    my %prog_state=&Apache::lonhtmlcommon::Create_PrgWin
-        ($r,'Processing Student Submissions',
-         'Processing Student Submissions', 
-         scalar(@$Students),'inline',undef,'Statistics','stats_status');
+    my %prog_state=&Apache::lonhtmlcommon::Create_PrgWin($r,scalar(@$Students));
     foreach my $student (@$Students) {
         foreach my $partid (@{$resource->parts}) {
             my @response_ids   = $resource->responseIds($partid);
@@ -275,8 +274,7 @@ sub compile_student_answers {
                 }
             }
         }
-        &Apache::lonhtmlcommon::Increment_PrgWin($r,\%prog_state,
-                                                 'last student');
+        &Apache::lonhtmlcommon::Increment_PrgWin($r,\%prog_state,'last student');
     }
     &Apache::lonhtmlcommon::Close_PrgWin($r,\%prog_state);
     return;
@@ -626,7 +624,6 @@ sub CreateInterface {
         $output_selector.= '>'.$output_format->{'text'}.'</option>'.$/;
     }
     $output_selector .= '</select>'.$/;
-    $Str .= &Apache::lonhtmlcommon::breadcrumbs('Student Submission Reports');
     $Str .= '<p>';
     $Str .= &Apache::loncommon::start_data_table();
     $Str .= &Apache::loncommon::start_data_table_header_row();
@@ -668,11 +665,6 @@ sub CreateInterface {
     $Str .= &Apache::loncommon::end_data_table_row();
     $Str .= &Apache::loncommon::end_data_table();
     #
-    $Str .= '<p><span class="LC_nobreak">'
-           .&mt('Status: [_1]',
-                    '<input type="text" name="stats_status"'
-                   .' size="60" value="" readonly="readonly" />')
-           .'</span></p>';
     $Str .= '</p>';
     ##
     return $Str;
