@@ -9,7 +9,7 @@
 #  and displays a page showing the results of the action.
 #
 #
-# $Id: loncfile.pm,v 1.106.2.1 2011/11/07 13:38:45 raeburn Exp $
+# $Id: loncfile.pm,v 1.122 2014/06/14 21:40:05 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -37,7 +37,7 @@
 
 =head1 NAME
 
-Apache::loncfile - Construction space file management.
+Apache::loncfile - Authoring space file management.
 
 =head1 SYNOPSIS
  
@@ -68,7 +68,6 @@ use File::Basename;
 use File::Copy;
 use HTML::Entities();
 use Apache::Constants qw(:common :http :methods);
-use Apache::loncacc;
 use Apache::lonnet;
 use Apache::loncommon();
 use Apache::lonlocal;
@@ -102,7 +101,7 @@ my $r;				# Needs to be global for some stuff RF.
 =cut
 
 sub Debug {
-    # Put out the indicated message butonly if DEBUG is true.
+    # Put out the indicated message but only if DEBUG is true.
     if ($DEBUG) {
 	my ($r,$message) = @_;
 	$r->log_reason($message);
@@ -110,14 +109,15 @@ sub Debug {
 }
 
 sub done {
-   my ($url)=@_;
-   my $done=&mt("Done");
-   return(<<ENDDONE);
-<a href="$url">$done</a>
-<script type="text/javascript">
-location.href="$url";
-</script>
-ENDDONE
+    my ($url) = @_;
+    return
+       '<p>'
+      .&Apache::lonhtmlcommon::confirm_success(&mt("Done"))
+      .'<br /><a href="'.$url.'">'.&mt("Continue").'</a>'
+      .'<script type="text/javascript">'
+      .'location.href="'.$url.'";'
+      .'</script>'
+      .'</p>';
 }
 
 =pod
@@ -158,24 +158,28 @@ Global References
 sub URLToPath {
     my $Url = shift;
     &Debug($r, "UrlToPath got: $Url");
-    $Url=~ s/\/+/\//g;
-    $Url=~ s/^https?\:\/\/[^\/]+//;
-    $Url=~ s/^\///;
-    $Url=~ s/(\~|priv\/)($match_username)\//\/home\/$2\/public_html\//;
+    $Url=~ s{^https?\://[^/]+}{};
+    $Url=~ s{//+}{/}g;
+    $Url=~ s{^/}{};
+    $Url=$Apache::lonnet::perlvar{'lonDocRoot'}."/$Url";
     &Debug($r, "Returning $Url \n");
     return $Url;
 }
 
 sub url {
     my $fn=shift;
-    $fn=~s/^\/home\/($match_username)\/public\_html/\/priv\/$1/;
+    my $londocroot = $Apache::lonnet::perlvar{'lonDocRoot'};
+    $fn=~ s/^\Q$londocroot\E//;
+    $fn=~s{/\./}{/}g;
     $fn=&HTML::Entities::encode($fn,'<>"&');
     return $fn;
 }
 
 sub display {
     my $fn=shift;
-    $fn=~s-^/home/($match_username)/public_html-/priv/$1-;
+    my $londocroot = $Apache::lonnet::perlvar{'lonDocRoot'};
+    $fn=~s/^\Q$londocroot\E//;
+    $fn=~s{/\./}{/}g;
     return '<span class="LC_filename">'.$fn.'</span>';
 }
 
@@ -186,9 +190,9 @@ sub display {
 
 sub obsolete_unpub {
     my ($user,$domain,$construct)=@_;
+    my $londocroot = $Apache::lonnet::perlvar{'lonDocRoot'};
     my $published=$construct;
-    $published=~
-	s/^\/home\/$user\/public\_html\//\/home\/httpd\/html\/res\/$domain\/$user\//;
+    $published=~s{^\Q$londocroot/priv/\E}{$londocroot/res/};
     if (-e $published) {
 	if (&Apache::lonnet::metadata($published,'obsolete')) {
 	    return 1;
@@ -202,12 +206,13 @@ sub obsolete_unpub {
 # see if directory is empty
 # ignores any .meta, .save, .bak, and .log files created for a previously
 # published file, which has since been marked obsolete and deleted.
+# ignores a .DS_Store file put there when viewing directory via webDAV on MacOS. 
 sub empty_directory {
     my ($dirname,$phase) = @_;
     if (opendir DIR, $dirname) {
         my @files = grep(!/^\.\.?$/, readdir(DIR)); # ignore . and ..
         if (@files) { 
-            my @orphans = grep(/\.(meta|save|log|bak)$/,@files);
+            my @orphans = grep(/\.(meta|save|log|bak|DS_Store)$/,@files);
             if (scalar(@files) - scalar(@orphans) > 0) { 
                 return 0;
             } else {
@@ -230,7 +235,7 @@ sub empty_directory {
 
 =item exists($user, $domain, $file)
 
-   Determine if a resource file name has been published or exists
+   Determine if a resource filename has been published or exists
    in the construction space.
 
  Parameters:
@@ -269,9 +274,9 @@ sub exists {
     my ($user, $domain, $construct, $creating) = @_;
     $creating ||= 'file';
 
+    my $londocroot = $Apache::lonnet::perlvar{'lonDocRoot'};
     my $published=$construct;
-    $published=~
-	s{^/home/$user/public_html/}{/home/httpd/html/res/$domain/$user/};
+    $published=~s{^\Q$londocroot/priv/\E}{$londocroot/res/};
     my ($type,$result);
     if ( -d $construct ) {
 	return ('error','<p><span class="LC_error">'.&mt('Error: destination for operation is an existing directory.').'</span></p>');
@@ -345,7 +350,7 @@ sub checksuffix {
 }
 
 sub cleanDest {
-    my ($request,$dest,$subdir,$fn,$uname)=@_;
+    my ($request,$dest,$subdir,$fn,$uname,$udom)=@_;
     #remove bad characters
     my $foundbad=0;
     my $error='';
@@ -360,10 +365,10 @@ sub cleanDest {
     }
     if ($dest=~m|/|) {
 	my ($newpath)=($dest=~m|(.*)/|);
-	($newpath,$error)=&relativeDest($fn,$newpath,$uname);
+	($newpath,$error)=&relativeDest($fn,$newpath,$uname,$udom);
 	if (! -d "$newpath") {
 	    $request->print('<p><span class="LC_warning">'
-                           .&mt("You have requested to create file in directory [_1] which doesn't exist. The requested directory path has been removed from the requested file name."
+                           .&mt("You have requested to create file in directory [_1] which doesn't exist. The requested directory path has been removed from the requested filename."
                                ,&display($newpath))
                            .'</span></p>');
 	    $dest=~s|.*/||;
@@ -389,30 +394,25 @@ sub cleanDest {
 }
 
 sub relativeDest {
-    my ($fn,$newfilename,$uname)=@_;
+    my ($fn,$newfilename,$uname,$udom)=@_;
     my $error = '';
     if ($newfilename=~/^\//) {
 # absolute, simply add path
-	$newfilename='/home/'.$uname.'/public_html/';
+        my $londocroot = $Apache::lonnet::perlvar{'lonDocRoot'};
+	$newfilename="$londocroot/res/$udom/$uname/";
     } else {
 	my $dir=$fn;
-	$dir=~s/\/[^\/]+$//;
+	$dir=~s{/[^/]+$}{};
 	$newfilename=$dir.'/'.$newfilename;
     }
-    $newfilename=~s://+:/:g; # remove duplicate /
-    while ($newfilename=~m:/\.\./:) {
-	$newfilename=~ s:/[^/]+/\.\./:/:g; #remove dir/..
+    $newfilename=~s{//+}{/}g; # remove duplicate /
+    while ($newfilename=~m{/\.\./}) {
+	$newfilename=~ s{/[^/]+/\.\./}{/}g; #remove dir/..
     }
-    if ($newfilename =~ m{^/home/($match_username)/(?:public\_html|priv)/}) {
-        my $otheruname = $1;
-        unless ($otheruname eq $uname) {
-            my ($authorname,$authordom)=
-                &Apache::loncacc::constructaccess($newfilename,$env{'request.role.domain'});
-            unless (($authorname eq $otheruname) && ($authordom ne '')) {
-                my $otherdir = &display($newfilename);
-                $error = &mt('Access denied to [_1]',$otherdir);
-            }
-        }
+    my ($authorname,$authordom)=&Apache::lonnet::constructaccess($newfilename);
+    unless (($authorname) && ($authordom)) {
+       my $otherdir = &display($newfilename);
+       $error = &mt('Access denied to [_1]',$otherdir);
     }
     return ($newfilename,$error);
 }
@@ -437,9 +437,9 @@ Parameters:
 
 sub CloseForm1 {
     my ($request,  $fn) = @_;
-    $request->print('<p><input type="submit" value="'.&mt('Continue').'" /></p></form>');
-    $request->print('<form action="'.&url($fn).
-		    '" method="post"><p><input type="submit" value="'.&mt('Cancel').'" /></p></form>');
+    $request->print('<input type="submit" value="'.&mt('Continue').'" /></form>');
+    $request->print(' <form action="'.&url($fn).'" method="post">'.
+                    '<input type="submit" value="'.&mt('Cancel').'" /></form>');
 }
 
 
@@ -700,9 +700,20 @@ sub Copy1 {
 	my ($type,$return)=&exists($user, $domain, $newfilename);
 	$request->print($return);
 	if ($type eq 'error') {
-	    $request->print('<br /><a href="'.&url($fn).'">'.&mt('Cancel').'</a>');
+	    $request->print('<br /><a href="'.&url($fn).'">'.&mt('Cancel').'</a></form>');
 	    return;
 	}
+# Check if there is enough space.
+        my @fileinfo = stat($fn);
+        my ($dir,$fname) = ($fn =~ m{^(.+/)([^/]+)$});
+        my $filesize = $fileinfo[7];
+        $filesize = int($filesize/1000); #expressed in kb
+        my $output = &Apache::loncommon::excess_filesize_warning($user,$domain,'author',
+                                                                 $fname,$filesize,'copy');
+        if ($output) {
+            $request->print($output.'<br /><a href="'.&url($dir).'">'.&mt('Cancel').'</a></form>');
+            return;
+        }
     $request->print(
         '<input type="hidden" name="newfilename"'
        .' value="'.$newfilename.'" />'
@@ -825,7 +836,7 @@ Parameters:
 
 =item   $domain   - Name of the domain of the user
 
-=item   $fn      - Source file name
+=item   $fn      - Source filename
 
 =item   $newfilename
                   - Name of the file to be created; no path information
@@ -838,7 +849,7 @@ Side Effects:
 =item 2 new forms are displayed.  Clicking on the confirmation button
 causes the browser to attempt to load the specfied URL, allowing the
 proper handler to take care of file creation. There is also a Cancel
-button which returns you to the driectory listing you came from
+button which returns you to the directory listing you came from
 
 =back
 
@@ -880,7 +891,7 @@ sub NewFile1 {
 		'<form name="fileaction" action="/adm/cfile" method="post">'.
                 '<input type="hidden" name="qualifiedfilename" value="'.$fn.'" />'.
 		'<input type="hidden" name="action" value="newfile" />'.
-	        '<span class ="LC_nobreak">'.&mt('Enter a file name: ').'<input type="text" name="newfilename" value="Type Name Here" onfocus="if (this.value == '."'Type Name Here') this.value=''".'" />&nbsp;<input type="submit" value="Go" />'.
+	        '<span class ="LC_nobreak">'.&mt('Enter a filename: ').'<input type="text" name="newfilename" value="Type Name Here" onfocus="if (this.value == '."'Type Name Here') this.value=''".'" />&nbsp;<input type="submit" value="Go" />'.
                 '</span></form></p>'.
                 '<p><form action="'.&url($fn).
                 '" method="post"><p><input type="submit" value="'.&mt('Cancel').'" /></form></p>');
@@ -950,9 +961,9 @@ sub phaseone {
     my $doingdir=0;
     if ($env{'form.action'} eq 'newdir') { $doingdir=1; }
     my ($newfilename,$error) = 
-        &cleanDest($r,$env{'form.newfilename'},$doingdir,$fn,$uname);
+        &cleanDest($r,$env{'form.newfilename'},$doingdir,$fn,$uname,$udom);
     unless ($error) {
-        ($newfilename,$error)=&relativeDest($fn,$newfilename,$uname);
+        ($newfilename,$error)=&relativeDest($fn,$newfilename,$uname,$udom);
     }
     if ($error) {
         my $dirlist;
@@ -962,8 +973,8 @@ sub phaseone {
             $dirlist=$fn; 
         }
         $r->print('<div class="LC_error">'.$error.'</div>'.
-                  '<h3><a href="'.&url($dirlist).'">'.&mt('Return to Directory').
-                  '</a></h3>');
+                  '<p><a href="'.&url($dirlist).'">'.&mt('Return to Directory').
+                  '</a></p>');
         return;
     }
     $r->print('<form action="/adm/cfile" method="post">'.
@@ -1099,7 +1110,7 @@ sub Rename2 {
 	}
     } else {
         $request->print(
-            '<p>'
+            '<p class="LC_error">'
            .&mt('No such file: [_1]',
                 &display($oldfile))
            .'</p></form>'
@@ -1387,11 +1398,19 @@ sub phasetwo {
 	$dest = $newdir."/";
     }
     if ( ($env{'form.action'} eq 'newdir') && ($env{'form.phase'} eq 'two') && ( ($env{'form.callingmode'} eq 'testbank') || ($env{'form.callingmode'} eq 'imsimport') ) ) {
-	$r->print('<h3><a href="javascript:self.close()">'.&mt('Done').'</a></h3>');
+        $r->print(
+            '<p>'
+           .&Apache::lonhtmlcommon::confirm_success(&mt('Done'))
+           .'<br /><a href="javascript:self.close()">'.&mt('Continue').'</a>'
+           .'</p>'
+        );
     } else {
         if ($env{'form.action'} eq 'rename') {
-            $r->print('<h3><a href="'.&url($dest).'">'.&mt('Return to Directory').'</a></h3>');
-            $r->print('<h3><a href="'.&url($dest_newname).'">'.$disp_newname.'</a></h3>');
+            $r->print(
+                 '<p>'.&Apache::lonhtmlcommon::confirm_success(&mt('Done')).'</p>'
+                .&Apache::lonhtmlcommon::actionbox(
+                     ['<a href="'.&url($dest).'">'.&mt('Return to Directory').'</a>',
+                      '<a href="'.&url($dest_newname).'">'.$disp_newname.'</a>']));
         } else {
 	    $r->print(&done(&url($dest)));
 	}
@@ -1441,14 +1460,11 @@ sub handler {
     } 
 
 # ----------------------------------------------------------- Start page output
-    my $uname;
-    my $udom;
 
-    ($uname,$udom)=
-	&Apache::loncacc::constructaccess($fn,$r->dir_config('lonDefDomain'));
+    my ($uname,$udom) = &Apache::lonnet::constructaccess($fn);
     &Debug($r, 
 	   "loncfile::handler constructaccess uname = $uname domain = $udom");
-    unless (($uname) && ($udom)) {
+    if (($uname eq '') || ($udom eq '')) {
 	$r->log_reason($uname.' at '.$udom.
 		       ' trying to manipulate file '.$env{'form.filename'}.
 		       ' ('.$fn.') - not authorized', 
@@ -1476,27 +1492,31 @@ function writeDone() {
 	$loaditem{'onload'} = "writeDone()";
     }
 
+    my $londocroot = $r->dir_config('lonDocRoot');
+    my $trailfile = $fn;
+    $trailfile =~ s{^/(priv/)}{$londocroot/$1};
+    
     # Breadcrumbs
     &Apache::lonhtmlcommon::clear_breadcrumbs();
     &Apache::lonhtmlcommon::add_breadcrumb({
-        'text'  => 'Construction Space',
-        'href'  => &Apache::loncommon::authorspace(),
+        'text'  => 'Authoring Space',
+        'href'  => &Apache::loncommon::authorspace($fn),
     });
     &Apache::lonhtmlcommon::add_breadcrumb({
         'text'  => 'File Operation',
-        'title' => 'Construction Space File Operation',
+        'title' => 'Authoring Space File Operation',
         'href'  => '',
     });
 
-    $r->print(&Apache::loncommon::start_page('Construction Space File Operation',
+    $r->print(&Apache::loncommon::start_page('Authoring Space File Operation',
 					     $js,
 					     {'add_entries' => \%loaditem,})
              .&Apache::lonhtmlcommon::breadcrumbs()
              .&Apache::loncommon::head_subbox(
-                  &Apache::loncommon::CSTR_pageheader($fn))
+                  &Apache::loncommon::CSTR_pageheader($trailfile))
     );
   
-    $r->print('<h3>'.&mt('Location').': '.&display($fn).'</h3>');
+    $r->print('<p>'.&mt('Location').': '.&display($fn).'</p>');
   
     if (($uname ne $env{'user.name'}) || ($udom ne $env{'user.domain'})) {
         $r->print('<p class="LC_info">'
@@ -1507,37 +1527,35 @@ function writeDone() {
 
 
     &Debug($r, "loncfile::handler Form action is $env{'form.action'} ");
-    if ($env{'form.action'} eq 'delete') {
-      	$r->print('<h3>'.&mt('Delete').'</h3>');
-    } elsif ($env{'form.action'} eq 'rename') {
-	$r->print('<h3>'.&mt('Rename').'</h3>');
-    } elsif ($env{'form.action'} eq 'move') {
-	$r->print('<h3>'.&mt('Move').'</h3>');
-    } elsif ($env{'form.action'} eq 'newdir') {
-	$r->print('<h3>'.&mt('New Directory').'</h3>');
-    } elsif ($env{'form.action'} eq 'decompress') {
-	$r->print('<h3>'.&mt('Decompress').'</h3>');
-    } elsif ($env{'form.action'} eq 'copy') {
-	$r->print('<h3>'.&mt('Copy').'</h3>');
-    } elsif ($env{'form.action'} eq 'newfile' ||
-	     $env{'form.action'} eq 'newhtmlfile' ||
-	     $env{'form.action'} eq 'newproblemfile' ||
-	     $env{'form.action'} eq 'newpagefile' ||
-	     $env{'form.action'} eq 'newsequencefile' ||
-	     $env{'form.action'} eq 'newrightsfile' ||
-	     $env{'form.action'} eq 'newstyfile' ||
-	     $env{'form.action'} eq 'newtaskfile' ||
-             $env{'form.action'} eq 'newlibraryfile' ||
-	     $env{'form.action'} eq 'Select Action' ) {
-	$r->print('<h3>'.&mt('New Resource').'</h3>');
+    my %action = &Apache::lonlocal::texthash(
+        'delete'          => 'Delete',
+        'rename'          => 'Rename',
+        'move'            => 'Move',
+        'newdir'          => 'New Directory',
+        'decompress'      => 'Decompress',
+        'copy'            => 'Copy',
+        'newfile'         => 'New Resource',
+	'newhtmlfile'     => 'New Resource',
+	'newproblemfile'  => 'New Resource',
+	'newpagefile'     => 'New Resource',
+	'newsequencefile' => 'New Resource',
+	'newrightsfile'   => 'New Resource',
+	'newstyfile'      => 'New Resource',
+	'newtaskfile'     => 'New Resource',
+        'newlibraryfile'  => 'New Resource',
+	'Select Action'   => 'New Resource',
+    );
+    if ($action{$env{'form.action'}}) {
+        $r->print('<h2>'.$action{$env{'form.action'}}.'</h2>');
     } else {
         $r->print('<p class="LC_error">'
-                 .&mt('Unknown Action').' '.$env{'form.action'}
+                 .&mt('Unknown Action: [_1]',$env{'form.action'})
                  .'</p>'
                  .&Apache::loncommon::end_page()
         );
-	return OK;  
+        return OK;
     }
+
     if ($env{'form.phase'} eq 'two') {
 	&Debug($r, "loncfile::handler  entering phase2");
 	&phasetwo($r,$fn,$uname,$udom);
