@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
-# Construction Space Directory Lister
+# Authoring Space Directory Lister
 #
-# $Id: lonpubdir.pm,v 1.125.4.1 2011/11/07 13:38:45 raeburn Exp $
+# $Id: lonpubdir.pm,v 1.158 2014/06/29 03:33:23 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -33,7 +33,6 @@ use strict;
 use Apache::File;
 use File::Copy;
 use Apache::Constants qw(:common :http :methods);
-use Apache::loncacc;
 use Apache::loncommon();
 use Apache::lonhtmlcommon();
 use Apache::londiff();
@@ -41,146 +40,242 @@ use Apache::lonlocal;
 use Apache::lonmsg;
 use Apache::lonmenu;
 use Apache::lonnet;
-use LONCAPA;
+use LONCAPA qw(:DEFAULT :match);
 
 sub handler {
 
-  my $r=shift;
+    my $r=shift;
 
-  my $fn;
+    # Validate access to the construction space and get username:domain.
 
-
-
-  $fn = getEffectiveUrl($r);
-
-  # Validate access to the construction space and get username@domain.
-
-  my $uname;
-  my $udom;
-
-  ($uname,$udom)=
-    &Apache::loncacc::constructaccess(
-             $fn,$r->dir_config('lonDefDomain')); 
-  unless (($uname) && ($udom)) {
-     $r->log_reason($uname.':'.$udom.
-         ' trying to list directory '.$env{'form.filename'}.
-         ' ('.$fn.') - not authorized', 
-         $r->filename); 
-     return HTTP_NOT_ACCEPTABLE;
-  }
-
-  # Remove trailing / from directory name.
-
-  $fn=~s/\/$//;
-
-  unless ($fn) { 
-     $r->log_reason($env{'user.name'}.':'.$env{'user.domain'}.
-         ' trying to list empty directory', $r->filename); 
-     return HTTP_NOT_FOUND;
-  } 
+    my ($uname,$udom)=&Apache::lonnet::constructaccess($r->uri); 
+    unless (($uname) && ($udom)) {
+        return HTTP_NOT_ACCEPTABLE;
+    }
 
 # ----------------------------------------------------------- Start page output
 
-  my $thisdisfn=$fn;
-  $thisdisfn=~s/^\/home\/$uname\/public_html//;	# subdirectory part of
-                                                # construction space. 
-  my $docroot=$r->dir_config('lonDocRoot');     # Apache  londocument root.
+    my $fn=$r->filename;
+    $fn=~s/\/$//;
+    my $thisdisfn=$fn;
 
-  my $resdir=$docroot.'/res/'.$udom.'/'.$uname.$thisdisfn; # Resource directory
-  my $targetdir=$udom.'/'.$uname.$thisdisfn; # Publiction target directory.
-  my $linkdir='/priv/'.$uname.$thisdisfn;      # Full URL name of constr space.
+    my $docroot=$r->dir_config('lonDocRoot');     # Apache  londocument root.
+    if ($thisdisfn eq "$docroot/priv/$udom") {
+        if ((-d "/home/$uname/public_html/") && (!-e "$docroot/priv/$udom/$uname")) {
+            my ($version) = ($r->dir_config('lonVersion') =~ /^\'?(\d+\.\d+)\./);
+            &Apache::loncommon::content_type($r,'text/html');
+            $r->send_http_header;
 
-  my %bombs=&Apache::lonmsg::all_url_author_res_msg($uname,$udom);
-
-  &startpage($r, $uname, $udom, $thisdisfn);   # Put out the start of page.
-  if ($env{'environment.remote'} eq 'off') {
-      &dircontrols($r,$uname,$udom,$thisdisfn);    # Put out actions for directory, 
-                                               # browse/upload + new file page.
-  } else {
-      &pubbuttons($r,$uname,$thisdisfn);
-  }
-  &resourceactions($r,$uname,$udom,$thisdisfn); #Put out form used for printing/deletion etc.
-
-  my $numdir = 0;
-  my $numres = 0;
-  
-  # Start off the directory table.
-  $r->print(&Apache::loncommon::start_data_table()
-           .&Apache::loncommon::start_data_table_header_row()
-           .'<th>'.&mt('Type').'</th>'
-           .'<th>'.&mt('Actions').'</th>'
-           .'<th>'.&mt('Name').'</th>'
-           .'<th>'.&mt('Title').'</th>'
-           .'<th colspan="2">'.&mt('Status').'</th>'
-           .'<th>'.&mt('Last Modified').'</th>'
-           .&Apache::loncommon::end_data_table_header_row()
-  );
-
-  my $filename;
-  my $dirptr=16384;		# Mask indicating a directory in stat.cmode.
-
-  opendir(DIR,$fn);
-  my @files=sort {uc($a) cmp uc($b)} (readdir(DIR));
-  foreach my $filename (@files) {
-     my ($cdev,$cino,$cmode,$cnlink,
-         $cuid,$cgid,$crdev,$csize,
-         $catime,$cmtime,$cctime,
-         $cblksize,$cblocks)=stat($fn.'/'.$filename);
-
-     my $extension='';
-     if ($filename=~/\.(\w+)$/) { $extension=$1; }
-     if ($cmode&$dirptr) {
-	 putdirectory($r, $thisdisfn, $linkdir, $filename, $cmtime,$targetdir,\%bombs,\$numdir);
-     } elsif (&Apache::loncommon::fileembstyle($extension) ne 'hdn') {
-	 putresource($r, $udom, $uname, $filename, $thisdisfn, $resdir, 
-		     $targetdir, $linkdir, $cmtime,\%bombs,\$numres);
-     } else {
-	# "hidden" extension and not a directory, so hide it away.
-     }
-  }
-  closedir(DIR);
-
-  $r->print(&Apache::loncommon::end_data_table()
-           .&Apache::loncommon::end_page()
-  );
-  return OK;  
-}
-#
-#  Gets the effective URL of the request and returns it:
-#    $effn = getEffectiveUrl($r);
-#       $r  - The Apache Request object.
-sub getEffectiveUrl {
-    my $r = shift;
-    my $fn;
-    
-    if ($env{'form.filename'}) {	# If a form filename is defined.
-	$fn=$env{'form.filename'};
-	#
-	#   Replace the ~username of the URL with /home/username/public_html
-	#   so that we don't have to worry about ~ expansion internally.
-	#
-	$fn=~s/^https?\:\/\/[^\/]+\///;
-        $fn=~s/^\///;
-        $fn=~s{~($LONCAPA::username_re)}{/home/$1/public_html};
-	
-	#  Remove trailing / strings (?) 
-	
-	$fn=~s/\/[^\/]+$//;
-    } else {
-	#   If no form is defined, use request filename.
-	$fn = $r->filename();
-	my $lonDocRoot=$r->dir_config('lonDocRoot');
-	if ( $fn =~ /$lonDocRoot/ ) {
-	    #internal authentication, needs fixup.
-	    $fn = $r->uri(); # non users do not get the full path request
-                             # through SCRIPT_FILENAME
-	    $fn=~s{^/~($LONCAPA::username_re)}{/home/$1/public_html};
-	}
+            &Apache::lonhtmlcommon::clear_breadcrumbs();
+            $r->print(&Apache::loncommon::start_page('Authoring Space').
+                      '<div class="LC_error">'.
+                      '<br /><p>'.
+                      &mt('Your Authoring Space is currently in the location used by LON-CAPA version 2.10 and older, but your domain is using a newer LON-CAPA version ([_1]).',$version).'</p>'.
+                      '<p>'.
+                      &mt('Please ask your Domain Coordinator to move your Authoring Space to the new location.').
+                      '</p>'.
+                      '</div>'.
+                      &Apache::loncommon::end_page());
+            return OK;
+        }
     }
-    $fn=~s/\/+/\//g;
-    return $fn;
+    $thisdisfn=~s/^\Q$docroot\E\/priv//;
+    
+    my $resdir=$docroot.'/res'.$thisdisfn; # Resource directory
+    my $targetdir='/res'.$thisdisfn; # Publication target directory.
+    my $linkdir='/priv'.$thisdisfn;      # Full URL name of constr space.
+
+    my %bombs=&Apache::lonmsg::all_url_author_res_msg($uname,$udom);
+
+    &startpage($r, $uname, $udom, $thisdisfn);  # Put out the start of page.
+
+    if (!-d $fn) {
+        if (-e $fn) {
+            $r->print('<p class="LC_info">'.&mt('Requested item is a file not a directory.').'</p>');
+        } else {
+            $r->print('<p class="LC_info">'.&mt('The requested subdirectory does not exist.').'</p>');
+        }
+        $r->print(&Apache::loncommon::end_page());
+        return OK;
+    }
+    my @files;
+    if (opendir(DIR,$fn)) {
+        @files = grep(!/^\.+$/,readdir(DIR));
+        closedir(DIR);
+    } else {
+        $r->print('<p class="LC_error">'.&mt('Could not open directory.').'</p>');
+        $r->print(&Apache::loncommon::end_page());
+        return OK;
+    }
+
+    &dircontrols($r,$uname,$udom,$thisdisfn);   # Put out actions for directory, 
+                                                # browse/upload + new file page.
+    &resourceactions($r,$uname,$udom,$thisdisfn); # Put out form used for printing/deletion etc.
+
+    my $numdir = 0;
+    my $numres = 0;
+  
+    # Retrieving value for "sortby" and "sortorder" from QUERY_STRING
+    &Apache::loncommon::get_unprocessed_cgi($ENV{'QUERY_STRING'},
+        ['sortby','sortorder']);
+
+    # Sort by name as default, not reversed
+    if (! exists($env{'form.sortby'})) { $env{'form.sortby'} = 'filename' }
+    if (! exists($env{'form.sortorder'})) { $env{'form.sortorder'} = '' }
+
+    my $sortby = $env{'form.sortby'};
+    my $sortorder = $env{'form.sortorder'};
+
+    if ((@files == 0) && ($thisdisfn =~ m{^/$match_domain/$match_username})) {
+        if ($thisdisfn =~ m{^/$match_domain/$match_username$}) {
+            $r->print('<p class="LC_info">'.&mt('This Authoring Space is currently empty.').'</p>');
+        } else {
+            $r->print('<p class="LC_info">'.&mt('This subdirectory is currently empty.').'</p>');
+        }
+        $r->print(&Apache::loncommon::end_page());
+        return OK;
+    }
+
+    # Start off the directory table.
+    $r->print(&Apache::loncommon::start_data_table()
+        .&Apache::loncommon::start_data_table_header_row()
+        .'<th><a href="'.$linkdir.'/?sortby=filetype&sortorder='
+            .((($sortby eq "filetype") && ($sortorder ne 'rev')) ? 'rev' : '') 
+            .'">'.&mt('Type')
+            .'<span class="LC_fontsize_small"> &#9660;</span></a></th>'
+        .'<th>'.&mt('Actions').'</th>'
+        .'<th><a href="'.$linkdir.'/?sortby=filename&sortorder='
+            .((($sortby eq "filename") && ($sortorder ne 'rev')) ? 'rev' : '') 
+            .'">'.&mt('Name')
+            .'<span class="LC_fontsize_small"> &#9660;</span></a></th>'
+        .'<th><a href="'.$linkdir.'/?sortby=title&sortorder='
+            .((($sortby eq "title") && ($sortorder ne 'rev')) ? 'rev' : '') 
+            .'">'.&mt('Title')
+            .'<span class="LC_fontsize_small"> &#9660;</span></a></th>'
+        .'<th colspan="2"><a href="'.$linkdir.'/?sortby=pubstatus&sortorder='
+            .((($sortby eq "pubstatus") && ($sortorder ne 'rev')) ? 'rev' : '') 
+            .'">'.&mt('Status')
+            .'<span class="LC_fontsize_small"> &#9660;</span></a></th>'
+        .'<th><a href="'.$linkdir.'/?sortby=cmtime&sortorder='
+            .((($sortby eq "cmtime") && ($sortorder ne 'rev')) ? 'rev' : '') 
+            .'">'.&mt('Last Modified')
+            .'<span class="LC_fontsize_small"> &#9660;</span></a></th>'
+        .'<th><a href="'.$linkdir.'/?sortby=size&sortorder='
+            .((($sortby eq "size") && ($sortorder ne 'rev')) ? 'rev' : '') 
+            .'">'.&mt('Size').' (kB)'
+            .'<span class="LC_fontsize_small"> &#9660;</span></a></th>'
+        .&Apache::loncommon::end_data_table_header_row()
+    );
+
+    my $dirptr=16384;		# Mask indicating a directory in stat.cmode.
+    my $filehash = {};
+    foreach my $filename (@files) {
+        # Skip .DS_Store and hidden files
+        my ($extension) = ($filename=~/\.(\w+)$/);
+        next if (($filename eq '.DS_Store')
+                || (&Apache::loncommon::fileembstyle($extension) eq 'hdn')
+                || ($filename =~ /^\._/));
+
+        my ($cmode,$csize,$cmtime)=(stat($fn.'/'.$filename))[2,7,9];
+        my $linkfilename = &HTML::Entities::encode('/priv'.$thisdisfn.'/'.$filename,'<>&"');
+        # Identify type of file according to icon used
+        my ($filetype) = (&Apache::loncommon::icon($filename) =~ m{/(\w+).gif$}); 
+        my $cstr_dir = $r->dir_config('lonDocRoot').'/priv'.$thisdisfn;
+        my $meta_same = &isMetaSame($cstr_dir, $resdir, $filename);
+        
+        # Store size, title, and status for files but not directories
+        my $size = (!($cmode&$dirptr)) ? $csize/1024. : 0;
+        my ($status, $pubstatus, $title, $fulltitle);
+        if (!($cmode&$dirptr)) {
+            ($status, $pubstatus) = &getStatus($resdir, $targetdir, $cstr_dir, 
+                $filename, $linkfilename, $cmtime, $meta_same);
+            ($fulltitle, $title) = &getTitle($resdir, $targetdir, $filename, 
+                                        $linkfilename, $meta_same, \%bombs);
+        } else {
+            ($status, $pubstatus) = ('','');
+            ($fulltitle, $title) = ('','');
+        }
+
+        # This hash will allow sorting
+        $filehash->{ $filename } = {
+            "cmtime"            => $cmtime,
+            "size"              => $size,
+            "cmode"             => $cmode,
+            "filetype"          => $filetype,
+            "title"             => $title,
+            "fulltitle"         => $fulltitle,
+            "status"            => $status,
+            "pubstatus"         => $pubstatus,
+            "linkfilename"      => $linkfilename,
+        }
+    }
+   
+    my @sorted_files;
+    # Sorting by something other than "Name".  Name is the secondary key.
+    if ($sortby =~ m{cmtime|size}) {    # Numeric fields
+        # First check if order should be reversed
+        if ($sortorder eq "rev") {
+            @sorted_files = sort {
+                $filehash->{$a}->{$sortby} <=> $filehash->{$b}->{$sortby}
+                    or
+                uc($a) cmp uc($b)
+            } (keys(%{$filehash}));
+        } else {
+            @sorted_files = sort {
+                $filehash->{$b}->{$sortby} <=> $filehash->{$a}->{$sortby}
+                    or
+                uc($a) cmp uc($b)
+            } (keys(%{$filehash}));
+        }
+    } elsif ($sortby =~ m{filetype|title|status}) {     # String fields
+        if ($sortorder eq "rev") {
+            @sorted_files = sort {
+                $filehash->{$b}->{$sortby} cmp $filehash->{$a}->{$sortby}
+                    or
+                uc($a) cmp uc($b)
+            } (keys(%{$filehash}));
+        } else {
+            @sorted_files = sort {
+                $filehash->{$a}->{$sortby} cmp $filehash->{$b}->{$sortby}
+                    or
+                uc($a) cmp uc($b)
+            } (keys(%{$filehash}));
+        }
+
+    # Sort by "Name" is the default
+    } else { 
+        if ($sortorder eq "rev") {
+            @sorted_files = sort {uc($b) cmp uc($a)} (keys(%{$filehash}));
+        } else {
+            @sorted_files = sort {uc($a) cmp uc($b)} (keys(%{$filehash}));
+        }
+    }
+
+    # Print the sorted resources
+    foreach my $filename (@sorted_files) {
+        if ($filehash->{$filename}->{"cmode"}&$dirptr) {        # Directories
+            &putdirectory($r, $thisdisfn, $linkdir, $filename, 
+                $filehash->{$filename}->{"cmtime"}, 
+                $targetdir, \%bombs, \$numdir);
+        } else {                                                # Files
+            &putresource($r, $udom, $uname, $filename, $thisdisfn, $resdir,
+                $targetdir, $linkdir, $filehash->{$filename}->{"cmtime"}, 
+                $filehash->{$filename}->{"size"}, \$numres, 
+                $filehash->{$filename}->{"linkfilename"},
+                $filehash->{$filename}->{"fulltitle"},
+                $filehash->{$filename}->{"status"},
+                $filehash->{$filename}->{"pubstatus"});
+        }
+    }
+
+    $r->print( &Apache::loncommon::end_data_table()
+        .&Apache::loncommon::end_page() );
+
+    return OK;
 }
-#
+
+
+
 #   Output the header of the page.  This includes:
 #   - The HTML header 
 #   - The H1/H3  stuff which includes the directory.
@@ -193,42 +288,44 @@ sub getEffectiveUrl {
 
 sub startpage {
     my ($r, $uname, $udom, $thisdisfn) = @_;
-    my $currdir = '/priv/'.$uname.$thisdisfn;
     &Apache::loncommon::content_type($r,'text/html');
     $r->send_http_header;
 
-    my $formaction='/priv/'.$uname.$thisdisfn.'/';
+    my $formaction='/priv'.$thisdisfn.'/';
     $formaction=~s|/+|/|g;
     &Apache::lonhtmlcommon::store_recent('construct',$formaction,$formaction);
 
     &Apache::lonhtmlcommon::clear_breadcrumbs();
     &Apache::lonhtmlcommon::add_breadcrumb({
-        'text'  => 'Construction Space',
-        'href'  => &Apache::loncommon::authorspace(),
+        'text'  => 'Authoring Space',
+        'href'  => &Apache::loncommon::authorspace($formaction),
     });
     # breadcrumbs (and tools) will be created 
     # in start_page->bodytag->innerregister
 
-    if ($env{'environment.remote'} eq 'off') {
-        $env{'request.noversionuri'}=$currdir.'/';
-        $r->print(&Apache::loncommon::start_page('Construction Space',undef));
-    } else {
-        $r->print(&Apache::loncommon::start_page('Construction Space',undef,
-                                                 { 'only_body' => 1,}));
-    }
+    $env{'request.noversionuri'}=$formaction;
+    $r->print(&Apache::loncommon::start_page('Authoring Space'));
+
+    my $londocroot = $Apache::lonnet::perlvar{'lonDocRoot'};
+    my $current_disk_usage = &Apache::lonnet::diskusage($udom,$uname,"$londocroot/priv/$udom/$uname");
+    my $disk_quota = &Apache::loncommon::get_user_quota($uname,$udom,'author'); #expressed in MB
+    $disk_quota = 1000 * $disk_quota; # convert from MB to kB
 
     $r->print(&Apache::loncommon::head_subbox(
-                &Apache::loncommon::CSTR_pageheader()));
+                     '<div style="float:right;padding-top:0;margin-top;0">'
+                    .&Apache::lonhtmlcommon::display_usage($current_disk_usage,$disk_quota)
+                    .'</div>'
+                    .&Apache::loncommon::CSTR_pageheader()));
 
     my $esc_thisdisfn = &Apache::loncommon::escape_single($thisdisfn);
-    my $doctitle = 'LON-CAPA '.&mt('Construction Space');
+    my $doctitle = 'LON-CAPA '.&mt('Authoring Space');
     my $newname = &mt('New Name');
     my $pubdirscript=(<<ENDPUBDIRSCRIPT);
 <script type="text/javascript">
 top.document.title = '$esc_thisdisfn/ - $doctitle';
 // Store directory location for menu bar to find
 
-parent.lastknownpriv='/~$uname$esc_thisdisfn/';
+parent.lastknownpriv='/priv$esc_thisdisfn/';
 
 // Confirmation dialogues
 
@@ -331,11 +428,6 @@ parent.lastknownpriv='/~$uname$esc_thisdisfn/';
 </script>
 ENDPUBDIRSCRIPT
     $r->print($pubdirscript);
-
-    if ((($uname ne $env{'user.name'}) || ($udom ne $env{'user.domain'})) &&
-	$env{'environment.remote'} ne 'off') {
-        $r->print('<h3>'.&mt('Co-Author [_1]',$uname.':'.$udom).'</h3>');
-    }
 }
 
 sub dircontrols {
@@ -388,7 +480,7 @@ sub dircontrols {
             <option value="printdir">$lt{'prnt'}</option>
             <option value="delete">$lt{'dedr'}</option>
         </select>
-        <input type="hidden" name="filename" value="/~$uname$thisdisfn/" />
+        <input type="hidden" name="filename" value="/priv$thisdisfn/" />
       </fieldset>
     </form>
     <form name="publishdir" method="post" action="/adm/publish" target="_parent">
@@ -404,7 +496,7 @@ sub dircontrols {
     <form name="upublisher" enctype="multipart/form-data" method="post" action="/adm/upload" target="_parent">
       <fieldset>
         <legend>$lt{'updc'}</legend>
-        <input type="hidden" name="filename" value="/~$uname$thisdisfn/" />
+        <input type="hidden" name="filename" value="/priv$thisdisfn/" />
         <input type="file" name="upfile" size="20" />
         <input type="button" value="$lt{'uplo'}"  onclick="checkUpload(this.form)" />
       </fieldset>
@@ -416,7 +508,7 @@ sub dircontrols {
       <fieldset>
               <legend>$lt{'crea'}</legend>
 	      <span class="LC_nobreak">
-		<input type="hidden" name="filename" value="/~$uname$thisdisfn/" />
+		<input type="hidden" name="filename" value="/priv$thisdisfn/" />
                   <script type="text/javascript">
                     function validate_go() {
                         var selected = document.fileaction.action.selectedIndex;
@@ -448,17 +540,6 @@ sub dircontrols {
 END
 }
 
-sub pubbuttons {
-    my ($r,$uname,$thisdisfn) = @_;
-    $r->print('<form method="post" action="/adm/publish" target="_parent">'.
-              '<table><tr><td><input type="hidden" name="filename" value="/~'.
-               $uname.$thisdisfn.'/" />'.
-              '<input type="submit" value="'.&mt('Publish Directory').'" /></td><td>'.
-'<input type="button" onclick="window.location='."'/~".
-               $uname.$thisdisfn."/default.meta'".'" value="'.
-&mt('Edit Directory Metadata').'" /></td></tr></table></form>');
-}
-
 sub resourceactions {
     my ($r,$uname,$udom,$thisdisfn) = @_;
     $r->print(<<END);
@@ -517,38 +598,45 @@ sub getSourceRightString {
 }
 #
 #  Put out a directory table row:
-#    putdirectory(r, base, here, dirname, modtime)
-#      r       - Apache request object.
-#      reqfile - File in request.
-#      here    - Where we are in directory tree.
-#      dirname - Name of directory special file.
-#      modtime - Encoded modification time.
-# 
+#    putdirectory(r, base, here, dirname, modtime, targetdir, bombs, numdir)
+#      r         - Apache request object.
+#      reqfile   - File in request.
+#      here      - Where we are in directory tree.
+#      dirname   - Name of directory special file.
+#      modtime   - Encoded modification time.
+#      targetdir - Publication target directory.
+#      bombs     - Reference to hash of URLs with runtime error messages.
+#      numdir    - Reference to scalar used to track number of sub-directories
+#                  in directory (used in form name for each "actions" dropdown).
+#
 sub putdirectory {
-    my ($r, $reqfile, $here, $dirname, $modtime, $resdir, $bombs, $numdir) = @_;
-    # construct the display filename: the directory name unless ..:
-    
+    my ($r, $reqfile, $here, $dirname, $modtime, $targetdir, $bombs, $numdir) = @_;
+
+# construct the display filename: the directory name unless ..:
+   
+    my $actionitem;
+ 
     my $disfilename = $dirname;
-    if ($dirname eq '..') {
-	$disfilename = '<i>'.&mt('Parent Directory').'</i>';
-    }
-    unless ( (($dirname eq '..') && ($reqfile eq '')) || ($dirname eq '.')) {
-	my $kaputt=0;
-	foreach (keys %{$bombs}) {
-	    if ($_=~m:^\Q$resdir\E/\Q$disfilename\E/:) { $kaputt=1; last; }
-	}
+# Don't display directory itself, and there is no way up from root directory
+    unless ((($dirname eq '..') && ($reqfile=~/^\/[^\/]+\/[^\/]+$/)) || ($dirname eq '.')) {
+        my $kaputt=0;
+        if (ref($bombs) eq 'HASH') {
+            foreach my $key (keys(%{$bombs})) {
+                my $currentdir = &Apache::lonnet::declutter("$targetdir/$disfilename");
+                if (($key) =~ m{^\Q$currentdir\E/}) { $kaputt=1; last; }
+            }
+        }
+#
+# Get the metadata from that directory's default.meta to display titles
+#
 	%Apache::lonpublisher::metadatafields=();
 	%Apache::lonpublisher::metadatakeys=();
-	my $construct=$here;
-	$construct=~s{^/priv/($LONCAPA::username_re)$}{/home/$1/public_html};
-        my $dirpath = $here;
-        $dirpath=~s{^/priv/}{/~};
-	&Apache::lonpublisher::metaeval(&Apache::lonnet::getfile(
-       				 $construct.'/'.$dirname.'/default.meta'
-								 ));
-        my $actionitem = '';
+	&Apache::lonpublisher::metaeval(
+                 &Apache::lonnet::getfile($r->dir_config('lonDocRoot').$here.'/'.$dirname.'/default.meta')
+                                       );
         if ($dirname eq '..') {
             $actionitem = &mt('Go to ...');
+            $disfilename = '<i>'.&mt('Parent Directory').'</i>';
         } else {
             $actionitem = 
                     '<form name="dirselect_'.$$numdir.
@@ -561,7 +649,7 @@ sub putdirectory {
                       '<option value="printdir">'.&mt('Print directory').'</option>'.
                       '<option value="delete">'.&mt('Delete directory').'</option>'.
                     '</select>'.
-                     '<input type="hidden" name="filename" value="'.&HTML::Entities::encode($dirpath.'/'.$dirname,'<>&"').'/" />'.
+                     '<input type="hidden" name="filename" value="'.&HTML::Entities::encode($here.'/'.$dirname,'<>&"').'/" />'.
                      '<input type="hidden" name="openname" value="'.$here.'/'.$dirname.'/" />'.
                      '<input type="hidden" name="postdata" value="" />'.
                    '</form>';
@@ -573,7 +661,7 @@ sub putdirectory {
 		  '<td>'.$actionitem.'</td>'.
 		  '<td><span class="LC_filename"><a href="'.&HTML::Entities::encode($here.'/'.$dirname,'<>&"').'/" target="_parent">'.
 		  $disfilename.'</a></span></td>'.
-		        '<td colspan="3">'.($kaputt?&Apache::lonhtmlcommon::authorbombs($resdir.'/'.$disfilename.'/'):'').$Apache::lonpublisher::metadatafields{'title'});
+		        '<td colspan="3">'.($kaputt?&Apache::lonhtmlcommon::authorbombs($targetdir.'/'.$disfilename.'/'):'').$Apache::lonpublisher::metadatafields{'title'});
 	if ($Apache::lonpublisher::metadatafields{'subject'} ne '') {
 	    $r->print(' <i>'.
 		      $Apache::lonpublisher::metadatafields{'subject'}.
@@ -581,53 +669,71 @@ sub putdirectory {
 	}
 	$r->print($Apache::lonpublisher::metadatafields{'keywords'}.'</td>'.
 		  '<td>'.&Apache::lonlocal::locallocaltime($modtime).'</td>'.
+	          '<td></td>'.
 		  "</tr>\n");
     }
-    return OK;
+    return;
 }
-#
-#   Put a table row for a file resource.
-#
-sub putresource {
-    my ($r, $udom, $uname, $filename, $thisdisfn, 
-	$resdir, $targetdir, $linkdir,
-	$cmtime,$bombs,$numres) = @_;
-    &Apache::lonnet::devalidate_cache_new('meta',$targetdir.'/'.$filename);
-    my $pubstatus = 'unpublished';
-    my $status=&mt('Unpublished');
-    my $css_class='LC_browser_file';
-    my $title='&nbsp;';
-    my $publish_button=&mt('Publish');
-    my $cstr_dir = '/home/'.$uname.'/public_html/'.$thisdisfn.'/';
-#    my $action_buttons=
-#        '<br /><a target="_parent" href="/adm/cfile?action=delete&filename=/~'.
-#	$uname.'/'.$thisdisfn.'/'.$filename.'">'.
-#	&mt('Delete').'</a>';
+
+sub getTitle {
+    my ($resdir, $targetdir, $filename, $linkfilename, $meta_same, $bombs) = @_;
+    my $title='';
+    my $titleString = &getTitleString($targetdir.'/'.$filename);
     if (-e $resdir.'/'.$filename) {
-        my $same=0;
-	my ($rdev,$rino,$rmode,$rnlink,
-	    $ruid,$rgid,$rrdev,$rsize,
-	    $ratime,$rmtime,$rctime,
-	    $rblksize,$rblocks)=stat($resdir.'/'.$filename);
-        if ($rmtime>=$cmtime) {
-           $same=1;
+	$title = '<a href="'.$targetdir.'/'.$filename.
+	    '.meta" target="cat">'.$titleString.'</a>';
+        if (!$meta_same) {
+	    $title = &mt('Metadata Modified').'<br />'.$title.
+		'<br />'.
+                &Apache::loncommon::modal_link(
+                    '/adm/diff?filename='.$linkfilename.'.meta'.'&amp;versiontwo=priv',
+                    &mt('Metadata Diffs'),600,500);
+	    $title.="\n".'<br />'.
+                &Apache::loncommon::modal_link(
+                    '/adm/retrieve?filename='.$linkfilename.'.meta&amp;inhibitmenu=yes&amp;add_modal=yes',
+                    &mt('Retrieve Metadata'),600,500);
+        } 
+    }
+    # Allow editing metadata of published and unpublished resources
+    $title .= "\n".'<br />' if ($title);
+    $title .= '<a href="'.$linkfilename.'.meta">'.
+              ($$bombs{&Apache::lonnet::declutter($targetdir.'/'.$filename)}?
+                  '<img src="/adm/lonMisc/bomb.gif" border="0" alt="'.&mt('bomb').'" />':
+                  &mt('Edit Metadata')).
+              '</a>';
+
+    return ($title, $titleString);
+}
+
+
+sub isMetaSame {
+    my ($cstr_dir, $resdir, $filename) = @_;
+    my $meta_cmtime = (stat($cstr_dir.'/'.$filename.'.meta'))[9];
+    my $meta_rmtime = (stat($resdir.'/'.$filename.'.meta'))[9];
+    return (&Apache::londiff::are_different_files($resdir.'/'.$filename.'.meta',
+            $cstr_dir.'/'.$filename.'.meta') && $meta_rmtime < $meta_cmtime) 
+        ? 0 : 1;
+}
+    
+
+sub getStatus {    
+    my ($resdir, $targetdir, $cstr_dir, $filename,  
+            $linkfilename, $cmtime, $meta_same) = @_;
+    my $pubstatus = 'unpublished';
+    my $status = &mt('Unpublished');
+
+    if (-e $resdir.'/'.$filename) {
+        my $same = 0;
+        if ((stat($resdir.'/'.$filename))[9] >= $cmtime) {
+            $same = 1;
         } else {
            if (&Apache::londiff::are_different_files($resdir.'/'.$filename,
 						     $cstr_dir.'/'.$filename)) {
-              $same=0;
+              $same = 0;
            } else {
-              $same=1;
+              $same = 1;
            }
         }
-	my $meta_cmtime = (stat($cstr_dir.'/'.$filename.'.meta'))[9];
-	my $meta_rmtime = (stat($resdir.'/'.$filename.'.meta'))[9];
-	my $meta_same = 1;
-	if ($meta_rmtime < $meta_cmtime
-	    && &Apache::londiff::are_different_files($resdir.'/'.$filename.'.meta',
-						     $cstr_dir.'/'.$filename.'.meta')) {
-	    $meta_same = 0;
-	}
-	$publish_button=&mt('Re-publish');
 
         my $rights_status =
             &mt(&getCopyRightString($targetdir.'/'.$filename)).', ';
@@ -639,9 +745,6 @@ sub putresource {
         $rights_status .=
             $lt_SourceRight{&getSourceRightString($targetdir.'/'.$filename)};
 
-	$title = '<a href="/res/'.$targetdir.'/'.$filename.
-	    '.meta" target="cat">'.
-	    &getTitleString($targetdir.'/'.$filename).'</a>';
 	if ($same) {
 	    if (&Apache::lonnet::metadata($targetdir.'/'.$filename,'obsolete')) {
                 $pubstatus = 'obsolete';
@@ -655,52 +758,52 @@ sub putresource {
 		$status=&mt('Published').
 		    '<br />'. $rights_status;
 	    }
-#	    } else {
-#		$action_buttons='';
-#	    }
 	} else {
             $pubstatus = 'modified';
 	    $status=&mt('Modified').
 		'<br />'. $rights_status;
-#	    $action_buttons='';
 	    if (&Apache::loncommon::fileembstyle(($filename=~/\.(\w+)$/)) eq 'ssi') {
-		$status.='<br /><a href="/adm/diff?filename=/~'.$uname.
-		    $thisdisfn.'/'.$filename.
-		    '&amp;versiontwo=priv" target="cat">'.&mt('Diffs').'</a>';
+		$status.='<br />'.
+                         &Apache::loncommon::modal_link(
+                             '/adm/diff?filename='.$linkfilename.'&amp;versiontwo=priv',
+                             &mt('Diffs'),600,500);
 	    }
 	} 
 
-	$title.="\n".'<br /><a href="/~'.$uname.$thisdisfn.'/'.$filename.'.meta">'. 
-	    ($$bombs{$targetdir.'/'.$filename}?'<img src="/adm/lonMisc/bomb.gif" border="0" alt="'.&mt('bomb').'" />':&mt('Edit Metadata')).'</a>';
-
-	if (!$meta_same) {
-	    $title = &mt('Metadata Modified').'<br />'.$title.
-		'<br /><a href="/adm/diff?filename=/~'.$uname.
-		$thisdisfn.'/'.$filename.'.meta'.
-		'&amp;versiontwo=priv" target="cat">'.&mt('Metadata Diffs').'</a>';
-	    $title.="\n".'<br /><a href="/adm/retrieve?filename=/~'.$uname.
-		$thisdisfn.'/'.$filename.'.meta" target="_parent">'.&mt('Retrieve Metadata').'</a>';
-	}
-	$status.="\n".'<br /><a href="/adm/retrieve?filename=/~'.$uname.
-	    $thisdisfn.'/'.$filename.'" target="_parent">'.&mt('Retrieve').'</a>';
+	$status.="\n".'<br />'.
+             &Apache::loncommon::modal_link(
+                 '/adm/retrieve?filename='.$linkfilename.'&amp;inhibitmenu=yes&amp;add_modal=yes',&mt('Retrieve'),600,500);
     }
+
+    return ($status, $pubstatus);
+}
+
+
+#
+#   Put a table row for a file resource.
+#
+sub putresource {
+    my ($r, $udom, $uname, $filename, $thisdisfn, $resdir, $targetdir, 
+            $linkdir, $cmtime, $size, $numres, $linkfilename, $title, 
+            $status, $pubstatus) = @_;
+    &Apache::lonnet::devalidate_cache_new('meta',$targetdir.'/'.$filename);
+
     my $editlink='';
     my $editlink2='';
     if ($filename=~/\.(xml|html|htm|xhtml|xhtm|sty)$/) {
-	$editlink=' <br />(<a href="'.$linkdir.'/'.$filename.'?forceedit=1" target="_parent">'.&mt('Edit').'</a>)';
+	$editlink=' <br />(<a href="'.$linkdir.'/'.$filename.'?editmode=Edit&amp;problemmode=edit">'.&mt('Edit').'</a>)';
     }
-    if ($filename=~/\.(problem|exam|quiz|assess|survey|form|library)$/) {
-	$editlink=' (<a href="'.$linkdir.'/'.$filename.'?forceedit=1" target="_parent">'.&mt('EditXML').'</a>)';
-	$editlink2=' <br />(<a href="'.$linkdir.'/'.$filename.'?forceColoredit=1" target="_parent">'.&mt('Edit').'</a>)';
+    if ($filename=~/$LONCAPA::assess_re/) {
+	$editlink=' (<a href="'.$linkdir.'/'.$filename.'?editmode=Edit&amp;problemmode=editxml">'.&mt('EditXML').'</a>)';
+	$editlink2=' <br />(<a href="'.$linkdir.'/'.$filename.'?editmode=Edit&amp;problemmode=edit">'.&mt('Edit').'</a>)';
     }
     if ($filename=~/\.(problem|exam|quiz|assess|survey|form|library|xml|html|htm|xhtml|xhtm|sty)$/) {
-	$editlink.=' (<a href="/adm/cleanup?filename=/~'.$uname.
-	    $thisdisfn.'/'.$filename.'" target="_parent">'.&mt('Clean Up').')</a>';
+	$editlink.=' (<a href="/adm/cleanup?filename='.$linkfilename.'" target="_parent">'.&mt('Clean Up').')</a>';
     }
     if ($filename=~/\.(zip|tar|bz2|gz|tar.gz|tar.bz2|tgz)$/) {
-	$editlink=' (<a target="_parent" href="/adm/cfile?decompress=/~'.
-	      $uname.$thisdisfn.'/'.$filename.'">'.&mt('Decompress').'</a>)';
+	$editlink=' (<a target="_parent" href="/adm/cfile?decompress='.$linkfilename.'">'.&mt('Decompress').'</a>)';
     }
+    my $publish_button = (-e $resdir.'/'.$filename) ? &mt('Re-publish') : &mt('Publish');
     my $pub_select = '';
     &create_pubselect($r,\$pub_select,$udom,$uname,$thisdisfn,$filename,$resdir,$pubstatus,$publish_button,$numres);
     $r->print(&Apache::loncommon::start_data_table_row().
@@ -715,9 +818,10 @@ sub putresource {
               '<td class="LC_browser_file_'.$pubstatus.'">&nbsp;&nbsp;</td>'. # Display publication status
               '<td>'.$status.'</td>'.
 	      '<td>'.&Apache::lonlocal::locallocaltime($cmtime).'</td>'.
+	      '<td>'.sprintf("%.1f",$size).'</td>'.
 	      &Apache::loncommon::end_data_table_row()
     );
-    return OK;
+    return;
 }
 
 sub create_pubselect {
@@ -747,8 +851,8 @@ sub create_pubselect {
 '<option value="cleanup">'.&mt('Clean up').'</option>'.
 '<option value="print">'.&mt('Print').'</option>'.
 '</select>
-<input type="hidden" name="filename" value="/~'.
- &HTML::Entities::encode($uname.$thisdisfn.'/'.$filename,'<>&"').'" />
+<input type="hidden" name="filename" value="/priv'.
+ &HTML::Entities::encode($thisdisfn.'/'.$filename,'<>&"').'" />
  <input type="hidden" name="dispfilename" value="'.
  &HTML::Entities::encode($filename).'" /></form>';
     $$numres ++;
@@ -770,48 +874,23 @@ sub check_for_versions {
             $versions ++;        
         }
     }
+    closedir(DIR);
     return $versions;
 }
 
-#
-#   Categorize files in the directory.
-#   For each file in a list of files in a file directory, 
-#   the  file categorized as one of:
-#    - directory  
-#    - sequence
-#    - problem 
-#    - Other resource.
-#
-#   For each file the modification date is determined as well.
-#   Returned is a list of sublists:
-#    (directories, sequences, problems, other)
-#   each of the sublists contains entries of the following form (sorted by
-#   filename):
-#     (filename, typecode, lastmodtime)
-#
-#   $list = CategorizeFiles($location, $files)
-#       $location   - Directory in which the files live (relative to our
-#                     execution.
-#       $files      - list of files.
-#
-sub CategorizeFiles {
-    my $location = shift;
-    my $files    = shift;
-}
-
 1;
 __END__
 
 
 =head1 NAME
 
-Apache::lonpubdir - Construction space directory lister
+Apache::lonpubdir - Authoring space directory lister
 
 =head1 SYNOPSIS
 
 Invoked (for various locations) by /etc/httpd/conf/srm.conf:
 
- <LocationMatch "^/\~.*/$">
+ <LocationMatch "^/+priv.*/$">
  PerlAccessHandler       Apache::loncacc
  SetHandler perl-script
  PerlHandler Apache::lonpubdir
@@ -884,38 +963,21 @@ Output the header of the page.  This includes:
     $str = getTitleString($fullname);
         $fullname - Fully qualified filename to check.
 
-=item putdirectory(r, base, here, dirname, modtime)
+=item putdirectory($r, $base, $here, $dirname, $modtime, $targetdir, $bombs,
+                   $numdir)
 
     Put out a directory table row:
     
-    putdirectory($r, $base, $here, $dirname, $modtime)
-        $r       - Apache request object.
-        $reqfile - File in request.
-        $here    - Where we are in directory tree.
-        $dirname - Name of directory special file.
-        $modtime - Encoded modification time.
-
-=item CategorizeFiles($location, $files)
-    
-    Categorize files in the directory.
-    For each file in a list of files in a file directory, 
-    the  file categorized as one of:
-        - directory  
-        - sequence
-        - problem 
-        - Other resource.
-
-    For each file the modification date is determined as well.
-    Returned is a list of sublists:
-        (directories, sequences, problems, other)
-    each of the sublists contains entries of the following form (sorted by filename):
-    (filename, typecode, lastmodtime)
-    
-        $list = CategorizeFiles($location, $files)
-        $location   - Directory in which the files live (relative to our execution)
-        $files      - list of files.
+        $r        - Apache request object.
+        $reqfile  - File in request.
+        $here     - Where we are in directory tree.
+        $dirname  - Name of directory special file.
+        $modtime  - Encoded modification time.
+        targetdir - Publication target directory.
+        bombs     - Reference to hash of URLs with runtime error messages.
+        numdir    - Reference to scalar used to track number of sub-directories
+                    in directory (used in form name for each "actions" dropdown).
 
 =back
 
 =cut
-
