@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # Publication Handler
 #
-# $Id: lonpublisher.pm,v 1.267.2.1 2011/11/07 13:38:45 raeburn Exp $
+# $Id: lonpublisher.pm,v 1.289 2014/01/15 18:49:56 bisitz Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -66,10 +66,10 @@ invocation by F<loncapa_apache.conf>:
 
 =head1 OVERVIEW
 
-Authors can only write-access the C</~authorname/> space. They can
-copy resources into the resource area through the publication step,
-and move them back through a recover step. Authors do not have direct
-write-access to their resource space.
+Authors can only write-access the C</priv/domain/authorname/> space. 
+They can copy resources into the resource area through the 
+publication step, and move them back through a recover step. 
+Authors do not have direct write-access to their resource space.
 
 During the publication step, several events will be
 triggered. Metadata is gathered, where a wizard manages default
@@ -102,8 +102,6 @@ to publication space.
 Many of the undocumented subroutines implement various magical
 parsing shortcuts.
 
-=over 4
-
 =cut
 
 ######################################################################
@@ -121,7 +119,6 @@ use HTML::LCParser;
 use HTML::Entities;
 use Encode::Encoder;
 use Apache::lonxml;
-use Apache::loncacc;
 use DBI;
 use Apache::lonnet;
 use Apache::loncommon();
@@ -150,6 +147,8 @@ my $lock;
 
 =pod
 
+=over 4
+
 =item B<metaeval>
 
 Evaluates a string that contains metadata.  This subroutine
@@ -266,9 +265,9 @@ sub metaread {
     my ($logfile,$fn,$prefix)=@_;
     unless (-e $fn) {
 	print($logfile 'No file '.$fn."\n");
-        return '<div><b>'
-              .&mt('No file: [_1]'
-                  ,'</b> <tt>'.&Apache::loncfile::display($fn).'</tt></div>');
+        return '<p class="LC_warning">'
+              .&mt('No file: [_1]',&Apache::loncfile::display($fn))
+              .'</p>';
     }
     print($logfile 'Processing '.$fn."\n");
     my $metastring;
@@ -277,9 +276,9 @@ sub metaread {
 	$metastring=join('',<$metafh>);
     }
     &metaeval($metastring,$prefix);
-    return '<div><b>'
-          .&mt('Processed file: [_1]'
-              ,'</b> <tt>'.&Apache::loncfile::display($fn).'</tt></div>');
+    return '<p class="LC_info">'
+          .&mt('Processed file: [_1]',&Apache::loncfile::display($fn))
+          .'</p>';
 }
 
 #########################################
@@ -791,10 +790,55 @@ sub fix_ids_and_indices {
 		}
 		if (!$endtag) { if ($token->[4]=~m:/>$:) { $endtag=' /'; }; }
 		$outstring.='<'.$tag.$newparmstring.$endtag.'>';
-		if ($lctag eq 'm' || $lctag eq 'script' || $lctag eq 'answer' 
-                    || $lctag eq 'display' || $lctag eq 'tex') {
+		if ($lctag eq 'm' || $lctag eq 'answer' || $lctag eq 'display' ||
+                    $lctag eq 'tex') {
 		    $outstring.=&get_all_text_unbalanced('/'.$lctag,\@parser);
-		}
+                } elsif ($lctag eq 'script') {
+                    if ($parms{'type'} eq 'loncapa/perl') {
+                        $outstring.=&get_all_text_unbalanced('/'.$lctag,\@parser);
+                    } else {
+                        my $script = &get_all_text_unbalanced('/'.$lctag,\@parser);
+                        if ($script =~ m{\.set\w+(Src|Swf)\(["']}i) {
+                            my @srcs = split(/\.set/i,$script);
+                            if (scalar(@srcs) > 1) {
+                                foreach my $item (@srcs) {
+                                    if ($item =~ m{^(FlashPlayerSwf|MediaSrc|XMPSrc|ConfigurationSrc|PosterImageSrc)\((['"])(?:(?!\2).)+\2\)}is) {
+                                        my $srctype = $1;
+                                        my $quote = $2;
+                                        my ($url) = ($item =~ m{^\Q$srctype($quote\E([^$quote]+)\Q$quote)\E});
+                                        $url = &urlfixup($url);
+                                        unless ($url=~m{^(?:http|https|ftp)://}) {
+                                            $allow{&absoluteurl($url,$target)}=1;
+                                            if ($srctype eq 'ConfigurationSrc') {
+                                                if ($url =~ m{^(.+/)configuration_express\.xml$}) {
+#
+# Camtasia 8.1: express_show/spritesheet.png needed, and included in zip archive.
+# Not referenced directly in <main>.html or <main>_player.html files,
+# so add this file to %allow (where <main> is name user gave to file/archive).
+#
+                                                    my $spritesheet = $1.'express_show/spritesheet.png';
+                                                    $allow{&absoluteurl($spritesheet,$target)}=1;
+                                                }
+                                            } elsif ($srctype eq 'PosterImageSrc') {
+                                                if ($url =~ m{^(.+)_First_Frame\.png$}) {
+                                                    my $prefix = $1;
+#
+# Camtasia 8.1: <main>_Thumbnails.png needed, and included in zip archive.
+# Not referenced directly in <main>.html or <main>_player.html files,
+# so add this file to %allow (where <main> is name user gave to file/archive).
+#
+                                                    my $thumbnail = $prefix.'_Thumbnails.png';
+                                                    $allow{&absoluteurl($thumbnail,$target)}=1;
+                                                }
+                                            }
+                                        }
+                                    }
+                                }
+                            }
+                        }
+                        $outstring .= $script
+                    }
+                }
 	    } elsif ($token->[0] eq 'E') {
 		if ($token->[2]) {
 		    unless ($token->[1] eq 'allow') {
@@ -1017,6 +1061,11 @@ sub publish {
 		   $outdep.= ' - <span class="LC_error">'.&mt('Currently not available').
 		       '</span>';
                } else {
+#
+# Store the fact that the dependency has been used by the target file
+# Unfortunately, usage is erroneously named sequsage in lonmeta.pm
+# The translation happens in lonmetadata.pm
+#
                    my %temphash=(&Apache::lonnet::declutter($target).'___'.
                              &Apache::lonnet::declutter($thisdep).'___usage'
                                  => time);
@@ -1080,16 +1129,16 @@ sub publish {
 # ------------------------------------------------ Check out directory hierachy
 
         my $thisdisfn=$source;
-        $thisdisfn=~s/^\/home\/\Q$cuname\E\///;
 
-        my @urlparts=split(/\//,$thisdisfn);
+        $thisdisfn=~s/^\Q$docroot\E\/priv\/\Q$cudom\E\/\Q$cuname\E\///;
+        my @urlparts=('.',split(/\//,$thisdisfn));
         $#urlparts--;
 
-        my $currentpath='/home/'.$cuname.'/';
+        my $currentpath=$docroot.'/priv/'.$cudom.'/'.$cuname.'/';
 
 	my $prefix='../'x($#urlparts);
-        foreach (@urlparts) {
-	    $currentpath.=$_.'/';
+        foreach my $subdir (@urlparts) {
+	    $currentpath.=$subdir.'/';
             $scrout.=&metaread($logfile,$currentpath.'default.meta',$prefix);
 	    $prefix=~s|^\.\./||;
         }
@@ -1223,14 +1272,12 @@ sub publish {
     my $intr_scrout.='<br />'
                     .'<form name="pubform" action="/adm/publish" method="post">';
     unless ($env{'form.makeobsolete'}) {
-       my $thissrc=$source;
-       $thissrc=~s{^/home/($match_username)/public_html}{/priv/$1};
        $intr_scrout.='<p class="LC_warning">'
                     .&mt('Searching for your resource will be based on the following metadata. Please provide as much data as possible.')
                     .'</p>'
                     .'<p><input type="submit" value="'
                     .&mt('Finalize Publication')
-                    .'" /> <a href="'.$thissrc.'">'.&mt('Cancel').'</a></p>';
+                    .'" /> <a href="'.&Apache::loncfile::url($source).'">'.&mt('Cancel').'</a></p>';
     }
     $intr_scrout.=&Apache::lonhtmlcommon::start_pick_box();
     $intr_scrout.=
@@ -1482,6 +1529,8 @@ Returns:
 0: fail
 1: success
 
+=back
+
 =cut
 
 #'stupid emacs
@@ -1767,27 +1816,22 @@ sub phasetwo {
 
 # ------------------------------------------------ Provide link to new resource
     unless ($batch) {
-        my $thissrc=$source;
-        $thissrc=~s{^/home/($match_username)/public_html}{/priv/$1};
         
+        my $thissrc=&Apache::loncfile::url($source);
         my $thissrcdir=$thissrc;
         $thissrcdir=~s/\/[^\/]+$/\//;
         
-        $r->print(&Apache::loncommon::head_subbox(
-            &Apache::lonhtmlcommon::start_funclist().
-            &Apache::lonhtmlcommon::add_item_funclist(
+        $r->print(
+            &Apache::lonhtmlcommon::actionbox([
                 '<a href="'.$thisdistarget.'">'.
                 &mt('View Published Version').
-                '</a>').
-            &Apache::lonhtmlcommon::add_item_funclist(
+                '</a>',
                 '<a href="'.$thissrc.'">'.
                 &mt('Back to Source').
-                '</a>').
-            &Apache::lonhtmlcommon::add_item_funclist(
+                '</a>',
                 '<a href="'.$thissrcdir.'">'.
                 &mt('Back to Source Directory').
-                '</a>').
-            &Apache::lonhtmlcommon::end_funclist())
+                '</a>'])
         );
     }
     return 1;
@@ -1839,8 +1883,6 @@ sub batchpublish {
     my %oldenv=%env;
     $srcfile=~s/\/+/\//g;
     $targetfile=~s/\/+/\//g;
-    my $thisdisfn=$srcfile;
-    $thisdisfn=~s/\/home\/korte\/public_html\///;
     $srcfile=~s/\/+/\//g;
 
     my $docroot=$r->dir_config('lonDocRoot');
@@ -1857,8 +1899,7 @@ sub batchpublish {
     my $thisembstyle=&Apache::loncommon::fileembstyle($thistype);
      
     $r->print('<h2>'
-             .&mt('Publishing [_1]'
-                 ,'<span class="LC_filename">'.$thisdisfn.'</span>')
+             .&mt('Publishing [_1]',&Apache::loncfile::display($srcfile))
              .'</h2>'
     );
 
@@ -1884,15 +1925,16 @@ sub publishdirectory {
     my ($r,$fn,$thisdisfn)=@_;
     $fn=~s/\/+/\//g;
     $thisdisfn=~s/\/+/\//g;
-    my $resdir=
-	$Apache::lonnet::perlvar{'lonDocRoot'}.'/res/'.$cudom.'/'.$cuname.'/'.
-	$thisdisfn;
-    $r->print(&Apache::lonhtmlcommon::start_pick_box()
+    my $thisdisresdir=$thisdisfn;
+    $thisdisresdir=~s/^\/priv\//\/res\//;
+    my $resdir = $r->dir_config('lonDocRoot').$thisdisresdir;
+    $r->print('<form name="pubdirpref" method="post" action="">'
+             .&Apache::lonhtmlcommon::start_pick_box()
              .&Apache::lonhtmlcommon::row_title(&mt('Directory'))
             .'<span class="LC_filename">'.$thisdisfn.'</span>'
             .&Apache::lonhtmlcommon::row_closure()
             .&Apache::lonhtmlcommon::row_title(&mt('Target'))
-            .'<span class="LC_filename">'.$resdir.'</span>'
+            .'<span class="LC_filename">'.$thisdisresdir.'</span>'
     );
 
     my $dirptr=16384;		# Mask indicating a directory in stat.cmode.
@@ -1901,16 +1943,16 @@ sub publishdirectory {
         $r->print(&Apache::lonhtmlcommon::row_closure()
                  .&Apache::lonhtmlcommon::row_title(&mt('Options'))
         );
-        $r->print('<form name="pubdirpref" method="post">'.
-		  &hiddenfield('phase','two').
+        $r->print(&hiddenfield('phase','two').
 		  &hiddenfield('filename',$env{'form.filename'}).
 		  &checkbox('pubrec','include subdirectories').
 		  &checkbox('forcerepub','force republication of previously published files').
                   &checkbox('obsolete','make file(s) obsolete').
-		  &checkbox('forceoverride','force directory level metadata over existing').
-		  '<br /><input type="submit" value="'.&mt('Publish Directory').'" /></form>');
+		  &checkbox('forceoverride','force directory level metadata over existing')
+        );
         $r->print(&Apache::lonhtmlcommon::row_closure(1)
                  .&Apache::lonhtmlcommon::end_pick_box()
+                 .'<br /><input type="submit" value="'.&mt('Publish Directory').'" /></form>'
         );
         $lock=0;
     } else {
@@ -1973,12 +2015,11 @@ sub publishdirectory {
 
 sub defaultmetapublish {
     my ($r,$fn,$cuname,$cudom)=@_;
-    $fn=~s/^\/\~$cuname\//\/home\/$cuname\/public_html\//;
     unless (-e $fn) {
        return HTTP_NOT_FOUND;
     }
     my $target=$fn;
-    $target=~s/^\/home\/$cuname\/public_html\//$Apache::lonnet::perlvar{'lonDocRoot'}\/res\/$cudom\/$cuname\//;
+    $target=~s/^\Q$Apache::lonnet::perlvar{'lonDocRoot'}\E\/priv\//\Q$Apache::lonnet::perlvar{'lonDocRoot'}\E\/res\//;
 
 
     &Apache::loncommon::content_type($r,'text/html');
@@ -2021,9 +2062,7 @@ sub defaultmetapublish {
 	$r->print($reply.'</p><br />');$r->rflush;
     }
 # ------------------------------------------------------------------- Link back
-    my $link=$fn;
-    $link=~s/^\/home\/$cuname\/public_html\//\/priv\/$cuname\//;
-    $r->print("<a href='$link'>".&mt('Back to Metadata').'</a>');
+    $r->print("<a href='".&Apache::loncfile::display($fn)."'>".&mt('Back to Metadata').'</a>');
     $r->print(&Apache::loncommon::end_page());
     return OK;
 }
@@ -2087,59 +2126,34 @@ sub handler {
 # -------------------------------------------------------------- Check filename
 
     my $fn=&unescape($env{'form.filename'});
+    ($cuname,$cudom)=&Apache::lonnet::constructaccess($fn);
+# ----------------------------------------------------- Do we have permissions?
+     unless (($cuname) && ($cudom)) {
+       $r->log_reason($env{'user.name'}.' at '.$env{'user.domain'}.
+                      ' trying to publish file '.$env{'form.filename'}.
+                      ' - not authorized', 
+                      $r->filename); 
+       return HTTP_NOT_ACCEPTABLE;
+     }
+# ----------------------------------------------------------------- Get docroot
+    $docroot=$r->dir_config('lonDocRoot');
 
-    ($cuname,$cudom)=
-	&Apache::loncacc::constructaccess($fn,$r->dir_config('lonDefDomain'));
 
 # special publication: default.meta file
     if ($fn=~/\/default.meta$/) {
 	return &defaultmetapublish($r,$fn,$cuname,$cudom); 
     }
     $fn=~s/\.meta$//;
-  
+
+# sanity test on the filename 
+ 
     unless ($fn) { 
 	$r->log_reason($cuname.' at '.$cudom.
 		       ' trying to publish empty filename', $r->filename); 
 	return HTTP_NOT_FOUND;
     } 
 
-    unless (($cuname) && ($cudom)) {
-	$r->log_reason($cuname.' at '.$cudom.
-		       ' trying to publish file '.$env{'form.filename'}.
-		       ' ('.$fn.') - not authorized', 
-		       $r->filename); 
-	return HTTP_NOT_ACCEPTABLE;
-    }
-
-    my $home=&Apache::lonnet::homeserver($cuname,$cudom);
-    my $allowed=0;
-    my @ids=&Apache::lonnet::current_machine_ids();
-    foreach my $id (@ids) { if ($id eq $home) { $allowed = 1; }  }
-    unless ($allowed) {
-	$r->log_reason($cuname.' at '.$cudom.
-		       ' trying to publish file '.$env{'form.filename'}.
-		       ' ('.$fn.') - not homeserver ('.$home.')', 
-		       $r->filename); 
-	return HTTP_NOT_ACCEPTABLE;
-    }
-
-    $fn=~s{^http://[^/]+}{};
-    $fn=~s{^/~($match_username)}{/home/$1/public_html};
-
-    my $targetdir='';
-    $docroot=$r->dir_config('lonDocRoot'); 
-    if ($1 ne $cuname) {
-	$r->log_reason($cuname.' at '.$cudom.
-		       ' trying to publish unowned file '.
-		       $env{'form.filename'}.' ('.$fn.')', 
-		       $r->filename); 
-	return HTTP_NOT_ACCEPTABLE;
-    } else {
-	$targetdir=$docroot.'/res/'.$cudom;
-    }
-                                 
-  
-    unless (-e $fn) { 
+    unless (-e $docroot.$fn) { 
 	$r->log_reason($cuname.' at '.$cudom.
 		       ' trying to publish non-existing file '.
 		       $env{'form.filename'}.' ('.$fn.')', 
@@ -2177,8 +2191,8 @@ sub handler {
     # Breadcrumbs
     &Apache::lonhtmlcommon::clear_breadcrumbs();
     &Apache::lonhtmlcommon::add_breadcrumb({
-        'text'  => 'Construction Space',
-        'href'  => &Apache::loncommon::authorspace(),
+        'text'  => 'Authoring Space',
+        'href'  => &Apache::loncommon::authorspace($fn),
     });
     &Apache::lonhtmlcommon::add_breadcrumb({
         'text'  => 'Resource Publication',
@@ -2191,34 +2205,24 @@ sub handler {
     $r->print(&Apache::loncommon::start_page('Resource Publication',$js)
              .&Apache::lonhtmlcommon::breadcrumbs()
              .&Apache::loncommon::head_subbox(
-                  &Apache::loncommon::CSTR_pageheader($fn))
+                  &Apache::loncommon::CSTR_pageheader($docroot.$fn))
     );
 
-
-    my $thisfn=$fn;
-
-    my $thistarget=$thisfn;
-      
-    $thistarget=~s/^\/home/$targetdir/;
-    $thistarget=~s/\/public\_html//;
-
-    my $thisdistarget=$thistarget;
-    $thisdistarget=~s/^\Q$docroot\E//;
-
-    my $thisdisfn=$thisfn;
-    $thisdisfn=~s/^\/home\/\Q$cuname\E\/public_html\///;
+    my $thisdisfn=&HTML::Entities::encode($fn,'<>&"');
+    my $thistarget=$fn;
+    $thistarget=~s/^\/priv\//\/res\//;
+    my $thisdistarget=&HTML::Entities::encode($thistarget,'<>&"');
 
     if ($fn=~/\/$/) {
 # -------------------------------------------------------- This is a directory
-	&publishdirectory($r,$fn,$thisdisfn);
-	$r->print('<hr /><a href="/priv/'
-		  .$cuname.'/'.$thisdisfn
-		  .'">'.&mt('Return to Directory').'</a>');
-
-
+	&publishdirectory($r,$docroot.$fn,$thisdisfn);
+        $r->print(
+            '<br /><br />'.
+            &Apache::lonhtmlcommon::actionbox([
+                '<a href="'.$thisdisfn.'">'.&mt('Return to Directory').'</a>']));
     } else {
 # ---------------------- Evaluate individual file, and then output information.
-	$thisfn=~/\.(\w+)$/;
+	$fn=~/\.(\w+)$/;
 	my $thistype=$1;
 	my $thisembstyle=&Apache::loncommon::fileembstyle($thistype);
         if ($thistype eq 'page') {  $thisembstyle = 'rat'; }
@@ -2242,7 +2246,7 @@ sub handler {
                  .'<tt>'
                  );
 	$r->print(<<ENDCAPTION);
-<a href='javascript:void(window.open("/~$cuname/$thisdisfn","cat","height=300,width=500,scrollbars=1,resizable=1,menubar=0,location=1"))'>
+<a href='javascript:void(window.open("$thisdisfn","cat","height=300,width=500,scrollbars=1,resizable=1,menubar=0,location=1"))'>
 $thisdisfn</a>
 ENDCAPTION
         $r->print('</tt>'
@@ -2265,7 +2269,7 @@ ENDCAPTION
             $r->print(&Apache::lonhtmlcommon::row_closure()
                      .&Apache::lonhtmlcommon::row_title(&mt('Diffs')));
 	    $r->print(<<ENDDIFF);
-<a href='javascript:void(window.open("/adm/diff?filename=/~$cuname/$thisdisfn&versiontwo=priv","cat","height=300,width=500,scrollbars=1,resizable=1,menubar=0,location=1"))'>
+<a href='javascript:void(window.open("/adm/diff?filename=$thisdisfn&amp;versiontwo=priv","cat","height=300,width=500,scrollbars=1,resizable=1,menubar=0,location=1"))'>
 ENDDIFF
             $r->print(&mt('Diffs with Current Version').'</a>');
 	}
@@ -2274,17 +2278,17 @@ ENDDIFF
                  .&Apache::lonhtmlcommon::end_pick_box()
                  );
   
-# ------------------ Publishing from $thisfn to $thistarget with $thisembstyle.
+# ---------------------- Publishing from $fn to $thistarget with $thisembstyle.
 
 	unless ($env{'form.phase'} eq 'two') {
 # ---------------------------------------------------------- Parse for problems
 	    my ($warningcount,$errorcount);
 	    if ($thisembstyle eq 'ssi') {
-		($warningcount,$errorcount)=&checkonthis($r,$thisfn);
+		($warningcount,$errorcount)=&checkonthis($r,$fn);
 	    }
 	    unless ($errorcount) {
 		my ($outstring,$error)=
-		    &publish($thisfn,$thistarget,$thisembstyle);
+		    &publish($docroot.$fn,$docroot.$thistarget,$thisembstyle);
 		$r->print($outstring);
 	    } else {
 		$r->print('<h3 class="LC_error">'.
@@ -2292,7 +2296,7 @@ ENDDIFF
 			  '</h3>');
 	    }
 	} else {
-	    &phasetwo($r,$thisfn,$thistarget,$thisembstyle,$thisdistarget); 
+	    &phasetwo($r,$docroot.$fn,$docroot.$thistarget,$thisembstyle,$thisdistarget); 
 	}
     }
     $r->print(&Apache::loncommon::end_page());
@@ -2307,7 +2311,5 @@ __END__
 
 =back
 
-=back
-
 =cut
 
