@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # TeX Conversion Module
 #
-# $Id: lontexconvert.pm,v 1.108 2011/06/06 16:08:45 raeburn Exp $
+# $Id: lontexconvert.pm,v 1.112.2.4 2014/06/18 06:48:47 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -50,6 +50,78 @@ use LONCAPA;
 use URI::Escape;
 use IO::Socket::INET;
 
+
+#
+# Table of substitutions to unicode characters.
+#
+
+my %unicode_harpoons = (
+                        '\rightleftharpoons'  => 0x21cc,
+                      );
+
+my %unicode_translations = (
+
+# Brackets - unicode for browsers/OS which support it.
+
+    '&#63726;'             => 0x23a1,
+    '&#63727;'             => 0x23a2,
+    '&#63728;'             => 0x23a3,
+    '&#63737;'             => 0x23a4,
+    '&#63738;'             => 0x23a5,
+    '&#63739;'             => 0x23a6,
+
+#  Parens - unicode for browsers/OS which support it
+
+    '&#63723;'              => 0x239b,
+    '&#63724;'              => 0x239c,
+    '&#63725;'              => 0x239d,
+    '&#63734;'              => 0x239e,
+    '&#63735;'              => 0x239f,
+    '&#63736;'              => 0x23a0,
+
+);
+
+my %ascii_8bit_translations = (
+
+# Brackets - pure 8-bit ascii ugliness for browsers/OS which can't handle unicode
+
+    '&#63726;'              => 0x5b,
+    '&#63727;'              => 0x5b,    # '['
+    '&#63728;'              => 0x5b,
+    '&#63737;'              => 0x5d,    # ']'
+    '&#63738;'              => 0x5d,
+    '&#63739;'              => 0x5d,
+
+# Parens - pure 8-bit ascii ugliness for browsers/OS which can't handle unicode
+
+    '&#63723;'              => 0x28,
+    '&#63724;'              => 0x28,      # '('
+    '&#63725;'              => 0x28,
+    '&#63734;'              => 0x29,
+    '&#63735;'              => 0x29,      # '('
+    '&#63736;'              => 0x29,
+
+);
+
+##
+# Utility to convert elements of a string to unicode:
+#
+# @param input - Input string
+# @param pattern - Pattern to convert
+# @param unicode - Unicode to substitute for pattern.
+#
+# @return string - resulting string.
+# 
+sub unicode_subst {
+    my ($input, $pattern, $unicode) = @_;
+    
+    my $char = pack('U', $unicode);
+
+    $input =~ s/$pattern/$char/g;
+
+    return $input;
+}
+
 # ====================================================================== Header
 
 sub init_tth {
@@ -106,15 +178,37 @@ sub convert_real {
     $xmlstring=~s/^\s*\<br clear\=\"all\"/\<br/s;
     $xmlstring=~s/^\s*//;
     $xmlstring=~s/\s*$//;
-    #
-    # \rightleftharpoons is not converted by tth but maps
-    # reasonably well to &#8660;.  If we get many more of these,
-    # we're going to need to have a translation sub.
-    #
-    my $lrharpoon = pack("U", 0x21cc);
-    $xmlstring=~s/\\rightleftharpoons/$lrharpoon/g;
-
     &Apache::lonxml::end_alarm();
+
+    #
+    # Several strings produced by tth require
+    # transliteration -> unicode equivalents to render reliably
+    # in browsers. %unicode_translations and %unicode_harpoons are tables of
+    # string->substitution which we now apply. (%ascii_8bit_translations used
+    # instead for Windows XP and mobile devices.
+
+    my $use_ascii;
+    if ($env{'browser.os'} eq 'win') {
+        if (($env{'browser.osversion'}) && ($env{'browser.osversion'} < 6.0)) {
+            $use_ascii = 1;
+        }
+    }
+    if ($env{'browser.mobile'}) {
+        $use_ascii = 1;
+    }
+
+    foreach my $pattern (keys(%unicode_translations)) {
+	my $unicode = $unicode_translations{$pattern};
+	if ($use_ascii) {
+	    $unicode = $ascii_8bit_translations{$pattern};
+	}
+	$xmlstring = &unicode_subst($xmlstring, $pattern, $unicode);
+    }
+
+    foreach my $pattern (keys(%unicode_harpoons)) {
+        $xmlstring = &unicode_subst($xmlstring, $pattern, $unicode_harpoons{$pattern});
+    }
+
     return ($xmlstring,$errorstring);
 }
 
@@ -174,6 +268,37 @@ sub displaystyle {
     return 0;
 }
 
+sub MathJax_converted {
+    my $texstring=shift;
+    my $tag='math/tex;';
+    if (&displaystyle($texstring)) { $tag='math/tex; mode=display'; }
+    &clean_out_math_mode($texstring);
+    return &MathJax_header().
+      '<script type="'.$tag.'">'.$$texstring.'</script>';
+}
+
+{
+    #Relies heavily on the previous jsMath installation
+    my @MathJax_sent_header;
+    sub MathJax_reset {
+        undef(@MathJax_sent_header);
+    }
+    sub MathJax_push {
+        push(@MathJax_sent_header,0);
+    }
+    sub MathJax_header {
+        if (!@MathJax_sent_header) {
+            &Apache::lonnet::logthis("mismatched calls of MathJax_header and MathJax_process");
+            return '';
+        }
+        return '' if $MathJax_sent_header[-1];
+        $MathJax_sent_header[-1]=1;
+        return
+          '<script type="text/javascript" src="/adm/MathJax/MathJax.js?config=TeX-AMS-MML_HTMLorMML"></script>'."\n";
+    }
+}
+
+
 sub jsMath_converted {
     my $texstring=shift;
     my $tag='span';
@@ -241,6 +366,11 @@ sub init_math_support {
 	($inherit_jsmath && &jsMath_state(-2))) {
 	return &Apache::lontexconvert::jsMath_header();
     }
+    &Apache::lontexconvert::MathJax_push();
+    if (lc(&tex_engine()) eq 'mathjax') { # ||
+        #($inherit_jsmath && &jsMath_state(-2))) {
+        return &Apache::lontexconvert::MathJax_header();
+    }
     return;
 }
 
@@ -301,6 +431,8 @@ sub converted {
 	return &tth_converted($string);
     } elsif ($mode =~ /jsmath/i) {
 	return &jsMath_converted($string);
+    } elsif ($mode =~ /mathjax/i) {
+        return &MathJax_converted($string);
     } elsif ($mode =~ /mimetex/i) {
 	return &mimetex_converted($string);
     } elsif ($mode =~ /raw/i) {
@@ -380,7 +512,7 @@ sub msgtexconverted {
     foreach my $fragment (split(/(?:\&lt\;|\<)\/*algebra\s*(?:\&gt\;|\>)/i,
 				$message)) {
 	if ($tex) {
-        my $algebra = &algebra($fragment, 'web', undef, undef, undef, undef, 'tth');
+        my $algebra = &algebra($fragment, 'web', undef, undef, undef, 'tth');
 	    if ($email) {
 		$outmessage.='</pre><tt>'.$algebra.'</tt><pre>';
 		$tex=0;
@@ -465,6 +597,8 @@ sub postprocess_algebra {
     $string =~s/\\lim\\left\((.+?),(.+?),(.+?)\\right\)/\\lim_{$2\\to $3}$1/gs;
     return $string;
 }
+
+
 1;
 __END__
 
@@ -509,6 +643,8 @@ Header
 
 =item jsMath_converted()
 
+=item MathJax_converted()
+        - Mimics the jsMath functionality
 
 =item tex_engine()
 
