@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # XML Parser Module 
 #
-# $Id: lonxml.pm,v 1.505.2.7 2011/12/01 04:41:55 raeburn Exp $
+# $Id: lonxml.pm,v 1.531.2.16 2014/06/21 23:39:27 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -113,6 +113,7 @@ use Apache::lonr();
 use Apache::lonlocal;
 use Apache::lonhtmlcommon();
 use Apache::functionplotresponse();
+use Apache::lonnavmaps();
 
 #====================================   Main subroutine: xmlparse  
 
@@ -143,7 +144,7 @@ $metamode = 0;
 # turns on and of run::evaluate actually derefencing var refs
 $evaluate = 1;
 
-# data structure for eidt mode, determines what tags can go into what other tags
+# data structure for edit mode, determines what tags can go into what other tags
 %insertlist=();
 
 # stores the list of active tag namespaces
@@ -210,8 +211,12 @@ sub xmlend {
     my $discussion;
     &Apache::loncommon::get_unprocessed_cgi($ENV{'QUERY_STRING'},
 					   ['LONCAPA_INTERNAL_no_discussion']);
-    if (! exists($env{'form.LONCAPA_INTERNAL_no_discussion'}) ||
-        $env{'form.LONCAPA_INTERNAL_no_discussion'} ne 'true') {
+    if (
+           (   (!exists($env{'form.LONCAPA_INTERNAL_no_discussion'})) 
+            || ($env{'form.LONCAPA_INTERNAL_no_discussion'} ne 'true')
+           ) 
+        && ($env{'form.inhibitmenu'} ne 'yes')
+       ) {
         $discussion=&Apache::lonfeedback::list_discussion($mode,$status);
     }
     if ($target eq 'tex') {
@@ -229,21 +234,21 @@ sub tokeninputfield {
     return (<<ENDINPUTFIELD)
 <script type="text/javascript">
     function updatetoken() {
-	var comp=new Array;
+        var comp=new Array;
         var barcode=unescape(document.tokeninput.barcode.value);
         comp=barcode.split('*');
         if (typeof(comp[0])!="undefined") {
-	    document.tokeninput.codeone.value=comp[0];
-	}
+            document.tokeninput.codeone.value=comp[0];
+        }
         if (typeof(comp[1])!="undefined") {
-	    document.tokeninput.codetwo.value=comp[1];
-	}
+            document.tokeninput.codetwo.value=comp[1];
+        }
         if (typeof(comp[2])!="undefined") {
             comp[2]=comp[2].toUpperCase();
-	    document.tokeninput.codethree.value=comp[2];
-	}
+            document.tokeninput.codethree.value=comp[2];
+        }
         document.tokeninput.barcode.value='';
-    }  
+    }
 </script>
 <form method="post" name="tokeninput" action="">
 <table border="2" bgcolor="#FFFFBB">
@@ -252,7 +257,7 @@ sub tokeninputfield {
 <table>
 <tr>
 <td>Scan in Barcode</td>
-<td><input type="text" size="22" name="barcode" 
+<td><input type="text" size="22" name="barcode"
 onchange="updatetoken()"/></td>
 </tr>
 <tr><td><i>or</i> Type in DocID</td>
@@ -261,7 +266,7 @@ onchange="updatetoken()"/></td>
 <b><font size="+2">*</font></b>
 <input type="text" size="5" name="codetwo" />
 <b><font size="+2">*</font></b>
-<input type="text" size="10" name="codethree" value="$defhost" 
+<input type="text" size="10" name="codethree" value="$defhost"
 onchange="this.value=this.value.toUpperCase()" />
 </td></tr>
 </table>
@@ -275,14 +280,13 @@ ENDINPUTFIELD
 sub maketoken {
     my ($symb,$tuname,$tudom,$tcrsid)=@_;
     unless ($symb) {
-	$symb=&Apache::lonnet::symbread();
+        $symb=&Apache::lonnet::symbread();
     }
     unless ($tuname) {
-	$tuname=$env{'user.name'};
+        $tuname=$env{'user.name'};
         $tudom=$env{'user.domain'};
         $tcrsid=$env{'request.course.id'};
     }
-
     return &Apache::lonnet::checkout($symb,$tuname,$tudom,$tcrsid);
 }
 
@@ -292,10 +296,10 @@ sub printtokenheader {
 
     my ($symb,$courseid,$domain,$name) = &Apache::lonnet::whichuser();
     unless ($tsymb) {
-	$tsymb=$symb;
+        $tsymb=$symb;
     }
     unless ($tuname) {
-	$tuname=$name;
+        $tuname=$name;
         $tudom=$domain;
         $tcrsid=$courseid;
     }
@@ -304,13 +308,13 @@ sub printtokenheader {
 
     if ($target eq 'web') {
         my %idhash=&Apache::lonnet::idrget($tudom,($tuname));
-	return 
+        return
  '<img align="right" src="/cgi-bin/barcode.png?encode='.$token.'" />'.
                &mt('Checked out for').' '.$plainname.
                '<br />'.&mt('User').': '.$tuname.' at '.$tudom.
-	       '<br />'.&mt('ID').': '.$idhash{$tuname}.
-	       '<br />'.&mt('CourseID').': '.$tcrsid.
-	       '<br />'.&mt('Course').': '.$env{'course.'.$tcrsid.'.description'}.
+               '<br />'.&mt('ID').': '.$idhash{$tuname}.
+               '<br />'.&mt('CourseID').': '.$tcrsid.
+               '<br />'.&mt('Course').': '.$env{'course.'.$tcrsid.'.description'}.
                '<br />'.&mt('DocID').': '.$token.
                '<br />'.&mt('Time').': '.&Apache::lonlocal::locallocaltime().'<hr />';
     } else {
@@ -396,7 +400,12 @@ sub xmlparse {
  &clean_safespace($safeeval);
 
  if (@script_var_displays) {
-     $finaloutput .= join('',@script_var_displays);
+     if ($finaloutput =~ m{</body>\s*</html>\s*$}s) {
+         my $scriptoutput = join('',@script_var_displays);
+         $finaloutput=~s{(</body>\s*</html>)\s*$}{$scriptoutput$1}s;
+     } else {
+         $finaloutput .= join('',@script_var_displays);
+     }
      undef(@script_var_displays);
  }
  &init_state();
@@ -572,11 +581,11 @@ sub inner_xmlparse {
   }
   if (($ENV{'QUERY_STRING'}) && ($target eq 'web')) {
     $finaloutput=&afterburn($finaloutput);
-  }	    
+  }
   if ($target eq 'modified') {
 # if modfied, handle startpart and endpart
      $finaloutput=~s/\<startpartmarker[^\>]*\>(.*)\<endpartmarker[^\>]*\>/<part>$1<\/part>/gs;
-  }
+  }	    
   return $finaloutput;
 }
 
@@ -738,8 +747,9 @@ sub setup_globals {
 
 sub init_safespace {
   my ($target,$safeeval,$safehole,$safeinit) = @_;
+  $safeeval->reval('use LaTeX::Table;');
   $safeeval->deny_only(':dangerous');
-  $safeeval->reval('use Math::Complex;');
+  $safeeval->reval('use LONCAPA::LCMathComplex;');
   $safeeval->permit_only(":default");
   $safeeval->permit("entereval");
   $safeeval->permit(":base_math");
@@ -886,6 +896,10 @@ sub init_safespace {
   $safehole->wrap(\&Apache::functionplotresponse::fpr_f,$safeeval,'&fpr_f');
   $safehole->wrap(\&Apache::functionplotresponse::fpr_dfdx,$safeeval,'&fpr_dfdx');
   $safehole->wrap(\&Apache::functionplotresponse::fpr_d2fdx2,$safeeval,'&fpr_d2fdx2');
+  $safehole->wrap(\&Apache::functionplotresponse::fpr_vectorcoords,$safeeval,'&fpr_vectorcoords');
+  $safehole->wrap(\&Apache::functionplotresponse::fpr_objectcoords,$safeeval,'&fpr_objectcoords');
+  $safehole->wrap(\&Apache::functionplotresponse::fpr_vectorlength,$safeeval,'&fpr_vectorlength');
+  $safehole->wrap(\&Apache::functionplotresponse::fpr_vectorangle,$safeeval,'&fpr_vectorangle');
 
 #  use Data::Dumper;
 #  $safehole->wrap(\&Data::Dumper::Dumper,$safeeval,'&LONCAPA_INTERNAL_Dumper');
@@ -938,7 +952,8 @@ sub initialize_rndseed {
 sub default_homework_load {
     my ($safeeval)=@_;
     &Apache::lonxml::debug('Loading default_homework');
-    my $default=&Apache::lonnet::getfile('/home/httpd/html/res/adm/includes/default_homework.lcpm');
+    my $default=&Apache::lonnet::getfile($Apache::lonnet::perlvar{'lonIncludes'}.
+                                         '/default_homework.lcpm');
     if ($default eq -1) {
 	&Apache::lonxml::error("<b>Unable to find <i>default_homework.lcpm</i></b>");
     } else {
@@ -1108,7 +1123,7 @@ the current 'bubble line number' is stored in the %env element
 The value of it is stored in $Apache:lonxml::counter when live and
 stored back to env after done.
 
-=item &increment_counter($increment);
+=item &increment_counter($increment, $part_response);
 
 Increments the internal counter environment variable a specified amount
 
@@ -1442,7 +1457,9 @@ sub do_registered_ssi {
 
 sub add_script_result {
     my ($display) = @_;
-    push(@script_var_displays, $display);
+    if ($display ne '') {
+        push(@script_var_displays, $display);
+    }
 }
 
 #
@@ -1585,39 +1602,70 @@ sub renderingoptions {
            {&Apache::lonlocal::texthash
                (''        => '',
                 'tth'     => 'tth (TeX to HTML)',
-                'jsMath'  => 'jsMath',
+                'MathJax' => 'MathJax',
+  		'jsMath'  => 'jsMath',
                 'mimetex' => 'mimetex (Convert to Images)')}).
      '</span>';
     return $output;
 }
 
 sub inserteditinfo {
-      my ($filecontents, $filetype, $filename)=@_;
+      my ($filecontents,$filetype,$filename,$symb,$itemtitle,$folderpath,$uri,$action) = @_;
       $filecontents = &HTML::Entities::encode($filecontents,'<>&"');
       my $xml_help = '';
       my $initialize='';
       my $textarea_id = 'filecont';
-      my $dragmath_button;
-      my ($add_to_onload, $add_to_onresize);
+      my ($dragmath_button,$deps_button,$context,$cnum,$cdom,$add_to_onload,
+          $add_to_onresize,$init_dragmath);
       $initialize=&Apache::lonhtmlcommon::spellheader();
-      if (($filetype eq 'html') && (&Apache::lonhtmlcommon::htmlareabrowser())) {
-          my $lang = &Apache::lonhtmlcommon::htmlarea_lang();
-          my %textarea_args = (
-                                fullpage => 'true',
-                                dragmath => 'math',
-                              );
-          $initialize .= &Apache::lonhtmlcommon::htmlareaselectactive(\%textarea_args);
+      if ($filetype eq 'html') {
+          if ($env{'request.course.id'}) {
+              $cdom = $env{'course.'.$env{'request.course.id'}.'.domain'};
+              $cnum = $env{'course.'.$env{'request.course.id'}.'.num'};
+              if ($uri =~ m{^\Q/uploaded/$cdom/$cnum/portfolio/syllabus/\E}) {
+                  $context = 'syllabus';
+              }
+          }
+          if (&Apache::lonhtmlcommon::htmlareabrowser()) {
+	      my $lang = &Apache::lonhtmlcommon::htmlarea_lang();
+              my %textarea_args = (
+                                    fullpage => 'true',
+                                    dragmath => 'math',
+                                  );
+              $initialize .= &Apache::lonhtmlcommon::htmlareaselectactive(\%textarea_args);
+              if ($context eq 'syllabus') {
+                  $init_dragmath = "editmath_visibility('filecont','none')";
+              }
+          }
       }
       $initialize .= (<<FULLPAGE);
 <script type="text/javascript">
 // <![CDATA[
     function initDocument() {
 	resize_textarea('$textarea_id','LC_aftertextarea');
+        $init_dragmath
     }
 // ]]>
 </script>
 FULLPAGE
+      my $textareaclass;
       if ($filetype eq 'html') {
+          if ($context eq 'syllabus') {
+              $deps_button = &Apache::lonhtmlcommon::dependencies_button()."\n";
+              $initialize .=
+                  &Apache::lonhtmlcommon::dependencycheck_js(undef,&mt('Syllabus'),
+                                                             $uri,undef,
+                                                             "/public/$cdom/$cnum/syllabus").
+                  "\n";
+              if (&Apache::lonhtmlcommon::htmlareabrowser()) {
+                  $textareaclass = 'class="LC_richDefaultOn"';
+              }
+          } elsif ($symb || $folderpath) {
+              $deps_button = &Apache::lonhtmlcommon::dependencies_button()."\n";
+              $initialize .=
+                  &Apache::lonhtmlcommon::dependencycheck_js($symb,$itemtitle,
+                                                             undef,$folderpath,$uri)."\n";
+          }
           $dragmath_button = '<span id="math_filecont">'.&Apache::lonhtmlcommon::dragmath_button('filecont',1).'</span>';
           $initialize .= "\n".&Apache::lonhtmlcommon::dragmath_js('EditMathPopup');
       }
@@ -1625,11 +1673,14 @@ FULLPAGE
       $add_to_onresize = "resize_textarea('$textarea_id','LC_aftertextarea');";
 
       if ($filetype eq 'html') {
-	  $xml_help=&Apache::loncommon::helpLatexCheatsheet();
+          my $not_author;
+          if ($uri =~ m{^/uploaded/}) {
+              $not_author = 1;
+          }
+	  $xml_help=&Apache::loncommon::helpLatexCheatsheet(undef,undef,$not_author);
       }
 
       my $titledisplay=&display_title();
-      my $textareaclass;
       my %lt=&Apache::lonlocal::texthash('st' => 'Save and Edit',
 					 'vi' => 'Save and View',
 					 'dv' => 'Discard Edits and View',
@@ -1645,13 +1696,19 @@ FULLPAGE
               $htmlerror='<span class="LC_error">'.$htmlerror.'</span>';
           }
           if (&Apache::lonhtmlcommon::htmlareabrowser()) {
-              $textareaclass = 'class="LC_richDefaultOff"';
+              unless ($textareaclass) {
+                  $textareaclass = 'class="LC_richDefaultOff"';
+              }
           }
       }
+      my $undo;
+      unless ($uri =~ m{^/uploaded/}) {
+          $undo = '<input type="submit" name="Undo" accesskey="u" value="'.$lt{'un'}.'" />'."\n";
+      }
       my $editfooter=(<<ENDFOOTER);
 $initialize
 <a name="editsection" />
-<form $form_events method="post" name="xmledit">
+<form $form_events method="post" name="xmledit" action="$action">
   <div class="LC_edit_problem_editxml_header">
     <table class="LC_edit_problem_header_title"><tr><td>
         $filename
@@ -1661,8 +1718,7 @@ $initialize
     </table>
     <div class="LC_edit_problem_discards">
       <input type="submit" name="discardview" accesskey="d" value="$lt{'dv'}" />
-      <input type="submit" name="Undo" accesskey="u" value="$lt{'un'}" />
-      $htmlerror $dragmath_button
+      $undo $htmlerror $deps_button $dragmath_button
     </div>
     <div class="LC_edit_problem_saves">
       <input type="submit" name="savethisfile" accesskey="s" value="$lt{'st'}" />
@@ -1675,7 +1731,6 @@ $initialize
     $titledisplay
   </div>
 </form>
-</body>
 ENDFOOTER
       return ($editfooter,$add_to_onload,$add_to_onresize);;
 }
@@ -1746,6 +1801,24 @@ sub handler {
     } else {
 	$filetype='html';
     }
+    unless ($env{'request.uri'}) {
+        $env{'request.uri'}=$request->uri;
+        &Apache::loncommon::get_unprocessed_cgi($ENV{'QUERY_STRING'},
+                                                ['todocs']);
+    }
+    my ($cdom,$cnum);
+    if ($env{'request.course.id'}) {
+        $cdom = $env{'course.'.$env{'request.course.id'}.'.domain'};
+        $cnum = $env{'course.'.$env{'request.course.id'}.'.num'};
+        if ($filetype eq 'html') {
+            if ($request->uri =~ m{^\Q/uploaded/$cdom/$cnum/portfolio/syllabus/\E.+$}) {
+                if (&Apache::lonnet::allowed('mdc',$env{'request.course.id'})) {
+                    &Apache::loncommon::get_unprocessed_cgi($ENV{'QUERY_STRING'},
+                                                            ['editmode']);
+                }
+            }
+        }
+    }
     if ($filetype eq 'sty') {
         $breadcrumbtext = 'Style File Editor';
     } elsif ($filetype eq 'js') {
@@ -1845,26 +1918,22 @@ ENDNOTFOUND
                               '<tr><td><b><pre>'.$result.
                               '</pre></b></td></tr></table>';
                 }
-                if ($env{'environment.remote'} eq 'off') {
-                    my $brcrum;
-                    if ($env{'request.state'} eq 'construct') {
-                        $brcrum = [{'href' => &Apache::loncommon::authorspace(),
-                                    'text' => 'Construction Space'},
-                                   {'href' => '',
-                                    'text' => $breadcrumbtext}];
-                    } else {
-                        $brcrum = ''; # FIXME: Where are we?
-                    }
-                    my %options = ('bread_crumbs' => $brcrum,
-                                   'bgcolor'      => '#FFFFFF');
-                    $result =
-                        &Apache::loncommon::start_page(undef,undef,\%options)
-                       .$controls
-                       .$result
-                       .&Apache::loncommon::end_page();
+                my $brcrum;
+                if ($env{'request.state'} eq 'construct') {
+                    $brcrum = [{'href' => &Apache::loncommon::authorspace($request->uri),
+                                'text' => 'Authoring Space'},
+                               {'href' => '',
+                                'text' => $breadcrumbtext}];
                 } else {
-                    $result = $controls.$result;
+                    $brcrum = ''; # FIXME: Where are we?
                 }
+                my %options = ('bread_crumbs' => $brcrum,
+                               'bgcolor'      => '#FFFFFF');
+                $result =
+                    &Apache::loncommon::start_page(undef,undef,\%options)
+                   .$controls
+                   .$result
+                   .&Apache::loncommon::end_page();
             }
         }
     }
@@ -1875,11 +1944,37 @@ ENDNOTFOUND
     unless ($env{'request.state'} eq 'published') {
 	if ($env{'form.editmode'} && (!($env{'form.viewmode'})) && (!($env{'form.discardview'})))
 	{
-	    my $displayfile=$request->uri;
-	    $displayfile=~s/^\/[^\/]*//;
+            my ($displayfile,$url,$symb,$itemtitle,$action);
+	    $displayfile=$request->uri;
+            if ($request->uri =~ m{^/uploaded/}) {
+                if ($env{'request.course.id'}) {
+                    if ($request->uri =~ m{^\Q/uploaded/$cdom/$cnum/supplemental/\E}) {
+                        &Apache::loncommon::get_unprocessed_cgi($ENV{'QUERY_STRING'},
+                                                                ['folderpath','title']);
+                    } elsif ($request->uri =~ m{^\Q/uploaded/$cdom/$cnum/portfolio/syllabus/\E(.+)$}) {
+                        my $filename = $1;
+                        if ($1 eq 'loncapa.html') {
+                            $displayfile = &mt('Syllabus (minimal template)');
+                            $action = $request->uri.'?forceedit=1';
+                        } else {
+                            $displayfile = &mt('Syllabus file: [_1]',$1);
+                        }
+                        $itemtitle = &mt('Syllabus');
+                    }
+                }
+                unless ($itemtitle) {
+                    ($symb,$itemtitle,$displayfile) = 
+                        &get_courseupload_hierarchy($request->uri,
+                                                    $env{'form.folderpath'},
+                                                    $env{'form.title'});
+                }
+            } else {
+	        $displayfile=~s/^\/[^\/]*//;
+            }
 
 	    my ($edit_info, $add_to_onload, $add_to_onresize)=
-		&inserteditinfo($filecontents,$filetype,$displayfile);
+		&inserteditinfo($filecontents,$filetype,$displayfile,$symb,
+                                $itemtitle,$env{'form.folderpath'},$request->uri,$action);
 
 	    my %options = 
 		('add_entries' =>
@@ -1888,17 +1983,13 @@ ENDNOTFOUND
             my $header;
             if ($env{'request.state'} eq 'construct') {
                 $options{'bread_crumbs'} = [{
-                            'href' => &Apache::loncommon::authorspace(),
-                            'text' => 'Construction Space'},
+                            'href' => &Apache::loncommon::authorspace($request->uri),
+                            'text' => 'Authoring Space'},
                            {'href' => '',
                             'text' => $breadcrumbtext}];
                 $header = &Apache::loncommon::head_subbox(
                               &Apache::loncommon::CSTR_pageheader());
             }
-	    if ($env{'environment.remote'} ne 'off') {
-		$options{'bgcolor'}   = '#FFFFFF';
-		$options{'only_body'} = 1;
-	    }
 	    my $js =
 		&Apache::edit::js_change_detection().
 		&Apache::loncommon::resize_textarea_js();
@@ -1928,11 +2019,47 @@ sub display_title {
 	    $title = substr($title, rindex($title, '/') + 1);
 	}
         $result = "<script type='text/javascript'>top.document.title = '$title - LON-CAPA "
-                  .&mt('Construction Space')."';</script>";
+                  .&mt('Authoring Space')."';</script>";
     }
     return $result;
 }
 
+sub get_courseupload_hierarchy {
+    my ($url,$folderpath,$title) = @_;
+    my ($symb,$itemtitle,$displaypath);
+    if ($env{'request.course.id'}) {
+        if ($folderpath =~ /^supplemental/) {
+            my @folders = split(/\&/,$folderpath);
+            my @pathitems;
+            while (@folders) {
+                my $folder=shift(@folders);
+                my $foldername=shift(@folders);
+                $foldername =~ s/\:(\d*)\:(\w*)\:(\w*):(\d*)\:?(\d*)$//;
+                push(@pathitems,&unescape($foldername));
+            }
+            if ($title) {
+                push(@pathitems,&unescape($title));
+            }
+            $displaypath = join(' &raquo; ',@pathitems);
+        } else {
+            $symb = &Apache::lonnet::symbread($url);
+            my ($map,$id,$res)=&Apache::lonnet::decode_symb($symb);
+            my $navmap=Apache::lonnavmaps::navmap->new;
+            if (ref($navmap)) {
+                my $res = $navmap->getBySymb($symb);
+                if (ref($res)) {
+                    my @pathitems =
+                        &Apache::loncommon::get_folder_hierarchy($navmap,$map,1);
+                    $itemtitle = $res->compTitle();
+                    push(@pathitems,$itemtitle);
+                    $displaypath = join(' &raquo; ',@pathitems);
+                }
+            }
+        }
+    }
+    return ($symb,$itemtitle,$displaypath);
+}
+
 sub debug {
     if ($Apache::lonxml::debug eq "1") {
 	$|=1;
@@ -1949,8 +2076,9 @@ sub debug {
 }
 
 sub show_error_warn_msg {
-    if ($env{'request.filename'} eq '/home/httpd/html/res/lib/templates/simpleproblem.problem' &&
-	&Apache::lonnet::allowed('mdc',$env{'request.course.id'})) {
+    if (($env{'request.filename'} eq 
+         $Apache::lonnet::perlvar{'lonDocRoot'}.'/res/lib/templates/simpleproblem.problem') &&
+        (&Apache::lonnet::allowed('mdc',$env{'request.course.id'}))) {
 	return 1;
     }
     return (($Apache::lonxml::debug eq 1) ||
@@ -2014,7 +2142,7 @@ sub error {
 	    my @userlist;
 	    foreach (keys %users) {
 		my ($user,$domain) = split(/:/, $_);
-		push(@userlist,"$user\@$domain");
+		push(@userlist,"$user:$domain");
 		my $key=$declutter.'_'.$user.'_'.$domain;
 		my %lastnotified=&Apache::lonnet::get('nohist_xmlerrornotifications',
 						      [$key],
@@ -2037,7 +2165,7 @@ sub error {
 		$errormsg=&mt("An error occurred while processing this resource. The instructor has been notified.");
 	    }
 	}
-	push(@Apache::lonxml::error_messages,"<b>$errormsg</b> <br />");
+	push(@Apache::lonxml::error_messages,"<span class=\"LC_warning\">$errormsg</span><br />");
     }
 }
 
@@ -2082,13 +2210,18 @@ sub add_messages {
 }
 
 sub get_param {
-    my ($param,$parstack,$safeeval,$context,$case_insensitive) = @_;
+    my ($param,$parstack,$safeeval,$context,$case_insensitive, $noelide) = @_;
     if ( ! $context ) { $context = -1; }
     my $args ='';
     if ( $#$parstack > (-2-$context) ) { $args=$$parstack[$context]; }
     if ( ! $Apache::lonxml::usestyle ) {
 	$args=$Apache::lonxml::style_values.$args;
     }
+
+    if ($noelide) {
+        $args =~ s/'\$/'\\\$/g;
+    }
+
     if ( ! $args ) { return undef; }
     if ( $case_insensitive ) {
 	if ($args =~ s/(my (?:.*))(\$\Q$param\E[,\)])/$1.lc($2)/ei) {
@@ -2260,7 +2393,7 @@ sub description {
 sub helpinfo {
     my ($token)=@_;
     my $tag = &get_tag($token);
-    return ($insertlist{$tag.'.helpfile'}, $insertlist{$tag.'.helpdesc'});
+    return ($insertlist{$tag.'.helpfile'}, &mt($insertlist{$tag.'.helpdesc'}));
 }
 
 sub get_tag {
@@ -2286,7 +2419,7 @@ sub get_tag {
 =item &print_pdf_radiobutton(fieldname, value)
 
 Returns a latexline to generate a PDF-Form-Radiobutton.
-Note: Radiobuttons with equal names are automaticly grouped
+Note: Radiobuttons with equal names are automaticly grouped 
       in a selection-group.
 
 $fieldname: PDF internalname of the radiobutton(group)
