@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # <script> definiton
 #
-# $Id: scripttag.pm,v 1.151 2008/12/11 21:28:59 raeburn Exp $
+# $Id: scripttag.pm,v 1.169 2013/10/01 00:52:58 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -31,6 +31,8 @@ package Apache::scripttag;
 use strict;
 use Apache::lonnet;
 use Apache::lonlocal;
+use Apache::lonxml();
+use Apache::londefdef();
 use Apache::style();
 
 #Globals
@@ -41,7 +43,7 @@ use Apache::style();
 BEGIN {
   &Apache::lonxml::register('Apache::scripttag',
 			    ('script','scriptlib','parserlib','import',
-			     'window','display','storetc','physnet',
+			     'window','windowlink','togglebox','display','storetc','physnet',
 			     'standalone','comment','num','parse','algebra',
 			     'LONCAPA_INTERNAL_TURN_STYLE_ON',
 			     'LONCAPA_INTERNAL_TURN_STYLE_OFF'));
@@ -102,31 +104,16 @@ sub start_script {
 	    ($Apache::lonhomework::viewgrades == 'F')) {
 	    $Apache::lonxml::evaluate--;
 	    my (undef,undef,$udom,$uname)=&Apache::lonnet::whichuser();
-	    my $windowopen=&Apache::lonhtmlcommon::javascript_docopen();
-	    my $start_page =
-		&Apache::loncommon::start_page('Script Vars', undef,
-					       {'only_body' => 1,
-						'bgcolor'   => '#FFFFFF',
-						'js_ready'  => 1,});
-	    my $end_page =
-		&Apache::loncommon::end_page({'js_ready' => 1,});
-
 	    $uname =~s/\W//g;
 	    $udom  =~s/\W//g;
 	    my $function_name = 
 		join('_','LONCAPA_scriptvars',$uname,$udom,
 		     $env{'form.counter'},$Apache::lonxml::curdepth);
-	    my $script_var ="<script type=\"text/javascript\">
-// <![CDATA[
-    function $function_name() {newWindow=open('','new_W','width=500,height=500,scrollbars=1,resizable=yes');newWindow.$windowopen;newWindow.document.writeln('$start_page<pre>";
-	    my $listing=&Apache::run::dump($target,$safeeval);
-	    $listing=~s/\\/\\\\/g;
-	    $listing=~s/\'/\\\'/g;
-	    $script_var.=$listing;
-	    $script_var.= "<\\/pre>$end_page');newWindow.document.close();newWindow.focus()}
-// ]]>
-</script><a href=\"javascript:$function_name();void(0);\">".&mt('Script Vars')."</a><br />";
-	    &Apache::lonxml::add_script_result($script_var);
+            &Apache::lonxml::add_script_result(
+	             &Apache::loncommon::modal_adhoc_window($function_name,500,500,
+                            '<pre style="background-color:#ffffff;">'.
+                            &Apache::run::dump($target,$safeeval).'</pre>',
+                            &mt('Script Vars'))."<br />");
 	}
     } elsif ($target eq "edit" ) {
       #&Apache::run::run($bodytext,$safeeval);
@@ -142,6 +129,68 @@ sub start_script {
       if ($target ne "meta" && $target ne 'tex' && $target ne 'answer') {
 	  $result = $token->[4];
 	  $result.=$bodytext;
+          my $src=&Apache::lonxml::get_param('src',$parstack,$safeeval,undef,1);
+          my $url=&Apache::lonnet::hreflocation('',$env{'request.filename'});
+          my $cdom = $env{'course.'.$env{'request.course.id'}.'.domain'};
+          my $cnum = $env{'course.'.$env{'request.course.id'}.'.num'};
+          if ($src ne '') {
+              if ($src !~ m{^(/|https?://)}) {
+                  my $cleanhref = &Apache::londefdef::clean_docs_httpref($src,$url,$cdom,$cnum);
+                  if ($cleanhref) {
+                      &Apache::lonxml::extlink($cleanhref);
+                  }
+              }
+          } elsif (($type eq 'text/javascript') && ($bodytext ne '')) {
+              if ($url =~ m{^\Q/uploaded/$cdom/$cnum/docs/\E}) {
+                  if ($bodytext =~ m{\.set\w+(Src|Swf)\(["']}i) {
+                      my @srcs = split(/\.set/,$bodytext);
+                      if (scalar(@srcs) > 1) {
+                          foreach my $item (@srcs) {
+                              if ($item =~ m{^(FlashPlayerSwf|MediaSrc|XMPSrc|ConfigurationSrc|PosterImageSrc)\((['"])(?:(?!\2).)+\2\)}is) {
+                                  my $srctype = $1;
+                                  my $quote = $2;
+                                  my ($fname) = ($item =~ m{^\Q$srctype($quote\E([^$quote]+)\Q$quote)\E}); 
+                                  my $cleanhref =
+                                      &Apache::londefdef::clean_docs_httpref($fname,$url,$cdom,$cnum);
+                                  if ($cleanhref) {
+                                      &Apache::lonxml::extlink($cleanhref);
+                                      if ($srctype eq 'ConfigurationSrc') {
+                                          if ($cleanhref =~ m{^(.+/)configuration_express\.xml$}) {
+#
+# Camtasia 8.1: express_show/spritesheet.png needed, and included in zip archive.
+# Not referenced directly in <main>.html or <main>_player.html files,
+# so call lonxml::extlink() here to include httpref for the uploaded file.
+# (where <main> is name user gave to file/archive).
+#
+
+                                              my $spritesheet = $1.'express_show/spritesheet.png';
+                                              if (&Apache::lonnet::repcopy_userfile($spritesheet) eq 'ok') {
+                                                  &Apache::lonxml::extlink($spritesheet);
+                                              }
+                                          }
+                                      } elsif ($srctype eq 'PosterImageSrc') {
+                                          if ($fname =~ m{^(.+)_First_Frame\.png$}) {
+                                              my $prefix = $1;
+                                              my ($path) = ($cleanhref =~ m{^(.+/)\Q$fname\E});
+#
+# Camtasia 8.1: <main>_Thumbnails.png needed, and included in zip archive.
+# Not referenced directly in <main>.html or <main>_player.html files,
+# so call lonxml::extlink() here to include httpref for the uploaded file
+# (where <main> is name user gave to file/archive).
+#
+                                              my $thumbnail = $path.$prefix.'_Thumbnails.png';
+                                              if (&Apache::lonnet::repcopy_userfile($thumbnail) eq 'ok') {
+                                                  &Apache::lonxml::extlink($thumbnail);
+                                              }
+                                          }
+                                      }
+                                  }
+                              }
+                          }
+                      }
+                  }
+              }
+          }
       }
   }
   return $result;
@@ -303,19 +352,24 @@ sub start_window {
     if ($target eq 'web' || $target eq 'webgrade') {
 	&Apache::lonxml::startredirection;
     } elsif ($target eq 'tex') {
-	$result = '\unskip\footnote{';
+        my $printtext=&Apache::lonxml::get_param('printtext',$parstack,$safeeval);
+        if ($printtext=~/\w/) {
+# If printtext is given, do not output any intervening information
+           &Apache::lonxml::startredirection;
+        } else {
+           $result = '\unskip\footnote{';
+        }
     } elsif ($target eq 'edit') {
 	$result.=&Apache::edit::tag_start($target,$token);
 	$result.=&Apache::edit::text_arg('Text of Link:','linktext',$token,70);
 	$result.=&Apache::edit::text_arg('Height:','height',$token,5);
 	$result.=&Apache::edit::text_arg('Width:','width',$token,5);
-	$result.=&Apache::edit::text_arg('Mime Type:','mimetype',$token,5);
+        $result.=&Apache::edit::text_arg('Printed text (optional):','printtext',$token,20);
 	$result .=&Apache::edit::end_row().&Apache::edit::start_spanning_row();
     } elsif ($target eq 'modified') {
 	my $constructtag=&Apache::edit::get_new_args($token,$parstack,
 						     $safeeval,'linktext',
-						     'width','height',
-						     'mimetype');
+						     'width','height');
 	if ($constructtag) { $result=&Apache::edit::rebuild_tag($token); }
     }
     return $result;  
@@ -326,53 +380,123 @@ sub end_window {
   my $result;
   if ($target eq 'web' || $target eq 'webgrade') {
     my $output=&Apache::lonxml::endredirection;
-    $output =~ s/[\n\r]/ /g;
-#    $output = &HTML::Entities::encode($output,'<>&"\'');
-    $output =~ s/\'/\\\'/g;
     my $linktext= &Apache::lonxml::get_param('linktext',$parstack,$safeeval);
     if (!$linktext) { $linktext='<sup>*</sup>'; }
     my $width= &Apache::lonxml::get_param('width',$parstack,$safeeval);
     if (!$width) { $width='500'; }
     my $height= &Apache::lonxml::get_param('height',$parstack,$safeeval);
     if (!$height) { $height='200'; }
-    my $mimetype= &Apache::lonxml::get_param('mimetype',$parstack,$safeeval)
-	          || 'text/html';
-
-    my ($start_page,$end_page);
-    if ($mimetype eq 'text/html') {
-	$start_page =
-	    &Apache::loncommon::start_page($linktext, undef,
-					   {'only_body' => 1,
-					    'bgcolor'   => '#FFFFFF',
-					    'js_ready'  => 1,});
-	$end_page =
-	    &Apache::loncommon::end_page({'js_ready' => 1,});
-    }
-    $result = "<script type=\"text/javascript\">
-//<!--
- function LONCAPA_newwindow_$Apache::lonxml::curdepth() {
-newWindow=open(".&Apache::lonhtmlcommon::javascript_nothing().",'new_W');
-newWindow.close();
-newWindow=open(".&Apache::lonhtmlcommon::javascript_nothing().",'new_W','width=$width,height=$height,scrollbars=1');
-newWindow.".&Apache::lonhtmlcommon::javascript_docopen($mimetype).";
-newWindow.document.writeln('$start_page $output $end_page');
-newWindow.document.close();}
-//-->
-</script><a href=\"javascript:LONCAPA_newwindow_$Apache::lonxml::curdepth();void(0);\">$linktext</a>";
+    $result=&Apache::loncommon::modal_adhoc_window
+           ("LONCAPA_newwindow_$Apache::lonxml::curdepth",$width,$height,$output,$linktext);
   } elsif ($target eq 'tex') {
-      $result = '}';
+      my $printtext=&Apache::lonxml::get_param('printtext',$parstack,$safeeval);
+      if ($printtext=~/\w/) {
+# If a "printtext" is given, proceed to retrieve all intervening information and trash it
+         my $output=&Apache::lonxml::endredirection;
+# Use printtext instead
+         $result=$printtext;
+      } else {
+         $result='}';
+      }
   } else {
       $result = '';
   }
   return $result; 
 }
 
+
+sub start_windowlink {
+    my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
+    my $result = '';
+    if ($target eq 'web' || $target eq 'webgrade') {
+        &Apache::lonxml::startredirection;
+    } elsif ($target eq 'edit') {
+        $result.=&Apache::edit::tag_start($target,$token);
+        $result.=&Apache::edit::text_arg('Link:','href',$token,70);
+        $result.=&Apache::edit::text_arg('Height:','height',$token,5);
+        $result.=&Apache::edit::text_arg('Width:','width',$token,5);
+        $result .=&Apache::edit::end_row().&Apache::edit::start_spanning_row();
+    } elsif ($target eq 'modified') {
+        my $constructtag=&Apache::edit::get_new_args($token,$parstack,
+                                                     $safeeval,'href',
+                                                     'width','height');
+        if ($constructtag) { $result=&Apache::edit::rebuild_tag($token); }
+    }
+    return $result;
+}
+
+sub end_windowlink {
+  my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
+  my $result;
+  if ($target eq 'web' || $target eq 'webgrade') {
+    my $output=&Apache::lonxml::endredirection;
+    my $href= &Apache::lonxml::get_param('href',$parstack,$safeeval);
+    if (!$href) { $href='/adm/rat/empty.html'; }
+    my $width= &Apache::lonxml::get_param('width',$parstack,$safeeval);
+    if (!$width) { $width='500'; }
+    my $height= &Apache::lonxml::get_param('height',$parstack,$safeeval);
+    if (!$height) { $height='200'; }
+    $result=&Apache::loncommon::modal_link($href,$output,$width,$height);
+  } else {
+      $result = '';
+  }
+  return $result;
+}
+
+
+sub start_togglebox {
+    my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
+    my $result = '';
+    if ($target eq 'web' || $target eq 'webgrade') {
+        my $id="LONCAPA_togglebox_$Apache::lonxml::curdepth";
+        my $heading=&Apache::lonxml::get_param('heading',$parstack,$safeeval);
+        unless ($heading) { $heading=''; } else { $heading.=' '; }
+        my $showtext=&Apache::lonxml::get_param('showtext',$parstack,$safeeval);
+        my $hidetext=&Apache::lonxml::get_param('hidetext',$parstack,$safeeval);
+        my $headerbg=&Apache::lonxml::get_param('headerbg',$parstack,$safeeval);
+        $result=&Apache::loncommon::start_togglebox($id,$heading,$headerbg,$hidetext,$showtext);
+    } elsif ($target eq 'tex') {
+        my $heading=&Apache::lonxml::get_param('heading',$parstack,$safeeval);
+        unless ($heading) { $heading=''; } else { $heading.=' '; }
+        $result = "\n\n".'\fbox{{\bf '.$heading.'} \qquad '."\n";
+    } elsif ($target eq 'edit') {
+        $result.=&Apache::edit::tag_start($target,$token);
+        $result.=&Apache::edit::text_arg('Heading:','heading',$token,70);
+        $result.=&Apache::edit::text_arg('Header Background:','headerbg',$token,7);
+        $result.=&Apache::edit::text_arg('Show text:','showtext',$token,10);
+        $result.=&Apache::edit::text_arg('Hide text:','hidetext',$token,10);
+        $result .=&Apache::edit::end_row().&Apache::edit::start_spanning_row();
+    } elsif ($target eq 'modified') {
+        my $constructtag=&Apache::edit::get_new_args($token,$parstack,
+                                                     $safeeval,'heading',
+                                                     'showtext','hidetext',
+                                                     'headerbg','textbg');
+        if ($constructtag) { $result=&Apache::edit::rebuild_tag($token); }
+    }
+    return $result;
+}
+
+sub end_togglebox {
+  my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
+  my $result;
+  if ($target eq 'web' || $target eq 'webgrade') {
+    $result=&Apache::loncommon::end_togglebox();
+  } elsif ($target eq 'tex') {
+      $result = "}\n\n";
+  } else {
+      $result = '';
+  }
+  return $result;
+}
+
+
+
 sub start_import {
   my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
   my $bodytext=$$parser[$#$parser]->get_text("/import");
   my $result ="";
 
-  $bodytext=Apache::run::evaluate($bodytext,$safeeval,$$parstack[$#$parstack]);
+  $bodytext=&Apache::run::evaluate($bodytext,$safeeval,$$parstack[$#$parstack]);
 
   if ($target eq 'web' ||  $target eq 'webgrade' || $target eq 'grade' 
       || $target eq 'answer' || $target eq 'tex' || $target eq 'analyze' ) {
@@ -384,7 +508,19 @@ sub start_import {
       &Apache::lonxml::error("<b> Unable to find <i>$bodytext as $location</i> for import</b>");
       return "";
     }
-
+    my $importmode=&Apache::lonxml::get_param('importmode',$parstack,$safeeval);
+    if (($importmode eq 'problem') || ($importmode eq 'part')) {
+# We are using import to import published problems
+       if (($importmode eq 'problem') || ($file=~/<part[^<]*>/s)) {
+# We explicitly don't want this to be a separate part or the problem already has parts
+          $file=~s/^\s*<problem>/<library>/s;
+	  $file=~s/<\/problem>\s*$/<\/library>/s;
+       } else {
+# We want this to be a separate part, but it currently is not
+          $file=~s/^\s*<problem>/<library><part>/s;
+	  $file=~s/<\/problem>\s*$/<\/part><\/library>/s;
+       }
+    }
     my $dir=$location;
     $dir=~s:/[^/]*$::;
     #  &Apache::lonxml::debug("directory $dir $location file $file \n<b>END</b>\n");
@@ -397,20 +533,38 @@ sub start_import {
 
   } elsif ($target eq "edit" ) {
     $result.=&Apache::edit::tag_start($target,$token);
-    $result.=&Apache::edit::editline($token->[1],$bodytext,'',40);
+    my $location=$token->[1];
+    $location=~s/^\s*//s;
+    $location=~s/\s*$//s;
+    $result.=&Apache::edit::editline($location,$bodytext,'',40);
     $result.=&Apache::edit::browse(undef,'textnode');
+    $result.= '&nbsp;<label>'.&mt('Import as:').
+              '<select name="importmode_'.$Apache::lonxml::curdepth.'">';
+    my %options=&Apache::lonlocal::texthash(''        => 'as standard library',
+                                            'problem' => 'as problem',
+                                            'part'    => 'as problem part(s)');
+    foreach my $option (sort(keys(%options))) {
+       $result.='<option value="'.$option.'"';
+       if ($option eq &Apache::lonxml::get_param('importmode',$parstack,$safeeval)) {
+          $result.=' selected="selected"';
+       }
+       $result.='>'.$options{$option}.'</option>';
+    }
+    $result.='</select></label>';
     #FIXME this need to convert $bodytext to be a contruction space reference
     #my $location=&Apache::lonnet::filelocation($Apache::lonxml::pwd['-1'],$bodytext);
     #$result.="Click<a href=\"$location\">here</a> to edit<br />"
   } elsif ($target eq 'modified') {
-      $result=$token->[4].&Apache::edit::modifiedfield("/import",$parser);
+      &Apache::edit::get_new_args($token,$parstack,$safeeval,'importmode');
+      $result='<import id="'.$token->[2]{'id'}.'" importmode="'.$token->[2]{'importmode'}.'">';
+      $result.=&Apache::edit::modifiedfield("/import",$parser);
   } elsif ($target eq 'meta') {
     my $id= &Apache::lonxml::get_id($parstack,$safeeval);
     $result.='<import part="'.$Apache::inputtags::part;
     if ($id) {
       $result.='" id="'.$id;
     }
-    $result.='">';
+    $result.='" importmode="'.$token->[2]{'importmode'}.'">';
     $result.=$bodytext;
     $result.='</import>';
   }
