@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # Page Handler
 #
-# $Id: lonpage.pm,v 1.95 2011/07/28 02:53:33 raeburn Exp $
+# $Id: lonpage.pm,v 1.104 2014/03/08 18:27:39 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -36,6 +36,7 @@ use strict;
 use Apache::Constants qw(:common :http);
 use Apache::lonnet;
 use Apache::loncommon();
+use Apache::lonhtmlcommon;
 use Apache::lonxml();
 use Apache::lonlocal;
 use Apache::lonmenu;
@@ -214,6 +215,10 @@ sub handler {
                   my $lcm=1;
                   my $contents=0;
                   my $nforms=0;
+                  my $nuploads=0;
+                  my %turninpaths;
+                  my %multiresps;
+                  my $turninparent;
                   
                   my %ssibody=();
                   my %ssibgcolor=();
@@ -223,6 +228,7 @@ sub handler {
                   my %ssialink=();
      
                   my %cellemb=();
+                  my %cellexternal=();
 
                   my $allscript='';
                   my $allmeta='';
@@ -240,18 +246,26 @@ sub handler {
                       $lcm*=($#colcont+1)/euclid($lcm,($#colcont+1));
                       foreach (@colcont) {
                           my $src=$hash{'src_'.$_};
-			  my ($extension)=($src=~/\.(\w+)$/);
+                          my ($extension)=($src=~/\.(\w+)$/);
+			  $cellexternal{$_}=($hash{'ext_'.$_} eq 'true:');
 			  if ($hash{'encrypted_'.$_}) {
 			      $src=&Apache::lonenc::encrypted($src);
 			  }
                           $cellemb{$_}=
 			      &Apache::loncommon::fileembstyle($extension);
-                          if ($cellemb{$_} eq 'ssi') {
+                          if ($cellexternal{$_}) {
+                              unless (($target eq 'tex') || ($target eq 'tex_answer')) {
+                                  $ssibody{$_} = <<ENDEXT;
+<iframe src="$src" width="100%">No iframe support!</iframe>
+ENDEXT
+                              }
+                          } elsif ($cellemb{$_} eq 'ssi') {
 # --------------------------------------------------------- This is an SSI cell
 			      my ($mapid,$resid)=split(/\./,$_);
 			      my $symb=&Apache::lonnet::encode_symb($hash{'map_id_'.$mapid},$resid,$src);
 			      
 			      my $prefix=$_.'_';
+                              my $idprefix= join('_',($mapid,$resid,''));
                               my %posthash=('request.prefix' => $prefix,
 					    'LONCAPA_INTERNAL_no_discussion' => 'true',
 					    'symb' => $symb);
@@ -352,8 +366,23 @@ sub handler {
 				  $nforms++;
                                   $output=~s/\<form[^\>]*\>//gsi;
                                   $output=~s/\<\/form[^\>]*\>//gsi;
+                                  if ($output=~/\<input[^\>]+name\s*=\s*[\'\"]*HWFILE/) {
+                                      $nuploads++;
+                                  }
                                   $output=~
 				      s/\<((?:input|select|button|textarea)[^\>]+)name\s*\=\s*[\'\"]*([^\'\"]+)[\'\"]*([^\>]*)\>/\<$1 name="$prefix$2" $3\>/gsi;
+                                  $output=~
+                                      s/\<((?:input|select|button|textarea)[^\>]+)id\s*\=\s*[\'\"]*([^\'\"]+)[\'\"]*([^\>]*)\>/\<$1 id="$idprefix$2" $3\>/gsi;
+                                  if ($nuploads) {
+                                      $output=~
+                                          s/\<(input[^\>]+name=\"\Q$prefix\EHWFILE[^\>]+)\s*id\s*\=\s*[\'\"]*([^\'\"]+)[\'\"]*([^\)]*)\>/\<$1 id="$prefix$2" $3\>/gsi;
+                                       ($turninpaths{$prefix},$multiresps{$prefix}) = 
+                                           &Apache::loncommon::get_turnedin_filepath($symb,$env{'user.name'},$env{'user.domain'});
+                                       if ($turninparent eq '') {
+                                           $turninparent = $turninpaths{$prefix};
+                                           $turninparent =~ s{(/[^/]+)$}{}; 
+                                       }
+                                  }
                                   $output=~
                                       s/\<((?:input|select)[^\>]+\Qjavascript:setSubmittedPart\E)\(\s*[\'\"]([^\'\"]+)[\'\"]*\s*\)/\<$1('$2','$prefix')/gsi;
                               }
@@ -403,15 +432,33 @@ sub handler {
 				  $allscript.
 				  "\n</script>\n";
 			  }
+                          if (($nforms) && ($nuploads)) {
+                              $allscript .= &Apache::lonhtmlcommon::file_submissionchk_js(\%turninpaths,\%multiresps);
+                          }
+                          if (($nforms) && (&Apache::lonhtmlcommon::htmlareabrowser())) {
+                              my %textarea_args = (
+                                  dragmath => 'math',
+                              );
+                              $allscript .= &Apache::lonhtmlcommon::htmlareaselectactive(\%textarea_args);
+                          }
 # ------------------------------------------------------------------ Start body
 			  $r->print(&Apache::loncommon::start_page(undef,$allscript,
 								   {'force_register' => 1,
 								    'bgcolor'        => '#ffffff',}));
 # ------------------------------------------------------------------ Start form
 			  if ($nforms) {
-			      $r->print('<form name="lonhomework" method="post"  enctype="multipart/form-data" action="'.
+			      my $fmtag = '<form name="lonhomework" method="post"  enctype="multipart/form-data"';
+                              if ($nuploads) {
+                                  my $multi;
+                                  if ($nuploads > 1) {
+                                      $multi = 1;
+                                  }
+                                  $fmtag .= 'onsubmit="return file_submission_check(this,'."'$turninparent','$multi'".');"';
+                              }
+                              $fmtag .= ' action="'.
 					&Apache::lonenc::check_encrypt($requrl)
-					.'">');
+					.'">';
+                              $r->print($fmtag);
 			  }
 		      } elsif (($target eq 'tex') || ($target eq 'tex_answer')) {
 			  #  I think this is not needed as the header
@@ -443,12 +490,11 @@ sub handler {
                           my $avespan=$lcm/($#colcont+1);
                           for ($j=0;$j<=$#colcont;$j++) {
                               my $rid=$colcont[$j];
-
 			      my $metainfo =&get_buttons(\%hash,$rid).'<br />';
 			    unless (($target eq 'tex') || ($target eq 'tex_answer')) {
 				$r->print('<td colspan="'.$avespan.'"');
 			    }
-                              if ($cellemb{$rid} eq 'ssi') {
+                              if (($cellemb{$rid} eq 'ssi') || ($cellexternal{$rid})) {
 				  unless (($target eq 'tex') || ($target eq 'tex_answer')) {
 				      if ($ssibgcolor{$rid}) {
 					  $r->print(' bgcolor="'.
@@ -470,7 +516,10 @@ sub handler {
 				      }             
 				      $r->print('>');
 				  }
-                                  $r->print($ssibody{$rid});	
+                                  unless (($cellexternal{$rid}) && 
+                                          ($target eq 'tex') && ($target eq 'tex_answer')) {
+                                      $r->print($ssibody{$rid});
+                                  }
 				  unless (($target eq 'tex') || ($target eq 'tex_answer')) {
 				      $r->print('</font>');
                                   }
@@ -488,9 +537,15 @@ sub handler {
                                   $r->print('>'.$metainfo.'<embed src="'.
                                     $hash{'src_'.$rid}.'"></embed>');
                               } elsif (&Apache::lonnet::declutter($hash{'src_'.$rid}) !~/\.(sequence|page)$/) {
-                                  $r->print($metainfo.'<b>'.$hash{'title_'.$rid}.'</b><br />'.
-                                  &mt('It is recommended that you use an up-to-date virus scanner before handling this file.').'</p><p><table>'.
-                                  &Apache::londocs::entryline(0,&mt("Click to download or use your browser's Save Link function"),'/'.&Apache::lonnet::declutter($hash{'src_'.$rid})).'</table></p><br />');
+                                  $r->print($metainfo.'<b>'.$hash{'title_'.$rid}.'</b><br />');
+                                  unless ($cellemb{$rid} eq 'wrp') {
+                                      $r->print(&mt('It is recommended that you use an up-to-date virus scanner before handling this file.'));
+                                  }
+                                  $r->print('</p><p><table>'.
+                                            &Apache::londocs::entryline(0,
+                                                                        &mt("Click to download or use your browser's Save Link function"),
+                                                                        '/'.&Apache::lonnet::declutter($hash{'src_'.$rid})).
+                                                                        '</table></p><br />');
                               }
 			      unless (($target eq 'tex') || ($target eq 'tex_answer')) {
 				  $r->print('</td>');
@@ -515,7 +570,7 @@ sub handler {
 # ---------------------------------------------------------------- Submit, etc.
                       if ($nforms) {
                           $r->print(
-	                  '<input name="all_submit" value="Submit All" type="'.
+	                  '<input name="all_submit" value="'.&mt('Submit All').'" type="'.
 			  (($nforms>1)?'submit':'hidden').'"></input></form>');
                       }
 		      unless (($target eq 'tex') || ($target eq 'tex_answer')) {
@@ -566,20 +621,28 @@ sub get_buttons {
 	$esrc=&Apache::lonenc::encrypted($esrc);
     }
     if ($hash->{'src_'.$rid} !~ m-^/uploaded/-
+        && $hash->{'src_'.$rid} !~ m{^https?://}
 	&& !$env{'request.enc'}
 	&& ($env{'request.role.adv'}
 	    || !$hash->{'encrypted_'.$rid})) { 
 	$metainfo .='<a name="'.&escape($symb).'" />'.
 	    '<a href="'.$hash->{'src_'.$rid}.'.meta'.'" target="LONcatInfo">'.
-	    '<img src="/adm/lonMisc/cat_button.png" class="LC_icon" />'.
+            '<img src="/res/adm/pages/catalog.png" class="LC_icon"'.
+            ' alt="'.&mt('Show Metadata').'"'.
+            ' title="'.&mt('Show Metadata').'" />'.
 	    '</a>';
     }
-    $metainfo .= '<a href="/adm/evaluate?postdata='.
-	&escape($esrc).
-	'" target="LONcatInfo">'.
-	'<img src="/adm/lonMisc/eval_button.png" class="LC_icon" />'.
-	'</a>';
-    if (($hash->{'src_'.$rid}=~/\.(problem|exam|quiz|assess|survey|form)$/) &&
+    if (($hash->{'src_'.$rid} !~ m{^/uploaded/}) &&
+        ($hash->{'src_'.$rid} !~ m{^https?://})) {
+        $metainfo .= '<a href="/adm/evaluate?postdata='.
+	    &escape($esrc).
+	    '" target="LONcatInfo">'.
+            '<img src="/res/adm/pages/eval.png" class="LC_icon"'.
+            ' alt="'.&mt('Provide my evaluation of this resource').'"'.
+            ' title="'.&mt('Provide my evaluation of this resource').'" />'.
+	    '</a>';
+    }
+    if (($hash->{'src_'.$rid}=~/$LONCAPA::assess_re/) &&
 	($hash->{'src_'.$rid} !~ m-^/uploaded/-)) {
 
 	if (&Apache::lonnet::allowed('mgr',$env{'request.course.id'})) {
@@ -587,12 +650,16 @@ sub get_buttons {
 		'<a href="/adm/grades?symb='.&escape($symb).
 #               '&command=submission" target="LONcatInfo">'.
 		'&command=submission">'.
-		'<img src="/adm/lonMisc/subm_button.png" class="LC_icon" />'.
+                '<img src="/adm/lonMisc/subm_button.png" class="LC_icon"'.
+                ' alt="'.&mt('View Submissions for a Student or a Group of Students').'"'.
+                ' title="'.&mt('View Submissions for a Student or a Group of Students').'" />'.
 		'</a>'.
 		'<a href="/adm/grades?symb='.&escape($symb).
 #               '&command=gradingmenu" target="LONcatInfo">'.
 		'&command=gradingmenu">'.
-		'<img src="/adm/lonMisc/pgrd_button.png" class="LC_icon" />'.
+                '<img src="/res/adm/pages/pgrd.png" class="LC_icon"'.
+                ' alt="'.&mt('Content Grades').'"'.
+                ' title="'.&mt('Content Grades').'" />'.
 		'</a>';
 	}
 	if (&Apache::lonnet::allowed('opa',$env{'request.course.id'})) {
@@ -600,10 +667,32 @@ sub get_buttons {
 		'<a href="/adm/parmset?symb='.&escape($symb).
 #               '" target="LONcatInfo">'.
 		'" >'.
-		'<img src="/adm/lonMisc/pprm_button.png" class="LC_icon" />'.
+                '<img src="/adm/lonMisc/pprm_button.png" class="LC_icon"'.
+                ' alt="'.&mt('Content Settings').'"'.
+                ' title="'.&mt('Content Settings').'" />'.
 		'</a>';
 	}
     }
+    if (($env{'request.course.id'}) && (&Apache::lonnet::allowed('mdc',$env{'request.course.id'}))) {
+        my $cdom = $env{'course.'.$env{'request.course.id'}.'.domain'};
+        my $cnum = $env{'course.'.$env{'request.course.id'}.'.num'};
+        my $file=&Apache::lonnet::declutter($hash->{'src_'.$rid});
+        my ($cfile,$home,$switchserver,$forceedit,$forceview) =
+            &Apache::lonnet::can_edit_resource($file,$cnum,$cdom,$hash->{'src_'.$rid},$symb);
+        if ($cfile ne '') {
+            my $jscall = &Apache::lonhtmlcommon::jump_to_editres($cfile,$home,$switchserver,
+                                                                 $forceedit,1,$symb,undef,
+                                                                 &escape($env{'form.title'}));
+            if ($jscall) {
+                my $icon = 'pcstr.png';
+                my $label = &mt('Edit');
+                my $title = &mt('Edit this resource');
+                my $pic = '<img src="'.&Apache::loncommon::lonhttpdurl('/res/adm/pages/'.$icon).'"'.
+                          ' class="LC_icon" alt="'.$label.'" title="'.$title.'" />';
+                $metainfo .= '&nbsp;<a href="javascript:'.$jscall.';">'.$pic.'</a>';
+            }
+        }
+    }
     return $metainfo;
 }
 
