@@ -2,7 +2,7 @@
 #
 # Page flip handler
 #
-# $Id: lonpageflip.pm,v 1.79 2009/04/23 17:31:25 bisitz Exp $
+# $Id: lonpageflip.pm,v 1.80.8.6 2014/06/26 18:28:25 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -36,6 +36,7 @@ use LONCAPA;
 use Apache::Constants qw(:common :http REDIRECT);
 use Apache::lonnet;
 use Apache::loncommon();
+use Apache::lonlocal;
 use HTML::TokeParser;
 use GDBM_File;
 
@@ -217,26 +218,6 @@ sub get_next_possible_move {
               return ($next,$mapurl);
 }
 
-sub navlaunch {
-    my ($r)=@_;
-    &Apache::loncommon::content_type($r,'text/html');
-    &Apache::loncommon::no_cache($r);
-    $r->send_http_header;
-    $r->print(&Apache::loncommon::start_page('Launched'));   
-    $r->print(<<ENDNAV);
-    <p><a href="/adm/flip?postdata=firstres%3a">Goto first resource</a></p>
-    <script type="text/javascript">
-	function collapse() {
-	    menu=window.open("/adm/navmaps?collapseExternal","loncapanav",
-			     "height=600,width=400,scrollbars=1");
-	    this.document.location='/adm/navmaps?turningOffExternal';
-	}
-    </script>
-    <p><a href="javascript:collapse();">Collapse external navigation window</a></p>
-ENDNAV
-    $r->print(&Apache::loncommon::end_page());
-}
-
 sub first_accessible_resource {
     my $furl;
     if (tie(%hash,'GDBM_File',$env{'request.course.fn'}.'.db',
@@ -302,7 +283,7 @@ sub handler {
 	     
 	  return REDIRECT;
       }
-      if ($direction eq 'return' || $direction eq 'navlaunch') {
+      if ($direction eq 'return') { 
 # -------------------------------------------------------- Return to last known
          my $last;
          if (tie(%hash,'GDBM_File',$env{'request.course.fn'}.'_symb.db',
@@ -325,17 +306,12 @@ sub handler {
             untie %hash;
          } else {
 	    $newloc='/adm/navmaps';
-         }  
-	 if ($newloc eq '/adm/navmaps' && $direction eq 'navlaunch') {
-	     &navlaunch($r);
-	     return OK;
-	 } else {
-	     &Apache::loncommon::content_type($r,'text/html');
-	     $r->header_out(Location => 
-			    &Apache::lonnet::absolute_url().$newloc);
+         }
+	 &Apache::loncommon::content_type($r,'text/html');
+	 $r->header_out(Location => 
+			&Apache::lonnet::absolute_url().$newloc);
 	     
-	     return REDIRECT;
-	 }
+	 return REDIRECT;
       }
       $currenturl=~s/^https?\:\/\///;
       $currenturl=~s/^[^\/]+//;
@@ -352,16 +328,11 @@ sub handler {
          if ($last) {
 	     $currenturl=&Apache::lonnet::clutter((&Apache::lonnet::decode_symb($last))[2]);
 	 } else {
-	     if ($direction eq 'return') {
-		 &Apache::loncommon::content_type($r,'text/html');
-		 $r->header_out(Location => 
-				&Apache::lonnet::absolute_url().
-				'/adm/noidea.html');
-		 return REDIRECT;
-	     } else {
-		 &navlaunch($r);
-		 return OK;
-	     }
+	     &Apache::loncommon::content_type($r,'text/html');
+	     $r->header_out(Location => 
+			    &Apache::lonnet::absolute_url().
+			    '/adm/noidea.html');
+	     return REDIRECT;
          }
       }
 # ------------------------------------------- Do we have any idea where we are?
@@ -434,24 +405,13 @@ sub handler {
 		  if ($direction eq 'back') {
                      &Apache::lonnet::linklog($redirecturl,$currenturl);
 		  }
-# ------------------------------------------------- Check for critical messages
-		  if ((time-$env{'user.criticalcheck.time'})>300) {
-                     my @what=&Apache::lonnet::dump
-                                  ('critical',$env{'user.domain'},
-                                              $env{'user.name'});
-                     if ($what[0]) {
-	                if (($what[0] ne 'con_lost') && 
-                            ($what[0]!~/^error\:/)) {
-	                   $redirecturl='/adm/email?critical=display';
-			   $redirectsymb='';
-                        }
-                     }
-                     &Apache::lonnet::appenv({'user.criticalcheck.time'=>time});
-		  }
-
-		  &Apache::loncommon::content_type($r,'text/html');
-		  my $url=&Apache::lonnet::absolute_url().$redirecturl;
-		  $url = &add_get_param($url, { 'symb' => $redirectsymb});
+# ------------------------------------- Check for and display critical messages
+                  my ($redirect, $url) = &Apache::loncommon::critical_redirect(300);
+                  unless ($redirect) {
+                      $url=&Apache::lonnet::absolute_url().$redirecturl;
+                      $url = &add_get_param($url, { 'symb' => $redirectsymb});
+                  }
+                  &Apache::loncommon::content_type($r,'text/html');
                   $r->header_out(Location => $url);
                   return REDIRECT;
 	      } else {
@@ -462,13 +422,16 @@ sub handler {
 						     'explain' =>
 						     'You have reached the end of the sequence of materials.',
 						     'back' => 'Go Back',
-						     'nav' => 'Navigate Course Content',
+						     'nav' => 'Course Contents',
 						     'wherenext' =>
 						     'There are several possibilities of where to go next',
 						     'pick' =>
 						     'Please click on the the resource you intend to access',
 						     'titleheader' => 'Title',
 						     'type' => 'Type');
+                  if (&Apache::loncommon::course_type() eq 'Community') {
+                      $lt{'nav'} = &mt('Community Contents');
+                  }
                   if ($#possibilities>0) {
 		      my $start_page=
 			  &Apache::loncommon::start_page('Multiple Resources');
@@ -494,21 +457,17 @@ ENDSTART
                      }
                      $r->print('</table>');
                   } else {
-		      my $start_page=
-			  &Apache::loncommon::start_page('No Resource');
-		      $r->print(<<ENDNONE);
-$start_page
-<h3>$lt{'title'}</h3>
-<p>$lt{'explain'}</p>
-ENDNONE
+                      $r->print(
+                          &Apache::loncommon::start_page('No Resource')
+                         .'<h2>'.$lt{'title'}.'</h2>'
+                         .'<p>'.$lt{'explain'}.'</p>');
 		  }
-		  $r->print(<<ENDMENU);
-<ul>
-<li><a href="/adm/flip?postdata=return:">$lt{'back'}</a></li>
-<li><a href="/adm/navmaps">$lt{'nav'}</a></li>
-</ul>
-ENDMENU
-                  $r->print(&Apache::loncommon::end_page());
+                  $r->print(
+                      &Apache::lonhtmlcommon::actionbox(
+                          ['<a href="/adm/flip?postdata=return:">'.$lt{'back'}.'</a></li>',
+                           '<a href="/adm/navmaps">'.$lt{'nav'}.'</a></li>'
+                          ])
+                     .&Apache::loncommon::end_page());
                   return OK;
 	      }
 	  } else {
@@ -519,6 +478,7 @@ ENDMENU
       } else {
 # ---------------------------------------- No, could not determine where we are
 	  $r->internal_redirect('/adm/ambiguous');
+          return OK;
       }
   } else {
 # -------------------------- Class was not initialized or page fliped strangely
@@ -561,8 +521,6 @@ described at http://www.lon-capa.org.
 
 =item get_next_possible_move()
 
-=item navlaunch()
-
 =item first_accessible_resource()
 
 =item handler()
