@@ -2,7 +2,7 @@
 # Build menu bar for Advanced RAT. Uses javascript code originally in
 # static file: rat/client/code.html, now in  rat/client/ratcode.js  
 #
-# $Id: lonratmenu.pm,v 1.15 2009/12/04 15:13:04 bisitz Exp $
+# $Id: lonratmenu.pm,v 1.18.8.3 2013/06/05 15:30:52 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -47,8 +47,8 @@ sub handler {
     $readfile=~s/\/loadonly\/adveditmenu$//;
 
     # Breadcrumbs
-    my $brcrum = [{'href' => &Apache::loncommon::authorspace(),
-                   'text' => 'Construction Space'},
+    my $brcrum = [{'href' => &Apache::loncommon::authorspace($r->uri),
+                   'text' => 'Authoring Space'},
                   {'href' => '',
                    'text' => 'RAT'},
                   {'href' => '',
@@ -108,8 +108,8 @@ sub handler {
            '<a href="javascript:groupimport();">'
           .&mt('Import').'</a>')
       .&Apache::lonhtmlcommon::add_item_funclist(
-           '<a href="javascript:bookmarkopen();">'
-          .&mt('Import Bookmarks').'</a>')
+           '<a href="javascript:open_StoredLinks_Import('."'advanced'".');">'
+          .&mt('Import from Stored Links').'</a>')
       .&Apache::lonhtmlcommon::add_item_funclist(
            '<a href="javascript:groupcopy();">'
           .&mt('Copy').'</a>')
